/* Ast.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_ast_
#define _H_qu_ast_ 1


/** Ast
	The final Ast class.
*/
extern qu_class QuAstClass;


/** qu_ast
	qu_ast represents an instance of a Ast class.
	It is a pointer to the instance struct qu_ast_s.

	We could create tons of Ast subclasses but that will need a tremendous
	amount of code just to print them. So we don't do it. Until know this
	only Ast class works just fine.

	@warn
	v{cursor} needs to point to the source code String thus the String must
	be referenced. The root Ast and the source code String is stored in
	the corresponding ={Module} object.

	@p
	The v{line} field is there so that we don't need to look it up when
	printing an error message or the Ast. This is important if you consider
	that a program might call C{eval} a lot of times.
*/
struct qu_ast_s
{
	QU_OBJECT_HEAD;
	qu_ast      next;   /* Sibling node */
	qu_ast      child;  /* First child node */
	int         type;   /* One of QU_AST_XXX */
	const char *cursor; /* Position in source code */
	long        line;   /* Line number */
	long        op;     /* Flag, operator, etc. */
	qu_any      thing;  /* Any object */
};


/** qu_ast_is
	Whether or not the object is a Ast.
*/
#define qu_ast_is(o) (qu_classof (o) == QuAstClass)


/** qu_ast_newr qu_ast_newp
	Create a new qu_ast object.

	v{type} must be one of #{QU_AST_XXX}.

	v{cursor} is the cursor position in the source code.

	v{line} is the line number.

	Use qu_ast_newr to create a "rule" node.
	This node does not contain information about the cursor position in the
	source code.

	Use qu_ast_newp to create a "production" node.
	This node contains information about the cursor position in the source code.

	The theory is that we should never display error messages using "rule" nodes.
	The two functions is more efficient than just one.
*/
extern qu_ast qu_ast_newr (int type);
extern qu_ast qu_ast_newp (int type, const char *cursor, long line);


/** qu_ast_name
	Get name of a specified node type.
*/
extern qu_string qu_ast_name (int type);


/** QU_AST
	These are the node types.
*/
#define QU_AST_IN_HEADER
#define n_(name) QU_AST_##name,
enum
{
#include "Asts.h"
};
#undef n_
#undef QU_AST_IN_HEADER


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_ast__setup ();
extern void qu_ast__init  ();

#endif /*_H_qu_ast_*/
