/* Class.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_class_
#define _H_qu_class_ 1


/** Class qu_class
	The final Class class.
*/
extern qu_class QuClassClass;


/** qu_class_s
	This represents an instance of the Class object, a k{class}.

	v{accel} is used for fast C classes attribute lookup. It is automatically
	built by ={qu_class_enddecl}. You must never set it yourself.

	v{init} is set by ={qu_class_method} to speed-up things a bit, i.e. if set
	then we don't need to look it up.

	v{get} and v{set} are used for C classes and may be set by the C classes
	explicitly. They are the get/set attribute functions called at run-time.
	Defaults values are set when created. They may never be NULL.
*/
typedef qu_any (*qu_attrget_f) (qu_any self, qu_string name, int restrict);
typedef qu_any (*qu_attrset_f) (qu_any self, qu_string name, qu_any val, int restrict);

typedef struct qu_claccel_s qu_claccel_s;
struct qu_class_s
{
	QU_SYMBOL_HEAD;
	long          id;      /* class ID - one of QU_CLASS_ID_XXX*/
	qu_class      super;   /* Super class, 0 if none */
	size_t        size;    /* size of instance struct */
	qu_destroy_f  dtor;    /* free allocated memory, may be NULL */
	qu_mark_f     mark;    /* mark referenced objects, may be NULL */
	qu_claccel_s *accel;   /* Initially NULL, see below */
	qu_array      mixin;   /* Mixin classes, initially NULL */
	qu_attrget_f  get;     /* Get instance attribute, never NULL */
	qu_attrset_f  set;     /* set instance attribute, never NULL */
	qu_sub        init;    /* sub __init, borrowed from attr, may be NULL */
	qu_sub        fnew;    /* sub class new, borrowed from attr, never NULL */
};

struct qu_claccel_s
{
	qu_sub1_f fas;       /* sub __as       (x)       -> Any      */
	qu_sub0_f fclear;    /* sub __clear              -> self     */
	qu_sub1_f fcmp;      /* sub __cmp      (x)       -> Int      */
	qu_sub0_f fcopy;     /* sub __copy               -> Any      */
	qu_sub0_f fdecr;     /* sub __decr               -> Any      */
	qu_sub1_f fdel;      /* sub __del      (x)       -> Int      */
	qu_sub1_f fdiv;      /* sub __div      (x)       -> Any      */
	qu_sub0_f feach;     /* sub __each               -> Iterator */
	qu_sub1_f feq;       /* sub __eq       (x)       -> Boolean  */
	qu_sub1_f fformat;   /* sub __format   (f)       -> String   */
	qu_sub1_f fgetitem;  /* sub __getitem  (i)       -> Any      */
	qu_sub2_f fgetslice; /* sub __getslice (i, l)    -> Any      */
	qu_sub0_f fhash;     /* sub __hash               -> Int      */
	qu_sub1_f fiand;     /* sub __iand     (x)       -> Any      */
	qu_sub1_f fidiv;     /* sub __idiv     (x)       -> Integer  */
	qu_sub1_f fin;       /* sub __in       (x)       -> Boolean  */
	qu_sub0_f fincr;     /* sub __incr               -> Any      */
	qu_subn_f finit;     /* sub __init     (*arg)    -> Null     */
	qu_sub0_f finot;     /* sub __inot               -> Any      */
	qu_sub1_f fior;      /* sub __ior      (x)       -> Any      */
	qu_sub1_f fixor;     /* sub __ixor     (x)       -> Any      */
	qu_sub0_f flen;      /* sub __len                -> Int      */
	qu_sub1_f flike;     /* sub __like     (x)       -> Boolean  */
	qu_sub1_f flshift;   /* sub __lshift   (x)       -> Any      */
	qu_sub1_f fminus;    /* sub __minus    (x)       -> Any      */
	qu_sub1_f fmul;      /* sub __mul      (x)       -> Any      */
	qu_sub0_f fneg;      /* sub __neg                -> Any      */
	qu_sub1_f fof;       /* sub __of       (x)       -> Any      */
	qu_sub1_f fplus;     /* sub __plus     (x)       -> Any      */
	qu_sub0_f fpop;      /* sub __pop                -> Any      */
	qu_sub0_f fpos;      /* sub __pos                -> Any      */
	qu_sub1_f fpow;      /* sub __pow      (x)       -> Any      */
	qu_sub0_f fprint;    /* sub __print              -> Any      */
	qu_sub1_f fpush;     /* sub __push     (x)       -> self     */
	qu_sub1_f frem;      /* sub __rem      (x)       -> Any      */
	qu_sub1_f frshift;   /* sub __rshift   (x)       -> Any      */
	qu_sub2_f fsetitem;  /* sub __setitem  (i, v)    -> Any      */
	qu_sub3_f fsetslice; /* sub __setslice (i, l, v) -> Any      */
	qu_sub0_f fstring;   /* sub __string             -> String   */
	qu_sub0_f ftrue;     /* sub __true               -> Boolean  */
};



/** QU_CLASS_FLAG
	Flags used by Class objects.

	Some values are shared by the ={Sub} and ={Var} classes.
	See ={Flag}.

	Do not access directly.
	Use the accessors provided instead.
*/
#define QU_CLASS_FLAG_DOC      QU_SYMBOL_FLAG_DOC      /* document compilation? */
#define QU_CLASS_FLAG_EXTERN   QU_SYMBOL_FLAG_EXTERN   /* C class */
#define QU_CLASS_FLAG_PRIVATE  QU_SYMBOL_FLAG_PRIVATE  /* is module private? */
#define QU_CLASS_FLAG_LOCAL    QU_SYMBOL_FLAG_LOCAL    /* local to block? - must be w/ QU_CLASS_FLAG_PRIVATE */
#define QU_CLASS_FLAG_FINAL    QU_SYMBOL_FLAG_FINAL    /* is __final? */
#define QU_CLASS_FLAG_ABSTRACT QU_SYMBOL_FLAG_ABSTRACT /* is __abstract? */
#define QU_CLASS_FLAG_DYNAMIC  QU_FLAG_10              /* can create attributes at run-time? */
#define QU_CLASS_FLAG_DESTROY  QU_FLAG_11              /* defines __destroy? */
#define QU_CLASS_FLAG_AUTO     QU_FLAG_12              /* __auto create attribute? */
#define QU_CLASS_FLAG_MIXIN    QU_FLAG_13              /* is __mixin? */
#define QU_CLASS_FLAG_NOCATCH  QU_FLAG_14              /* is non-catchable exception class? */
#define QU_CLASS_FLAG_PURE     QU_FLAG_15              /* don't inherit from Object */
#define QU_CLASS_FLAG_AUTOITER QU_FLAG_16              /* interpreter uses internal iterator */
#define QU_CLASS_FLAG_NOWEAK   QU_FLAG_17              /* cant create weak references */
#define QU_CLASS_FLAG_CAST     QU_FLAG_18              /* Number, Decimal, etc. method new can be called */
#define QU_CLASS_FLAG_CALLABLE QU_FLAG_19              /* Class, Sub, Method, and Struct */
#define QU_CLASS_FLAG_SYMBOL   QU_FLAG_20              /* Module, Class, Sub, Var are symbol objects */
#define QU_CLASS_FLAG_LITERAL  QU_FLAG_21              /* non-container and instance can be made static? */


/** qu_class_is_literal
	Is non-Container class and instance can be made into a static object.
*/
#define qu_class_is_literal(self) qu_flag_test (self, QU_CLASS_FLAG_LITERAL)


/** qu_class_is_symbol
	Whether the class is a symbol: Module, Class, Sub, Var.
*/
#define qu_class_is_symbol(self) qu_flag_test (self, QU_CLASS_FLAG_SYMBOL)


/** qu_class_callable
	Whether the class is callable: Class, Sub, Method, Struct.
*/
#define qu_class_callable(self) qu_flag_test (self, QU_CLASS_FLAG_CALLABLE)


/** qu_class_noweak
	Whether we are allowed to make ={Weak} references to instance of the class.
*/
#define qu_class_noweak(self) qu_flag_test (self, QU_CLASS_FLAG_NOWEAK)


/** qu_class_autoiter
	Whether the interpreter can use an internal iterator.
*/
#define qu_class_autoiter(self) qu_flag_test (self, QU_CLASS_FLAG_AUTOITER)


/** qu_class_is_cast
	The abstract class's f{new} method can be called since they return an
	instance of a real class: Number, Real, Integer, and Decimal.
*/
#define qu_class_is_cast(self) qu_flag_test (self, QU_CLASS_FLAG_CAST)


/** qu_class_is_extern
	Whether or not this is a C level class.

	You may not set/unset this flag.
*/
#define qu_class_is_extern(self) qu_flag_test (self, QU_CLASS_FLAG_EXTERN)


/** qu_class_is_private
	Whether or not this Class should only be visible within the module where it
	is declared.

	This flag must be coupled with QU_CLASS_FLAG_LOCAL because we only allow a class
	to be declared in module level.
*/
#define qu_class_is_private(self) qu_flag_test (self, QU_CLASS_FLAG_PRIVATE)


/** qu_class_is_final
	Whether or not this Class is final (can not be extended).
*/
#define qu_class_is_final(self) qu_flag_test (self, QU_CLASS_FLAG_FINAL)


/** qu_class_is_abstract
	Whether or not this Class is an abstract (can not instantiate).
*/
#define qu_class_is_abstract(self) qu_flag_test (self, QU_CLASS_FLAG_ABSTRACT)


/** qu_class_is_mixin
	Whether or not this Class is a mixin class.
*/
#define qu_class_is_mixin(self) qu_flag_test (self, QU_CLASS_FLAG_MIXIN)


/** qu_class_is_dynamic qu_class_is_auto
	qu_class_is_dynamic checks whether or not this Class allows creation of attributes
	at run-time.

	qu_class_is_auto checks whether the Class allows automatic creation of instance
	attributes at run-time, e.g. C{x.foo = 1} automatically creates the instance
	attribute v{foo} if it has not been declared.
*/
#define qu_class_is_dynamic(self) qu_flag_test (self, QU_CLASS_FLAG_DYNAMIC)
#define qu_class_is_auto(self)    qu_flag_test (self, QU_CLASS_FLAG_AUTO)


/** qu_class_catchable
	Whether this exception Class can not be catch-ed.
*/
#define qu_class_catchable(self) (!qu_flag_test (self, QU_CLASS_FLAG_NOCATCH))


/** qu_class_has_destroy
	Whether the Class defines the __destroy instance method.

	The QU_CLASS_FLAG_DESTROY flag is set by qu_class_enddecl by checking whether or not
	the class declares the __destroy method.

	The purpose is to help GC performance so it does not have to do an attribute
	lookup everytime an object is destroyed.

	This means that a __destroy attribute created at run-time will not be respected
	by the GC, which we think is favorable.
*/
#define qu_class_has_destroy(self) qu_flag_test (self, QU_CLASS_FLAG_DESTROY)


/** qu_class_is_doc
	Whether or not the Class comes from a documentation compilation.

	This is a documentation hack.

	When compiling documentation, program level classes are flagged QU_CLASS_FLAG_EXTERN
	to make things easier and more efficient (no need to do some extra checks)
	but they are also flagged QU_CLASS_FLAG_DOC for recognition.
*/
#define qu_class_is_doc(self) qu_flag_test (self, QU_CLASS_FLAG_DOC)


/** qu_class_is_pure
	Whether or not the class defines the __pure decorator which means that it
	must not inherit from the Object class (do not lookup Object if an attribute
	is not found).
*/
#define qu_class_is_pure(self) qu_flag_test (self, QU_CLASS_FLAG_PURE)


/** qu_class_super
	Get the superclass of this class if any. Return 0 if the class is a top-level
	class.
*/
#define qu_class_super(self) ((self)->super)


/** qu_classid
	Class IDs are currently used simply to speed things up a bit in some places.

	qu_classid returns the class ID of the object.

	qu_class_sysid returns the class ID of the toplevel builtin class if there is
	one, otherwise the class ID.
*/
#define QU_CLASS_ID_USER              0
#define QU_CLASS_ID_Object            1
#define QU_CLASS_ID_Class             2
#define QU_CLASS_ID_Module            10
#define QU_CLASS_ID_Var               20
#define QU_CLASS_ID_Function          30
#    define QU_CLASS_ID_Sub           31
#    define QU_CLASS_ID_Method        32
#define QU_CLASS_ID_Exception         40
#define QU_CLASS_ID_Null              41
#define QU_CLASS_ID_Boolean           42
#define QU_CLASS_ID_String            50
#define QU_CLASS_ID_Number            60
#    define QU_CLASS_ID_Real          61
#        define QU_CLASS_ID_Integer   62
#            define QU_CLASS_ID_Int   63
#            define QU_CLASS_ID_Long  64
#        define QU_CLASS_ID_Rational  65
#        define QU_CLASS_ID_Decimal   66
#            define QU_CLASS_ID_Big   67
#            define QU_CLASS_ID_Float 68
#        define QU_CLASS_ID_Inf       69
#    define QU_CLASS_ID_Complex       70
#define QU_CLASS_ID_Container         100
#define QU_CLASS_ID_Iterator          110
#define QU_CLASS_ID_Array             120
#define QU_CLASS_ID_Dict              130
#    define QU_CLASS_ID_Arg           131
#define QU_CLASS_ID_Set               140
#define QU_CLASS_ID_Buffer            150
#define QU_CLASS_ID_Queue             160
#define QU_CLASS_ID_Range             170
#define QU_CLASS_ID_Record            180
#define QU_CLASS_ID_Struct            190
#define QU_CLASS_ID_Stream            200
#    define QU_CLASS_ID_File          201
#    define QU_CLASS_ID_Socket        202
#define QU_CLASS_ID_Thread            210
#define QU_CLASS_ID_Dlib              220
#define QU_CLASS_ID_Opaque            230
#define QU_CLASS_ID_Pointer           240
#define QU_CLASS_ID_Cfunc             250
#define QU_CLASS_ID_Proc              260
#define QU_CLASS_ID_Regex             270
#define QU_CLASS_ID_Task              280
#define QU_CLASS_ID_Tokenizer         500
#define QU_CLASS_ID_Ast               510
#define QU_CLASS_ID_Code              520
#define QU_CLASS_ID_Frame             530
#define QU_CLASS_ID_Weak              540
#define QU_CLASS_ID_Flow              550
#define QU_CLASS_ID_GObject           600

/* for a Class just say foo->id */
#define qu_classid(o) (qu_int_is(o) ? QU_CLASS_ID_Int : ((qu_obj)(o))->klass->id)
extern int qu_class_sysid (qu_any o);


/** qu_class_as
	Various representations.
*/
#define qu_class_as_string(self)  qu_object_default_string (self)
#define qu_class_as_boolean(self) QuTrue


/***/
/*----------------------------------------------------------------------------
	methods designed for the compiler/interpreter
----------------------------------------------------------------------------*/

/** qu_class_declare qu_class_decl_attr qu_class_search_attr
	These functions are specifically designed for the compiler. If you are not
	working with the compiler then you should not call these functions.

	qu_class_declare allocates a new Class object. The compiler must set the
	module attributes itself via ={qu_module_setattr}.

	qu_class_decl_attr is used to declare an attribute (Var or Sub), class or
	instance attribute. It checks for signatures and return 1 on success,
	0 on failure.

	qu_class_search_attr looks for an attribute in the super class and mixin
	classes, the Class class, and finally the Object class.
	It looks for class and instance attribute.

	qu_class_search_attrx does the same as qu_class_search_attr but looks in
	the Object class first then the Class class.

	See ={qu_attr_search} for how Qu looks for attributes.
*/
extern qu_class qu_class_declare      (qu_module mod, qu_string name, long flags);
extern int      qu_class_decl_attr    (qu_class self, qu_string name, qu_symbol value);
extern qu_any   qu_class_search_attr  (qu_class self, qu_string name);
extern qu_any   qu_class_search_attrx (qu_class self, qu_string name);


/** qu_class_check_prototype qu_class_del_accel
	qu_class_check_prototype check for a correct prototype.
	If listed as a standard name (e.g. __mul), returns the static string version
	of the name on success, NULL on failure.
	Otherwise this simply returns v{name}.

	qu_class_del_accel deletes the accelerator if it exists.
	This is necessary if the accelerator (which must be a C function) is replaced
	by a Qu function. Only Code should be calling this.
*/
extern qu_string qu_class_check_prototype (qu_class self, qu_string name, qu_symbol attr);
extern void      qu_class_del_accel       (qu_class self, qu_string name);



/***/
/*----------------------------------------------------------------------------
	methods designed for C modules
----------------------------------------------------------------------------*/

/** qu_class_new
	This function is designed for C modules to create a new Class object.

	qu_class_newz is simply a convenience function.

	This function automatically sets the attribute in the associated module
	object, i.e. C{mod->attrs}. No questions asked. No checks are made.
	It assumes that the C programmer is smart enough to know that declaring
	attributes with the same name more than once is an error.

	@warn
	Builtin classes must not call qu_class_new from their xxx__setup function.
	Creating classes in initialization phase may only be done from the
	xxx__init function.

	The v{name} field in qu_class_newz must be a static C string, e.g. C{"foo"}.
	It may not be a dynamic pointer.
*/
extern qu_class qu_class_new (qu_module o, qu_string name, long flags, size_t size, qu_destroy_f f, qu_mark_f m);

#define qu_class_newz(mod, name, struc, ffree, fmark) \
	qu_class_new (mod, qu_string_new_staticz (name), 0, sizeof (struc), \
		(qu_destroy_f) ffree, (qu_mark_f) fmark)

#define qu_class_new_finalz(mod, name, struc, ffree, fmark) \
	qu_class_new (mod, qu_string_new_staticz (name), QU_CLASS_FLAG_FINAL, sizeof (struc), \
		(qu_destroy_f) ffree, (qu_mark_f) fmark)

#define qu_class_new_abstractz(mod, name, struc, ffree, fmark) \
	qu_class_new (mod, qu_string_new_staticz (name), QU_CLASS_FLAG_ABSTRACT, sizeof (struc), \
		(qu_destroy_f) ffree, (qu_mark_f) fmark)

/** qu_class_prop
	Create a C class property.

	@warn
	The v{namez} field in the macros must be a static C string, e.g. C{"foo"}.
	It may not be a dynamic pointer.
*/
extern qu_var qu_class_prop (qu_class self, qu_string name, qu_any value, long flags);

#define qu_class_prop_class(self, name, val) \
	qu_class_prop (self, name, val, QU_VAR_FLAG_CLASS)

#define qu_class_prop_inst(self, name, val) \
	qu_class_prop (self, name, val, QU_VAR_FLAG_INST)

#define qu_class_const(self, name, val) \
	qu_class_prop (self, name, val, QU_VAR_FLAG_CLASS | QU_VAR_FLAG_FROZEN | QU_VAR_FLAG_FINAL)


#define qu_class_propz(self, namez, val, flags) \
	qu_class_prop (self, qu_string_new_staticz (namez), val, flags)

#define qu_class_prop_classz(self, namez, val) \
	qu_class_propz (self, namez, val, QU_VAR_FLAG_CLASS)

#define qu_class_prop_instz(self, namez, val) \
	qu_class_propz (self, namez, val, QU_VAR_FLAG_INST)

#define qu_class_constz(self, namez, val) \
	qu_class_propz (self, namez, val, QU_VAR_FLAG_CLASS | QU_VAR_FLAG_FROZEN | QU_VAR_FLAG_FINAL)



/** qu_class_method
	Create a C class method.

	@warn
	The v{namez} field in the macros must be a static C string, e.g. C{"foo"}.
	It may not be a dynamic pointer.

	These functions will issue a ={QU_FATAL} error if the attribute is a standard
	attribute, e.g. __cmp, and the signature does not match the predefined
	prototype.
*/
extern qu_sub qu_class_method (qu_class self, qu_string name, void *cfunc, int argc, long flags);

#define qu_class_method_static(self, name, faddr, argc) \
	qu_class_method (self, name, (void *) faddr, argc, QU_SUB_FLAG_STATIC)

#define qu_class_method_class(self, name, faddr, argc) \
	qu_class_method (self, name, (void *) faddr, argc, QU_SUB_FLAG_CLASS)

#define qu_class_method_inst(self, name, faddr, argc) \
	qu_class_method (self, name, (void *) faddr, argc, QU_SUB_FLAG_INST)


#define qu_class_methodz(self, namez, faddr, argc, flags) \
	qu_class_method (self, qu_string_new_staticz (namez), (void *) faddr, argc, flags)

#define qu_class_method_staticz(self, namez, faddr, argc) \
	qu_class_methodz (self, namez, faddr, argc, QU_SUB_FLAG_STATIC)

#define qu_class_method_classz(self, namez, faddr, argc) \
	qu_class_methodz (self, namez, faddr, argc, QU_SUB_FLAG_CLASS)

#define qu_class_method_instz(self, namez, faddr, argc) \
	qu_class_methodz (self, namez, faddr, argc, QU_SUB_FLAG_INST)


#define qu_class_method_staticzf(self, namez, faddr, argc) \
	qu_class_methodz (self, namez, faddr, argc, QU_SUB_FLAG_STATIC | QU_SUB_FLAG_FINAL)

#define qu_class_method_classzf(self, namez, faddr, argc) \
	qu_class_methodz (self, namez, faddr, argc, QU_SUB_FLAG_CLASS | QU_SUB_FLAG_FINAL)

#define qu_class_method_instzf(self, namez, faddr, argc) \
	qu_class_methodz (self, namez, faddr, argc, QU_SUB_FLAG_INST | QU_SUB_FLAG_FINAL)


/** qu_class_alias
	Create an alias for an existing attribute.
*/
extern qu_any qu_class_alias  (qu_class self, qu_string name, qu_string alias);

#define qu_class_aliasz(self, namez, aliasz) \
	qu_class_alias (self, qu_string_new_staticz (namez), qu_string_new_staticz (aliasz))


/** qu_class_set_super qu_class_set_mixin qu_class_enddecl
	These functions are designed for C modules to set the super class or
	mixin class. They are also used by the compiler.

	qu_class_set_super sets the super class of a qu_class.

	qu_class_set_mixin sets the class as a mixin class.

	qu_class_enddecl must be called after qu_class_set_super and qu_class_set_mixin to
	finalize declaration. It sets up the v{accel} field and mixes the
	attributes for faster lookup.

	All return 1 on success, throw an exception and returns 0 on failure.
	Failure includes, for example, when a method tagged __final is overridden,
	a class tagged __final is subclassed, etc.

	@xmp
		class Foo is Bar (Baz)
		#            |    |
		#            |    qu_class_set_mixin (Foo, Baz)
		#            |
		#            qu_class_set_super (Foo, Bar)
*/
extern int qu_class_set_super (qu_class self, qu_class sup);
extern int qu_class_set_mixin (qu_class self, qu_class klass);
extern int qu_class_enddecl  (qu_class self);


/** qu_class_find_class
	Find a superclass or a mixin by name.
*/
extern qu_class qu_class_find_class (qu_class self, qu_string name);


/** qu_class_mixes
	Whether the class mixins a specified class.
*/
#define qu_class_mixes(self, c) (qu__object__mixes (self, c) == QuTrue)


/** qu_class_mixins
	Return a frozen -{Array} of the classes used as mixins.

	This is intended for Object.
*/
#define qu_class_mixins(self) qu__object__mixins (self)


/** qu_class_tree
	Return an ={Array} consiting all the classes in this class's hierachy.
	The first item is this class.
*/
#define qu_class_tree(self) qu__object__tree (self)


/** qu_class_init_new
	qu_class_init_new is a convenience function designed to be called from
	within the f[new} methods of C classes that allow subclassing.

	It calls the __init method if one is defined, returns v{inst} on success,
	0 otherwise (__init throws an exception). It does nothing if the class
	does not declare an __init method and simply returns v{inst}.
*/
extern qu_any qu_class_init_new (qu_class self, qu_any inst, int argc, qu_any *argv);


/***/
/*----------------------------------------------------------------------------
	methods for general purposes
----------------------------------------------------------------------------*/

/** qu_class_get_module
	Get the ={Module} object where the class is declared.

	For builtin classes, this always return ={QuSysModule}.
*/
#define qu_class_get_module(self) ((qu_module) ((self)->parent))


/** qu_classof
	Get the Class of an object.

	If the object is a Class then this returns QuClassClass.
*/
#define qu_classof(self) (qu_int_is(self) ? QuIntClass : ((qu_obj)(self))->klass)


/** qu_class_is qu_class_isa qu_class_is_super_or_mixin
	qu_class_is tests whether or not an object is a Class.

	qu_class_isa tests whether or not the Class is a super class of another Class.
	It is provided because it is more efficient than the script level method
	Class.isa.

	qu_class_is_super_or_mixin checks whether v{x} is a mixin or superclass of self.
*/
#define qu_class_is(self) (qu_int_is(self) ? 0 : (((qu_obj)(self))->klass == QuClassClass))
extern int qu_class_isa   (qu_class self, qu_class x);
extern int qu_class_is_super_or_mixin (qu_class self, qu_class x);


/** qu_class_depth
	Return the depth of the Class in its class hierarchy. A top-level class
	such as ={Number}, for example, will have a value of 0.
*/
extern int qu_class_depth (qu_class self);


/***/
/*----------------------------------------------------------------------------
	methods specifically designed for builtin classes
----------------------------------------------------------------------------*/

/** qu_class__setup
	This is called internally by builtin classes to setup the ={qu_class_s} struct.

	This function must be called by all builtin classes from their xxx__setup
	function. Never from xxx__init.
*/
extern void qu_class__setup (qu_class self, long flags, size_t size, qu_destroy_f f, qu_mark_f m);

#define qu_class_setup(klass, struc, ffree, fmark) \
	qu_class__setup (klass, 0, sizeof (struc), (qu_destroy_f) ffree, (qu_mark_f) fmark)

#define qu_class_setup_sys(klass, struc, ffree, fmark) \
	qu_class__setup (klass, QU_CLASS_FLAG_FINAL | QU_CLASS_FLAG_ABSTRACT, sizeof (struc), (qu_destroy_f) ffree, (qu_mark_f) fmark)

#define qu_class_setup_final(klass, struc, ffree, fmark) \
	qu_class__setup (klass, QU_CLASS_FLAG_FINAL, sizeof (struc), (qu_destroy_f) ffree, (qu_mark_f) fmark)

#define qu_class_setup_abstract(klass, struc) \
	qu_class__setup (klass, QU_CLASS_FLAG_ABSTRACT, sizeof (struc), 0, 0)

#define qu_class_setup_base(klass, struc) \
	qu_class__setup (klass, QU_CLASS_FLAG_FINAL | QU_CLASS_FLAG_ABSTRACT, sizeof (struc), 0, 0)

#define qu_class_setup_mixin(klass, struc) \
	qu_class__setup (klass, QU_CLASS_FLAG_MIXIN, sizeof (struc), 0, 0)


/***/
/*--------------------------------------------------------------------------*/

/* called internally by the main program to initialize all builtin classes */
extern void qu_class__init ();

/* for Reduce */
extern int qu_class_can_access (qu_class self, qu_symbol from);

/***/
#define QU_ATTRS_IN_HEADER
#include "attrs.h"

#endif /*_H_qu_class_*/
