/* Complex.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_complex_
#define _H_qu_complex_ 1


/** Complex
	This is the final Complex class.
*/
extern qu_class QuComplexClass;


/** qu_complex
	qu_complex represents an instance of a Complex class.
	It is a pointer to the instance struct qu_complex_s.

	Both the real and imaginary parts may be any ={Real} objects including
	={QuInfPos} or ={QuInfNeg}.
*/
struct qu_complex_s
{
	QU_OBJECT_HEAD;
	qu_real real;   /* real part */
	qu_real imag;   /* imaginary part */
};


/** QuComplexN
	Predefined system Complex objects.
*/
/* the real and imaginary parts are Int objects */
extern qu_complex QuComplex00;       /*  0 + 0j */
extern qu_complex QuComplex01;       /*  0 + 1j */
extern qu_complex QuComplex0_1;      /*  0 - 1j */
extern qu_complex QuComplex10;       /*  1 + 0j */
extern qu_complex QuComplex11;       /*  1 + 1j */
extern qu_complex QuComplex1_1;      /*  1 - 1j */
extern qu_complex QuComplex_10;      /* -1 + 0j */
extern qu_complex QuComplex_11;      /* -1 + 1j */
extern qu_complex QuComplex_1_1;     /* -1 - 1j */

/* the real and imaginary parts are Big objects */
extern qu_complex QuComplexBig00;
extern qu_complex QuComplexBig01;
extern qu_complex QuComplexBig0_1;
extern qu_complex QuComplexBig10;
extern qu_complex QuComplexBig11;
extern qu_complex QuComplexBig1_1;
extern qu_complex QuComplexBig_10;
extern qu_complex QuComplexBig_11;
extern qu_complex QuComplexBig_1_1;

extern qu_complex QuComplexInfp;    /* infp + 0j */
extern qu_complex QuComplexInfn;    /* infn + 0j */


/** qu_complex_to_static
	Create a new system Complex object from an existing one.
	The object is not marked by the GC.
*/
extern qu_complex qu_complex_to_static (qu_complex x);


/** __qu_complex__
	If the ={qu_number_context} specifies REALIFY_COMPLEX and the imaginary part is
	zero then this function returns the real part.

	Otherwise if the context does not allow Complex numbers to be returned then
	this function throws EComplex, return a Complex otherwise.

	@warn
	You must always use this to return a Complex result to Qu programs.
*/
extern qu_number __qu_complex__ (qu_real r, qu_real i);


/** qu_complex_new
	Create a new Complex object from various C types and other objects.

	All always succeed except when the argument is an infinity and the
	={qu_number_context} does not allow infinities, in which case EInf is thrown.
	If you don't want EInf then use qu_complex_new.
*/
extern qu_complex qu_complex_new  (qu_real real, qu_real imag);
extern qu_complex qu_complex_newr (qu_real real, qu_real imag);
extern qu_complex qu_complex_new1 (qu_real real);

#define           qu_complex_newh(v)   qu_complex_new1 ((qu_real) qu_int_newh (v))
#define           qu_complex_newuh(v)  qu_complex_new1 ((qu_real) qu_int_newuh (v))
#define           qu_complex_newi(v)   qu_complex_new1 ((qu_real) qu_integer_newi (v))
#define           qu_complex_newui(v)  qu_complex_new1 ((qu_real) qu_integer_newui (v))
#define           qu_complex_newl(v)   qu_complex_new1 ((qu_real) qu_integer_newl (v))
#define           qu_complex_newul(v)  qu_complex_new1 ((qu_real) qu_integer_newul (v))
extern qu_complex qu_complex_newli     (long v);
#define           qu_complex_newd(v)   qu_complex_new1 ((qu_real) qu_float_newd (v))
#ifdef HAVE_LONG_LONG
#define           qu_complex_newll(v)  qu_complex_new1 ((qu_real) qu_integer_newll (v))
#define           qu_complex_newull(v) qu_complex_new1 ((qu_real) qu_integer_newull (v))
#endif
#ifdef HAVE_LONG_DOUBLE
#define           qu_complex_newld(v)  qu_complex_new1 ((qu_real) qu_decimal_newld (v))
#endif

#define           qu_complex_new_int(x)      qu_complex_new1 ((qu_real) x)
#define           qu_complex_new_long(x)     qu_complex_new1 ((qu_real) x)
#define           qu_complex_new_rational(x) qu_complex_new1 ((qu_real) x)
#define           qu_complex_new_float(x)    qu_complex_new1 ((qu_real) x)
#define           qu_complex_new_big(x)      qu_complex_new1 ((qu_real) x)
#define           qu_complex_new_complex(x)  (x)
#define           qu_complex_new_integer(x)  qu_complex_new1 ((qu_real) x)
#define           qu_complex_new_decimal(x)  qu_complex_new1 ((qu_real) x)
#define           qu_complex_new_real(x)     qu_complex_new1 (x)
extern qu_complex qu_complex_new_number      (qu_number x);
#define           qu_complex_new_boolean(x)  (((x) == QuTrue) ? QuComplex10 : QuComplex00);
extern qu_complex qu_complex_new_string      (qu_string x);


/** qu_complex_to
	Convert a Complex into a C type.

	Sets errno on failure.
*/
#define qu_complex_toh(x)    qu_real_toh   ((x)->real)
#define qu_complex_touh(x)   qu_real_touh  ((x)->real)
#define qu_complex_toi(x)    qu_real_toi   ((x)->real)
#define qu_complex_toui(x)   qu_real_toui  ((x)->real)
#define qu_complex_toli(x)   qu_real_toli  ((x)->real)
#define qu_complex_tol(x)    qu_real_tol   ((x)->real)
#define qu_complex_toul(x)   qu_real_toul  ((x)->real)
#define qu_complex_tod(x)    qu_real_tod   ((x)->real)
#ifdef HAVE_LONG_LONG
#define qu_complex_toll(x)   qu_real_toll  ((x)->real)
#define qu_complex_toull(x)  qu_real_toull ((x)->real)
#endif
#ifdef HAVE_LONG_DOUBLE
#define qu_complex_told(x)   qu_real_told  ((x)->real)
#endif


/** qu_complex_as
	Convert a Complex to another object.

	Return NULL and throws on failure.
*/
#define qu_complex_as_int(x)      qu_real_as_int      ((x)->real)
#define qu_complex_as_long(x)     qu_real_as_long     ((x)->real)
#define qu_complex_as_rational(x) qu_real_as_rational ((x)->real)
#define qu_complex_as_float(x)    qu_real_as_float    ((x)->real)
#define qu_complex_as_big(x)      qu_real_as_big      ((x)->real)
#define qu_complex_as_complex(x)  (x)
#define qu_complex_as_integer(x)  qu_real_as_integer ((x)->real)
#define qu_complex_as_decimal(x)  qu_real_as_decimal ((x)->real)
#define qu_complex_as_real(x)     ((x)->real)
#define qu_complex_as_number(x)   ((qu_number) x)
#define qu_complex_as_boolean(x)  (qu_complex_iszero (x) ? QuFalse : QuTrue)
#define qu_complex_as_string(x)   qu__complex__string(x)


/** qu_complex_sign
	Return -1, 0, 1 if the real part is negative, 0, positive respectively.
	The imaginary part is ignored.
*/
#define qu_complex_sign(x)  qu_real_sign ((x)->real)


/** qu_complex_is
	Identity functions.

	qu_complex_is tests whether or not an object is a Complex.

	qu_complex_isneg return 1 if the real part is negative, and 0 otherwise.
	The imaginary part is ignored.

	qu_complex_iszero return 1 if both the real and imaginary parts are 0.
	Return 0 oherwise.

	qu_complex_isone return 1 if the real part is 1 or -1.
	Return 0 otherwise.
	The imaginary part is ignored.

	qu_complex_iswhole return 1 if the real part is a whole number.
	Return 0 otherwise.
	The imaginary part is ignored.

	qu_complex_isfrac return 1 if if 0 < |x.real| < 1.
	Returns 0 otherwise.
	The imaginary part is ignored.

	qu_complex_isodd return 1 if the real part is a whole and odd number.
	Return 0 otherwise.
	The imaginary part is ignored.

	qu_complex_isreal return 1 if the imaginary part is 0.
	Return 0 otherwise.
*/
#define    qu_complex_is(x)      (qu_classof (x) == QuComplexClass)
extern int qu_complex_iszero     (qu_complex x);
#define    qu_complex_isneg(x)   qu_real_isneg   ((x)->real)
#define    qu_complex_isone(x)   qu_real_isone   ((x)->real)
#define    qu_complex_is1(x)     qu_real_is1     ((x)->real)
#define    qu_complex_iswhole(x) qu_real_iswhole ((x)->real)
#define    qu_complex_isodd(x)   qu_real_isodd   ((x)->real)
#define    qu_complex_isfrac(x)  qu_real_isfrac  ((x)->real)
#define    qu_complex_isreal(x)  qu_real_iszero  ((x)->imag)


/** qu_complex_unary
	Unary functions.

	qu_complex_pos returns the number itself.

	qu_complex_neg returns a Complex with both the real and imaginary parts negated.

	qu_complex_incr returns a Complex with both the real and imaginary parts incremented by 1.

	qu_complex_decr returns a Complex with both the real and imaginary parts decremented by 1.
*/
#define qu_complex_pos(x)  (x)
#define qu_complex_neg(x)  qu__complex__neg  (x)
#define qu_complex_incr(x) qu__complex__incr (x)
#define qu_complex_decr(x) qu__complex__decr (x)
#define qu_complex_abs(x)  qu__complex_abs   (x)


/** qu_complex_eq
	Whether the value is the same as another.
	All return 0 or 1.

	For Real objects then 1 is returned if the real part is the same as the
	object and the imaginary part is 0.

	For Complex objects, 1 is returned if both real and imaginary parts are the same.
*/
extern int qu_complex_eq_int      (qu_complex x, qu_int y);
extern int qu_complex_eq_long     (qu_complex x, qu_long y);
extern int qu_complex_eq_rational (qu_complex x, qu_rational y);
extern int qu_complex_eq_float    (qu_complex x, qu_float y);
extern int qu_complex_eq_big      (qu_complex x, qu_big y);
extern int qu_complex_eq_complex  (qu_complex x, qu_complex y);
extern int qu_complex_eq_inf      (qu_complex x, qu_inf y);
extern int qu_complex_eq_integer  (qu_complex x, qu_integer y);
extern int qu_complex_eq_decimal  (qu_complex x, qu_decimal y);
extern int qu_complex_eq_real     (qu_complex x, qu_real y);
extern int qu_complex_eq_number   (qu_complex x, qu_number y);


/** qu_complex_like
	Whether the absolute value is the same as the absolute value of another.
	All return 0 or 1.

	For Real objects then 1 is returned if the absolute value of the real part
	is the same as the absolute value of the object and the imaginary part is 0.

	For Complex objects, 1 is returned if both real and imaginary parts have the same
	absolute values.
*/
extern int qu_complex_like_int      (qu_complex x, qu_int y);
extern int qu_complex_like_long     (qu_complex x, qu_long y);
extern int qu_complex_like_rational (qu_complex x, qu_rational y);
extern int qu_complex_like_float    (qu_complex x, qu_float y);
extern int qu_complex_like_big      (qu_complex x, qu_big y);
extern int qu_complex_like_complex  (qu_complex x, qu_complex y);
extern int qu_complex_like_inf      (qu_complex x, qu_inf y);
extern int qu_complex_like_integer  (qu_complex x, qu_integer y);
extern int qu_complex_like_decimal  (qu_complex x, qu_decimal y);
extern int qu_complex_like_real     (qu_complex x, qu_real y);
extern int qu_complex_like_number   (qu_complex x, qu_number y);


/** qu_complex_cmp
	Compare with another.
	All return -1, 0 or 1 in the usual manner.

	For Real objects then the value is compared to the real part. If it is the same
	then the sign of the imaginary part is returned.

	For Complex objects, both real and imaginary parts are compared.
*/
extern int qu_complex_cmp_int      (qu_complex x, qu_int y);
extern int qu_complex_cmp_long     (qu_complex x, qu_long y);
extern int qu_complex_cmp_rational (qu_complex x, qu_rational y);
extern int qu_complex_cmp_float    (qu_complex x, qu_float y);
extern int qu_complex_cmp_big      (qu_complex x, qu_big y);
extern int qu_complex_cmp_complex  (qu_complex x, qu_complex y);
extern int qu_complex_cmp_inf      (qu_complex x, qu_inf y);
extern int qu_complex_cmp_integer  (qu_complex x, qu_integer y);
extern int qu_complex_cmp_decimal  (qu_complex x, qu_decimal y);
extern int qu_complex_cmp_real     (qu_complex x, qu_real y);
extern int qu_complex_cmp_number   (qu_complex x, qu_number y);


/** qu_complex_cmpabs
	Compare the absolute value with the absolute value of another.
	All return -1, 0, or -1 in the usual manner.

	The absolute value of the Complex number is used to do the comparisson, which
	formula is C{sqrt (z.real * z.real + z.imag * z.imag)}.
*/
extern int qu_complex_cmpabs_int      (qu_complex x, qu_int y);
extern int qu_complex_cmpabs_long     (qu_complex x, qu_long y);
extern int qu_complex_cmpabs_rational (qu_complex x, qu_rational y);
extern int qu_complex_cmpabs_float    (qu_complex x, qu_float y);
extern int qu_complex_cmpabs_big      (qu_complex x, qu_big y);
extern int qu_complex_cmpabs_complex  (qu_complex x, qu_complex y);
extern int qu_complex_cmpabs_inf      (qu_complex x, qu_inf y);
extern int qu_complex_cmpabs_integer  (qu_complex x, qu_integer y);
extern int qu_complex_cmpabs_decimal  (qu_complex x, qu_decimal y);
extern int qu_complex_cmpabs_real     (qu_complex x, qu_real y);
extern int qu_complex_cmpabs_number   (qu_complex x, qu_number y);


/** qu_complex_plus
	Add a with another.
*/
extern qu_number qu_complex_plus_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_plus_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_plus_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_plus_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_plus_big      (qu_complex x, qu_big y);
extern qu_number qu_complex_plus_complex  (qu_complex x, qu_complex y);
extern qu_number qu_complex_plus_inf      (qu_complex x, qu_inf y);
extern qu_number qu_complex_plus_integer  (qu_complex x, qu_integer y);
extern qu_number qu_complex_plus_decimal  (qu_complex x, qu_decimal y);
extern qu_number qu_complex_plus_real     (qu_complex x, qu_real y);
extern qu_number qu_complex_plus_number   (qu_complex x, qu_number y);


/** qu_complex_minus
	Subtract with another.
*/
extern qu_number qu_complex_minus_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_minus_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_minus_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_minus_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_minus_big      (qu_complex x, qu_big y);
extern qu_number qu_complex_minus_complex  (qu_complex x, qu_complex y);
extern qu_number qu_complex_minus_inf      (qu_complex x, qu_inf y);
extern qu_number qu_complex_minus_integer  (qu_complex x, qu_integer y);
extern qu_number qu_complex_minus_decimal  (qu_complex x, qu_decimal y);
extern qu_number qu_complex_minus_real     (qu_complex x, qu_real y);
extern qu_number qu_complex_minus_number   (qu_complex x, qu_number y);


/** qu_complex_rminus
	Subtract another with this one.
	This is provided for efficiency.
*/
extern qu_number qu_complex_rminus_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_rminus_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_rminus_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_rminus_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_rminus_big      (qu_complex x, qu_big y);


/** qu_complex_mul
	Multiply with another.
*/
extern qu_number qu_complex_mul_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_mul_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_mul_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_mul_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_mul_big      (qu_complex x, qu_big y);
extern qu_number qu_complex_mul_complex  (qu_complex x, qu_complex y);
extern qu_number qu_complex_mul_inf      (qu_complex x, qu_inf y);
extern qu_number qu_complex_mul_integer  (qu_complex x, qu_integer y);
extern qu_number qu_complex_mul_decimal  (qu_complex x, qu_decimal y);
extern qu_number qu_complex_mul_real     (qu_complex x, qu_real y);
extern qu_number qu_complex_mul_number   (qu_complex x, qu_number y);


/** qu_complex_div
	Divide with another.
*/
extern qu_number qu_complex_div_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_div_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_div_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_div_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_div_big      (qu_complex x, qu_big y);
extern qu_number qu_complex_div_complex  (qu_complex x, qu_complex y);
extern qu_number qu_complex_div_inf      (qu_complex x, qu_inf y);
extern qu_number qu_complex_div_integer  (qu_complex x, qu_integer y);
extern qu_number qu_complex_div_decimal  (qu_complex x, qu_decimal y);
extern qu_number qu_complex_div_real     (qu_complex x, qu_real y);
extern qu_number qu_complex_div_number   (qu_complex x, qu_number y);


/** qu_complex_rdiv
	Divide another with this one.
	This is provided for efficiency.
*/
extern qu_number qu_complex_rdiv_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_rdiv_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_rdiv_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_rdiv_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_rdiv_big      (qu_complex x, qu_big y);
extern qu_number qu_complex_rdiv_complex  (qu_complex x, qu_complex y);


/** qu_complex_idiv
	Integer division with another.
*/
extern qu_integer qu_complex_idiv_int      (qu_complex x, qu_int y);
extern qu_integer qu_complex_idiv_long     (qu_complex x, qu_long y);
extern qu_integer qu_complex_idiv_rational (qu_complex x, qu_rational y);
extern qu_integer qu_complex_idiv_float    (qu_complex x, qu_float y);
extern qu_integer qu_complex_idiv_big      (qu_complex x, qu_big y);
extern qu_integer qu_complex_idiv_complex  (qu_complex x, qu_complex y);
extern qu_integer qu_complex_idiv_inf      (qu_complex x, qu_inf y);
extern qu_integer qu_complex_idiv_integer  (qu_complex x, qu_integer y);
extern qu_integer qu_complex_idiv_decimal  (qu_complex x, qu_decimal y);
extern qu_integer qu_complex_idiv_real     (qu_complex x, qu_real y);
extern qu_integer qu_complex_idiv_number   (qu_complex x, qu_number y);


/** qu_complex_ridiv
	Integer division of another with this one.
	This is provided for efficiency.
*/
extern qu_integer qu_complex_ridiv_int      (qu_complex x, qu_int y);
extern qu_integer qu_complex_ridiv_long     (qu_complex x, qu_long y);
extern qu_integer qu_complex_ridiv_rational (qu_complex x, qu_rational y);
extern qu_integer qu_complex_ridiv_float    (qu_complex x, qu_float y);
extern qu_integer qu_complex_ridiv_big      (qu_complex x, qu_big y);


/** qu_complex_rem
	Remainder of dividing with another.
*/
extern qu_number qu_complex_rem_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_rem_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_rem_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_rem_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_rem_big      (qu_complex x, qu_big y);
extern qu_number qu_complex_rem_complex  (qu_complex x, qu_complex y);
extern qu_number qu_complex_rem_inf      (qu_complex x, qu_inf y);
extern qu_number qu_complex_rem_integer  (qu_complex x, qu_integer y);
extern qu_number qu_complex_rem_decimal  (qu_complex x, qu_decimal y);
extern qu_number qu_complex_rem_real     (qu_complex x, qu_real y);
extern qu_number qu_complex_rem_number   (qu_complex x, qu_number y);


/** qu_complex_rrem
	Remainder of dividing another with this one.
	This is provided for efficiency.
*/
extern qu_number qu_complex_rrem_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_rrem_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_rrem_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_rrem_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_rrem_big      (qu_complex x, qu_big y);


/** qu_complex_pow
	Raise to the power of another.
*/
extern qu_number qu_complex_pow_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_pow_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_pow_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_pow_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_pow_big      (qu_complex x, qu_big y);
extern qu_number qu_complex_pow_complex  (qu_complex x, qu_complex y);
extern qu_number qu_complex_pow_inf      (qu_complex x, qu_inf y);
extern qu_number qu_complex_pow_integer  (qu_complex x, qu_integer y);
extern qu_number qu_complex_pow_decimal  (qu_complex x, qu_decimal y);
extern qu_number qu_complex_pow_real     (qu_complex x, qu_real y);
extern qu_number qu_complex_pow_number   (qu_complex x, qu_number y);


/** qu_complex_rpow
	Raise another to the power of this one.
	This is provided for efficiency.
*/
extern qu_number qu_complex_rpow_int      (qu_complex x, qu_int y);
extern qu_number qu_complex_rpow_long     (qu_complex x, qu_long y);
extern qu_number qu_complex_rpow_rational (qu_complex x, qu_rational y);
extern qu_number qu_complex_rpow_float    (qu_complex x, qu_float y);
extern qu_number qu_complex_rpow_big      (qu_complex x, qu_big y);


/** qu_complex_round
	Rounding functions.

	qu_complex_ceil returns the result of rounding the real and imaginary parts to the
	nearest integer towards positive infinity.

	qu_complex_floor returns the result of rounding the real and imaginary parts to the
	nearest integer towards negative infinity.

	qu_complex_trunc returns the result of rounding the real and imaginary parts to the
	nearest integer towards zero, that is, it removes the fraction part.
*/
#define qu_complex_floor(x) qu__complex_floor (x)
#define qu_complex_ceil(x)  qu__complex_ceil  (x)
#define qu_complex_trunc(x) qu__complex_trunc (x)
#define qu_complex_frac(x)  qu__complex_frac  (x)
extern qu_number qu_complex_round (qu_complex x, int prec);


/** qu_complex_format
	Format a qu_complex.

	See ={Format} for details;

	This function is provided for efficiency between core modules.
	It is not intended to be called by users directly.
*/
extern qu_string qu_complex_format (qu_complex x, qu_format_s *f);


/***/
/*--------------------------------------------------------------------------*/

/* These are called internally by qu_class.c to initialize the qu_complex class */
extern void qu_complex__setup ();
extern void qu_complex__init  ();

/* for efficiency - may only be called from within this file */
extern qu_complex qu__complex__neg    (qu_complex self);
extern qu_complex qu__complex__incr   (qu_complex self);
extern qu_complex qu__complex__decr   (qu_complex self);
extern qu_string  qu__complex__string (qu_complex x);
extern qu_real    qu__complex_abs     (qu_complex x);
extern qu_number  qu__complex_floor   (qu_complex x);
extern qu_number  qu__complex_ceil    (qu_complex x);
extern qu_number  qu__complex_trunc   (qu_complex x);
extern qu_number  qu__complex_frac    (qu_complex x);

/* set by the Math module when loaded */
typedef qu_number (*qu_complexpowbig_f) (qu_real ar, qu_real ai, qu_real br, qu_real bi);
typedef qu_real   (*qu_complexabs_f   ) (qu_complex x);
typedef qu_real   (*qu_complexarg_f   ) (qu_complex x);

extern qu_complexpowbig_f __QuComplexPowComplexMath;
extern qu_complexabs_f    __QuComplexAbsMath;
extern qu_complexarg_f    __QuComplexArgMath;

/* for Sys */
extern qu_number qu_complex_return (qu_any r, qu_any i);

#endif /*_H_qu_complex_*/
