/* Decimal.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_decimal_
#define _H_qu_decimal_ 1


/** Decimal qu_decimal
	This is the abstract parent class of ={Float} and ={Big}.

	qu_decimal represents an instance of a Decimal class.
*/
extern qu_class QuDecimalClass;


/** qu_decimalN
	Predefined system objects.
*/
#define QuDecimal0  ((qu_decimal) QuFloat0)   /*  0.0 */
#define QuDecimal1  ((qu_decimal) QuFloat1)   /*  1.0 */
#define QuDecimal2  ((qu_decimal) QuFloat2)   /*  2.0 */
#define QuDecimal_1 ((qu_decimal) QuFloat_1)  /* -1.0 */
#define QuDecimal_2 ((qu_decimal) QuFloat_2)  /* -2.0 */


/** qu_decimal_new
	Create a new Decimal from other objects.

	For C type conversions, all functions always succeed and the returned
	value is either a ={Float}. Only qu_decimal_newld returns a ={Big}.

	For object conversions, an exception is thrown and NULL is returned on failure.
	On success either a ={Float} or ={Big} is returned.
*/
#define qu_decimal_newh(v)   qu_decimal_newd (v)
#define qu_decimal_newuh(v)  qu_decimal_newd (v)
#define qu_decimal_newi(v)   qu_decimal_newd (v)
#define qu_decimal_newui(v)  qu_decimal_newd (v)
#define qu_decimal_newli(v)  qu_decimal_newd (v)
#define qu_decimal_newl(v)   qu_decimal_newd (v)
#define qu_decimal_newul(v)  qu_decimal_newd (v)
#define qu_decimal_newd(v)   ((qu_decimal) (qu_float_newd (v)))
#if HAVE_LONG_LONG
#define qu_decimal_newll(v)  qu_decimal_newd (v)
#define qu_decimal_newull(v) qu_decimal_newd (v)
#endif
#ifdef HAVE_LONG_DOUBLE
extern qu_decimal qu_decimal_newld  (long double v);
#endif

#define qu_decimal_new_int(x)     qu_int_as_decimal (x)
#define qu_decimal_new_long       qu_long_as_decimal (x)
#define qu_decimal_new_rational   qu_rational_as_decimal (x)
#define qu_decimal_new_float(x)   qu_float_as_decimal (x)
#define qu_decimal_new_big(x)     qu_big_as_decimal (x)
#define qu_decimal_new_complex(x) qu_real_new_complex ((x)->real)
#define qu_decimal_new_integer(x) qu_integer_as_decimal (x)
#define qu_decimal_new_decimal(x) ((qu_decimal) x)
#define qu_decimal_new_real(x)    qu_real_as_decimal (x)
#define qu_decimal_new_number(x)  qu_number_as_decimal (x)


/** qu_decimal_newdx
	Create a new Decimal/Real object from a C double by checking whether or not the
	value is NaN or infinity.

	If the value is NaN then QuEValue is thrown and NULL is returned.

	If the value is infinity then QuEInf is thrown and NULL is returned unless the
	={qu_number_context} allows infinities to be returned, in which case one of
	={QuInfPos} or ={QuInfNeg} is returned based on the sign.
*/
extern qu_real qu_decimal_newdx (double v);
#ifdef HAVE_LONG_DOUBLE
extern qu_real qu_decimal_newldx (long double v);
#endif


/** qu_decimal_to
	Convert a Decimal into a C type.

	All accept any object. If the object is not a Decimal then errno is set to EDOM and
	0 or NULL is returned. Otherwise errno is set to ERANGE if the value does not fit
	into the specified type or object. Otherwise errno is set to zero indicating success.
*/
#define                   qu_decimal_toh(x)  qu_number__ltoh   (qu_decimal_tol  (x))
#define                   qu_decimal_touh(x) qu_number__ultouh (qu_decimal_toul (x))
#define                   qu_decimal_toi(x)  qu_number__ltoi   (qu_decimal_tol  (x))
#define                   qu_decimal_toui(x) qu_number__ultoui (qu_decimal_toul (x))
#define                   qu_decimal_toli(x) qu_number__ltoli  (qu_decimal_tol  (x))
extern long               qu_decimal_tol     (qu_any x);
extern unsigned long      qu_decimal_toul    (qu_any x);
extern float              qu_decimal_tof     (qu_any x);
extern double             qu_decimal_tod     (qu_any x);
#ifdef HAVE_LONG_LONG
extern long long          qu_decimal_toll    (qu_any x);
extern unsigned long long qu_decimal_toull   (qu_any x);
#endif
#ifdef HAVE_LONG_DOUBLE
extern long double        qu_decimal_told    (qu_any x);
#endif


/** qu_decimal_as
	Convert a Decimal into other objects.

	NULL is returned on failure.

	These are just convenient wrappers around ={qu_float_as} and ={qu_big_as}.
	If you already know what the object is, then it is more efficient to call them
	instead of these functions.
*/
extern qu_int      qu_decimal_as_int        (qu_decimal x);
extern qu_long     qu_decimal_as_long       (qu_decimal x);
extern qu_rational qu_decimal_as_rational   (qu_decimal x);
extern qu_float    qu_decimal_as_float      (qu_decimal x);
extern qu_big      qu_decimal_as_big        (qu_decimal x);
#define            qu_decimal_as_complex(x) qu_complex_new1 ((qu_real) (x))
extern qu_integer  qu_decimal_as_integer    (qu_decimal x);
#define            qu_decimal_as_decimal(x) (x)
#define            qu_decimal_as_real(x)    ((qu_real) (x))
#define            qu_decimal_as_number(x)  ((qu_number) (x))


/** qu_decimal_sign
	Returns -1, 0, 1 if the value is negative, 0, positive respectively.
*/
extern int qu_decimal_sign (qu_decimal x);


/** qu_decimal_is
	Identity functions.

	qu_decimal_is tests whether an object is a Decimal.

	qu_decimal_isneg returns 1 if the value is negative.
	Returns 0 otherwise.

	qu_decimal_iszero returns 1 if the value is 0.
	Returns 0 otherwise.

	qu_decimal_isone returns 1 if the value is 1 or -1.
	Returns 0 otherwise.

	qu_decimal_is1 returns 1 if the value is 1.
	Returns 0 otherwise.

	qu_decimal_iswhole returns 1 if the value does not have a fraction.
	Returns 0 otherwise.

	qu_decimal_isfrac returns 1 if 0 < |x| < 1.
	Returns 0 otherwise.

	qu_decimal_isodd returns 1 if the value is a whole and odd number.
	Returns 0 otherwise.
*/
#define qu_decimal_is(x) (qu_float_is (x) || qu_big_is (x))
extern int qu_decimal_iszero  (qu_decimal x);
extern int qu_decimal_isneg   (qu_decimal x);
extern int qu_decimal_isone   (qu_decimal x);
extern int qu_decimal_is1     (qu_decimal x);
extern int qu_decimal_iswhole (qu_decimal x);
extern int qu_decimal_isfrac  (qu_decimal x);
extern int qu_decimal_isodd   (qu_decimal x);


/** qu_decimal_unary
	Unary functions.

	qu_decimal_pos returns the number itself, obviously.

	qu_decimal_neg returns a negated value.

	qu_decimal_incr returns the number incremented by 1.

	qu_decimal_decr returns the number decremented by 1.

	These are just convenient wrappers around
	={qu_float_pos} ={qu_float_neg} ={qu_float_incr} ={qu_float_decr}
	={qu_big_pos} ={qu_big_neg} ={qu_big_incr} ={qu_big_decr}.
*/
extern qu_decimal qu_decimal_pos  (qu_decimal x);
extern qu_decimal qu_decimal_neg  (qu_decimal x);
extern qu_real    qu_decimal_incr (qu_decimal x);
extern qu_real    qu_decimal_decr (qu_decimal x);


/** qu_decimal_eq
	Whether the value is the same as another.
	All return 0 or 1.

	These are just convenient wrappers around ={qu_float_eq} and ={qu_big_eq}.
*/
extern int qu_decimal_eq_int      (qu_decimal x, qu_int y);
extern int qu_decimal_eq_long     (qu_decimal x, qu_long y);
extern int qu_decimal_eq_rational (qu_decimal x, qu_rational y);
extern int qu_decimal_eq_float    (qu_decimal x, qu_float y);
extern int qu_decimal_eq_big      (qu_decimal x, qu_big y);
extern int qu_decimal_eq_complex  (qu_decimal x, qu_complex y);
extern int qu_decimal_eq_inf      (qu_decimal x, qu_inf y);
extern int qu_decimal_eq_integer  (qu_decimal x, qu_integer y);
extern int qu_decimal_eq_decimal  (qu_decimal x, qu_decimal y);
extern int qu_decimal_eq_real     (qu_decimal x, qu_real y);
extern int qu_decimal_eq_number   (qu_decimal x, qu_number y);


/** qu_decimal_like
	Whether the absolute value is the same as the absolute value of another.
	All return 0 or 1.

	These are just convenient wrappers around ={qu_float_like} and ={qu_big_like}.
*/
extern int qu_decimal_like_int      (qu_decimal x, qu_int y);
extern int qu_decimal_like_long     (qu_decimal x, qu_long y);
extern int qu_decimal_like_rational (qu_decimal x, qu_rational y);
extern int qu_decimal_like_float    (qu_decimal x, qu_float y);
extern int qu_decimal_like_big      (qu_decimal x, qu_big y);
extern int qu_decimal_like_complex  (qu_decimal x, qu_complex y);
extern int qu_decimal_like_inf      (qu_decimal x, qu_inf y);
extern int qu_decimal_like_integer  (qu_decimal x, qu_integer y);
extern int qu_decimal_like_decimal  (qu_decimal x, qu_decimal y);
extern int qu_decimal_like_real     (qu_decimal x, qu_real y);
extern int qu_decimal_like_number   (qu_decimal x, qu_number y);


/** qu_decimal_cmp
	Compare with another.
	All return -1, 0, or -1 in the usual manner.

	These are just convenient wrappers around ={qu_float_cmp} and ={qu_big_cmp}.
*/
extern int qu_decimal_cmp_int      (qu_decimal x, qu_int y);
extern int qu_decimal_cmp_long     (qu_decimal x, qu_long y);
extern int qu_decimal_cmp_rational (qu_decimal x, qu_rational y);
extern int qu_decimal_cmp_float    (qu_decimal x, qu_float y);
extern int qu_decimal_cmp_big      (qu_decimal x, qu_big y);
extern int qu_decimal_cmp_complex  (qu_decimal x, qu_complex y);
extern int qu_decimal_cmp_inf      (qu_decimal x, qu_inf y);
extern int qu_decimal_cmp_integer  (qu_decimal x, qu_integer y);
extern int qu_decimal_cmp_decimal  (qu_decimal x, qu_decimal y);
extern int qu_decimal_cmp_real     (qu_decimal x, qu_real y);
extern int qu_decimal_cmp_number   (qu_decimal x, qu_number y);


/** qu_decimal_cmpabs
	Compare the absolute value with the absolute value of another.
	All return -1, 0, or -1 in the usual manner.

	These are just convenient wrappers around ={qu_float_cmpabs} and ={qu_big_cmpabs}.
*/
extern int qu_decimal_cmpabs_int      (qu_decimal x, qu_int y);
extern int qu_decimal_cmpabs_long     (qu_decimal x, qu_long y);
extern int qu_decimal_cmpabs_rational (qu_decimal x, qu_rational y);
extern int qu_decimal_cmpabs_float    (qu_decimal x, qu_float y);
extern int qu_decimal_cmpabs_big      (qu_decimal x, qu_big y);
extern int qu_decimal_cmpabs_complex  (qu_decimal x, qu_complex y);
extern int qu_decimal_cmpabs_inf      (qu_decimal x, qu_inf y);
extern int qu_decimal_cmpabs_integer  (qu_decimal x, qu_integer y);
extern int qu_decimal_cmpabs_decimal  (qu_decimal x, qu_decimal y);
extern int qu_decimal_cmpabs_real     (qu_decimal x, qu_real y);
extern int qu_decimal_cmpabs_number   (qu_decimal x, qu_number y);


/** qu_decimal_plus
	Add with another.

	These are just convenient wrappers around ={qu_float_plus} and ={qu_big_plus}.
*/
extern qu_real    qu_decimal_plus_int      (qu_decimal x, qu_int y);
extern qu_real    qu_decimal_plus_long     (qu_decimal x, qu_long y);
extern qu_real    qu_decimal_plus_rational (qu_decimal x, qu_rational y);
extern qu_real    qu_decimal_plus_float    (qu_decimal x, qu_float y);
extern qu_real    qu_decimal_plus_big      (qu_decimal x, qu_big y);
extern qu_complex qu_decimal_plus_complex  (qu_decimal x, qu_complex y);
extern qu_real    qu_decimal_plus_inf      (qu_decimal x, qu_inf y);
extern qu_real    qu_decimal_plus_integer  (qu_decimal x, qu_integer y);
extern qu_real    qu_decimal_plus_decimal  (qu_decimal x, qu_decimal y);
extern qu_real    qu_decimal_plus_real     (qu_decimal x, qu_real y);
extern qu_number  qu_decimal_plus_number   (qu_decimal x, qu_number y);


/** qu_decimal_minus
	Subtract with another.

	These are just convenient wrappers around ={qu_float_minus} and ={qu_big_minus}.
*/
extern qu_real    qu_decimal_minus_int      (qu_decimal x, qu_int y);
extern qu_real    qu_decimal_minus_long     (qu_decimal x, qu_long y);
extern qu_real    qu_decimal_minus_rational (qu_decimal x, qu_rational y);
extern qu_real    qu_decimal_minus_float    (qu_decimal x, qu_float y);
extern qu_real    qu_decimal_minus_big      (qu_decimal x, qu_big y);
extern qu_complex qu_decimal_minus_complex  (qu_decimal x, qu_complex y);
extern qu_real    qu_decimal_minus_inf      (qu_decimal x, qu_inf y);
extern qu_real    qu_decimal_minus_integer  (qu_decimal x, qu_integer y);
extern qu_real    qu_decimal_minus_decimal  (qu_decimal x, qu_decimal y);
extern qu_real    qu_decimal_minus_real     (qu_decimal x, qu_real y);
extern qu_number  qu_decimal_minus_number   (qu_decimal x, qu_number y);


/** qu_decimal_mul
	Multiply with another.

	These are just convenient wrappers around ={qu_float_minus} and ={qu_big_minus}.
*/
extern qu_real    qu_decimal_mul_int      (qu_decimal x, qu_int y);
extern qu_real    qu_decimal_mul_long     (qu_decimal x, qu_long y);
extern qu_real    qu_decimal_mul_rational (qu_decimal x, qu_rational y);
extern qu_real    qu_decimal_mul_float    (qu_decimal x, qu_float y);
extern qu_real    qu_decimal_mul_big      (qu_decimal x, qu_big y);
extern qu_complex qu_decimal_mul_complex  (qu_decimal x, qu_complex y);
extern qu_real    qu_decimal_mul_inf      (qu_decimal x, qu_inf y);
extern qu_real    qu_decimal_mul_integer  (qu_decimal x, qu_integer y);
extern qu_real    qu_decimal_mul_decimal  (qu_decimal x, qu_decimal y);
extern qu_real    qu_decimal_mul_real     (qu_decimal x, qu_real y);
extern qu_number  qu_decimal_mul_number   (qu_decimal x, qu_number y);


/** qu_decimal_div
	Divide with another.

	These are just convenient wrappers around ={qu_float_div} and ={qu_big_div}.
*/
extern qu_real    qu_decimal_div_int      (qu_decimal x, qu_int y);
extern qu_real    qu_decimal_div_long     (qu_decimal x, qu_long y);
extern qu_real    qu_decimal_div_rational (qu_decimal x, qu_rational y);
extern qu_real    qu_decimal_div_float    (qu_decimal x, qu_float y);
extern qu_real    qu_decimal_div_big      (qu_decimal x, qu_big y);
extern qu_complex qu_decimal_div_complex  (qu_decimal x, qu_complex y);
extern qu_real    qu_decimal_div_inf      (qu_decimal x, qu_inf y);
extern qu_real    qu_decimal_div_integer  (qu_decimal x, qu_integer y);
extern qu_real    qu_decimal_div_decimal  (qu_decimal x, qu_decimal y);
extern qu_real    qu_decimal_div_real     (qu_decimal x, qu_real y);
extern qu_number  qu_decimal_div_number   (qu_decimal x, qu_number y);


/** qu_decimal_idiv
	Integer division with another.

	These are just convenient wrappers around ={qu_float_idiv} and ={qu_big_idiv}.
*/
extern qu_integer qu_decimal_idiv_int      (qu_decimal x, qu_int y);
extern qu_integer qu_decimal_idiv_long     (qu_decimal x, qu_long y);
extern qu_integer qu_decimal_idiv_rational (qu_decimal x, qu_rational y);
extern qu_integer qu_decimal_idiv_float    (qu_decimal x, qu_float y);
extern qu_integer qu_decimal_idiv_big      (qu_decimal x, qu_big y);
extern qu_integer qu_decimal_idiv_complex  (qu_decimal x, qu_complex y);
extern qu_integer qu_decimal_idiv_inf      (qu_decimal x, qu_inf y);
extern qu_integer qu_decimal_idiv_integer  (qu_decimal x, qu_integer y);
extern qu_integer qu_decimal_idiv_decimal  (qu_decimal x, qu_decimal y);
extern qu_integer qu_decimal_idiv_real     (qu_decimal x, qu_real y);
extern qu_integer qu_decimal_idiv_number   (qu_decimal x, qu_number y);


/** qu_decimal_rem
	Remainder of dividing with another.

	These are just convenient wrappers around ={qu_float_rem} and ={qu_big_rem}.
*/
extern qu_real    qu_decimal_rem_int      (qu_decimal x, qu_int y);
extern qu_real    qu_decimal_rem_long     (qu_decimal x, qu_long y);
extern qu_real    qu_decimal_rem_rational (qu_decimal x, qu_rational y);
extern qu_real    qu_decimal_rem_float    (qu_decimal x, qu_float y);
extern qu_real    qu_decimal_rem_big      (qu_decimal x, qu_big y);
extern qu_complex qu_decimal_rem_complex  (qu_decimal x, qu_complex y);
extern qu_decimal qu_decimal_rem_inf      (qu_decimal x, qu_inf y);
extern qu_real    qu_decimal_rem_integer  (qu_decimal x, qu_integer y);
extern qu_real    qu_decimal_rem_decimal  (qu_decimal x, qu_decimal y);
extern qu_real    qu_decimal_rem_real     (qu_decimal x, qu_real y);
extern qu_number  qu_decimal_rem_number   (qu_decimal x, qu_number y);


/** qu_decimal_pow
	Raise to the power of another.

	These are just convenient wrappers around ={qu_float_pow} and ={qu_big_pow}.
*/
extern qu_real    qu_decimal_pow_int      (qu_decimal x, qu_int y);
extern qu_real    qu_decimal_pow_long     (qu_decimal x, qu_long y);
extern qu_number  qu_decimal_pow_rational (qu_decimal x, qu_rational y);
extern qu_number  qu_decimal_pow_float    (qu_decimal x, qu_float y);
extern qu_number  qu_decimal_pow_big      (qu_decimal x, qu_big y);
extern qu_complex qu_decimal_pow_complex  (qu_decimal x, qu_complex y);
extern qu_real    qu_decimal_pow_inf      (qu_decimal x, qu_inf y);
extern qu_number  qu_decimal_pow_integer  (qu_decimal x, qu_integer y);
extern qu_number  qu_decimal_pow_decimal  (qu_decimal x, qu_decimal y);
extern qu_number  qu_decimal_pow_real     (qu_decimal x, qu_real y);
extern qu_number  qu_decimal_pow_number   (qu_decimal x, qu_number y);


/** qu_decimal_format
	Format a Decimal.

	See ={Format} for details.

	This is just a wrapper around ={qu_float_format} and ={qu_big_format}.

	This function is provided for efficiency between core modules.
	It is not intended to be called by users directly.
*/
extern qu_string qu_decimal_format (qu_decimal x, qu_format_s *f);


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_decimal__setup ();
extern void qu_decimal__init  ();

#endif /*_H_qu_decimal_*/
