/* Dlib.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_dlib_
#define _H_qu_dlib_ 1


/** Dlib
	The final Dlib class.

	Dlib enables access to shared (dynamic) library functions and variables
	from within Qu programs.

	The internals are intentionally hidden since you should not be using Dlib
	from within C code.
*/
extern qu_class QuDlibClass;


/** qu_dlib
	qu_dlib represents an instance of the Dlib class.
	It is a pointer to the instance struct qu_dlib_s.
*/
struct qu_dlib_s
{
	QU_OBJECT_HEAD;
	void  *handle;  /* handle as returned by dlopen */
};


/** qu_dlib_is
	Whether an object is a Dlib.
*/
#define qu_dlib_is(x) (qu_classof (x) == QuDlibClass)


/** qu_dlib_check_spec
	Check for valid call specification.

	Return one of the following:
	@ul
	.-1 invalid spec
	.-2 missing C{":"}
	.-3 too many arguments
	.>= 0 okay, the number of arguments
*/
extern int qu_dlib_check_spec (char *s, long len);



/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_dlib__setup ();
extern void qu_dlib__init  ();

/* for C modules */
extern qu_any    qu_dlib_call         (void *dlhandle, int argc, qu_any *argv);
extern qu_method qu_dlib_cfunc_static (void *dlhandle, qu_string name, qu_string spec);

/* to support Glib more efficiently */
#ifdef QU_SUPPORT_GLIB
extern qu_class QuGObjectClass;
typedef struct qu_gobject_s qu_gobject_s;
struct qu_gobject_s
{
	QU_OBJECT_HEAD;
	void    *ptr;        /* pointer to GObject */
	qu_any   callbacks;  /* Dict of callbacks, might be NULL */
};
typedef struct qu_gobject_s *qu_gobject;
#define qu_gobject_is(x) qu_object_instof (x, QuGObjectClass)
typedef qu_gobject (*qu_gobject_new_f) (void *gobject);
extern qu_gobject_new_f __QuGObjectNew;
#endif


#endif /*_H_qu_dlib_*/
