/* Float.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_float_
#define _H_qu_float_ 1


/** Float qu
	This is the final Float class.
*/
extern qu_class QuFloatClass;


/** qu_float
	qu_float represents an instance of the Float class.
	It is a pointer to the instance struct qu_float_s.
*/
struct qu_float_s
{
	QU_OBJECT_HEAD;
	double value; /* may not be NaN or infinities */
};


/** QU_FLOAT_FLAG
	Flags used by Float objects.
*/
/* all reserved */
#define QU_FLOAT_FLAG_FOO  QU_FLAG_0  /* just to annoy you */


/** QuFloatN
	Predefined system Float objects.

	See ={qu_float_new_static} to create your own.

	@warn
	QU_FLOAT_MIN is a negative value thus contrary to DBL_MIN which is a positive
	value.
*/
extern qu_float QU_FLOAT_MIN; /* -DBL_MAX */
extern qu_float QU_FLOAT_MAX; /* +DBL_MAX */
extern qu_float QuFloat0;     /*  0.0 */
extern qu_float QuFloat1;     /*  1.0 */
extern qu_float QuFloat2;     /*  2.0 */
extern qu_float QuFloat_1;    /* -1.0 */
extern qu_float QuFloat_2;    /* -2.0 */


/** qu_float_new
	Create a new Float object from various C types or other objects.

	@warn
	You may not pass a NaN or infinity to qu_float_new.
*/
#define         qu_float_new       qu_float_newd
#define         qu_float_newh(v)   qu_float_newd (v)
#define         qu_float_newuh(v)  qu_float_newd (v)
#define         qu_float_newi(v)   qu_float_newd (v)
#define         qu_float_newui(v)  qu_float_newd (v)
#define         qu_float_newl(v)   qu_float_newd (v)
#define         qu_float_newul(v)  qu_float_newd (v)
#define         qu_float_newli(v)  qu_float_newd (v)
extern qu_float qu_float_newd      (double v);
#ifdef HAVE_LONG_LONG
#define         qu_float_newll(v)  qu_float_newd (v)
#define         qu_float_newull(v) qu_float_newd (v)
#endif
#ifdef HAVE_LONG_DOUBLE
extern qu_float qu_float_newld     (long double v);
#endif

#define         qu_float_new_int(x)      qu_float_newd (qu_int_tol (x))
extern qu_float qu_float_new_long        (qu_long x);
extern qu_float qu_float_new_rational    (qu_rational x);
#define         qu_float_new_float(x)    (x)
#define         qu_float_new_big(x)      qu_big_as_float (x)
#define         qu_float_new_complex(x)  qu_complex_as_float (x)
extern qu_float qu_float_new_integer     (qu_integer x);
extern qu_float qu_float_new_decimal     (qu_decimal x);
extern qu_float qu_float_new_real        (qu_real x);
extern qu_float qu_float_new_number      (qu_number x);
extern qu_float qu_float_new_string      (qu_string x);
#define         qu_float_new_boolean(x)  (((x) == QuTrue) ? QuFloat1 : QuFloat0)


/** qu_float_newdx
	Create a new Float/Real object from a C double by checking whether or not the
	value is NaN or infinity.

	If the value is NaN then QuEValue is thrown and NULL is returned.

	If the value is infinity then QuEInf is thrown and NULL is returned unless the
	={qu_number_context} allows infinities to be returned, in which case one of
	={QuInfPos} or ={QuInfNeg} is returned based on the sign.
*/
extern qu_real qu_float_newdx (double v);
#ifdef HAVE_LONG_DOUBLE
extern qu_real qu_float_newldx (long double v);
#endif


/** qu_float_new_static
	Create a new system Float object not marked by the GC.

	qu_float_to_static creates one from an existing Float.
*/
extern qu_float qu_float_new_static (double v);
extern qu_float qu_float_to_static  (qu_float x);


/** qu_float_to
	Convert a Float to a C type.

	The functions set v{errno} to zero on success, to EDOM if conversion is not possible
	or ERANGE if the value falls out of the Float range. The I{unsigned} versions fail
	with ERANGE if the value is negative.
*/
#define                   qu_float_toh(x)   qu_number__ltoh   (qu_float_tol  (x))
#define                   qu_float_touh(x)  qu_number__ultouh (qu_float_toul (x))
#define                   qu_float_toi(x)   qu_number__ltoi   (qu_float_tol  (x))
#define                   qu_float_toui(x)  qu_number__ultoui (qu_float_toul (x))
#define                   qu_float_toli(x)  qu_number__ltoli  (qu_float_tol  (x))
extern long               qu_float_tol      (qu_float x);
extern unsigned long      qu_float_toul     (qu_float x);
extern float              qu_float_tof      (qu_float x);
#define                   qu_float_tod(x)   ((x)->value)
#ifdef HAVE_LONG_LONG
extern long long          qu_float_toll     (qu_float x);
extern unsigned long long qu_float_toull    (qu_float x);
#endif
#ifdef HAVE_LONG_DOUBLE
extern long double        qu_float_told     (qu_float x);
#endif


/** qu_float_as
	Convert a Float to another object.

	Only qu_float_as_int can fail (in which case NULL is returned and an exception set)
	because it is always possible to convert a Float into a ={Long}, ={Big}, etc.
*/
extern qu_int     qu_float_as_int         (qu_float x);
#define           qu_float_as_float(x)    (x)
#define           qu_float_as_long(x)     qu_long_newd ((x)->value)
#define           qu_float_as_rational(x) qu_rational_newd ((x)->value)
#define           qu_float_as_big(x)      qu_big_newd ((x)->value)
#define           qu_float_as_complex(x)  qu_complex_new1 ((qu_real) (x))
extern qu_integer qu_float_as_integer     (qu_float x);
#define           qu_float_as_decimal(x)  ((qu_decimal) (x))
#define           qu_float_as_real(x)     ((qu_real) (x))
#define           qu_float_as_number(x)   ((qu_number) x)
#define           qu_float_as_boolean(x)  ((x)->value ? QuTrue : QuFalse)
#define           qu_float_as_string(x)   qu__float__string (x)


/** qu_float_sign
	Returns -1, 0, or 1 if the number is negative, 0, or positive respectively.
*/
extern int qu_float_sign (qu_float x);


/** qu_float_is
	Identity functions.

	qu_float_is tests whether or not an object is a Float.

	qu_float_isneg returns 1 if the number is negative.
	Returns 0 otherwise.

	qu_float_iszero returns 1 if the number is zero.
	Returns 0 otherwise.

	qu_float_isone returns 1 if the number is -1 or 1.
	Returns 0 otherwise.

	qu_float_is1 return 1 if the number is 1.
	Returns 0 otherwise.

	qu_float_iswhole return 1 if the number is whole.
	Returns 0 otherwise.
	The system function f{modf} is used to test.

	qu_float_isfrac return 1 if 0 < x < 1.
	Returns 0 otherwise.

	qu_float_isodd return 1 if the number is odd.
	Returns 0 otherwise.
	The system function f{frexp} and f{fmod} is used to test.
*/
#define    qu_float_is(x)      (qu_classof (x) == QuFloatClass)
#define    qu_float_iszero(x)  ((x)->value == 0)
#define    qu_float_isneg(x)   ((x)->value < 0)
#define    qu_float_is1(x)     ((x)->value == 1)
extern int qu_float_isfrac     (qu_float x);
#define    qu_float_iswhole(x) (modf ((x)->value, &__QuFloatIsWholeHelper) == 0)
extern int qu_float_isone      (qu_float x);
extern int qu_float_isodd      (qu_float x);

/*helper*/
extern double __QuFloatIsWholeHelper; /* do not use! */


/** qu_float_unary
	Unary functions.

	qu_float_pos returns the number itself, obviously.

	qu_float_neg returns a negated value.
	The result is a qu_float.

	qu_float_incr returns the qu_float incremented by 1.
	The result is either a qu_float or a qu_big.

	qu_float_decr returns the qu_float decremented by 1.
	The result is either an qu_float or a qu_big.

	Note that all always succeed.
*/
#define qu_float_pos(x)   (x)
#define qu_float_neg(x)   qu__float__neg  (x)
#define qu_float_incr(x)  qu__float__incr (x)
#define qu_float_decr(x)  qu__float__decr (x)


/** qu_float_eq
	Whether the value is the same as another.
	All return 0 or 1.
*/
#define    qu_float_eq_int(x, y)     ((x)->value == qu_int_tod (y))
extern int qu_float_eq_long          (qu_float x, qu_long y);
extern int qu_float_eq_rational      (qu_float x, qu_rational y);
#define    qu_float_eq_float(x, y)   ((x)->value == (y)->value)
#define    qu_float_eq_big(x, y)     qu_big_eq_float (y, x)
#define    qu_float_eq_complex(x, y) qu_complex_eq_float (y, x)
#define    qu_float_eq_inf(x, y)     (0)
extern int qu_float_eq_integer       (qu_float x, qu_integer y);
extern int qu_float_eq_decimal       (qu_float x, qu_decimal y);
extern int qu_float_eq_real          (qu_float x, qu_real y);
extern int qu_float_eq_number        (qu_float x, qu_number y);


/** qu_float_like
	Whether the absolute value is the same as another.
	All return 0 or 1.
*/
#define    qu_float_like_int(x, y)     (qu_float_cmpabs_int (x, y) == 0)
#define    qu_float_like_long(x, y)    (qu_float_cmpabs_long (x, y) == 0)
extern int qu_float_like_rational      (qu_float x, qu_rational y);
extern int qu_float_like_float         (qu_float x, qu_float y);
#define    qu_float_like_big(x, y)     qu_big_like_float (y, x)
#define    qu_float_like_complex(x, y) qu_complex_like_float (y, x)
#define    qu_float_like_inf(x, y)     (0)
extern int qu_float_like_integer       (qu_float x, qu_integer y);
extern int qu_float_like_decimal       (qu_float x, qu_decimal y);
extern int qu_float_like_real          (qu_float x, qu_real y);
extern int qu_float_like_number        (qu_float x, qu_number y);


/** qu_float_cmp
	Compare with another.
	All return -1, 0 or 1 in the usual manner.
*/
extern int qu_float_cmp_int           (qu_float x, qu_int y);
extern int qu_float_cmp_long          (qu_float x, qu_long y);
extern int qu_float_cmp_rational      (qu_float x, qu_rational y);
extern int qu_float_cmp_float         (qu_float x, qu_float y);
#define    qu_float_cmp_big(x, y)     (-qu_big_cmp_float (y, x))
#define    qu_float_cmp_complex(x, y) (-qu_complex_cmp_float (y, x))
#define    qu_float_cmp_inf(x, y)     (-qu_inf_cmp_float (y, x))
extern int qu_float_cmp_integer       (qu_float x, qu_integer y);
extern int qu_float_cmp_decimal       (qu_float x, qu_decimal y);
extern int qu_float_cmp_real          (qu_float x, qu_real y);
extern int qu_float_cmp_number        (qu_float x, qu_number y);


/** qu_float_cmpabs
	Compare the absolute value with the absolute value of another.
	All return -1, 0 or 1 in the usual manner.
*/
extern int qu_float_cmpabs_int           (qu_float x, qu_int y);
extern int qu_float_cmpabs_long          (qu_float x, qu_long y);
extern int qu_float_cmpabs_rational      (qu_float x, qu_rational y);
extern int qu_float_cmpabs_float         (qu_float x, qu_float y);
#define    qu_float_cmpabs_big(x, y)     (-qu_big_cmpabs_float (y, x))
#define    qu_float_cmpabs_complex(x, y) (-qu_complex_cmpabs_float (y, x))
#define    qu_float_cmpabs_inf(x, y)     (-1)
extern int qu_float_cmpabs_integer       (qu_float x, qu_integer y);
extern int qu_float_cmpabs_decimal       (qu_float x, qu_decimal y);
extern int qu_float_cmpabs_real          (qu_float x, qu_real y);
extern int qu_float_cmpabs_number        (qu_float x, qu_number y);


/** qu_float_plus
	Add with another.
*/
extern qu_decimal qu_float_plus_int           (qu_float x, qu_int y);
extern qu_real    qu_float_plus_long          (qu_float x, qu_long y);
extern qu_real    qu_float_plus_rational      (qu_float x, qu_rational y);
extern qu_decimal qu_float_plus_float         (qu_float x, qu_float y);
#define           qu_float_plus_big(x, y)     qu_big_plus_float (y, x)
#define           qu_float_plus_complex(x, y) qu_complex_plus_float (y, x)
#define           qu_float_plus_inf(x, y)     qu_inf_plus_float (y, x)
extern qu_real    qu_float_plus_integer       (qu_float x, qu_integer y);
extern qu_real    qu_float_plus_decimal       (qu_float x, qu_decimal y);
extern qu_real    qu_float_plus_real          (qu_float x, qu_real y);
extern qu_number  qu_float_plus_number        (qu_float x, qu_number y);


/** qu_float_minus
	Subtract with another.
*/
extern qu_decimal qu_float_minus_int           (qu_float x, qu_int y);
extern qu_real    qu_float_minus_long          (qu_float x, qu_long y);
extern qu_real    qu_float_minus_rational      (qu_float x, qu_rational y);
extern qu_decimal qu_float_minus_float         (qu_float x, qu_float y);
#define           qu_float_minus_big(x, y)     qu_big_rminus_float (y, x)
#define           qu_float_minus_complex(x, y) qu_complex_rminus_float (y, x)
#define           qu_float_minus_inf(x, y)     qu_inf_plus_float (qu_inf_neg (y), x)
extern qu_real    qu_float_minus_integer       (qu_float x, qu_integer y);
extern qu_real    qu_float_minus_decimal       (qu_float x, qu_decimal y);
extern qu_real    qu_float_minus_real          (qu_float x, qu_real y);
extern qu_number  qu_float_minus_number        (qu_float x, qu_number y);


/** qu_float_rminus
	Subtract another with this one.
	Provided for efficieny.
*/
extern qu_decimal qu_float_rminus_int      (qu_float x, qu_int y);
extern qu_real    qu_float_rminus_long     (qu_float x, qu_long y);
extern qu_real    qu_float_rminus_rational (qu_float x, qu_rational y);


/** qu_float_mul
	Multiply with another.
*/
extern qu_decimal qu_float_mul_int           (qu_float x, qu_int y);
extern qu_real    qu_float_mul_long          (qu_float x, qu_long y);
extern qu_real    qu_float_mul_rational      (qu_float x, qu_rational y);
extern qu_decimal qu_float_mul_float         (qu_float x, qu_float y);
#define           qu_float_mul_big(x, y)     qu_big_mul_float (y, x)
#define           qu_float_mul_complex(x, y) qu_complex_mul_float (y, x)
#define           qu_float_mul_inf(x, y)     qu_inf_mul_float (y, x)
extern qu_real    qu_float_mul_integer       (qu_float x, qu_integer y);
extern qu_real    qu_float_mul_decimal       (qu_float x, qu_decimal y);
extern qu_real    qu_float_mul_real          (qu_float x, qu_real y);
extern qu_number  qu_float_mul_number        (qu_float x, qu_number y);


/** qu_float_div
	Divide with another.
*/
extern qu_real   qu_float_div_int           (qu_float x, qu_int y);
extern qu_real   qu_float_div_long          (qu_float x, qu_long y);
extern qu_real   qu_float_div_rational      (qu_float x, qu_rational y);
extern qu_real   qu_float_div_float         (qu_float x, qu_float y);
#define          qu_float_div_big(x, y)     qu_big_rdiv_float (y, x)
#define          qu_float_div_complex(x, y) qu_complex_rdiv_float (y, x)
extern qu_float  qu_float_div_inf           (qu_float x, qu_inf y);
extern qu_real   qu_float_div_integer       (qu_float x, qu_integer y);
extern qu_real   qu_float_div_decimal       (qu_float x, qu_decimal y);
extern qu_real   qu_float_div_real          (qu_float x, qu_real y);
extern qu_number qu_float_div_number        (qu_float x, qu_number y);


/** qu_float_rdiv
	Divide another with this one.
	Provided for efficieny.
*/
extern qu_real qu_float_rdiv_int      (qu_float x, qu_int y);
extern qu_real qu_float_rdiv_long     (qu_float x, qu_long y);
extern qu_real qu_float_rdiv_rational (qu_float x, qu_rational y);


/** qu_float_idiv
	Integer division with another.
*/
extern qu_integer qu_float_idiv_int           (qu_float x, qu_int y);
extern qu_integer qu_float_idiv_long          (qu_float x, qu_long y);
extern qu_integer qu_float_idiv_rational      (qu_float x, qu_rational y);
extern qu_integer qu_float_idiv_float         (qu_float x, qu_float y);
#define           qu_float_idiv_big(x, y)     qu_big_ridiv_float (y, x)
#define           qu_float_idiv_complex(x, y) qu_complex_ridiv_float (y, x)
extern qu_integer qu_float_idiv_inf           (qu_float x, qu_inf y);
extern qu_integer qu_float_idiv_integer       (qu_float x, qu_integer y);
extern qu_integer qu_float_idiv_decimal       (qu_float x, qu_decimal y);
extern qu_integer qu_float_idiv_real          (qu_float x, qu_real y);
extern qu_integer qu_float_idiv_number        (qu_float x, qu_number y);


/** qu_float_ridiv
	Integer division of another with this one.
	Provided for efficieny.
*/
extern qu_integer qu_float_ridiv_int      (qu_float x, qu_int y);
extern qu_integer qu_float_ridiv_long     (qu_float x, qu_long y);
extern qu_integer qu_float_ridiv_rational (qu_float x, qu_rational y);


/** qu_float_rem
	Remainder of dividing with another.
*/
extern qu_real    qu_float_rem_int           (qu_float x, qu_int y);
extern qu_real    qu_float_rem_long          (qu_float x, qu_long y);
extern qu_real    qu_float_rem_rational      (qu_float x, qu_rational y);
extern qu_real    qu_float_rem_float         (qu_float x, qu_float y);
#define           qu_float_rem_big(x, y)     qu_big_rrem_float (y, x)
#define           qu_float_rem_complex(x, y) qu_complex_rrem_float (y, x)
extern qu_float   qu_float_rem_inf           (qu_float x, qu_inf y);
extern qu_real    qu_float_rem_integer       (qu_float x, qu_integer y);
extern qu_real    qu_float_rem_decimal       (qu_float x, qu_decimal y);
extern qu_real    qu_float_rem_real          (qu_float x, qu_real y);
extern qu_number  qu_float_rem_number        (qu_float x, qu_number y);


/** qu_float_rrem
	Remainder of dividing another with this one.
	Provided for efficieny.
*/
extern qu_real qu_float_rrem_int      (qu_float x, qu_int y);
extern qu_real qu_float_rrem_long     (qu_float x, qu_long y);
extern qu_real qu_float_rrem_rational (qu_float x, qu_rational y);


/** qu_float_pow
	Raise to the power of another.
*/
extern qu_number qu_float_pow_int           (qu_float x, qu_int y);
extern qu_number qu_float_pow_long          (qu_float x, qu_long y);
extern qu_number qu_float_pow_rational      (qu_float x, qu_rational y);
extern qu_number qu_float_pow_float         (qu_float x, qu_float y);
#define          qu_float_pow_big(x, y)     qu_big_rpow_float (y, x)
#define          qu_float_pow_complex(x, y) qu_complex_rpow_float (y, x)
extern qu_real   qu_float_pow_inf           (qu_float x, qu_inf y);
extern qu_number qu_float_pow_integer       (qu_float x, qu_integer y);
extern qu_number qu_float_pow_decimal       (qu_float x, qu_decimal y);
extern qu_number qu_float_pow_real          (qu_float x, qu_real y);
extern qu_number qu_float_pow_number        (qu_float x, qu_number y);


/** qu_float_rpow
	Raise another to the power of this one.
	Provided for efficieny.
*/
extern qu_number qu_float_rpow_int      (qu_float x, qu_int y);
extern qu_number qu_float_rpow_long     (qu_float x, qu_long y);
extern qu_number qu_float_rpow_rational (qu_float x, qu_rational y);


/** qu_float_round
	Rounding functions.

	qu_float_ceil rounds the number to the nearest integer towards positive infinity.

	qu_float_floor rounds the number to the nearest integer towards negative infinity.

	qu_float_trunc rounds the number towards zero, that is, it removes the fraction part.

	qu_float_round rounds a Float to the specified number of fraction digits.

	Note that all of these functions always succeed.
*/
#define qu_float_ceil(x)   qu__float_ceil  (x)
#define qu_float_floor(x)  qu__float_floor (x)
#define qu_float_trunc(x)  qu__float_trunc (x)
extern qu_float qu_float_round (qu_float x, int prec);


/** qu_float_frac
	Return the fractional part.
*/
#define qu_float_frac(x) qu__float_frac (x)


/** qu_float_scan
	Scan a Float number.

	v{s} is the start of the string.
	If v{neg} is true then the scanner will not skip leading blanks and
	will not read the sign, that is, the string may not contain the sign
	nor leading spaces.

	On success qu_float_scand returns a C double value while qu_float_scan returns a
	Float object. Both update v{s} to the next cursor position after the number.

	On failure qu_float_scand sets v{errno} and returns NULL while qu_float_scan throws
	ESyntax and return NULL.
*/
extern double   qu_float_scand (char **s, int neg);
extern qu_float qu_float_scan  (char **s, int neg);


/** qu_float_format
	Format a Float.

	See ={Format} for details.

	This function is provided for efficiency between core modules.
	It is not intended to be called by users directly.
*/
extern qu_string qu_float_format (qu_float x, qu_format_s *f);


/** qu_float_hash
	Return the hash value of a C double.

	This is the function used to calculate the hash of a Float object.
	The value should match the value of other ={Number} objects which have
	the same value (the equality test returns 1).
*/
#define qu_float_hashd(v) ((long) (v))


/** qu_float_integer_factor
	Return the Integer which must be multiplied to the Float to get an Integer.

	@xmp
		qu_float_integer_factor (1.2) = 10
		qu_float_integer_factor (1.23) = 100
*/
extern qu_integer qu_float_integer_factor (qu_float x);


/** qu_double_isinf qu_double_isnan qu_double_isfinite \
	qu_longdouble_isinf qu_longdouble_isnan qu_longdouble_isfinite
	Use these instead of the buggy C declarations (isfinite, isnan, etc.).

	Note that the qu_long_double_xxx functions are only available if long
	double is supported (HAVE_LONG_DOUBLE is defined).
*/
#define qu_double_isnan(v)        qu_double__ne (v, v)
#define qu_double_isinf(v)        ((v) == __QuDoubleInfp || (v) == __QuDoubleInfn)
#define qu_double_isfinite(v)     (!(qu_double_isinf (v) || qu_double_isnan (v)))
#ifdef HAVE_LONG_DOUBLE
#define qu_longdouble_isnan(v)    qu_longdouble__ne (v, v)
#define qu_longdouble_isinf(v)    ((v) == __QuLongDoubleInfp || (v) == __QuLongDoubleInfn)
#define qu_longdouble_isfinite(v) (!(qu_longdouble_isinf (v) || qu_longdouble_isnan (v)))
#endif
/***/
extern int qu_double__ne (double a, double b);
extern double __QuDoubleInfp;
extern double __QuDoubleInfn;
#ifdef HAVE_LONG_DOUBLE
extern int qu_longdouble__ne (long double a, long double b);
extern long double __QuLongDoubleInfp;
extern long double __QuLongDoubleInfn;
#endif


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_float__setup ();
extern void qu_float__init ();

/* for efficiency - may only be called from within this file */
extern qu_string  qu__float__string (qu_float self);
extern qu_float   qu__float__neg    (qu_float self);
extern qu_decimal qu__float__incr   (qu_float self);
extern qu_decimal qu__float__decr   (qu_float self);
extern qu_decimal qu__float_ceil    (qu_float self);
extern qu_decimal qu__float_floor   (qu_float self);
extern qu_float   qu__float_trunc   (qu_float self);
extern qu_float   qu__float_frac    (qu_float self);

#endif /*_H_qu_float_*/
