/* Inf.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_inf_
#define _H_qu_inf_ 1


/** Inf qu_inf
	This is the final Inf class.

	qu_inf represents an instance of the Inf class.
	This is either QuInfPos or QuInfNeg.
*/
extern qu_class QuInfClass;


/** QuInfPos QuInfNeg
	These are system objects representing positive and negative infinity.
	They are the only two instances of Inf (like true and false of Boolean).

	They are the keywords infp and infn in program level.
*/
extern qu_inf QuInfPos;
extern qu_inf QuInfNeg;


/** QuInfPosSTR QuInfNegSTR
	These are the String representations.
*/
extern qu_string QuInfPosSTR;
extern qu_string QuInfNegSTR;


/** qu_inf_to
	These are required by other macros.

	All conversions fail with errno set to ERANGE and return the minimum or maximum
	possible value for the type based on the sign of v{x}.
*/
extern short              qu_inf_toh   (qu_inf x);
extern unsigned short     qu_inf_touh  (qu_inf x);
extern int                qu_inf_toi   (qu_inf x);
extern unsigned int       qu_inf_toui  (qu_inf x);
extern long               qu_inf_tol   (qu_inf x);
extern unsigned long      qu_inf_toul  (qu_inf x);
extern long               qu_inf_toli  (qu_inf x);
extern float              qu_inf_tof   (qu_inf x);
extern double             qu_inf_tod   (qu_inf x);
#ifdef HAVE_LONG_LONG
extern long long          qu_inf_toll  (qu_inf x);
extern unsigned long long qu_inf_toull (qu_inf x);
#endif
#ifdef HAVE_LONG_DOUBLE
extern long double        qu_inf_told  (qu_inf x);
#endif


/** qu_inf_as
	All object conversions may fail and throw an exception.
	Those calling qu__inf__toc always throw QuEValue.
	Those calling __qu_inf__ throw QuEInf if the ={qu_number_context} does not allow
	infinities.
*/
#define           qu_inf_as_int(x)      qu__inf__toc ()
#define           qu_inf_as_long(x)     qu__inf__toc ()
#define           qu_inf_as_rational(x) qu__inf__toc ()
#define           qu_inf_as_float(x)    qu__inf__toc ()
#define           qu_inf_as_big(x)      qu__inf__toc ()
extern qu_complex qu_inf_as_complex     (qu_inf x);
#define           qu_int_as_inf(x)      __qu_inf__ (qu_inf_sign (x))
#define           qu_inf_as_integer(x)  qu__inf__toc ()
#define           qu_inf_as_decimal(x)  qu__inf__toc ()
#define           qu_inf_as_real(x)     ((qu_real) __qu_inf__ (qu_inf_sign (x)))
#define           qu_inf_as_number(x)   ((qu_number) qu_inf_as_real(x))
#define           qu_inf_as_string(x)   (((x) == QuInfPos) ? QuInfPosSTR : QuInfNegSTR)
#define           qu_inf_as_boolean(x)  QuTrue
/*helper*/
extern qu_any qu__inf__toc ();


/** qu_inf_sign
	Returns 1 if v{x} is positive infinity, -1 otherwise.
*/
#define qu_inf_sign(x) (((x) == QuInfPos) ? 1 : -1)



/** qu_inf_is
	Infinity identity functions.

	qu_inf_is returns 1 if v{x} is an infinity.
	Returns 0 otherwise.

	qu_inf_isneg returns 1 if v{x} is negative infinity.
	Returns 0 otherwise.

	qu_inf_iswhole always return 1.
	This is probably wrong.

	All the others return 0, obviously.
*/
#define qu_inf_is(x)      (qu_classof (x) == QuInfClass)
#define qu_inf_isneg(x)   (((x) == QuInfPos) ? 0 : 1)
#define qu_inf_iszero(x)  (0)
#define qu_inf_isone(x)   (0)
#define qu_inf_is1(x)     (0)
#define qu_inf_iswhole(x) (1)
#define qu_inf_isfrac(x)  (0)
#define qu_inf_isodd(x)   (0)


/** qu_inf_unary
	Infinity unary functions.
*/
#define qu_inf_pos(x)   (x)
#define qu_inf_neg(x)   ((x) == QuInfPos ? QuInfNeg : QuInfPos)
#define qu_inf_incr(x)  __qu_inf__ (qu_inf_sign (x))
#define qu_inf_decr(x)  __qu_inf__ (qu_inf_sign (x))


/** qu_inf_eq
	Infinity equality tests.
	All return 0 or 1.
*/
#define    qu_inf_eq_int(x, y)      (0)
#define    qu_inf_eq_float(x, y)    (0)
#define    qu_inf_eq_long(x, y)     (0)
#define    qu_inf_eq_rational(x, y) (0)
#define    qu_inf_eq_big(x, y)      (0)
extern int qu_inf_eq_complex        (qu_inf x, qu_complex y);
#define    qu_inf_eq_inf(x, y)      ((x) == (y))
extern int qu_inf_eq_integer        (qu_inf x, qu_integer y);
extern int qu_inf_eq_decimal        (qu_inf x, qu_decimal y);
extern int qu_inf_eq_real           (qu_inf x, qu_real y);
extern int qu_inf_eq_number         (qu_inf x, qu_number y);


/** qu_inf_like
	Infinity absolute value equality tests.
	All return 0 or 1.
*/
#define    qu_inf_like_int(x, y)      (0)
#define    qu_inf_like_long(x, y)     (0)
#define    qu_inf_like_rational(x, y) (0)
#define    qu_inf_like_float(x, y)    (0)
#define    qu_inf_like_big(x, y)      (0)
extern int qu_inf_like_complex        (qu_inf x, qu_complex y);
#define    qu_inf_like_inf(x, y)      (1)
extern int qu_inf_like_integer        (qu_inf x, qu_integer y);
extern int qu_inf_like_decimal        (qu_inf x, qu_decimal y);
extern int qu_inf_like_real           (qu_inf x, qu_real y);
extern int qu_inf_like_number         (qu_inf x, qu_number y);


/** qu_inf_cmp
	Infinity comparisson functions.
	All return -1, 0 or 1 in the usual manner.
*/
#define    qu_inf_cmp_int(x, y)      (((x) == QuInfPos) ? 1 : -1)
#define    qu_inf_cmp_long(x, y)     (((x) == QuInfPos) ? 1 : -1)
#define    qu_inf_cmp_rational(x, y) (((x) == QuInfPos) ? 1 : -1)
#define    qu_inf_cmp_float(x, y)    (((x) == QuInfPos) ? 1 : -1)
#define    qu_inf_cmp_big(x, y)      (((x) == QuInfPos) ? 1 : -1)
extern int qu_inf_cmp_complex        (qu_inf x, qu_complex y);
extern int qu_inf_cmp_inf            (qu_inf x, qu_inf y);
extern int qu_inf_cmp_integer        (qu_inf x, qu_integer y);
extern int qu_inf_cmp_decimal        (qu_inf x, qu_decimal y);
extern int qu_inf_cmp_real           (qu_inf x, qu_real y);
extern int qu_inf_cmp_number         (qu_inf x, qu_number y);


/** qu_inf_cmpabs
	Infinity absolute value comparisson functions.
	All return -1, 0 or 1 in the usual manner.
*/
#define    qu_inf_cmpabs_int(x, y)      (1)
#define    qu_inf_cmpabs_long(x, y)     (1)
#define    qu_inf_cmpabs_rational(x, y) (1)
#define    qu_inf_cmpabs_float(x, y)    (1)
#define    qu_inf_cmpabs_big(x, y)      (1)
extern int qu_inf_cmpabs_complex        (qu_inf x, qu_complex y);
#define    qu_inf_cmpabs_inf(x, y)      (0)
extern int qu_inf_cmpabs_integer        (qu_inf x, qu_integer y);
extern int qu_inf_cmpabs_decimal        (qu_inf x, qu_decimal y);
extern int qu_inf_cmpabs_real           (qu_inf x, qu_real y);
extern int qu_inf_cmpabs_number         (qu_inf x, qu_number y);


/** qu_inf_div
	Divide with another.
*/
#define           qu_inf_div_int(x, y)      qu__inf__divr (x, qu_int_sign (y))
#define           qu_inf_div_float(x, y)    qu__inf__divr (x, qu_float_sign (y))
#define           qu_inf_div_long(x, y)     qu__inf__divr (x, qu_long_sign (y))
#define           qu_inf_div_rational(x, y) qu__inf__divr (x, qu_rational_sign (y))
#define           qu_inf_div_big(x, y)      qu__inf__divr (x, qu_big_sign (y))
extern qu_number  qu_inf_div_complex        (qu_inf x, qu_complex y);
extern qu_any     qu_inf_div_inf            (qu_inf x, qu_inf y);
extern qu_inf     qu_inf_div_integer        (qu_inf x, qu_integer y);
extern qu_inf     qu_inf_div_decimal        (qu_inf x, qu_decimal y);
extern qu_inf     qu_inf_div_real           (qu_inf x, qu_real y);
extern qu_number  qu_inf_div_number         (qu_inf x, qu_number y);
/*helper*/
extern qu_inf qu__inf__divr (qu_inf x, int sign);


/** qu_inf_idiv
	Integer division with another.

	All throw EIdiv.
*/
#define qu_inf_idiv_int(x, y)      qu__inf__idivr ()
#define qu_inf_idiv_float(x, y)    qu__inf__idivr ()
#define qu_inf_idiv_long(x, y)     qu__inf__idivr ()
#define qu_inf_idiv_rational(x, y) qu__inf__idivr ()
#define qu_inf_idiv_big(x, y)      qu__inf__idivr ()
#define qu_inf_idiv_complex(x, y)  qu__inf__idivr ()
#define qu_inf_idiv_inf(x, y)      qu__inf__idivr ()
#define qu_inf_idiv_integer(x, y)  qu__inf__idivr ()
#define qu_inf_idiv_decimal(x, y)  qu__inf__idivr ()
#define qu_inf_idiv_real(x, y)     qu__inf__idivr ()
#define qu_inf_idiv_number(x, y)   qu__inf__idivr ()
/*helper*/
extern qu_integer qu__inf__idivr ();


/** qu_inf_minus
	Subtract with another.
*/
#define           qu_inf_minus_int(x, y)      qu__inf__minusr(x)
#define           qu_inf_minus_float(x, y)    qu__inf__minusr(x)
#define           qu_inf_minus_long(x, y)     qu__inf__minusr(x)
#define           qu_inf_minus_rational(x, y) qu__inf__minusr(x)
#define           qu_inf_minus_big(x, y)      qu__inf__minusr(x)
extern qu_number  qu_inf_minus_complex        (qu_inf x, qu_complex y);
extern qu_inf     qu_inf_minus_inf            (qu_inf x, qu_inf y);
extern qu_inf     qu_inf_minus_integer        (qu_inf x, qu_integer y);
extern qu_inf     qu_inf_minus_decimal        (qu_inf x, qu_decimal y);
extern qu_inf     qu_inf_minus_real           (qu_inf x, qu_real y);
extern qu_number  qu_inf_minus_number         (qu_inf x, qu_number y);
/*helper*/
extern qu_inf qu__inf__minusr (qu_inf x);


/** qu_inf_mul
	Multiply with another.
*/
#define          qu_inf_mul_int(x, y)      qu__inf__mulr (x, qu_int_sign (y))
#define          qu_inf_mul_float(x, y)    qu__inf__mulr (x, qu_float_sign (y))
#define          qu_inf_mul_long(x, y)     qu__inf__mulr (x, qu_long_sign (y))
#define          qu_inf_mul_rational(x, y) qu__inf__mulr (x, qu_rational_sign (y))
#define          qu_inf_mul_big(x, y)      qu__inf__mulr (x, qu_big_sign (y))
#define          qu_inf_mul_complex(x, y)  qu_complex_mul_inf (y, x)
extern qu_inf    qu_inf_mul_inf            (qu_inf x, qu_inf y);
extern qu_inf    qu_inf_mul_integer        (qu_inf x, qu_integer y);
extern qu_inf    qu_inf_mul_decimal        (qu_inf x, qu_decimal y);
extern qu_inf    qu_inf_mul_real           (qu_inf x, qu_real y);
extern qu_number qu_inf_mul_number         (qu_inf x, qu_number y);
/*helper*/
extern qu_number qu__inf__mulr (qu_inf x, int sign);


/** qu_inf_plus
	Add with another.
*/
#define           qu_inf_plus_int(x, y)      qu__inf__plusr (x)
#define           qu_inf_plus_long(x, y)     qu__inf__plusr (x)
#define           qu_inf_plus_rational(x, y) qu__inf__plusr (x)
#define           qu_inf_plus_float(x, y)    qu__inf__plusr (x)
#define           qu_inf_plus_big(x, y)      qu__inf__plusr (x)
extern qu_number  qu_inf_plus_complex        (qu_inf x, qu_complex y);
extern qu_inf     qu_inf_plus_inf            (qu_inf x, qu_inf y);
extern qu_inf     qu_inf_plus_integer        (qu_inf x, qu_integer y);
extern qu_inf     qu_inf_plus_decimal        (qu_inf x, qu_decimal y);
extern qu_inf     qu_inf_plus_real           (qu_inf x, qu_real y);
extern qu_number  qu_inf_plus_number         (qu_inf x, qu_number y);
/*helper*/
extern qu_inf qu__inf__plusr (qu_inf x);


/** qu_inf_pow
	Raise to the power of another.
*/
extern qu_real   qu_inf_pow_int      (qu_inf x, qu_int y);
extern qu_real   qu_inf_pow_long     (qu_inf x, qu_long y);
extern qu_number qu_inf_pow_rational (qu_inf x, qu_rational y);
extern qu_number qu_inf_pow_float    (qu_inf x, qu_float y);
extern qu_number qu_inf_pow_big      (qu_inf x, qu_big y);
extern qu_number qu_inf_pow_complex  (qu_inf x, qu_complex y);
extern qu_real   qu_inf_pow_inf      (qu_inf x, qu_inf y);
extern qu_real   qu_inf_pow_integer  (qu_inf x, qu_integer y);
extern qu_number qu_inf_pow_decimal  (qu_inf x, qu_decimal y);
extern qu_number qu_inf_pow_real     (qu_inf x, qu_real y);
extern qu_number qu_inf_pow_number   (qu_inf x, qu_number y);


/** qu_inf_rem
	Remainder of dividing with another.

	All throw ERem.
*/
#define qu_inf_rem_int(x,y)      qu_inf__remr ()
#define qu_inf_rem_float(x,y)    qu_inf__remr ()
#define qu_inf_rem_long(x,y)     qu_inf__remr ()
#define qu_inf_rem_rational(x,y) qu_inf__remr ()
#define qu_inf_rem_big(x,y)      qu_inf__remr ()
#define qu_inf_rem_complex(x, y) qu_inf__remr ()
#define qu_inf_rem_inf(x, y)     qu_inf__remr ()
#define qu_inf_rem_integer(x, y) qu_inf__remr ()
#define qu_inf_rem_decimal(x, y) qu_inf__remr ()
#define qu_inf_rem_real(x, y)    qu_inf__remr ()
#define qu_inf_rem_number(x, y)  qu_inf__remr ()
/*helper*/
extern qu_any qu_inf__remr ();


/** qu_inf_round
	Rounding functions.

	These are only provided because the macros are needed by some other modules.
	You should have no reason to call these since it is not possible to round
	an infinity to an integer and thus all throw EMath.
*/
#define qu_inf_floor(x)       qu__inf_floor (x)
#define qu_inf_ceil(x)        qu__inf_ceil  (x)
#define qu_inf_trunc(x)       qu__inf_trunc (x)
#define qu_inf_frac(x)        qu__inf_frac  (x)
#define qu_inf_round(x, prec) qu__inf_round (x, QuInteger0)


/** qu_inf_format
	Format an infinity.

	See ={Format} for details.

	This function is provided for efficiency between core modules.
	It is not intended to be called by users directly.
*/
extern qu_string qu_inf_format (qu_inf x, qu_format_s *f);


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_inf__setup ();
extern void qu_inf__init  ();

/* for efficiency - may only be called from within this file */
extern qu_any qu__inf_floor (qu_inf self);
extern qu_any qu__inf_ceil  (qu_inf self);
extern qu_any qu__inf_trunc (qu_inf self);
extern qu_any qu__inf_frac  (qu_inf self);
extern qu_any qu__inf_round (qu_inf self, qu_integer n);

/* for Sys */
extern qu_inf qu_inf_return (qu_any x);

#endif /*_H_qu_inf_*/
