/* Inlines.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_inlines_
#define _H_qu_inlines_ 1


inline static qu_any qu_isys_as (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel && c->accel->fas)
		return c->accel->fas (self, x);
	return qu_object_call1 (self, QU_NAME__as, x);
}



inline static qu_any qu_isys_clear (qu_any self)
{
	qu_class c;
	qu_any   x;

	c = qu_classof (self);
	if (c->accel && c->accel->fclear)
		x = c->accel->fclear (self);
	else
		x = qu_object_call0 (self, QU_NAME__clear);

	QU_ENSURE (x);
	if (self == x)
		return x;

	QU_SYS_WARN_RETVAL (self, "__clear", "self");
	QU_THROW (QuEImplement);
}



inline static qu_int qu_isys_cmp (qu_any self, qu_any x)
{
	if (qu_int_is (self) && qu_int_is (x))
	{
		long u = qu_int_tol (self);
		long v = qu_int_tol (x);

		return (u < v) ? QuInt_1 : ((u > v) ? QuInt1 : QuInt0);
	}
	else
	{
		qu_class c = qu_classof (self);

		if (c->accel && c->accel->fcmp)
		{
			return c->accel->fcmp (self, x);
		}
		else
		{
			qu_any r = qu_object_call1 (self, QU_NAME__cmp, x);

			if (r && qu_int_is (r))
				return r;

			QU_SYS_WARN_RETVAL (self, "__cmp", "Int");
			return qu_object_default_cmp (self, x);
		}
	}
}



inline static qu_string qu_isys_concat (qu_any self, qu_any x)
{
	return qu_string_cat (qu_sys_string (self), qu_sys_string (x), NULL);
}



inline static qu_any qu_isys_copy (qu_any self)
{
	qu_class c;
	qu_any   x;

	c = qu_classof (self);
	if (c->accel && c->accel->fcopy)
		x = c->accel->fcopy (self);
	else
		x = qu_object_call0 (self, QU_NAME__copy);

	QU_ENSURE (x);
	if (qu_classof (x) == c)
		return x;

	QU_SYS_WARN_RETVAL (self, "__copy", c->name->str);
	QU_THROW (QuEImplement);
}



inline static qu_int qu_isys_del (qu_any self, qu_any x)
{
	qu_class c;

	c = qu_classof (self);
	if (c->accel && c->accel->fdel)
		x = c->accel->fdel (self, x);
	else
		x = qu_object_call0 (self, QU_NAME__del);

	QU_ENSURE (x);
	if (qu_int_is (x))
		return x;

	QU_SYS_WARN_RETVAL (self, "__del", "Int");
	QU_THROW (QuEImplement);
}



inline static qu_any qu_isys_decr (qu_any self)
{
	qu_class c;

	c = qu_classof (self);
	if (c->accel && c->accel->fdecr)
		return c->accel->fdecr (self);

	return qu_object_call0 (self, QU_NAME__decr);
}



inline static qu_any qu_isys_div (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel && c->accel->fdiv)
		return c->accel->fdiv (self, x);

	return qu_object_call1 (self, QU_NAME__div, x);
}



inline static qu_boolean qu_isys_eq (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel && c->accel->feq)
		return c->accel->feq (self, x);

	x = qu_object_call1 (self, QU_NAME__eq, x);
	if (x == QuTrue || x == QuFalse)
		return x;

	QU_SYS_WARN_RETVAL (self, "__eq", "Boolean");
	return QuFalse;
}



inline static qu_any qu_isys_each (qu_any self)
{
	register qu_class c = qu_classof (self);

	if (c->accel)
	{
		if (c->accel->feach)
			return c->accel->feach (self);
	}
	else
	{
		register qu_any x = qu_object_call0 (self, QU_NAME__each);

		if (x)
		{
			c = qu_classof (x);
			if (qu_class_autoiter (c))
				return x;
		}
		QU_SYS_WARN_RETVAL (self, "__each", "valid iterator");
	}
	QU_SYS_EIMPLEMENT (self, "__each");
}



inline static qu_any qu_isys_format (qu_any self, int argc, qu_any *argv)
{
	return qu_sys_format (self, argc, argv);
}



inline static qu_boolean qu_isys_ge (qu_any self, qu_any x)
{
	if (qu_int_is (self) && qu_int_is (x))
	{
		return ((long) self >= (long) x) ? QuTrue : QuFalse;
	}
	else
	{
		qu_class c = qu_classof (self);
		if (c->accel)
		{
			if (c->accel->fcmp)
			{
				x = c->accel->fcmp (self, x);
				return (x == QuInt0) ? QuTrue : ((long) x < 0) ? QuFalse : QuTrue;
			}
		}
		else
		{
			qu_any r = qu_object_call1 (self, QU_NAME__cmp, x);

			if (r && qu_int_is (r))
				return (r == QuInt0) ? QuTrue : ((long) r < 0) ? QuFalse : QuTrue;

			QU_SYS_WARN_RETVAL (self, "__cmp", "Int");
		}
		return (qu_object_default_cmpi (self, x) >= 0) ? QuTrue : QuFalse;
	}
}



inline static int qu_isys_attr_restrict (qu_any self, qu_frame f)
{
	if (self == f->self)
	{
		return 0;
	}
	else if (qu_module_is (self))
	{
		return self != (qu_any) qu_frame_get_symbol (f);
	}
	else
	{
		qu_class  c = qu_classof (self);
		qu_symbol b;

		if (c == QuClassClass)
			return 1;

		if (qu_class_is_auto (c) || /* special case */ c == QuRecordClass)
			return 0;

		b = qu_frame_get_symbol (f);

		if (qu_sub_is (b)) /* && !qu_sub_is_inst (b)) */
		{
			qu_class k = qu_sub_get_class ((qu_sub) b);

			if (k)
			{
				if (c == k || qu_class_isa (c, k))
					return 0;

				while (c)
				{
					if (c->mixin && qu_array_contains (c->mixin, k))
						return 0;
					c = c->super;
				}
				return 0;
			}
		}
		return 1;
	}
}



inline static qu_any qu_isys_getattrx (qu_any self, qu_string name, qu_frame f)
{
	return qu_classof (self)->get (self, name, qu_isys_attr_restrict (self, f));
}



inline static qu_any qu_isys_getattr (qu_any self, qu_any x, qu_frame f)
{
	QU_ARG_CHECKZ (qu_string_is (x), "attribute name must be a String");
	return qu_classof (self)->get (self, x, qu_isys_attr_restrict (self, f));
}



inline static qu_any qu_isys_getitem (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fgetitem)
			return c->accel->fgetitem (self, x);
	}
	return qu_object_call1 (self, QU_NAME__getitem, x);
}



inline static qu_any qu_isys_getslice (qu_any self, qu_any i, qu_any len)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fgetslice)
			return c->accel->fgetslice (self, i, len);
	}
	return qu_object_call2 (self, QU_NAME__getslice, i, len);
}



inline static qu_boolean qu_isys_gt (qu_any self, qu_any x)
{
	if (qu_int_is (self) && qu_int_is (x))
		return ((long) self > (long) x) ? QuTrue : QuFalse;
	else
	{
		qu_class c = qu_classof (self);
		if (c->accel)
		{
			if (c->accel->fcmp)
			{
				x = c->accel->fcmp (self, x);
				return (x == QuInt0) ? QuFalse : ((long) x < 0) ? QuFalse : QuTrue;
			}
		}
		else
		{
			qu_any r = qu_object_call1 (self, QU_NAME__cmp, x);
			if (r && qu_int_is (r))
				return (r == QuInt0) ? QuFalse : ((long) r < 0) ? QuFalse : QuTrue;
			QU_SYS_WARN_RETVAL (self, "__cmp", "Int");
		}
		return (qu_object_default_cmpi (self, x) > 0) ? QuTrue : QuFalse;
	}
}



inline static qu_any qu_isys_iand (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fiand)
			return c->accel->fiand (self, x);
	}
	return qu_object_call1 (self, QU_NAME__iand, x);
}



inline static qu_any qu_isys_idiv (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fidiv)
			return c->accel->fidiv (self, x);
	}
	return qu_object_call1 (self, QU_NAME__idiv, x);
}



inline static qu_boolean qu_isys_in (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel)
	{
		if (c->accel->fin)
			return c->accel->fin (self, x);
	}
	x = qu_object_call1 (self, QU_NAME__in, x);
	if (x)
	{
		if (qu_boolean_is (x))
			return x;
		QU_SYS_WARN_RETVAL (self, "__in", "Boolean");
	}
	QU_SYS_EIMPLEMENT (self, "__in");
}



inline static qu_any qu_isys_incr (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fincr)
			return c->accel->fincr (self);
	}
	return qu_object_call0 (self, QU_NAME__incr);
}



inline static qu_any qu_isys_inot (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->finot)
			return c->accel->finot (self);
	}
	return qu_object_call0 (self, QU_NAME__inot);
}



inline static qu_any qu_isys_ior (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fior)
			return c->accel->fior (self, x);
	}
	return qu_object_call1 (self, QU_NAME__ior, x);
}


inline static qu_boolean qu_isys_is (qu_any self, qu_any x)
{
	if (qu_record_is (self) && qu_struct_is (x))
	{
		if (qu_record_of (self, x))
			return QuTrue;
	}
	else if (qu_struct_is (self) && qu_struct_is (x))
	{
		if (self == x)
			return QuTrue;
	}
	else if (qu_sub_is (x))
	{
		/* validator, e.g. _int_ */
		if (((qu_sub) x)->argc == 1 || ((qu_sub) x)->argc == -1 )
		{
			qu_any r = qu_object_call_x1 (x, self);
			if (r == 0 || r == QuFalse)
				return QuFalse;
			return QuTrue;
		}
	}
	else if (qu_method_is (x))
	{
		qu_any r = qu_method_call1 (x, self);
		if (r == 0 || r == QuFalse)
			return QuFalse;
		return QuTrue;
	}
	else if (qu_module_is (x))
	{
		if (self == x)
			return QuTrue;
		else if (qu_module_is (self))
			return QuFalse;
		else
		{
			qu_class c = qu_classof (self);
			x = qu_attr_find (((qu_module) x)->attrs, c->name);
			if (x == (qu_any) c)
				return QuTrue;
		}
	}
	else
	{
		if (qu_object_isa (self, x))
			return QuTrue;
	}
	return QuFalse;
}



inline static qu_any qu_isys_ixor (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fixor)
			return c->accel->fixor (self, x);
	}
	return qu_object_call1 (self, QU_NAME__ixor, x);
}



inline static qu_boolean qu_isys_le (qu_any self, qu_any x)
{
	if (qu_int_is (self) && qu_int_is (x))
		return ((long) self > (long) x) ? QuFalse : QuTrue;
	else
	{
		qu_class c = qu_classof (self);
		if (c->accel)
		{
			if (c->accel->fcmp)
			{
				x = c->accel->fcmp (self, x);
				return (x == QuInt0) ? QuTrue : ((long) x < 0) ? QuTrue : QuFalse;
			}
		}
		else
		{
			qu_any r = qu_object_call1 (self, QU_NAME__cmp, x);
			if (r && qu_int_is (r))
				return (r == QuInt0) ? QuTrue : ((long) r < 0) ? QuTrue : QuFalse;
			QU_SYS_WARN_RETVAL (self, "__cmp", "Int");
		}
		return (qu_object_default_cmpi (self, x) <= 0) ? QuTrue : QuFalse;
	}
}



inline static qu_int qu_isys_len (qu_any self)
{
	qu_class c;
	qu_int   x;

	c = qu_classof (self);
	if (c->accel && c->accel->flen)
		x = c->accel->flen (self);
	else
		x = qu_object_call0 (self, QU_NAME__len);

	QU_ENSURE (x);
	if (qu_int_is (x))
		return x;

	QU_SYS_WARN_RETVAL (self, "__len", "Int");
	QU_THROW (QuEImplement);
}



inline static qu_boolean qu_isys_like (qu_any self, qu_any x)
{
	qu_class   c = qu_classof (self);
	qu_boolean r;

	if (c->accel)
	{
		if (c->accel->flike)
			return c->accel->flike (self, x);
	}
	r = qu_object_call1 (self, QU_NAME__like, x);
	if (r)
	{
		if (qu_boolean_is (r))
			return r;
		QU_SYS_WARN_RETVAL (self, "__like", "Boolean");
	}
	return QuFalse;
}



inline static qu_any qu_isys_lshift (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->flshift)
			return c->accel->flshift (self, x);
	}
	return qu_object_call1 (self, QU_NAME__lshift, x);
}



inline static qu_boolean qu_isys_lt (qu_any self, qu_any x)
{
	if (qu_int_is (self) && qu_int_is (x))
		return ((long) self >= (long) x) ? QuFalse : QuTrue;
	else
	{
		qu_class c = qu_classof (self);
		if (c->accel)
		{
			if (c->accel->fcmp)
			{
				x = c->accel->fcmp (self, x);
				return (x == QuInt0) ? QuFalse : ((long) x < 0) ? QuTrue : QuFalse;
			}
		}
		else
		{
			qu_any r = qu_object_call1 (self, QU_NAME__cmp, x);
			if (r && qu_int_is (r))
				return (r == QuInt0) ? QuFalse : ((long) r < 0) ? QuTrue : QuFalse;
			QU_SYS_WARN_RETVAL (self, "__cmp", "Int");
		}
		return (qu_object_default_cmpi (self, x) < 0) ? QuTrue : QuFalse;
	}
}



inline static qu_any qu_isys_minus (qu_any self, qu_any x)
{
	if (qu_int_is (self) && qu_int_is (x))
	{
		register long r = qu_int_tol (self) - qu_int_tol (x);
		if (QU_INT_MIN <= r && r <= QU_INT_MAX)
			return qu_int_new (r);
		return qu_long_newl (r);
	}
	else
	{
		qu_class c = qu_classof (self);
		if (c->accel)
		{
			if (c->accel->fminus)
				return c->accel->fminus (self, x);
		}
		return qu_object_call1 (self, QU_NAME__minus, x);
	}
}



inline static qu_any qu_isys_rem (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->frem)
			return c->accel->frem (self, x);
	}
	return qu_object_call1 (self, QU_NAME__rem, x);
}



inline static qu_any qu_isys_mul (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel)
	{
		if (c->accel->fmul)
			return c->accel->fmul (self, x);
	}
	return qu_object_call1 (self, QU_NAME__mul, x);
}



inline static qu_boolean qu_isys_ne (qu_any self, qu_any x)
{
	if (self == x)
	{
		return QuFalse;
	}
	else
	{
		qu_class c = qu_classof (self);

		if (c->accel)
		{
			if (c->accel->feq)
				return c->accel->feq (self, x) == QuTrue ? QuFalse : QuTrue;
		}
		else
		{
			qu_boolean r = qu_object_call1 (self, QU_NAME__eq, x);
			if (r)
			{
				if (qu_boolean_is (r))
					return r == QuTrue ? QuFalse : QuTrue;
			}
			QU_SYS_WARN_RETVAL (self, "__eq", "Boolean");
		}
	}
	return QuTrue;
}



inline static qu_any qu_isys_neg (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fneg)
			return c->accel->fneg (self);
	}
	return qu_object_call0 (self, QU_NAME__neg);
}



inline static qu_boolean qu_isys_not (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->ftrue)
			return (c->accel->ftrue (self) == QuTrue) ? QuFalse : QuTrue;
	}
	else
	{
		qu_boolean x = qu_object_call0 (self, QU_NAME__true);
		if (x == QuTrue)
			return QuFalse;
		if (x == QuFalse)
			return QuTrue;
		QU_SYS_WARN_RETVAL (self, "__true", "Boolean");
	}
	return QuFalse;
}



inline static qu_any qu_isys_of (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel)
	{
		if (c->accel->fof)
			return c->accel->fof (self, x);
	}
	return qu_object_call1 (self, QU_NAME__of, x);
}



inline static qu_any qu_isys_plus (qu_any self, qu_any x)
{
	if (qu_int_is (self) && qu_int_is (x))
	{
		register long r = qu_int_tol (self) + qu_int_tol (x);
		if (QU_INT_MIN <= r && r <= QU_INT_MAX)
			return qu_int_new (r);
		return qu_long_newl (r);
	}
	else
	{
		qu_class c = qu_classof (self);
		if (c->accel && c->accel->fplus)
			return c->accel->fplus (self, x);
		return qu_object_call1 (self, QU_NAME__plus, x);
	}
}



inline static qu_any qu_isys_pop (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fpop)
			return c->accel->fpop (self);
	}
	return qu_object_call0 (self, QU_NAME__pop);
}



inline static qu_any qu_isys_pos (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fpos)
			return c->accel->fpos (self);
	}
	return qu_object_call0 (self, QU_NAME__pos);
}



inline static qu_any qu_isys_pow (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel)
	{
		if (c->accel->fpow)
			return c->accel->fpow (self, x);
	}
	return qu_object_call1 (self, QU_NAME__pow, x);
}



inline static qu_any qu_isys_push (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fpush)
			return c->accel->fpush (self, x);
	}
	return qu_object_call1 (self, QU_NAME__push, x);
}



inline static qu_any qu_isys_range (qu_any self, qu_any x)
{
	QU_ARG_CHECK (qu_int_is (self) && qu_int_is (x));
	return qu_range_new_ex (self, x);
}



inline static qu_any qu_isys_ratio (qu_any self, qu_any x)
{
	return qu_rational_new_ratio (self, x);
}



inline static qu_boolean qu_isys_rin (qu_any x, qu_any self)
{
	qu_class c = qu_classof (self);

	if (c->accel)
	{
		if (c->accel->fin)
			return c->accel->fin (self, x);
	}
	x = qu_object_call1 (self, QU_NAME__in, x);
	if (x)
	{
		if (qu_boolean_is (x))
			return x;
		QU_SYS_WARN_RETVAL (self, "__in", "Boolean");
	}
	QU_SYS_EIMPLEMENT (self, "__in");
}



inline static qu_any qu_isys_rshift (qu_any self, qu_any x)
{
	qu_class c = qu_classof (self);

	if (c->accel)
	{
		if (c->accel->frshift)
			return c->accel->frshift (self, x);
	}
	return qu_object_call1 (self, QU_NAME__rshift, x);
}



inline static qu_any qu_isys_setitem (qu_any self, qu_any x, qu_any v)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fsetitem)
			return c->accel->fsetitem (self, x, v);
	}
	return qu_object_call2 (self, QU_NAME__setitem, x, v);
}



inline static qu_any qu_isys_setslice (qu_any self, qu_any from, qu_any len, qu_any v)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fsetslice)
			return c->accel->fsetslice (self, from, len, v);
	}
	return qu_object_call3 (self, QU_NAME__setslice, from, len, v);
}



inline static qu_any qu_isys_setattrx (qu_any self, qu_string name, qu_any val, qu_frame f)
{
	return qu_classof (self)->set (self, name, val, qu_isys_attr_restrict (self, f));
}



inline static qu_any qu_isys_setattr (qu_any self, qu_any x, qu_any val, qu_frame f)
{
	QU_ARG_CHECKZ (qu_string_is (x), "attribute name must be a String");
	return qu_classof (self)->set (self, x, val, qu_isys_attr_restrict (self, f));
}



inline static qu_string qu_isys_string (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->fstring)
			return c->accel->fstring (self);
	}
	else
	{
		qu_any x = qu_object_call0 (self, QU_NAME__string);
		if (x)
		{
			if (qu_string_is (x))
				return x;
		}
		QU_SYS_WARN_RETVAL (self, "__string", "String");
	}
	return qu_object_default_string (self);
}



inline static qu_boolean qu_isys_true (qu_any self)
{
	qu_class c = qu_classof (self);
	if (c->accel)
	{
		if (c->accel->ftrue)
			return c->accel->ftrue (self);
	}
	else
	{
		qu_boolean x = qu_object_call0 (self, QU_NAME__true);
		if (x == QuTrue || x == QuFalse)
			return x;
		QU_SYS_WARN_RETVAL (self, "__true", "Boolean");
	}
	return QuTrue;
}



inline static qu_any qu_isys_xplus (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_isys_plus (r, *argv++);
	}
	return r;
}



inline static qu_any qu_isys_xminus (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_isys_minus (r, *argv++);
	}
	return r;
}



inline static qu_any qu_isys_xdiv (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_isys_div (r, *argv++);
	}
	return r;
}


inline static qu_any qu_isys_xmul (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_sys_mul (r, *argv++);
	}
	return r;
}



inline static qu_any qu_isys_xrem (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_isys_rem (r, *argv++);
	}
	return r;
}



inline static qu_string qu_isys_xcat (int argc, qu_any *argv)
{
	return qu_sys_cat (argc, argv);
}



inline static qu_boolean qu_isys_xeq (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc)
	{
		if (qu_isys_eq (r, *argv++) != QuTrue)
			return QuFalse;
	}
	return QuTrue;
}



inline static qu_boolean qu_isys_xne (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc)
	{
		if (qu_isys_ne (r, *argv++) != QuTrue)
			return QuFalse;
	}
	return QuTrue;
}



inline static qu_boolean qu_isys_xlike (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc)
	{
		if (qu_isys_like (r, *argv++) != QuTrue)
			return QuFalse;
	}
	return QuTrue;
}



inline static qu_any qu_isys_xixor (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_isys_ixor (r, *argv++);
	}
	return r;
}



inline static qu_any qu_isys_xiand (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_isys_iand (r, *argv++);
	}
	return r;
}



inline static qu_any qu_isys_xior (int argc, qu_any *argv)
{
	qu_any r;

	assert (argc > 0);
	r = *argv++;
	while (--argc && r)
	{
		r = qu_isys_ior (r, *argv++);
	}
	return r;
}


#endif /*_H_qu_inlines_*/
