/* Integer.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_integer_
#define _H_qu_integer_ 1


/** Integer qu_integer
	This is the abstract parent class of ={Int} and ={Long}.

	qu_integer represent an instance of an Integer class.
*/
extern qu_class QuIntegerClass;


/** QuIntegerN
	Predefined system objects.
*/
#define QuInteger0  ((qu_integer) QuInt0)   /*  0 */
#define QuInteger1  ((qu_integer) QuInt1)   /*  1 */
#define QuInteger2  ((qu_integer) QuInt2)   /*  2 */
#define QuInteger_1 ((qu_integer) QuInt_1)  /* -1 */
#define QuInteger_2 ((qu_integer) QuInt_2)  /* -2 */


/** qu_integer_new
	Functions to convert C types or other objects into an Integer object.

	For C type conversions, all functions always succeed and the returned
	value is either an ={Int} or ={Long}.

	For object conversions, an exception is thrown and NULL is returned on failure.
*/
#define           qu_integer_newh(v)  qu_integer_newl  (v)
#define           qu_integer_newuh(v) qu_integer_newul (v)
#define           qu_integer_newi(v)  qu_integer_newl  (v)
#define           qu_integer_newui(v) qu_integer_newul (v)
#define           qu_integer_newli(v) qu_integer_newl  (v)
extern qu_integer qu_integer_newl     (long v);
extern qu_integer qu_integer_newul    (unsigned long v);
extern qu_integer qu_integer_newd     (double v);
#if HAVE_LONG_LONG
extern qu_integer qu_integer_newll    (long long v);
extern qu_integer qu_integer_newull   (unsigned long long v);
#endif
#ifdef HAVE_LONG_DOUBLE
extern qu_integer qu_integer_newld    (long double v);
#endif

#define qu_integer_new_int(x)      qu_int_as_integer (x)
#define qu_integer_new_long(x)     qu_float_as_integer (x)
#define qu_integer_new_rational(x) qu_rational_as_integer (x)
#define qu_integer_new_float(x)    qu_float_as_integer (x)
#define qu_integer_new_big(x)      qu_big_as_integer (x)
#define qu_integer_new_complex(x)  qu_complex_as_integer (x)
#define qu_integer_new_integer(x)  (x)
#define qu_integer_new_decimal(x)  qu_decimal_as_integer (x)
#define qu_integer_new_real(x)     qu_real_as_integer (x)
#define qu_integer_new_number(x)   qu_number_as_integer (x)


/** qu_integer_to
	Convert an Integer into a C type.

	The functions can take any object. They all set v{errno} to zero on success.
	On failure, v{errno} is set to EDOM if v{x} is conversion is not possible,
	to ERANGE if the value falls out of the C type range.

	The I{unsigned} versions fail with ERANGE if the value is negative.
*/
#define                   qu_integer_tob(x)    qu_number__ltob     (qu_integer_tol  (x))
#define                   qu_integer_toub(x)   qu_number__ultoub   (qu_integer_toul (x))
#define                   qu_integer_toh(x)    qu_number__ltoh     (qu_integer_tol  (x))
#define                   qu_integer_touh(x)   qu_number__ultouh   (qu_integer_toul (x))
#define                   qu_integer_toi(x)    qu_number__ltoi     (qu_integer_tol  (x))
#define                   qu_integer_toui(x)   qu_number__ultoui   (qu_integer_toul (x))
#define                   qu_integer_toli(x)   qu_number__ltoli    (qu_integer_tol  (x))
#define                   qu_integer_touli(x)  qu_number__ltouli   (qu_integer_tol  (x))
extern long               qu_integer_tol      (qu_any x);
extern unsigned long      qu_integer_toul     (qu_any x);
extern float              qu_integer_tof      (qu_any x);
extern double             qu_integer_tod      (qu_any x);
#ifdef HAVE_LONG_LONG
extern long long          qu_integer_toll     (qu_any x);
extern unsigned long long qu_integer_toull    (qu_any x);
#endif
#ifdef HAVE_LONG_DOUBLE
extern long double        qu_integer_told     (qu_any x);
#endif


/** qu_integer_as
	Convert an Integer into other objects.

	An exception is thrown and NULL is returned on failure.
*/
extern qu_int      qu_integer_as_int         (qu_integer x);
extern qu_long     qu_integer_as_long        (qu_integer x);
#define            qu_integer_as_rational(x) qu_rational_new1 (x)
extern qu_float    qu_integer_as_float       (qu_integer x);
extern qu_big      qu_integer_as_big         (qu_integer x);
#define            qu_integer_as_complex(x)  qu_complex_new1 ((qu_real) (x))
#define            qu_integer_as_integer(x)  (x)
#define            qu_integer_as_real(x)     ((qu_real) (x))
extern qu_decimal  qu_integer_as_decimal     (qu_integer x);
#define            qu_integer_as_number(x)   ((qu_number) (x))
extern qu_string   qu_integer_as_string      (qu_integer x);
extern qu_boolean  qu_integer_as_boolean     (qu_integer x);


/** qu_integer_sign
	Returns -1, 0, 1 if the value is negative, 0, positive respectively.
*/
extern int qu_integer_sign (qu_integer x);


/** qu_integer_is
	Identity functions.

	qu_integer_is checks whether an object is an Integer.

	qu_integer_isneg returns 1 if the value is negative.
	Returns 0 otherwise.

	qu_integer_iszero returns 1 if the value is 0.
	Returns 0 otherwise.

	qu_integer_isone returns 1 if the value is 1 or -1.
	Returns 0 oherwise.

	qu_integer_iswhole always return 1, obviosuly.

	qu_integer_isfrac always return 0, obviosuly.

	qu_integer_isodd return 1 if the value is an odd number.
	Returns 0 otherwise.
*/
#define qu_integer_is(x) (qu_int_is (x) || qu_long_is (x))
extern int qu_integer_iszero  (qu_integer x);
extern int qu_integer_isneg   (qu_integer x);
extern int qu_integer_isone   (qu_integer x);
extern int qu_integer_is1     (qu_integer x);
extern int qu_integer_iswhole (qu_integer x);
extern int qu_integer_isfrac  (qu_integer x);
extern int qu_integer_isodd   (qu_integer x);


/** qu_integer_unary
	Unary functions.

	qu_integer_pos returns the number itself, obviously.

	qu_integer_neg returns a negated value.
	The result is an qu_int.

	qu_integer_incr returns the qu_integer incremented by 1.
	On success, the result is an Int, a Long or ={QuInfPos}.
	QuEBig is thrown if the={qu_number_context} does not allow infinities.

	qu_integer_decr returns the qu_integer decremented by 1.
	On success, the result is an Int, a Long or ={QuInfNeg}.
	QuEBig is thrown if the={qu_number_context} does not allow infinities.

	qu_integer_inot returns a bit-not (inverted) value.
	The result is either an Int or a Long and always succeed.

	These are just convenient wrappers around
	={qu_int_pos} ={qu_int_neg} ={qu_int_incr} ={qu_int_decr} ={qu_int_inot}
	={qu_long_pos} ={qu_long_neg} ={qu_long_incr} ={qu_long_decr} ={qu_long_inot}.
*/
extern qu_integer qu_integer_pos  (qu_integer x);
extern qu_integer qu_integer_neg  (qu_integer x);
extern qu_real    qu_integer_incr (qu_integer x);
extern qu_real    qu_integer_decr (qu_integer x);
extern qu_integer qu_integer_inot (qu_integer x);


/** qu_integer_eq
	Whether the value is the same as another.
	All return 0 or 1.

	These are just convenient wrappers around ={qu_int_eq} and ={qu_long_eq}.
*/
extern int qu_integer_eq_int      (qu_integer x, qu_int y);
extern int qu_integer_eq_long     (qu_integer x, qu_long y);
extern int qu_integer_eq_rational (qu_integer x, qu_rational y);
extern int qu_integer_eq_float    (qu_integer x, qu_float y);
extern int qu_integer_eq_big      (qu_integer x, qu_big y);
extern int qu_integer_eq_complex  (qu_integer x, qu_complex y);
extern int qu_integer_eq_inf      (qu_integer x, qu_inf y);
extern int qu_integer_eq_integer  (qu_integer x, qu_integer y);
extern int qu_integer_eq_decimal  (qu_integer x, qu_decimal y);
extern int qu_integer_eq_real     (qu_integer x, qu_real y);
extern int qu_integer_eq_number   (qu_integer x, qu_number y);


/** qu_integer_like
	Whether the absolute value is the same as the absolute value of another.
	All return 0 or 1.

	These are just convenient wrappers around ={qu_int_like} and ={qu_long_like}.
*/
extern int qu_integer_like_int      (qu_integer x, qu_int y);
extern int qu_integer_like_long     (qu_integer x, qu_long y);
extern int qu_integer_like_rational (qu_integer x, qu_rational y);
extern int qu_integer_like_float    (qu_integer x, qu_float y);
extern int qu_integer_like_big      (qu_integer x, qu_big y);
extern int qu_integer_like_complex  (qu_integer x, qu_complex y);
extern int qu_integer_like_inf      (qu_integer x, qu_inf y);
extern int qu_integer_like_integer  (qu_integer x, qu_integer y);
extern int qu_integer_like_decimal  (qu_integer x, qu_decimal y);
extern int qu_integer_like_real     (qu_integer x, qu_real y);
extern int qu_integer_like_number   (qu_integer x, qu_number y);


/** qu_integer_cmp
	Compare with another.
	All return -1, 0, or -1 in the usual manner.

	These are just convenient wrappers around ={qu_int_cmp} and ={qu_long_cmp}.
*/
extern int qu_integer_cmp_int      (qu_integer x, qu_int y);
extern int qu_integer_cmp_long     (qu_integer x, qu_long y);
extern int qu_integer_cmp_rational (qu_integer x, qu_rational y);
extern int qu_integer_cmp_float    (qu_integer x, qu_float y);
extern int qu_integer_cmp_big      (qu_integer x, qu_big y);
extern int qu_integer_cmp_complex  (qu_integer x, qu_complex y);
extern int qu_integer_cmp_inf      (qu_integer x, qu_inf y);
extern int qu_integer_cmp_integer  (qu_integer x, qu_integer y);
extern int qu_integer_cmp_decimal  (qu_integer x, qu_decimal y);
extern int qu_integer_cmp_real     (qu_integer x, qu_real y);
extern int qu_integer_cmp_number   (qu_integer x, qu_number y);


/** qu_integer_cmpabs
	Compare the absolute value with the absolute value of another.
	All return -1, 0, or -1 in the usual manner.

	These are just convenient wrappers around ={qu_int_cmpabs} and ={qu_long_cmpabs}.
*/
extern int qu_integer_cmpabs_int      (qu_integer x, qu_int y);
extern int qu_integer_cmpabs_long     (qu_integer x, qu_long y);
extern int qu_integer_cmpabs_rational (qu_integer x, qu_rational y);
extern int qu_integer_cmpabs_float    (qu_integer x, qu_float y);
extern int qu_integer_cmpabs_big      (qu_integer x, qu_big y);
extern int qu_integer_cmpabs_complex  (qu_integer x, qu_complex y);
extern int qu_integer_cmpabs_inf      (qu_integer x, qu_inf y);
extern int qu_integer_cmpabs_integer  (qu_integer x, qu_integer y);
extern int qu_integer_cmpabs_decimal  (qu_integer x, qu_decimal y);
extern int qu_integer_cmpabs_real     (qu_integer x, qu_real y);
extern int qu_integer_cmpabs_number   (qu_integer x, qu_number y);


/** qu_integer_plus
	Add with another.

	These are just convenient wrappers around ={qu_int_plus} and ={qu_long_plus}.
*/
extern qu_real    qu_integer_plus_int      (qu_integer x, qu_int y);
extern qu_real    qu_integer_plus_long     (qu_integer x, qu_long y);
extern qu_real    qu_integer_plus_rational (qu_integer x, qu_rational y);
extern qu_real    qu_integer_plus_float    (qu_integer x, qu_float y);
extern qu_real    qu_integer_plus_big      (qu_integer x, qu_big y);
extern qu_complex qu_integer_plus_complex  (qu_integer x, qu_complex y);
extern qu_real    qu_integer_plus_inf      (qu_integer x, qu_inf y);
extern qu_real    qu_integer_plus_integer  (qu_integer x, qu_integer y);
extern qu_real    qu_integer_plus_decimal  (qu_integer x, qu_decimal y);
extern qu_real    qu_integer_plus_real     (qu_integer x, qu_real y);
extern qu_number  qu_integer_plus_number   (qu_integer x, qu_number y);


/** qu_integer_minus
	Subtract with another.

	These are just convenient wrappers around ={qu_int_minus} and ={qu_long_minus}.
*/
extern qu_real    qu_integer_minus_int      (qu_integer x, qu_int y);
extern qu_real    qu_integer_minus_long     (qu_integer x, qu_long y);
extern qu_real    qu_integer_minus_rational (qu_integer x, qu_rational y);
extern qu_real    qu_integer_minus_float    (qu_integer x, qu_float y);
extern qu_real    qu_integer_minus_big      (qu_integer x, qu_big y);
extern qu_complex qu_integer_minus_complex  (qu_integer x, qu_complex y);
extern qu_real    qu_integer_minus_inf      (qu_integer x, qu_inf y);
extern qu_real    qu_integer_minus_integer  (qu_integer x, qu_integer y);
extern qu_real    qu_integer_minus_decimal  (qu_integer x, qu_decimal y);
extern qu_real    qu_integer_minus_real     (qu_integer x, qu_real y);
extern qu_number  qu_integer_minus_number   (qu_integer x, qu_number y);


/** qu_integer_mul
	Multiply with another.

	These are just convenient wrappers around ={qu_int_mul} and ={qu_long_mul}.
*/
extern qu_real    qu_integer_mul_int      (qu_integer x, qu_int y);
extern qu_real    qu_integer_mul_long     (qu_integer x, qu_long y);
extern qu_real    qu_integer_mul_rational (qu_integer x, qu_rational y);
extern qu_real    qu_integer_mul_float    (qu_integer x, qu_float y);
extern qu_real    qu_integer_mul_big      (qu_integer x, qu_big y);
extern qu_complex qu_integer_mul_complex  (qu_integer x, qu_complex y);
extern qu_real    qu_integer_mul_inf      (qu_integer x, qu_inf y);
extern qu_real    qu_integer_mul_integer  (qu_integer x, qu_integer y);
extern qu_real    qu_integer_mul_decimal  (qu_integer x, qu_decimal y);
extern qu_real    qu_integer_mul_real     (qu_integer x, qu_real y);
extern qu_number  qu_integer_mul_number   (qu_integer x, qu_number y);


/** qu_integer_div
	Divide with another.

	These are just convenient wrappers around ={qu_int_div} and ={qu_long_div}.
*/
extern qu_real    qu_integer_div_int      (qu_integer x, qu_int y);
extern qu_real    qu_integer_div_long     (qu_integer x, qu_long y);
extern qu_real    qu_integer_div_rational (qu_integer x, qu_rational y);
extern qu_real    qu_integer_div_float    (qu_integer x, qu_float y);
extern qu_real    qu_integer_div_big      (qu_integer x, qu_big y);
extern qu_complex qu_integer_div_complex  (qu_integer x, qu_complex y);
extern qu_real    qu_integer_div_inf      (qu_integer x, qu_inf y);
extern qu_real    qu_integer_div_integer  (qu_integer x, qu_integer y);
extern qu_real    qu_integer_div_decimal  (qu_integer x, qu_decimal y);
extern qu_real    qu_integer_div_real     (qu_integer x, qu_real y);
extern qu_number  qu_integer_div_number   (qu_integer x, qu_number y);


/** qu_integer_idiv
	Integer division with another.

	These are just convenient wrappers around ={qu_int_idiv} and ={qu_long_idiv}.
*/
extern qu_integer qu_integer_idiv_int      (qu_integer x, qu_int y);
extern qu_integer qu_integer_idiv_long     (qu_integer x, qu_long y);
extern qu_integer qu_integer_idiv_rational (qu_integer x, qu_rational y);
extern qu_integer qu_integer_idiv_float    (qu_integer x, qu_float y);
extern qu_integer qu_integer_idiv_big      (qu_integer x, qu_big y);
extern qu_integer qu_integer_idiv_complex  (qu_integer x, qu_complex y);
extern qu_integer qu_integer_idiv_inf      (qu_integer x, qu_inf y);
extern qu_integer qu_integer_idiv_integer  (qu_integer x, qu_integer y);
extern qu_integer qu_integer_idiv_decimal  (qu_integer x, qu_decimal y);
extern qu_integer qu_integer_idiv_real     (qu_integer x, qu_real y);
extern qu_integer qu_integer_idiv_number   (qu_integer x, qu_number y);


/** qu_integer_rem
	Remainder of dividing with another.

	These are just convenient wrappers around ={qu_int_rem} and ={qu_long_rem}.
*/
extern qu_real    qu_integer_rem_int      (qu_integer x, qu_int y);
extern qu_real    qu_integer_rem_long     (qu_integer x, qu_long y);
extern qu_real    qu_integer_rem_rational (qu_integer x, qu_rational y);
extern qu_real    qu_integer_rem_float    (qu_integer x, qu_float y);
extern qu_real    qu_integer_rem_big      (qu_integer x, qu_big y);
extern qu_complex qu_integer_rem_complex  (qu_integer x, qu_complex y);
extern qu_integer qu_integer_rem_inf      (qu_integer x, qu_inf y);
extern qu_integer qu_integer_rem_integer  (qu_integer x, qu_integer y);
extern qu_real    qu_integer_rem_decimal  (qu_integer x, qu_decimal y);
extern qu_real    qu_integer_rem_real     (qu_integer x, qu_real y);
extern qu_number  qu_integer_rem_number   (qu_integer x, qu_number y);


/** qu_integer_pow
	Raise to the power of another.

	These are just convenient wrappers around ={qu_int_pow} and ={qu_long_pow}.
*/
extern qu_real    qu_integer_pow_int      (qu_integer x, qu_int y);
extern qu_real    qu_integer_pow_long     (qu_integer x, qu_long y);
extern qu_number  qu_integer_pow_rational (qu_integer x, qu_rational y);
extern qu_number  qu_integer_pow_float    (qu_integer x, qu_float y);
extern qu_number  qu_integer_pow_big      (qu_integer x, qu_big y);
extern qu_complex qu_integer_pow_complex  (qu_integer x, qu_complex y);
extern qu_real    qu_integer_pow_inf      (qu_integer x, qu_inf y);
extern qu_number  qu_integer_pow_integer  (qu_integer x, qu_integer y);
extern qu_number  qu_integer_pow_decimal  (qu_integer x, qu_decimal y);
extern qu_number  qu_integer_pow_real     (qu_integer x, qu_real y);
extern qu_number  qu_integer_pow_number   (qu_integer x, qu_number y);


/** qu_integer_iand
	Bit-and with another.

	These are just convenient wrappers around ={qu_int_iand} and ={qu_long_iand}.
*/
extern qu_integer qu_integer_iand_int     (qu_integer x, qu_int y);
extern qu_integer qu_integer_iand_long    (qu_integer x, qu_long y);
extern qu_integer qu_integer_iand_integer (qu_integer x, qu_integer y);


/** qu_integer_ior
	Bit-or with another.

	These are just convenient wrappers around ={qu_int_ior} and ={qu_long_ior}.
*/
extern qu_integer qu_integer_ior_int     (qu_integer x, qu_int y);
extern qu_integer qu_integer_ior_long    (qu_integer x, qu_long y);
extern qu_integer qu_integer_ior_integer (qu_integer x, qu_integer y);


/** qu_integer_ixor
	Bit-xor with another.

	These are just convenient wrappers around ={qu_int_ixor} and ={qu_long_ixor}.
*/
extern qu_integer qu_integer_ixor_int     (qu_integer x, qu_int y);
extern qu_integer qu_integer_ixor_iong    (qu_integer x, qu_long y);
extern qu_integer qu_integer_ixor_integer (qu_integer x, qu_integer y);


/** qu_integer_lshift
	Left-shift by the amount of another.

	These are just convenient wrappers around ={qu_int_lshift} and ={qu_long_lshift}.
*/
extern qu_integer qu_integer_lshift_int     (qu_integer x, qu_int y);
extern qu_integer qu_integer_lshift_long    (qu_integer x, qu_long y);
extern qu_integer qu_integer_lshift_integer (qu_integer x, qu_integer y);


/** qu_integer_rshift
	Right-shift by the amount of another.

	These are just convenient wrappers around ={qu_int_rshift} and ={qu_long_rshift}.
*/
extern qu_integer qu_integer_rshift_int     (qu_integer x, qu_int y);
extern qu_integer qu_integer_rshift_long    (qu_integer x, qu_long y);
extern qu_integer qu_integer_rshift_integer (qu_integer x, qu_integer y);


/** qu_integer_format
	Format an Integer.

	See ={Format} for details.

	This is just a convenient wrapper around ={qu_int_format} and ={qu_long_format}.

	This function is provided for efficiency between core modules.
	It is not intended to be called by users directly.
*/
extern qu_string qu_integer_format (qu_integer x, qu_format_s *f);


/** qu_integer_gcd
	Calculate the GCD (Greatest Common Divisor).

	Note that the signs are ignored and the result is a positive Integer.

	@warn
	Calling qu_integer_gcd with y == 0 throws EDiv.

	You may not call qu_integer_gcdl with y == 0. No checks are made.
*/
extern long       qu_integer_gcdl (long x, long y);
extern qu_integer qu_integer_gcd  (qu_integer x, qu_integer y);


/** int16
	Macros for 16-bits integers.

	Note that 16-bits integer is always available.
*/
#if SIZEOF_SHORT == 2
#define qu_i16_t           short
#define qu_u16_t           unsigned short
#define QU_I16_MIN         SHORT_MIN
#define QU_I16_MAX         SHORT_MAX
#define QU_U16_MIN         USHORT_MIN
#define QU_U16_MAX         USHORT_MAX
#define qu_integer_newi16  qu_integer_newh
#define qu_integer_newui16 qu_integer_newuh
#define qu_integer_toi16   qu_integer_toh
#define qu_integer_toui16  qu_integer_touh
#define QU_ARG_i16         QU_ARG_short
#define QU_ARG_ui16        QU_ARG_ushort
#define QU_ARG_vi16        QU_ARG_vshort
#define QU_ARG_vui16       QU_ARG_vushort
#else
#error no 16bits integer
#endif


/** int32
	Macros for 32-bits integers.

	Note that 32-bits integer is always available.
*/
#if SIZEOF_INT == 4
#define qu_i32_t           int
#define qu_u32_t           unsigned int
#define QU_I32_MIN         INT_MIN
#define QU_I32_MAX         INT_MAX
#define QU_UT32_MIN        UINT_MIN
#define QU_UT32_MAX        UINT_MAX
#define qu_integer_newi32  qu_integer_newi
#define qu_integer_newui32 qu_integer_newui
#define qu_integer_toi32   qu_integer_toi
#define qu_integer_toui32  qu_integer_toui
#define QU_ARG_i32         QU_ARG_int
#define QU_ARG_ui32        QU_ARG_uint
#define QU_ARG_vi32        QU_ARG_vint
#define QU_ARG_vui32       QU_ARG_vuint
#else
#error no 32bits integer
#endif


/** int64
	Macros for 64-bits integers.

	Note that 64-bits integer may not be available so test for whether
	C{qu_i64_t} is defined.
*/
#if SIZEOF_LONG == 8
#  define qu_i64_t           long
#  define qu_ui64_t          unsigned long
#  define QU_I64_MIN         LONG_MIN
#  define QU_I64_MAX         LONG_MAX
#  define QU_UI64_MIN        ULONG_MIN
#  define QU_UI64_MAX        ULONG_MAX
#  define qu_integer_newi64  qu_integer_newl
#  define qu_integer_newui64 qu_integer_newul
#  define qu_integer_toi64   qu_integer_tol
#  define qu_integer_toui64  qu_integer_toul
#  define QU_ARG_i64         QU_ARG_long
#  define QU_ARG_ui64        QU_ARG_ulong
#  define QU_ARG_vi64        QU_ARG_vlong
#  define QU_ARG_vui64       QU_ARG_vulong
#elif defined(HAVE_LONG_LONG) && SIZEOF_LONG_LONG == 8
#  define qu_i64_t           long long
#  define qu_u64_t           unsigned long long
#  define QU_I64_MIN         LONG_LONG_MIN
#  define QU_I64_MAX         LONG_LONG_MAX
#  define QU_UI64_MIN        ULONG_LONG_MIN
#  define QU_UI64_MAX        ULONG_LONG_MAX
#  define qu_integer_newi64  qu_integer_newll
#  define qu_integer_newui64 qu_integer_newull
#  define qu_integer_toi64   qu_integer_toll
#  define qu_integer_toui64  qu_integer_toull
#  define QU_ARG_i64         QU_ARG_longlong
#  define QU_ARG_ui64        QU_ARG_ulonglong
#  define QU_ARG_vi64        QU_ARG_vlonglong
#  define QU_ARG_vui64       QU_ARG_vulonglong
#endif


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_integer__setup ();
extern void qu_integer__init  ();

#endif /*_H_qu_integer_*/
