/* Method.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_method_
#define _H_qu_method_ 1


/** Method
	This is the final Method class.

	A Method is a ={Sub} which is bound to a self value. Calling means
	calling the Sub object passing the self object bound to it.
*/
extern qu_class QuMethodClass;


/** qu_method
	qu_method represents an instance of the Method class.
	It is a pointer to the instance struct qu_method_s.
*/
struct qu_method_s
{
	QU_OBJECT_HEAD;
	qu_sub   sub;    /* the Sub object */
	qu_any   self;   /* the 'self' */
	qu_array args;   /* arguments before formal arguments, initially 0 */
};


/** qu_method_is
	Whether or not the object is a Method.
*/
#define qu_method_is(o) (qu_classof(o) == QuMethodClass)


/** QU_METHOD_FLAG
	Flags used by Method objects.

	Do not access directly. Use the accessors instead.
*/
#define QU_METHOD_FLAG_DELEGATE  QU_FLAG_0  /* got it from __get */
#define QU_METHOD_FLAG_SETSELF   QU_FLAG_1  /* must set self on call? */


/** qu_method_is_delegate
	Whether the method was created by __get.
	This is to support delegation.

	@warn
	Don't use this. It's for Object.
*/
#define qu_method_is_delegate(self)   qu_flag_test (self, QU_METHOD_FLAG_DELEGATE)
#define qu_method_set_delegate(self)  qu_flag_set  (self, QU_METHOD_FLAG_DELEGATE)


/** qu_method_new
	Create a new Method.
*/
extern qu_method qu_method_new (qu_sub sub, qu_any self);


/** qu_method_get_module
	Get the ={Module} object where the Method is declared.
*/
extern qu_module qu_method_get_module (qu_method self);


/** qu_method_get_class
	Get the ={Class} object where the Method is declared.
*/
extern qu_class qu_method_get_class (qu_method self);


/** qu_method_call
	Call a method.
*/
extern qu_any qu_method_calln (qu_method self, int argc, qu_any *argv);
extern qu_any qu_method_call4 (qu_method self, qu_any a1, qu_any a2, qu_any a3, qu_any a4);
extern qu_any qu_method_call3 (qu_method self, qu_any a1, qu_any a2, qu_any a3);
extern qu_any qu_method_call2 (qu_method self, qu_any a1, qu_any a2);
extern qu_any qu_method_call1 (qu_method self, qu_any a1);
extern qu_any qu_method_call0 (qu_method self);


/** qu_method_as
	Various representation.
*/
#define qu_method_as_string(self)  qu__method__string (self)
#define qu_method_as_boolean(self) (QuTrue)


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_method__setup (void);
extern void qu_method__init  (void);

/* for efficiency - may only be called from within this file */
extern qu_string qu__method__string (qu_method self);

/*EXPERIMENTAL - currying - for Reduce.c */
extern qu_method qu_method_curry_compile_time (qu_any sub, qu_ast list);

/* for interp */
extern qu_any qu_method_calln_attr (qu_method self, qu_any theself, int argc, qu_any *argv);

/*EXPERIMENTAL - for extern */
extern qu_method qu_method_new_static (qu_sub sub, qu_any self);

#endif /*_H_qu_method_*/
