/* Opaque.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_opaque_
#define _H_qu_opaque_ 1


/** Opaque
	The Opaque class provides a convenient way to wrap C objects.
*/
extern qu_class QuOpaqueClass;


/** qu_opaquemark_f qu_opaquefree_f
	The mark and free method prototypes.
*/
typedef void (*qu_opaquemark_f) (qu_opaque o);
typedef void (*qu_opaquefree_f) (qu_opaque o);


/** QU_OPAQUE_HEAD QU_OPAQUE_HEADEX
	To create an Opaque object, use this instead of ={QU_OBJECT_HEAD}.

	@warn
	The f{opaquefree} function must B{not} free the object itself because
	that is done internally by Opaque.
*/
#define QU_OPAQUE_HEAD \
	QU_OBJECT_HEAD; \
	qu_opaquemark_f  mark;     /* called when marked, may be 0 */ \
	qu_opaquefree_f  free      /* called before freed, may be 0 */

#define QU_OPAQUE_HEADEX \
	QU_OBJECT_HEAD; \
	qu_opaquemark_f  mark;     /* called when marked, may be 0 */ \
	qu_opaquefree_f  free;     /* called before freed, may be 0 */ \
	qu_sub1_f        getitem;  /* to implement a C struct */ \
	qu_sub2_f        setitem   /* -ditto- */


/** qu_opaque
	qu_opaque represents an instance of the Opaque class.
	It is a pointer to the instance struct qu_opaque_s.
*/
struct qu_opaque_s
{
	QU_OPAQUE_HEAD;
};


/** qu_opaque_new qu_opaque_new_ex
	Create a new Opaque instance.

	@args
	.z  is the size of the struct.
	.g  is the __getitem C method.
	.s  is the __setitem C method.
	.m  is the mark C function to call to mark referred objects.
	.f  is the free C function to free memory.

	@p
	Use the macro for it is easier to use than qu_opaque__new.
*/
#define qu_opaque_new(m, f, z) \
	((qu_any) qu_opaque__new ((qu_opaquemark_f) m, (qu_opaquefree_f) f, z))

#define qu_opaque_new_ex(m, f, g, s, z) \
	((qu_any) qu_opaque__new ((qu_opaquemark_f) m, (qu_opaquefree_f) f, z, (qu_sub1_f) g, (qu_sub2_f) s))

extern qu_opaque qu_opaque__new    (qu_opaquemark_f m, qu_opaquefree_f f, size_t z);
extern qu_opaque qu_opaque__new_ex (qu_opaquemark_f m, qu_opaquefree_f f, size_t z, qu_sub1_f g, qu_sub2_f s);


/** qu_opaque_is
	qu_opaque_is checks whether or not an object is an Opaque object.

	QU_OPAQUE_CHECK checks whether or not an qu_opaque object is indeed your
	qu_opaque object. You can use either QU_OPAQUE_CHECK_MARK or QU_OPAQUE_CHECK_FREE.
*/
#define qu_opaque_is(o) (qu_classof(o) == QuOpaqueClass)
#define QU_OPAQUE_CHECK_MARK(o, markfunc) (((qu_opaque) o)->mark == (qu_opaquemark_f) markfunc)
#define QU_OPAQUE_CHECK_FREE(o, freefunc) (((qu_opaque) o)->free == (qu_opaquefree_f) freefunc)


/** QU_OPAQUE_FLAG
	Flags used by Opaque objects.

	Do not access directly. Use the accessors instead.
*/
/* 0-7 reserved by Opaque */
#define QU_OPAQUE_FLAG_EX       QU_FLAG_0 /* uses QU_OPAQUE_HEADEX */
#define QU_OPAQUE_FLAG_STRFIELD QU_FLAG_1 /* only call setitem/getitem if index is a String */


/** qu_opaque_is_ex
	Whether the instance struct uses QU_OPAQUE_HEADEX.
*/
#define qu_opaque_is_ex(self) qu_flag_test (self, QU_OPAQUE_FLAG_EX)


/** qu_opaque_strfield
	Whethter to call setitem/getitem only if index is a String.
*/
#define qu_opaque_is_strfield(self)  qu_flag_test (self, QU_OPAQUE_FLAG_STRFIELD)
#define qu_opaque_set_strfield(self) qu_flag_set  (self, QU_OPAQUE_FLAG_STRFIELD)


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_opaque__setup ();
extern void qu_opaque__init  ();

#endif /*_H_qu_opaque_*/
