/* Pointer.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_pointer_
#define _H_qu_pointer_ 1


/** Pointer
	This is the Pointer class.
*/
extern qu_class QuPointerClass;


/** qu_pointer
	qu_pointer represents an instance of the Pointer class.
	It is a pointer to the instance struct qu_pointer_s.
*/
#define QU_POINTER_HEAD \
	QU_OBJECT_HEAD; \
	long    size;  /* content length \                           */ \
	long    dummy; /* not used        >-- must match qu_string_s */ \
	void   *ptr;   /* the pointer    /                           */ \
	long    delta; /* incremental */ \
	qu_any  spec;  /* C struct specification: 0 or String or Struct or Dict */

struct qu_pointer_s
{
	QU_POINTER_HEAD
};


/** QU_POINTER_FLAG
	Flags used by Pointer objects.

	Do not access directly. Use the accessors instead.

	Flag-0 to Flag-10 are reserved for the Pointer class.
*/
#define QU_POINTER_FLAG_FROZEN    QU_FLAG_1  /* immutable? */
#define QU_POINTER_FLAG_ALLOCATED QU_FLAG_4  /* if is was allocated */


/** qu_pointer_freeze
	Freeze and unfreeze pointer.
*/
#define qu_pointer_frozen(self)   qu_flag_test  (self, QU_POINTER_FLAG_FROZEN)
#define qu_pointer_freeze(self)   qu_flag_set   (self, QU_POINTER_FLAG_FROZEN)
#define qu_pointer_unfreeze(self) qu_flag_unset (self, QU_POINTER_FLAG_FROZEN)


/** qu_pointer_allocated
	Whether the pointer address is allocated via ={qu_mem_alloc}.
*/
#define qu_pointer_allocated(self) qu_flag_test (self, QU_POINTER_FLAG_ALLOCATED)


/** qu_pointer_is
	Whether or not the object is a Pointer.
*/
#define qu_pointer_is(o) qu_object_instof (o, QuPointerClass)


/** QuPointerNULL
	The NULL pointer.
*/
extern qu_pointer QuPointerNULL;


/** qu_pointer_new
	Create a new Pointer object.

	With qu_pointer_new_allocated, v{m} must be allocated with ={qu_mem_alloc}.

	With qu_pointer_new_alloc, the memory pointed by v{ptr} is allocated with
	={qu_mem_alloc}.

	qu_pointer_new_calloc does the same and the content is initialized with zeroes.

	With qu_pointer_new, v{m} is any arbitrary memory address.
	The memory is not automatically freed.
*/
extern qu_pointer qu_pointer_new           (void *p);
extern qu_pointer qu_pointer_new_static    (void *p);
extern qu_pointer qu_pointer_new_allocated (void *m, long size);
extern qu_pointer qu_pointer_new_alloc     (long size);
extern qu_pointer qu_pointer_new_calloc    (long size);


/** qu_pointer_get qu_pointer_set
	Set or get value.

	See ={qu_struct_type_to_char} for valid types.

	qu_pointer_setrecord sets the record values.
*/
extern qu_any qu_pointer_get        (void *p, char type);
extern int    qu_pointer_set        (void *p, char type, qu_any x);
extern int    qu_pointer_set_record (void *p, qu_record r);
extern int    qu_pointer_get_record (void *p, qu_record r);


/** qu_pointer_as
	Various rerpresentation.
*/
#define qu_pointer_as_string(self)  qu__pointer__string (self)
#define qu_pointer_as_boolean(self) ((self)->ptr ? QuTrue : QuFalse)


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class. */
extern void qu_pointer__setup ();
extern void qu_pointer__init  ();

/* for efficiency - may only be called from within this file */
extern qu_string qu__pointer__string (qu_pointer self);

/* dont publish yet - used by Record as Pointer */
extern qu_pointer qu_pointer_new_record (qu_record r);

/* for implementors */
extern void qu__pointer__destroy (qu_pointer self);
extern void qu__pointer__mark    (qu_pointer self);

/* for the Pointers module */
extern qu_pointer qu_pointer_predefined (long size, char spec);

#endif /*_H_qu_pointer_*/
