/* Range.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_range_
#define _H_qu_range_ 1


/** Range qu_range
	This is the final Range class.
*/
extern qu_class QuRangeClass;


/** qu_range
	qu_range represents an instance of the Range class.
	It is a pointer to the instance struct qu_range_s.

	@warn
	All fields must be in the ={Int} class range.
*/
struct qu_range_s
{
	QU_OBJECT_HEAD;
	long first;  /* start of range */
	long last;   /* end of range */
	long step;   /* delta */
	long count;  /* number of items (pre-calculated by Range) */
};


/** qu_range_is
	Whether or not an object is a Range.
*/
#define qu_range_is(o) (qu_classof(o) == QuRangeClass)


/** qu_range_new
	Create a new Range object.

	Make sure the arguments are in the ={Int} range and does not produce a count
	more than ={GC_MAX_SIZE} otherwise you get an EArgValue exception.
*/
extern qu_range qu_range_new    (long first, long last, long step);
extern qu_range qu_range_new_ex (qu_any first, qu_any last);


/** qu_range_to_static
	Create a Range which is never garbage collected nor marked from an
	existing one.
*/
extern qu_range qu_range_to_static (qu_range self);


/** qu_range_cover
	Get the starting index in a container and the number of items covered
	by the Range. See C{Array.__getitem} for an example.
*/
extern long qu_range_cover (qu_range self, long len, long *start);


/** qu_range_contains
	Whether or not v{x} is within the Range.
	If v{x} is not an object then returns 0.
*/
extern int qu_range_contains (qu_range self, qu_any x);


/** qu_range_eq
	Whether or not a Range has the same property fields as another.
*/
extern int qu_range_eq (qu_range self, qu_range x);


/** qu_range_as
	Various representations.
*/
extern qu_array qu_range_as_array (qu_range self);
extern qu_set   qu_range_as_set   (qu_range self);
#define qu_range_as_string(self)  qu__range__string (self)
#define qu_range_as_boolean(self) (((self)->step > 0) ? QuTrue : QuFalse)


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_range__setup ();
extern void qu_range__init  ();

/* for efficiency - may only be called from within this file */
extern qu_string qu__range__string (qu_range self);

#endif /*_H_qu_range_*/
