/* Real.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_real_
#define _H_qu_real_ 1


/** Real qu_real
	This is the abstract parent class of ={Integer}, ={Decimal} and ={Inf}.

	qu_real is the type of a Real instance.
*/
extern qu_class QuRealClass;


/** QuRealN
	Predefined system objects.
*/
#define QuReal0  ((qu_real) QuInt0)   /*  0 */
#define QuReal1  ((qu_real) QuInt1)   /*  1 */
#define QuReal2  ((qu_real) QuInt2)   /*  2 */
#define QuReal_1 ((qu_real) QuInt_1)  /* -1 */
#define QuReal_2 ((qu_real) QuInt_2)  /* -2 */


/** qu_real_new
	Create a Real object from C types or other objects.

	These are just convenient wrappers around ={qu_integer_new} and ={qu_decimal_new}.
*/
#define qu_real_newh(v)   ((qu_real) qu_integer_newh   (v))
#define qu_real_newuh(v)  ((qu_real) qu_integer_newuh  (v))
#define qu_real_newi(v)   ((qu_real) qu_integer_newi   (v))
#define qu_real_newui(v)  ((qu_real) qu_integer_newui  (v))
#define qu_real_newl(v)   ((qu_real) qu_integer_newl   (v))
#define qu_real_newul(v)  ((qu_real) qu_integer_newul  (v))
#define qu_real_newli(v)  ((qu_real) qu_integer_newli  (v))
#define qu_real_newd(v)   ((qu_real) qu_decimal_newd   (v))
#ifdef HAVE_LONG_LONG
#define qu_real_newll(v)  ((qu_real) qu_integer_newll  (v))
#define qu_real_newull(v) ((qu_real) qu_integer_newull (v))
#endif
#ifdef HAVE_LONG_DOUBLE
#define qu_real_newld(v)  ((qu_real) qu_decimal_newld (v))
#endif

#define        qu_real_new_int(x)      ((qu_real) (x))
#define        qu_real_new_float(x)    ((qu_real) (x))
#define        qu_real_new_rational(x) ((qu_real) (x))
#define        qu_real_new_big(x)      ((qu_real) (x))
#define        qu_real_new_complex(x)  qu_complex_as_real (x)
#define        qu_real_new_inf(x)      ((qu_real) (x))
#define        qu_real_new_integer(x)  ((qu_real) (x))
#define        qu_real_new_decimal(x)  ((qu_real) (x))
#define        qu_real_new_real(x)     (x)
extern qu_real qu_real_new_number      (qu_number x);


/** qu_real_to
	Convert any object into C types.

	The functions set v{errno} to zero on success. On failure, v{errno} is set to EDOM
	if v{x} is conversion is not possible, to ERANGE if the value falls out of the
	C type range.

	The I{unsigned} versions fail with ERANGE if the value is negative.
*/
extern short              qu_real_toh   (qu_any x);
extern unsigned short     qu_real_touh  (qu_any x);
extern int                qu_real_toi   (qu_any x);
extern unsigned int       qu_real_toui  (qu_any x);
extern long               qu_real_tol   (qu_any x);
extern unsigned long      qu_real_toul  (qu_any x);
extern long               qu_real_toli  (qu_any x);
extern float              qu_real_tof   (qu_any x);
extern double             qu_real_tod   (qu_any x);
#ifdef HAVE_LONG_LONG
extern long long          qu_real_toll  (qu_any x);
extern unsigned long long qu_real_toull (qu_any x);
#endif
#ifdef HAVE_LONG_DOUBLE
extern long double        qu_real_told  (qu_any x);
#endif


/** qu_real_as
	Convert any object into other objects.

	An exception is thrown and NULL is returned on failure.
*/
extern qu_int      qu_real_as_int      (qu_any x);
extern qu_float    qu_real_as_float    (qu_any x);
extern qu_long     qu_real_as_long     (qu_any x);
extern qu_rational qu_real_as_rational (qu_any x);
extern qu_big      qu_real_as_big      (qu_any x);
extern qu_complex  qu_real_as_complex  (qu_any x);
extern qu_integer  qu_real_as_integer  (qu_any x);
extern qu_decimal  qu_real_as_decimal  (qu_any x);
extern qu_real     qu_real_as_real     (qu_any x);
extern qu_number   qu_real_as_number   (qu_any x);


/** qu_real_sign
	Returns -1, 0, 1 if the value is negative, 0, positive respectively.

	This is just a convenient wrapper around
	={qu_integer_sign} ={qu_decimal_sign} and ={qu_inf_sign}.
*/
extern int qu_real_sign (qu_real x);


/** qu_real_is
	Identity functions.

	qu_real_is tests whether an object is a Real.

	qu_real_isneg returns 1 if the value is negative.
	Returns 0 otherwise.

	qu_real_iszero returns 1 if the value is 0.
	Returns 0 otherwise.

	qu_real_isone returns 1 if the value is 1 or -1.
	Returns 0 otherwise.

	qu_real_is1 return 1 if the value is 1.
	Returns 0 otherwise.

	qu_real_iswhole return 1 if the value is a whole number.
	Returns 0 otherwise.

	qu_real_isfrac return 1 if 0 < |x| < 1.
	Returns 0 otherwise.

	qu_real_isodd return 1 if the value is a whole and odd number.
	Returns 0 otherwise.

	These are just convenient wrappers around
	={qu_integer_is} ={qu_decimal_is} and ={qu_inf_is}.
*/
extern int qu_real_is      (qu_any x);
extern int qu_real_iszero  (qu_real x);
extern int qu_real_isneg   (qu_real x);
extern int qu_real_isone   (qu_real x);
extern int qu_real_is1     (qu_real x);
extern int qu_real_iswhole (qu_real x);
extern int qu_real_isfrac  (qu_real x);
extern int qu_real_isodd   (qu_real x);


/** qu_real_unary
	Unary functions.

	qu_real_pos returns the number itself, obviously.

	qu_real_neg returns a negated value.

	qu_real_incr returns the number incremented by 1.

	qu_real_decr returns the number decremented by 1.

	These are just convenient wrappers around
	={qu_integer_pos} ={qu_integer_neg} ={qu_integer_incr} ={qu_integer_decr}
	={qu_decimal_pos} ={qu_decimal_neg} ={qu_decimal_incr} ={qu_decimal_decr}
	={qu_inf_pos}     ={qu_inf_neg}     ={qu_inf_incr}     ={qu_inf_decr}
*/
extern qu_real qu_real_pos  (qu_real x);
extern qu_real qu_real_neg  (qu_real x);
extern qu_real qu_real_incr (qu_real x);
extern qu_real qu_real_decr (qu_real x);


/** qu_real_eq
	Whether the value is the same as another.
	All return 0 or 1.

	These are just convenient wrappers around
	={qu_integer_eq} ={qu_decimal_eq} and ={qu_inf_eq}.
*/
extern int qu_real_eq_int      (qu_real x, qu_int y);
extern int qu_real_eq_float    (qu_real x, qu_float y);
extern int qu_real_eq_long     (qu_real x, qu_long y);
extern int qu_real_eq_rational (qu_real x, qu_rational y);
extern int qu_real_eq_big      (qu_real x, qu_big y);
extern int qu_real_eq_complex  (qu_real x, qu_complex y);
extern int qu_real_eq_inf      (qu_real x, qu_inf y);
extern int qu_real_eq_integer  (qu_real x, qu_integer y);
extern int qu_real_eq_decimal  (qu_real x, qu_decimal y);
extern int qu_real_eq_real     (qu_real x, qu_real y);
extern int qu_real_eq_number   (qu_real x, qu_number y);


/** qu_real_like
	Whether the absolute value is the same as the absolute value of another.
	All return 0 or 1.

	These are just convenient wrappers around
	={qu_integer_like} ={qu_decimal_like} and ={qu_inf_like}.
*/
extern int qu_real_like_int      (qu_real x, qu_int y);
extern int qu_real_like_float    (qu_real x, qu_float y);
extern int qu_real_like_long     (qu_real x, qu_long y);
extern int qu_real_like_rational (qu_real x, qu_rational y);
extern int qu_real_like_big      (qu_real x, qu_big y);
extern int qu_real_like_complex  (qu_real x, qu_complex y);
extern int qu_real_like_inf      (qu_real x, qu_inf y);
extern int qu_real_like_integer  (qu_real x, qu_integer y);
extern int qu_real_like_decimal  (qu_real x, qu_decimal y);
extern int qu_real_like_real     (qu_real x, qu_real y);
extern int qu_real_like_number   (qu_real x, qu_number y);


/** qu_real_cmp
	Compare with another.
	All return -1, 0, or -1 in the usual manner.

	These are just convenient wrappers around
	={qu_integer_cmp} ={qu_decimal_cmp} and ={qu_inf_cmp}.
*/
extern int qu_real_cmp_int      (qu_real x, qu_int y);
extern int qu_real_cmp_float    (qu_real x, qu_float y);
extern int qu_real_cmp_long     (qu_real x, qu_long y);
extern int qu_real_cmp_rational (qu_real x, qu_rational y);
extern int qu_real_cmp_big      (qu_real x, qu_big y);
extern int qu_real_cmp_complex  (qu_real x, qu_complex y);
extern int qu_real_cmp_inf      (qu_real x, qu_inf y);
extern int qu_real_cmp_integer  (qu_real x, qu_integer y);
extern int qu_real_cmp_decimal  (qu_real x, qu_decimal y);
extern int qu_real_cmp_real     (qu_real x, qu_real y);
extern int qu_real_cmp_number   (qu_real x, qu_number y);


/** qu_real_cmpabs
	Compare the absolute value with the absolute value of another.
	All return -1, 0, or -1 in the usual manner.

	These are just convenient wrappers around
	={qu_integer_cmpabs} ={qu_decimal_cmpabs} and ={qu_inf_cmpabs}.
*/
extern int qu_real_cmpabs_int      (qu_real x, qu_int y);
extern int qu_real_cmpabs_float    (qu_real x, qu_float y);
extern int qu_real_cmpabs_long     (qu_real x, qu_long y);
extern int qu_real_cmpabs_rational (qu_real x, qu_rational y);
extern int qu_real_cmpabs_big      (qu_real x, qu_big y);
extern int qu_real_cmpabs_complex  (qu_real x, qu_complex y);
extern int qu_real_cmpabs_inf      (qu_real x, qu_inf y);
extern int qu_real_cmpabs_integer  (qu_real x, qu_integer y);
extern int qu_real_cmpabs_decimal  (qu_real x, qu_decimal y);
extern int qu_real_cmpabs_real     (qu_real x, qu_real y);
extern int qu_real_cmpabs_number   (qu_real x, qu_number y);


/** qu_real_plus
	Add with another.

	These are just convenient wrappers around
	={qu_integer_plus} ={qu_decimal_plus} and ={qu_inf_plus}.
*/
extern qu_real    qu_real_plus_int      (qu_real x, qu_int y);
extern qu_real    qu_real_plus_float    (qu_real x, qu_float y);
extern qu_real    qu_real_plus_long     (qu_real x, qu_long y);
extern qu_real    qu_real_plus_rational (qu_real x, qu_rational y);
extern qu_real    qu_real_plus_big      (qu_real x, qu_big y);
extern qu_complex qu_real_plus_complex  (qu_real x, qu_complex y);
extern qu_real    qu_real_plus_inf      (qu_real x, qu_inf y);
extern qu_real    qu_real_plus_integer  (qu_real x, qu_integer y);
extern qu_real    qu_real_plus_decimal  (qu_real x, qu_decimal y);
extern qu_real    qu_real_plus_real     (qu_real x, qu_real y);
extern qu_number  qu_real_plus_number   (qu_real x, qu_number y);


/** qu_real_minus
	Subtract with another.

	These are just convenient wrappers around
	={qu_integer_minus} ={qu_decimal_minus} and ={qu_inf_minus}.
*/
extern qu_real    qu_real_minus_int      (qu_real x, qu_int y);
extern qu_real    qu_real_minus_float    (qu_real x, qu_float y);
extern qu_real    qu_real_minus_long     (qu_real x, qu_long y);
extern qu_real    qu_real_minus_rational (qu_real x, qu_rational y);
extern qu_real    qu_real_minus_big      (qu_real x, qu_big y);
extern qu_complex qu_real_minus_complex  (qu_real x, qu_complex y);
extern qu_real    qu_real_minus_inf      (qu_real x, qu_inf y);
extern qu_real    qu_real_minus_integer  (qu_real x, qu_integer y);
extern qu_real    qu_real_minus_decimal  (qu_real x, qu_decimal y);
extern qu_real    qu_real_minus_real     (qu_real x, qu_real y);
extern qu_number  qu_real_minus_number   (qu_real x, qu_number y);


/** qu_real_mul
	Multiply with another.

	These are just convenient wrappers around
	={qu_integer_mul} ={qu_decimal_mul} and ={qu_inf_mul}.
*/
extern qu_real    qu_real_mul_int      (qu_real x, qu_int y);
extern qu_real    qu_real_mul_float    (qu_real x, qu_float y);
extern qu_real    qu_real_mul_long     (qu_real x, qu_long y);
extern qu_real    qu_real_mul_rational (qu_real x, qu_rational y);
extern qu_real    qu_real_mul_big      (qu_real x, qu_big y);
extern qu_complex qu_real_mul_complex  (qu_real x, qu_complex y);
extern qu_real    qu_real_mul_inf      (qu_real x, qu_inf y);
extern qu_real    qu_real_mul_integer  (qu_real x, qu_integer y);
extern qu_real    qu_real_mul_decimal  (qu_real x, qu_decimal y);
extern qu_real    qu_real_mul_real     (qu_real x, qu_real y);
extern qu_number  qu_real_mul_number   (qu_real x, qu_number y);


/** qu_real_div
	Divide with another.

	These are just convenient wrappers around
	={qu_integer_div} ={qu_decimal_div} and ={qu_inf_div}.
*/
extern qu_real    qu_real_div_int      (qu_real x, qu_int y);
extern qu_real    qu_real_div_float    (qu_real x, qu_float y);
extern qu_real    qu_real_div_long     (qu_real x, qu_long y);
extern qu_real    qu_real_div_rational (qu_real x, qu_rational y);
extern qu_real    qu_real_div_big      (qu_real x, qu_big y);
extern qu_complex qu_real_div_complex  (qu_real x, qu_complex y);
extern qu_real    qu_real_div_inf      (qu_real x, qu_inf y);
extern qu_real    qu_real_div_integer  (qu_real x, qu_integer y);
extern qu_real    qu_real_div_decimal  (qu_real x, qu_decimal y);
extern qu_real    qu_real_div_real     (qu_real x, qu_real y);
extern qu_number  qu_real_div_number   (qu_real x, qu_number y);


/** qu_real_idiv
	Integer division of with another.

	These are just convenient wrappers around
	={qu_integer_idiv} ={qu_decimal_idiv} and ={qu_inf_div}.
*/
extern qu_integer qu_real_idiv_int      (qu_real x, qu_int y);
extern qu_integer qu_real_idiv_float    (qu_real x, qu_float y);
extern qu_integer qu_real_idiv_long     (qu_real x, qu_long y);
extern qu_integer qu_real_idiv_rational (qu_real x, qu_rational y);
extern qu_integer qu_real_idiv_big      (qu_real x, qu_big y);
extern qu_integer qu_real_idiv_complex  (qu_real x, qu_complex y);
extern qu_integer qu_real_idiv_inf      (qu_real x, qu_inf y);
extern qu_integer qu_real_idiv_integer  (qu_real x, qu_integer y);
extern qu_integer qu_real_idiv_decimal  (qu_real x, qu_decimal y);
extern qu_integer qu_real_idiv_real     (qu_real x, qu_real y);
extern qu_integer qu_real_idiv_number   (qu_real x, qu_number y);


/** qu_real_rem
	Remainder of dividing with another.

	These are just convenient wrappers around
	={qu_integer_rem} ={qu_decimal_rem} and ={qu_inf_rem}.
*/
extern qu_real    qu_real_rem_int      (qu_real x, qu_int y);
extern qu_real    qu_real_rem_float    (qu_real x, qu_float y);
extern qu_real    qu_real_rem_long     (qu_real x, qu_long y);
extern qu_real    qu_real_rem_rational (qu_real x, qu_rational y);
extern qu_real    qu_real_rem_big      (qu_real x, qu_big y);
extern qu_complex qu_real_rem_complex  (qu_real x, qu_complex y);
extern qu_real    qu_real_rem_inf      (qu_real x, qu_inf y);
extern qu_real    qu_real_rem_integer  (qu_real x, qu_integer y);
extern qu_real    qu_real_rem_decimal  (qu_real x, qu_decimal y);
extern qu_real    qu_real_rem_real     (qu_real x, qu_real y);
extern qu_number  qu_real_rem_number   (qu_real x, qu_number y);


/** qu_real_pow
	Raise to the power of another.

	These are just convenient wrappers around
	={qu_integer_pow} ={qu_decimal_pow} and ={qu_inf_pow}.
*/
extern qu_real    qu_real_pow_int      (qu_real x, qu_int y);
extern qu_real    qu_real_pow_float    (qu_real x, qu_float y);
extern qu_real    qu_real_pow_long     (qu_real x, qu_long y);
extern qu_real    qu_real_pow_rational (qu_real x, qu_rational y);
extern qu_real    qu_real_pow_big      (qu_real x, qu_big y);
extern qu_complex qu_real_pow_complex  (qu_real x, qu_complex y);
extern qu_real    qu_real_pow_inf      (qu_real x, qu_inf y);
extern qu_real    qu_real_pow_integer  (qu_real x, qu_integer y);
extern qu_real    qu_real_pow_decimal  (qu_real x, qu_decimal y);
extern qu_real    qu_real_pow_real     (qu_real x, qu_real y);
extern qu_number  qu_real_pow_number   (qu_real x, qu_number y);


/** qu_real_round
	Rounding functions.

	qu_real_ceil round the number to the nearest integer towards positive infinity.
	EHuge is thrown on failure.

	qu_real_floor round the number to the nearest integer towards negative infinity.
	EHuge is thrown on failure.

	qu_real_trunc round the number towards zero, that is, it removes the
	fraction part. This function throws EMath if the number is an infinity.

	These are just convenient wrappers around
	={qu_integer_round} ={qu_decimal_round} and ={qu_inf_round}.
*/
extern qu_real qu_real_floor (qu_real x);
extern qu_real qu_real_ceil  (qu_real x);
extern qu_real qu_real_trunc (qu_real x);
extern qu_real qu_real_frac  (qu_real x);
extern qu_real qu_real_round (qu_real x, int prec);


/** qu_real_format
	Format a Real.

	See ={Format} for details;

	This is just a convenient wrapper around
	={qu_integer_format} ={qu_decimal_format} and ={qu_inf_format}.

	This function is provided for efficiency between core modules.
	It is not intended to be called by users directly.
*/
extern qu_string qu_real_format (qu_real x, qu_format_s *f);


/** __qu_tiny__ __qu_huge__ __qu_inf__ __qu_zerodiv__
	__qu_tiny__ throws ETiny if the ={qu_number_context} does not allow rounding
	to zero. You must supply an Integer or Decimal zero value, i.e. QuInt0,
	QuLong0, QuRational0, QuFloat0 or QuBig0.

	__qu_huge__ throws EHuge if the number context does not allow infinities to
	be returned. v{sign} must be -1 for ={QuInfNeg}, anything else for ={QuInfPos}.

	__qu_inf__ returns an infinity or throw EInf if the number context does not allow
	infinities to be returned. v{sign} must be -1 for ={QuInfNeg}, anything
	else for ={QuInfPos}.

	__qu_zerodiv__ throws EDiv if the number context does not allow division by zero,
	returns an infinity otherwise.
*/
extern qu_real __qu_tiny__    (qu_real zero);
extern qu_inf  __qu_huge__    (int sign);
extern qu_inf  __qu_inf__     (int sign);
extern qu_inf  __qu_zerodiv__ (int sign);


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_real__setup ();
extern void qu_real__init  ();

#endif /*_H_qu_real_*/
