/* Record.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_record_
#define _H_qu_record_ 1

#include "Container.h"


/** Record
	This is the Record class.
*/
extern qu_class QuRecordClass;


/** qu_record
	qu_record represents an instance of the Record class.
	It is a pointer to the instance struct qu_record_s.

	See ={QU_CONTAINER_HEAD}.
*/
struct qu_record_s
{
	QU_CONTAINER_HEAD;
	qu_any    *objs;  /* values are allocated */
	qu_struct  struc; /* the associated qu_struct object */
};


/** QU_RECORD_FLAG
	Flags used by Record objects.

	Do not access directly. Use the accessors instead.
*/
#define QU_RECORD_FLAG_PROTECT QU_CONTAINER_FLAG_PROTECT /* protected by program level */
#define QU_RECORD_FLAG_FROZEN  QU_CONTAINER_FLAG_FROZEN  /* immutable? */


/** qu_record_frozen
	qu_record_frozen checks whether or not the Record is mutable.

	qu_record_freeze must only be used after you have created a new Record and
	don't want the Record to be modified by others.

	qu_record_unfreeze must only be used on a Record you have created.
	Normally, to modify the Record then freeze it again.
*/
#define qu_record_frozen(self)    qu_container_frozen   (self)
#define qu_record_freeze(self)    qu_container_freeze   (self)
#define qu_record_unfreeze(self)  qu_container_unfreeze (self)


/** qu_record_is
	Whether or not an object is a Record.
*/
#define qu_record_is(x) qu_object_instof (x, QuRecordClass)


/** qu_record_of
	Whether or not an object is a Record of a specified ={Struct}.

	@warn
	Make sure v{u} is a ={Struct}.
*/
#define qu_record_of(x, u) (qu_record_is (x) && ((qu_record) x)->struc == (u))


/** qu_record_new
	Create a new Record object.

	@warn
	If you supply the wrong number of arguments, you get an EArgCount.
	If you supply the wrong type of arguments, you get EArgValue.

	qu_record_new_ex does not ask any question, so be careful.
*/
extern qu_record qu_record_new    (qu_struct u, int argc, qu_any *argv);
extern qu_record qu_record_new_ex (qu_struct u, ...);


/** qu_record_len
	Return the number of items.
*/
#define qu_record_len(self) qu_struct_len ((self)->struct)


/** qu_record_as
	Various representations.
*/
extern qu_array qu_record_as_array (qu_record self);
extern qu_dict  qu_record_as_dict  (qu_record self);
extern qu_set   qu_record_as_set   (qu_record self);
#define qu_record_as_string(self)  qu__record__string (self)
#define qu_record_as_boolean(self) qu__record__true (self)


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_record__setup ();
extern void qu_record__init  ();

/* for efficiency - may only be called from within this file */
extern qu_string  qu__record__string (qu_record self);
extern qu_boolean qu__record__true   (qu_record self);

/* be careful - you must set the values your self */
extern qu_record qu_record_alloc (qu_struct u);

#endif /*_H_qu_record_*/
