/* Utf8.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_utf8_
#define _H_qu_utf8_ 1


/** QU_WCHAR_INVALID
	The invalid wide character.
*/
#define QU_WCHAR_INVALID ((wint_t) -1)


/** qu_utf8_charlen
	Return the number of bytes needed to encode a wide character.
*/
extern int qu_utf8_charlen (wint_t c);


/** qu_utf8_putchar
	Encode one wide charater and return the updated pointer.

	@warn
	Make sure there is enough room.
*/
extern char *qu_utf8_putchar (char *s, wint_t c);


/** qu_utf8_getchar
	Get a wide character and update the pointer.
	Return -1 if the wide character is invalid.
*/
extern wint_t qu_utf8_getchar (char **s, char *end);


/** qu_utf8_readchar
	Get a wide character.

	@warn
	This function assumes a valid pointer and valid UTF-8 string.
*/
extern wint_t qu_utf8_charat (char *s);


/** qu_utf8_new
	Creates a new String from raw bytes.
	Throw ESyntax if the bytes are not valid UTF-8 multibytes.
*/
extern qu_string qu_utf8_new (const char *z, long len);


/** qu_utf8_new_char
	Allocate a single UTF-8 character string.
*/
extern qu_string qu_utf8_new_char (wint_t c);


/** qu_utf8_width
	Count the number of wide characters in the string.
	Return -1 if the string is not a valid UTF-8 string.
*/
extern long qu_utf8_width (const char *z, long l);


/** qu_utf8_pos
	Get the position of the v{i}-th character, v{i} may be negative
	Return NULL if not found.
*/
extern char *qu_utf8_pos (const char *z, long l, long i);


/** qu_utf8_last
	Get the position of the last character.
	Return NULL if the v{l} is non-positive.

	@warn
	This function assumes a valid UTF-8 string.
*/
extern char *qu_utf8_last (const char *z, long l);


/** qu_utf8_getitem
	Return a String of the v{i}-th character, v{i} may be negative.
*/
extern qu_string qu_utf8_getitem (qu_string self, long i);


/** qu_utf8_getslice
	Get a slice of the string starting at the v{i}-th character, v{n} characters
	long, v{i} may be negative.
*/
extern qu_string qu_utf8_getslice (qu_string self, long i, long n);


/** qu_utf8_is
	Character tests.
*/
extern int qu_utf8_isalpha  (wint_t c);
extern int qu_utf8_isalnum  (wint_t c);
extern int qu_utf8_isblank  (wint_t c);
extern int qu_utf8_iscntrl  (wint_t c);
extern int qu_utf8_isdigit  (wint_t c);
extern int qu_utf8_isxdigit (wint_t c);
extern int qu_utf8_isgraph  (wint_t c);
extern int qu_utf8_islower  (wint_t c);
extern int qu_utf8_isprint  (wint_t c);
extern int qu_utf8_ispunct  (wint_t c);
extern int qu_utf8_isspace  (wint_t c);
extern int qu_utf8_isupper  (wint_t c);
extern int qu_utf8_iswide   (wint_t c);


/** qu_utf8_to
	Conversion functions.
*/
extern wint_t qu_utf8_toupper (wint_t c);
extern wint_t qu_utf8_tolower (wint_t c);
extern wint_t qu_utf8_totitle (wint_t c);

/***/
#endif /*_H_qu_utf8_*/
