/* Weak.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_weak_
#define _H_qu_weak_ 1


/** Weak
	The final Weak class implements weak references.

	A weak (or I{soft}) reference to an object does not prevent the object from
	being garbage-collected. The referee must check for the v{ref} field. When
	the garbage collector deletes the object then this field will be set to NULL.
*/
extern qu_class QuWeakClass;


/** qu_weak
	qu_weak represents an instance of the Weak class.
	It is a pointer to the instance struct qu_weak_s.
*/
struct qu_weak_s
{
	QU_OBJECT_HEAD;
	qu_obj  ref;  /* the referenced object, maybe 0 */
	long    hash; /* the object hash value */
};


/** QU_WEAK_FLAG
	Flags used by Weak objects.

	Do not access directly. Use the accessors instead.
*/
/* none but all are reserved */


/** qu_weak_is
	Whether or not an object is an instance of the Weak class.
*/
#define qu_weak_is(o) (qu_classof(o) == QuWeakClass)


/** qu_weak_new
	Create a new Weak object.

	@warn
	v{x} may not be a literal such as Number, String, etc.
*/
#define qu_weak_new(x) qu__weak_new (QuWeakClass, x)


/** qu_weak_unref
	Unreference the object.

	If the object is already gone then nothing happens.
	This function always succeed and return QuNil.
*/
#define qu_weak_unref(self) qu__weak_unref (self)


/***/
/*--------------------------------------------------------------------------*/

/* called internally by Class to initialize this class */
extern void qu_weak__setup ();
extern void qu_weak__init  ();

/* for efficiency - may only be called from within this file */
extern qu_weak qu__weak_new   (qu_class self, qu_any x);
extern qu_null qu__weak_unref (qu_weak self);

/* for Gc and Weak */
#define QU_WEAK_UNREF(self) QU_BEGIN ((self)->ref) = NULL; QU_END

#endif /*_H_qu_weak_*/
