/* err.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_err_
#define _H_qu_err_ 1

#include "base.h"


/** qu_err_do
	This is the function called by the macros. You should not call this
	function directly.

	@args
	.file  the file name, i.e. C{__FILE__}
	.func  the function name, i.e. C{__FUNCTION__}
	.kind  C{0}=remark, C{1}=warning, C{2}=experiment warning, C{3}=deprecated warning, C{4}=fatal error
	.fmt   format ala f{printf}
	....   the format arguments
*/
extern void qu_err_do (const char *file, const char *func, int kind, const char *fmt, ...);


/** QU_FATAL QU_WARN QU_REMARK QU_WARN_DEPRECATED QU_WARN_EXPERIMENTAL
	Print message to I{stderr}.

	QU_REMARK is enabled if DEBUG is defined. You can disable it by defining
	the macro DISABLE_REMARK.

	QU_WARN is enabled even if DEBUG is not defined. You can disable it by
	defining the macro DISABLE_WARN.

	QU_FATAL is always enabled. It calls C{abort()} so it never return.

	QU_WARN_DEPRECATED and QU_WARN_EXPERIMENTAL are always enabled.

	@note
		v{errno} is preserved, i.e. calling these macros does not change v{errno}.

	@xmp
		QU_FATAL ("%s is not %s", some_c_string, another_c_string);
*/
#define QU_FATAL(...)             qu_err_do (__FILE__, __FUNCTION__, 4, __VA_ARGS__)
#define QU_WARN_DEPRECATED(...)   qu_err_do (__FILE__, __FUNCTION__, 3, __VA_ARGS__)
#define QU_WARN_EXPERIMENTAL(...) qu_err_do (__FILE__, __FUNCTION__, 2, __VA_ARGS__)
#define QU_WARN(...)              qu_err_do (__FILE__, __FUNCTION__, 1, __VA_ARGS__)
#define QU_REMARK(...)            qu_err_do (__FILE__, __FUNCTION__, 0, __VA_ARGS__)

#if !defined(DEBUG) || defined(DISABLE_REMARK)
#undef QU_REMARK
#define QU_REMARK(...)
#endif

#if defined(DISABLE_WARN)
#undef QU_WARN
#define QU_WARN(...)
#endif

/***/
#endif /*_H_qu_err_*/
