/* qu.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_qu_
#define _H_qu_qu_ 1


/** Core
	Files included by qu.h.
*/
#include "err.h"
#include "ascii.h"

#include "Object.h"
#include "sig.h"
#include "attr.h"
#include "gc.h"
#include "interp.h"
#include "Sys.h"

#include "Tokenizer.h"
#include "Ast.h"
#include "Compile.h"

#include "Array.h"
#include "Big.h"
#include "Boolean.h"
#include "Buffer.h"
#include "Class.h"
#include "Code.h"
#include "Complex.h"
#include "Container.h"
#include "Decimal.h"
#include "Dict.h"
#include "Dlib.h"
#include "Exception.h"
#include "File.h"
#include "Float.h"
#include "Frame.h"
#include "Function.h"
#include "Int.h"
#include "Integer.h"
#include "Inf.h"
#include "Iterator.h"
#include "Long.h"
#include "Method.h"
#include "Module.h"
#include "Null.h"
#include "Number.h"
#include "Opaque.h"
#include "Proc.h"
#include "Pointer.h"
#include "Queue.h"
#include "Range.h"
#include "Rational.h"
#include "Real.h"
#include "Record.h"
#include "Set.h"
#include "Stream.h"
#include "String.h"
#include "Struct.h"
#include "Sub.h"
#include "Thread.h"
#include "Utf8.h"
#include "Var.h"
#include "Weak.h"


/** QU_DEBUG_LOCAL
	Define this before including C{qu.h} to activate f{assert} in your
	C source file.
*/
#ifdef QU_DEBUG_LOCAL
#undef assert
#define assert(expr) \
QU_BEGIN \
	if (!(expr)) \
		QU_FATAL ("%s line %d assertion failed: %s", __FUNCTION__, __LINE__, #expr); \
QU_END
#endif


/** QU_APP_DEBUG
	The __debug__ value.

	This is set with the C{-d} option.

	The default value is 0 which means __debug__ = false.

	Note that the __debug__ value can not be modified programmatically.
*/
extern int QU_APP_DEBUG;


/** QU_APP_VERBOSE
	Set by the C{-o} option.

	The default value is 0 (false) which means silent.
*/
extern int QU_APP_VERBOSE;


/** QU_APP_NO_WARNING
	Whether or not warning messages should be generated.

	Warnings are disabled with the C{-w} option.

	The default value is 1 (true).
*/
extern int QU_APP_NO_WARNING;


/** QU_APP_LINE_NUMBER
	Whether or not line numbers should be generated.

	Line numbers are generated by specifying the C{-l} option.

	The default value is 0 (false).
*/
extern int QU_APP_LINE_NUMBER;


/** QU_APP_INCLUDE_TEST
	Whether or not the code below the k{__test__} line should be included.

	They are enabled by specifying the C{-t} option.

	The default value is 0 (false).
*/
extern int QU_APP_INCLUDE_TEST;


/** QU_APP_KEEP_TREE
	Whether or not to keep the parsed trees.

	This is enabled with the C{-p} option.

	The default value is 0, thus free the parsed trees after compilation.
*/
extern int QU_APP_KEEP_TREE;


/** QU_APP_WITH_DOC
	Include documentation strings.

	This is enabled with the C{-D} option.

	The default value is 0 (false).
*/
extern int QU_APP_WITH_DOC;


/** QU_APP_HOME_DIR
	Where Qu is installed.

	This is where Qu should think where it is installed.
	It is set with the C{-M} option.

	The default value is QU_APP_DESTDIR as set in C{config.h}.
*/
extern const char *QU_APP_HOMEDIR;


/** QU_APP_OPTIMIZE_CODE
	Optimize code.

	Currently this implies that the code should be JIT-ed, if possible.
	It is set with the C{-O} option.

	The default value is 0 (false).
*/
extern int QU_APP_OPTIMIZE_CODE;


/** QU_APP_MORE_COMPILE_CHECK
	Perform more compile-time checks.

	If true then Qu must do more compile-time checks, e.g. check typed variables
	against constant properties, etc.

	The default value is 0 (false).
*/
extern int QU_APP_MORE_COMPILE_CHECK;


/** QU_APP_FREE_LEXICAL
	Free all lexical Var objects after compiling.

	This is set with the C{-x} option.

	The default value is 1 (true).
*/
extern int QU_APP_FREE_LEXICAL;


/** QU_APP_NOVALIDATION
	Deactivate all validators.

	The default value is 0 (false).
*/
extern int QU_APP_NOVALIDATION;


/** QU_APP_USEPACKAGE
	Set __package__ to this directory instead.

	This is set with the C{-K} option. Default is NULL.
*/
extern char *QU_APP_USEPACKAGE;


/** QU_APP_DAEMON
	Whether Qu was run with the -S option.
*/
extern int QU_APP_DAEMON;


#endif /*_H_qu_qu_*/
