/* sig.h - Copyright (c) Marc Krisnanto */

#ifndef _H_qu_sig_
#define _H_qu_sig_ 1

#include "base.h"


/** qu_sig_init_worker
	All worker threads must call this function just after they have been
	born into existence.

	If qu_sig__init was called with blocksig = 0 then this function does nothing,
	otherwise signals are blocked from the thread.
*/
extern void qu_sig_init_worker ();


/** qu_sig_got qu_sig_ack qu_sig_clear
	Each signal has a counter. When a signal is delivered, the count is
	incremented.

	qu_sig_got checks whether or not the signal has been delivered.

	qu_sig_ack decrements the counter.

	qu_sig_clear clears the counter.
*/
#define qu_sig_got(name)   (__QuSig##name != 0)
#define qu_sig_ack(name)   (--__QuSig##name)
#define qu_sig_clear(name) (__QuSig##name = 0)

/***/
extern volatile int __QuSigTERM;
extern volatile int __QuSigINT;
extern volatile int __QuSigHUP;
extern volatile int __QuSigALRM;
extern volatile int __QuSigPIPE;
extern volatile int __QuSigCHLD;
extern volatile int __QuSigUSR1;
extern volatile int __QuSigUSR2;


/** __qu_sig_begin __qu_sig_end
	Set and unset hook functions to call whenever a signal is caught.
	SIGHUP, SIGPIPE, SIGUSR1, or SIGUSR2 is caught.

	@xmp
		qu_sighook_f old;

		static void handler ()
		{
			...
		}

		__qu_sig_begin_int (old, handler);
		{
			...
		}
		__qu_sig_end_int (old);
*/
typedef void (*qu_sighook_f) ();

#define __qu_sig_begin(name, hook) \
QU_BEGIN \
	qu_sighook_f sIgOlD##name = __QuSigHook##name; \
	__QuSigHook##name = hook

#define __qu_sig_end(name) \
	__QuSigHook##name = sIgOlD##name; \
QU_END

/***/
extern volatile qu_sighook_f __QuSigHookINT;
extern volatile qu_sighook_f __QuSigHookCHLD;
extern volatile qu_sighook_f __QuSigHookALRM;
extern volatile qu_sighook_f __QuSigHookHUP;
extern volatile qu_sighook_f __QuSigHookPIPE;
extern volatile qu_sighook_f __QuSigHookUSR1;
extern volatile qu_sighook_f __QuSigHookUSR2;


/***/
/*--------------------------------------------------------------------------*/

/*
	Called by the main program to initialize signals.
	If blocksig is non-zero then signals are blocked.
*/
extern void qu_sig__init (int blocksig);

#endif /*_H_qu_sig_*/
