/* Generic.c - Copyright (c) Marc Krisnanto */

/*
	!!! THIS NOTICE SHOULD BE INCLUDED IN ALL DBD MODULES

	Thread synchronization is managed by the Db module. Functions that
	are calling the driver functions may be wrapped in __qu_blocking_begin/end.
	This means that driver functions may not allocate memory via qu_mem_alloc
	et.al., and may not create nor access objects.

	The only exception is 'connect'. When called, this function is not wrapped
	in __qu_blocking_begin/end. If the driver needs to allocate memory, create or
	access objects, the only place to do it is in the driver 'connect' function.

	The driver function 'free' is called by the GC. It may call qu_mem_free
	to free memory allocated with qu_mem_alloc et.al.

	Driver functions may, of course, use C malloc/free.
*/

#include "../Db.h"

static int _cap (int what)
{
	return 0;
}



static void *_connect (qu_dict data)
{
	return QuInt1;
}



static void _disconnect (void *c)
{
	return;
}



static void _free (void *c)
{
	return;
}



static int _errnum (void *c)
{
	return 0;
}



static const char *_errstr (void *c)
{
	return "Generic";
}



static void _finish (void *c)
{
	return;
}



static int _begin (void *c)
{
	return 0;
}



static int _rollback (void *c)
{
	return 0;
}



static int _commit (void *c)
{
	return 0;
}



static long _action (void *c, const char *sql)
{
	return -1;
}



static long _lastid (void *c)
{
	return -1;
}



static int _select (void *c, const char *sql)
{
	return -1;
}



static int _fetchrow (void *c, const char ***row)
{
	return -1;
}



static int _numfields (void *c)
{
	return -1;
}



static int _fieldtype (void *c, int index)
{
	return QU_DB_TYPE_UNKNOWN;
}



static const char *_fieldname (void *c, int index)
{
	return 0;
}



static int _escape (void *c, char *to, const char *from, int len)
{
	*to = '\0';
	return 0;
}



static qu_dbd_s GenericDbd =
{
	_cap,
	_connect,
	_disconnect,
	_free,
	_errnum,
	_errstr,
	_finish,
	_begin,
	_rollback,
	_commit,
	_action,
	_lastid,
	_select,
	_fetchrow,
	_numfields,
	_fieldtype,
	_fieldname,
	_escape,
};


qu_dbd_s *qu_dbd_Generic ()
{
	return &GenericDbd;
}
