#ifndef _SQLITE3_H_
#define _SQLITE3_H_

#include "../Db.h"

#include <stdarg.h>

#define SQLITE_OK           0
#define SQLITE_ERROR        1
#define SQLITE_INTERNAL     2
#define SQLITE_PERM         3
#define SQLITE_ABORT        4
#define SQLITE_BUSY         5
#define SQLITE_LOCKED       6
#define SQLITE_NOMEM        7
#define SQLITE_READONLY     8
#define SQLITE_INTERRUPT    9
#define SQLITE_IOERR       10
#define SQLITE_CORRUPT     11
#define SQLITE_NOTFOUND    12
#define SQLITE_FULL        13
#define SQLITE_CANTOPEN    14
#define SQLITE_PROTOCOL    15
#define SQLITE_EMPTY       16
#define SQLITE_SCHEMA      17
#define SQLITE_TOOBIG      18
#define SQLITE_CONSTRAINT  19
#define SQLITE_MISMATCH    20
#define SQLITE_MISUSE      21
#define SQLITE_NOLFS       22
#define SQLITE_AUTH        23
#define SQLITE_FORMAT      24
#define SQLITE_RANGE       25
#define SQLITE_NOTADB      26
#define SQLITE_ROW         100
#define SQLITE_DONE        101

#define SQLITE_INTEGER  1
#define SQLITE_FLOAT    2
#define SQLITE_TEXT     3
#define SQLITE_BLOB     4
#define SQLITE_NULL     5

typedef struct sqlite3 sqlite3;
typedef struct sqlite3_stmt sqlite3_stmt;

#if defined(_MSC_VER) || defined(__BORLANDC__)
  typedef __int64 sqlite_int64;
  typedef unsigned __int64 sqlite_uint64;
#else
  typedef long long int sqlite_int64;
  typedef unsigned long long int sqlite_uint64;
#endif

static int                  (*sqlite3_finalize         ) (sqlite3_stmt *pStmt);
static int                  (*sqlite3_open             ) (const char *filename, sqlite3 **ppDb);
static int                  (*sqlite3_close            ) (sqlite3 *);
static int                  (*sqlite3_errcode          ) (sqlite3 *db);
static const char          *(*sqlite3_errmsg           ) (sqlite3*);
static int                  (*sqlite3_prepare          ) (sqlite3 *db, const char *zSql, int nBytes, sqlite3_stmt **ppStmt, const char **pzTail);
static int                  (*sqlite3_step             ) (sqlite3_stmt*);
static int                  (*sqlite3_finalize         ) (sqlite3_stmt *pStmt);
static sqlite_int64         (*sqlite3_last_insert_rowid) (sqlite3*);
static int                  (*sqlite3_column_count     ) (sqlite3_stmt *pStmt);
static const unsigned char *(*sqlite3_column_text      ) (sqlite3_stmt*, int iCol);
static int                  (*sqlite3_column_count     ) (sqlite3_stmt *pStmt);
static int                  (*sqlite3_column_type      ) (sqlite3_stmt*, int iCol);
static const char          *(*sqlite3_column_name      ) (sqlite3_stmt*,int);
static char                *(*sqlite3_mprintf          ) (const char*,...);
static void                 (*sqlite3_free             ) (char *z);

#endif
