/* Netenum.c - Copyright (c) Marc Krisnanto */


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum TCP
		TCP_NODELAY
		TCP_MAXSEG
		TCP_CORK
	;;
	Constants to be used with #{getsockopt} or #{setsockopt}.

	See C{man 7 tcp} for details.
*/
static qu_var EnumTCP;


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum IP
		IP_OPTIONS
		IP_PKTINFO
		IP_RECVTOS
		IP_RECVTTL
		IP_RECVOPTS
		IP_RETOPTS
		IP_TOS
		IP_TTL
		IP_HDRINCL
		IP_RECVERR
		IP_PMTU_DISCOVER
		IP_MTU
		IP_ROUTER_ALERT
		IP_MULTICAST_TTL
		IP_MULTICAST_LOOP
		IP_ADD_MEMBERSHIP
		IP_DROP_MEMBERSHIP
		IP_MULTICAST_IF
	;;
	Constants to be used with #{getsockopt} or #{setsockopt}.
	See C{man 7 tcp} for details.
*/
static qu_var EnumIP;


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum IPPROTO
		IPPROTO_IP       : 0    # Dummy protocol for TCP.
		IPPROTO_HOPOPTS  : 0    # IPv6 Hop-by-Hop options.
		IPPROTO_ICMP     : 1    # Internet Control Message Protocol.
		IPPROTO_IGMP     : 2    # Internet Group Management Protocol.
		IPPROTO_IPIP     : 4    # IPIP tunnels (older KA9Q tunnels use 94).
		IPPROTO_TCP      : 6    # Transmission Control Protocol.
		IPPROTO_EGP      : 8    # Exterior Gateway Protocol.
		IPPROTO_PUP      : 12   # PUP protocol.
		IPPROTO_UDP      : 17   # User Datagram Protocol.
		IPPROTO_IDP      : 22   # XNS IDP protocol.
		IPPROTO_TP       : 29   # SO Transport Protocol Class 4.
		IPPROTO_IPV6     : 41   # IPv6 header.
		IPPROTO_ROUTING  : 43   # IPv6 routing header.
		IPPROTO_FRAGMENT : 44   # IPv6 fragmentation header.
		IPPROTO_RSVP     : 46   # Reservation Protocol.
		IPPROTO_GRE      : 47   # General Routing Encapsulation.
		IPPROTO_ESP      : 50   # Encapsulating security payload.
		IPPROTO_AH       : 51   # Authentication header.
		IPPROTO_ICMPV6   : 58   # ICMPv6.
		IPPROTO_NONE     : 59   # IPv6 no next header.
		IPPROTO_DSTOPTS  : 60   # IPv6 destination options.
		IPPROTO_MTP      : 92   # Multicast Transport Protocol.
		IPPROTO_ENCAP    : 98   # Encapsulation Header.
		IPPROTO_PIM      : 103  # Protocol Independent Multicast.
		IPPROTO_COMP     : 108  # Compression Header Protocol.
		IPPROTO_RAW      : 255  # Raw IP packets.
		IPPROTO_MAX
	;;
	Standard well-defined IP protocols.

	Note that the values are arbitrary and some may not be defined.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumIPPROTO;
#ifndef IPPROTO_IP
#define IPPROTO_IP 0
#endif


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum IPPORT
		IPPORT_ECHO         : 7    # Echo service.
		IPPORT_DISCARD      : 9    # Discard transmissions service.
		IPPORT_SYSTAT       : 11   # System status service.
		IPPORT_DAYTIME      : 13   # Time of day service.
		IPPORT_NETSTAT      : 15   # Network status service.
		IPPORT_FTP          : 21   # File Transfer Protocol.
		IPPORT_TELNET       : 23   # Telnet protocol.
		IPPORT_SMTP         : 25   # Simple Mail Transfer Protocol.
		IPPORT_TIMESERVER   : 37   # Timeserver service.
		IPPORT_NAMESERVER   : 42   # Domain Name Service.
		IPPORT_WHOIS        : 43   # Internet Whois service.
		IPPORT_MTP          : 57
		IPPORT_TFTP         : 69   # Trivial FTP.
		IPPORT_RJE          : 77
		IPPORT_FINGER       : 79   # Finger service.
		IPPORT_HTTP         : 80   # HTTP
		IPPORT_TTYLINK      : 87
		IPPORT_SUPDUP       : 95   # SUPDUP protocol.
		IPPORT_HTTPS        : 443  # HTTPS
		IPPORT_EXECSERVER   : 512  # execd service.
		IPPORT_LOGINSERVER  : 513  # rlogind service.
		IPPORT_CMDSERVER    : 514
		IPPORT_EFSSERVER    : 520

		IPPORT_BIFFUDP      : 512  # UDP ports.
		IPPORT_WHOSERVER    : 513
		IPPORT_ROUTESERVER  : 520

		IPPORT_RESERVED     : 1024 # < 1024 are reserved for privileged processes
		IPPORT_USERRESERVED : 5000 # > 5000 are reserved for (non-privileged) servers.
	;;
	Standard well-known ports.

	Note that the values are arbitrary and some may not be defined.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumIPPORT;
#ifndef IPPORT_HTTP
#define IPPORT_HTTP 80
#endif
#ifndef IPPORT_HTTPS
#define IPPORT_HTTPS 443
#endif


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum PF
		PF_UNSPEC    : 0   # Unspecified.
		PF_LOCAL     : 1   # Local to host (pipes and file-domain).
		PF_UNIX      : 1   # PF_LOCAL Old BSD name for PF_LOCAL.
		PF_FILE      : 1   # PF_LOCAL Another non-standard name for PF_LOCAL.
		PF_INET      : 2   # IP protocol family.
		PF_AX25      : 3   # Amateur Radio AX.25.
		PF_IPX       : 4   # Novell Internet Protocol.
		PF_APPLETALK : 5   # Appletalk DDP.
		PF_NETROM    : 6   # Amateur radio NetROM.
		PF_BRIDGE    : 7   # Multiprotocol bridge.
		PF_ATMPVC    : 8   # ATM PVCs.
		PF_X25       : 9   # Reserved for X.25 project.
		PF_INET6     : 10  # IP version 6.
		PF_ROSE      : 11  # Amateur Radio X.25 PLP.
		PF_DECnet    : 12  # Reserved for DECnet project.
		PF_NETBEUI   : 13  # Reserved for 802.2LLC project.
		PF_SECURITY  : 14  # Security callback pseudo AF.
		PF_KEY       : 15  # PF_KEY key management API.
		PF_NETLINK   : 16  # NETLINK
		PF_ROUTE     : 16  # PF_NETLINK Alias to emulate 4.4BSD.
		PF_PACKET    : 17  # Packet family.
		PF_ASH       : 18  # Ash.
		PF_ECONET    : 19  # Acorn Econet.
		PF_ATMSVC    : 20  # ATM SVCs.
		PF_SNA       : 22  # Linux SNA Project
		PF_IRDA      : 23  # IRDA sockets.
		PF_PPPOX     : 24  # PPPoX sockets.
		PF_WANPIPE   : 25  # Wanpipe API sockets.
		PF_BLUETOOTH : 31  # Bluetooth sockets.
		PF_MAX
	;;
	Network protocal family.

	Note that the values are arbitrary and some may not be defined.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumPF;
#ifndef PF_LOCAL
#define PF_LOCAL 1
#endif


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum AF
		AF_UNSPEC    : 0
		AF_LOCAL     : 1
		AF_UNIX      : 1
		AF_FILE      : 1
		AF_INET      : 2
		AF_AX25      : 3
		AF_IPX       : 4
		AF_APPLETALK : 5
		AF_NETROM    : 6
		AF_BRIDGE    : 7
		AF_ATMPVC    : 8
		AF_X25       : 9
		AF_INET6     : 10
		AF_ROSE      : 11
		AF_DECnet    : 12
		AF_NETBEUI   : 13
		AF_SECURITY  : 14
		AF_KEY       : 15
		AF_NETLINK   : 16
		AF_ROUTE     : 16
		AF_PACKET    : 17
		AF_ASH       : 18
		AF_ECONET    : 19
		AF_ATMSVC    : 20
		AF_SNA       : 22
		AF_IRDA      : 23
		AF_PPPOX     : 24
		AF_WANPIPE   : 25
		AF_BLUETOOTH : 31
		AF_MAX
	;;
	Network address family.

	Note that the values are arbitrary and some may not be defined.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumAF;
#ifndef AF_LOCAL
#define AF_LOCAL 1
#endif


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum NI
		NI_NOFQDN
			# If set, return only the hostname part of the FQDN for local hosts.

		NI_NUMERICHOST
			# If  set,  then  the  numeric form of the hostname is returned.  (When not
			# set, this will still happen in case the node's name cannot be looked up.)

		NI_NAMEREQD
			# If set, then a error is returned if the hostname cannot be looked up.

		NI_NUMERICSERV
			# If set, then the service address is returned in numeric form, for example
			# by its port number.

		NI_DGRAM
			# If set, then the service is datagram (UDP) based rather than stream (TCP)
			# based.  This  is required for the few ports (512-514) that have different
			# services for UDP and TCP.
	;;
	Flags for #{getnameinfo}.
*/
static qu_var EnumNI;



/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum INADDR
		INADDR_ANY             # Accept any incoming messages.
		INADDR_BROADCAST       # Send to all hosts.
		INADDR_NONE            # Indicating an error return.
		INADDR_LOOPBACK        # Loopback to local host 127.0.0.1
		INADDR_LOCALHOST       # INADDR_LOOPBACK
		INADDR_UNSPEC_GROUP    # Multicast 224.0.0.0
		INADDR_ALLHOSTS_GROUP  # Multicast 224.0.0.1
		INADDR_ALLRTRS_GROUP   # Multicast 224.0.0.2
		INADDR_MAX_LOCAL_GROUP # Multicast 224.0.0.255
	;;
	Reserved IPv4 addresses.

	Note that all of these values are defined.
*/
static qu_var EnumINADDR;
#ifndef INADDR_ANY
#define INADDR_ANY 0x00000000
#endif
#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif
#ifndef INADDR_BROADCAST
#define INADDR_BROADCAST 0xffffffff
#endif
#ifndef INADDR_LOOPBACK
#define INADDR_LOOPBACK 0x7F000001
#endif
#define INADDR_LOCALHOST INADDR_LOOPBACK
#ifndef INADDR_UNSPEC_GROUP
#define INADDR_UNSPEC_GROUP 0xe0000000
#endif
#ifndef INADDR_ALLHOSTS_GROUP
#define INADDR_ALLHOSTS_GROUP 0xe0000001
#endif
#ifndef INADDR_MAX_LOCAL_GROUP
#define INADDR_MAX_LOCAL_GROUP 0xe00000ff
#endif



/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum SOCK
		SOCK_STREAM    :  1  # Sequenced, reliable, two-way, connection-based byte streams
		SOCK_DGRAM     :  2  # Datagrams (connectionless, unreliable  messages)
		SOCK_RAW       :  3  # Raw network protocol access
		SOCK_RDM       :  4  # Reliable datagram layer, no ordering
		SOCK_SEQPACKET :  5  # Sequenced, reliable, two-way connection-based for datagrams
		SOCK_PACKET    : 10	 # Linux specific
		SOCK_MAX
	;;
	Socket types.

	Note that the values are arbitrary and some may not be defined.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumSOCK;



/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum SOL
		SOL_IP     : 0
		SOL_SOCKET : 1
		SOL_TCP    : 6
		SOL_UDP    : 17
		SOL_RAW    : 255
		SOL_DECNET : 261
		SOL_X25    : 262
		SOL_PACKET : 263
		SOL_ATM    : 264 # ATM layer (cell level).
		SOL_AAL    : 265 # ATM Adaption Layer (packet level).
		SOL_IRDA   : 266
		SOL_IPX
		SOL_AX25
		SOL_ATALK
		SOL_NETROM
		SOL_ROSE
	;;
	Socket levels.

	See C{man 2 getsockopt} for details.

	Note that the values are arbitrary and some may not be defined.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumSOL;
#ifndef SOL_IP
#define SOL_IP 0
#endif
#ifndef SOL_TCP
#define SOL_TCP 6
#endif
#ifndef SOL_UPD
#define SOL_UDP 17
#endif


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum SO
		SO_DEBUG                         : 1
		SO_REUSEADDR                     : 2
		SO_TYPE                          : 3
		SO_ERROR                         : 4
		SO_DONTROUTE                     : 5
		SO_BROADCAST                     : 6
		SO_SNDBUF                        : 7
		SO_RCVBUF                        : 8
		SO_KEEPALIVE                     : 9
		SO_OOBINLINE                     : 10
		SO_NO_CHECK                      : 11
		SO_PRIORITY                      : 12
		SO_LINGER                        : 13
		SO_BSDCOMPAT                     : 14
		SO_REUSEPORT                     : 15
		SO_PASSCRED                      : 16
		SO_PEERCRED                      : 17
		SO_RCVLOWAT                      : 18
		SO_SNDLOWAT                      : 19
		SO_RCVTIMEO                      : 20
		SO_SNDTIMEO                      : 21
		SO_SECURITY_AUTHENTICATION       : 22
		SO_SECURITY_ENCRYPTION_TRANSPORT : 23
		SO_SECURITY_ENCRYPTION_NETWORK   : 24
		SO_BINDTODEVICE                  : 25
		SO_ATTACH_FILTER                 : 26
		SO_DETACH_FILTER                 : 27
		SO_PEERNAME                      : 28
		SO_TIMESTAMP                     : 29
		SO_ACCEPTCONN                    : 30
		SO_NONBLOCK
	;;
	Socket options for #{getsockopt} and #{setsockopt}.

	See C{man 7 socket} for details.

	Note that some may not be defined and the values are arbitrary.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumSO;


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) enum MSG
		MSG_OOB       : 0x01    # Process out-of-band data.
		MSG_PEEK      : 0x02    # Peek at incoming messages.
		MSG_DONTROUTE : 0x04    # Don't use local routing.
		MSG_TRYHARD   : 0x04    # MSG_DONTROUTE
		MSG_CTRUNC    : 0x08    # Control data lost before delivery.
		MSG_PROXY     : 0x10    # Supply or ask second address.
		MSG_TRUNC     : 0x20
		MSG_DONTWAIT  : 0x40    # Nonblocking IO.
		MSG_EOR       : 0x80    # End of record.
		MSG_WAITALL   : 0x100   # Wait for a full request.
		MSG_FIN       : 0x200
		MSG_SYN       : 0x400
		MSG_CONFIRM   : 0x800   # Confirm path validity.
		MSG_RST       : 0x1000
		MSG_ERRQUEUE  : 0x2000  # Fetch message from error queue.
		MSG_NOSIGNAL  : 0x4000  # Do not generate SIGPIPE.
		MSG_MORE      : 0x8000  # Sender will send more.
		MSG_BTAG
		MSG_ETAG
	;;
	Bits in the v{flags} argument to #{send} and #{recv}.

	See C{man 2 send} and C{man 2 recv} for details.

	Note that the values are arbitrary and some may not be defined.
	The v{enum} is used simply for convenience.
*/
static qu_var EnumMSG;


/*) enum SHUT
		SHUT_RD
		SHUT_WR
		SHUT_RDWR
	;;
	Constatns for #{shutdown}.
*/
static qu_var EnumSHUT;




/*----------------------------------------------------------------------------
	init
----------------------------------------------------------------------------*/

#define e_(name) #name, name,

static void _net_init_enum (qu_module m)
{


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumTCP = qu_var_enum ((qu_symbol) m, "TCP",

	e_(TCP_NODELAY)
#ifdef TCP_MAXSEG
	e_(TCP_MAXSEG)
#endif
#ifdef TCP_CORK
	e_(TCP_CORK)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumIP = qu_var_enum ((qu_symbol) m, "IP",

	e_(IP_OPTIONS)
	e_(IP_PKTINFO)
	e_(IP_RECVTOS)
	e_(IP_RECVTTL)
	e_(IP_RECVOPTS)
	e_(IP_RETOPTS)
	e_(IP_TOS)
	e_(IP_TTL)
	e_(IP_HDRINCL)
	e_(IP_RECVERR)
#ifdef IP_PMTU_DISCOVER
	e_(IP_PMTU_DISCOVER)
#endif
#ifdef IP_MTU
	e_(IP_MTU)
#endif
	e_(IP_ROUTER_ALERT)
	e_(IP_MULTICAST_TTL)
	e_(IP_MULTICAST_LOOP)
	e_(IP_ADD_MEMBERSHIP)
	e_(IP_DROP_MEMBERSHIP)
	e_(IP_MULTICAST_IF)
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumIPPROTO = qu_var_enum ((qu_symbol) m, "IPPROTO",

#ifdef IPPROTO_IP
	e_(IPPROTO_IP)
#endif
#ifdef IPPROTO_HOPOPTS
	e_(IPPROTO_HOPOPTS)
#endif
#ifdef IPPROTO_ICMP
	e_(IPPROTO_ICMP)
#endif
#ifdef IPPROTO_IGMP
	e_(IPPROTO_IGMP)
#endif
#ifdef IPPROTO_IPIP
	e_(IPPROTO_IPIP)
#endif
#ifdef IPPROTO_TCP
	e_(IPPROTO_TCP)
#endif
#ifdef IPPROTO_EGP
	e_(IPPROTO_EGP)
#endif
#ifdef IPPROTO_PUP
	e_(IPPROTO_PUP)
#endif
#ifdef IPPROTO_UDP
	e_(IPPROTO_UDP)
#endif
#ifdef IPPROTO_IDP
	e_(IPPROTO_IDP)
#endif
#ifdef IPPROTO_TP
	e_(IPPROTO_TP)
#endif
#ifdef IPPROTO_IPV6
	e_(IPPROTO_IPV6)
#endif
#ifdef IPPROTO_ROUTING
	e_(IPPROTO_ROUTING)
#endif
#ifdef IPPROTO_FRAGMENT
	e_(IPPROTO_FRAGMENT)
#endif
#ifdef IPPROTO_RSVP
	e_(IPPROTO_RSVP)
#endif
#ifdef IPPROTO_GRE
	e_(IPPROTO_GRE)
#endif
#ifdef IPPROTO_ESP
	e_(IPPROTO_ESP)
#endif
#ifdef IPPROTO_AH
	e_(IPPROTO_AH)
#endif
#ifdef IPPROTO_ICMPV6
	e_(IPPROTO_ICMPV6)
#endif
#ifdef IPPROTO_NONE
	e_(IPPROTO_NONE)
#endif
#ifdef IPPROTO_DSTOPTS
	e_(IPPROTO_DSTOPTS)
#endif
#ifdef IPPROTO_MTP
	e_(IPPROTO_MTP)
#endif
#ifdef IPPROTO_ENCAP
	e_(IPPROTO_ENCAP)
#endif
#ifdef IPPROTO_PIM
	e_(IPPROTO_PIM)
#endif
#ifdef IPPROTO_COMP
	e_(IPPROTO_COMP)
#endif
#ifdef IPPROTO_RAW
	e_(IPPROTO_RAW)
#endif
#ifdef IPPROTO_MAX
	e_(IPPROTO_MAX)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumIPPORT = qu_var_enum ((qu_symbol) m, "IPPORT",

#ifdef IPPORT_ECHO
	e_(IPPORT_ECHO)
#endif
#ifdef IPPORT_DISCARD
	e_(IPPORT_DISCARD)
#endif
#ifdef IPPORT_SYSTAT
	e_(IPPORT_SYSTAT)
#endif
#ifdef IPPORT_DAYTIME
	e_(IPPORT_DAYTIME)
#endif
#ifdef IPPORT_NETSTAT
	e_(IPPORT_NETSTAT)
#endif
#ifdef IPPORT_FTP
	e_(IPPORT_FTP)
#endif
#ifdef IPPORT_TELNET
	e_(IPPORT_TELNET)
#endif
#ifdef IPPORT_SMTP
	e_(IPPORT_SMTP)
#endif
#ifdef IPPORT_TIMESERVER
	e_(IPPORT_TIMESERVER)
#endif
#ifdef IPPORT_NAMESERVER
	e_(IPPORT_NAMESERVER)
#endif
#ifdef IPPORT_WHOIS
	e_(IPPORT_WHOIS)
#endif
#ifdef IPPORT_MTP
	e_(IPPORT_MTP)
#endif
#ifdef IPPORT_TFTP
	e_(IPPORT_TFTP)
#endif
#ifdef IPPORT_RJE
	e_(IPPORT_RJE)
#endif
#ifdef IPPORT_FINGER
	e_(IPPORT_FINGER)
#endif
#ifdef IPPORT_HTTP
	e_(IPPORT_HTTP)
#endif
#ifdef IPPORT_TTYLINK
	e_(IPPORT_TTYLINK)
#endif
#ifdef IPPORT_SUPDUP
	e_(IPPORT_SUPDUP)
#endif
#ifdef IPPORT_HTTPS
	e_(IPPORT_HTTPS)
#endif
#ifdef IPPORT_EXECSERVER
	e_(IPPORT_EXECSERVER)
#endif
#ifdef IPPORT_LOGINSERVER
	e_(IPPORT_LOGINSERVER)
#endif
#ifdef IPPORT_CMDSERVER
	e_(IPPORT_CMDSERVER)
#endif
#ifdef IPPORT_EFSSERVER
	e_(IPPORT_EFSSERVER)
#endif
#ifdef IPPORT_BIFFUDP
	e_(IPPORT_BIFFUDP)
#endif
#ifdef IPPORT_WHOSERVER
	e_(IPPORT_WHOSERVER)
#endif
#ifdef IPPORT_ROUTESERVER
	e_(IPPORT_ROUTESERVER)
#endif
#ifdef IPPORT_RESERVED
	e_(IPPORT_RESERVED)
#endif
#ifdef IPPORT_USERRESERVED
	e_(IPPORT_USERRESERVED)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumPF = qu_var_enum ((qu_symbol) m, "PF",
#ifdef PF_UNSPEC
	e_(PF_UNSPEC)
#endif
#ifdef PF_LOCAL
	e_(PF_LOCAL)
#endif
#ifdef PF_UNIX
	e_(PF_UNIX)
#endif
#ifdef PF_FILE
	e_(PF_FILE)
#endif
#ifdef PF_INET
	e_(PF_INET)
#endif
#ifdef PF_AX25
	e_(PF_AX25)
#endif
#ifdef PF_IPX
	e_(PF_IPX)
#endif
#ifdef PF_APPLETALK
	e_(PF_APPLETALK)
#endif
#ifdef PF_NETROM
	e_(PF_NETROM)
#endif
#ifdef PF_BRIDGE
	e_(PF_BRIDGE)
#endif
#ifdef PF_ATMPVC
	e_(PF_ATMPVC)
#endif
#ifdef PF_X25
	e_(PF_X25)
#endif
#ifdef PF_INET6
	e_(PF_INET6)
#endif
#ifdef PF_ROSE
	e_(PF_ROSE)
#endif
#ifdef PF_DECnet
	e_(PF_DECnet)
#endif
#ifdef PF_NETBEUI
	e_(PF_NETBEUI)
#endif
#ifdef PF_SECURITY
	e_(PF_SECURITY)
#endif
#ifdef PF_KEY
	e_(PF_KEY)
#endif
#ifdef PF_NETLINK
	e_(PF_NETLINK)
#endif
#ifdef PF_ROUTE
	e_(PF_ROUTE)
#endif
#ifdef PF_PACKET
	e_(PF_PACKET)
#endif
#ifdef PF_ASH
	e_(PF_ASH)
#endif
#ifdef PF_ECONET
	e_(PF_ECONET)
#endif
#ifdef PF_ATMSVC
	e_(PF_ATMSVC)
#endif
#ifdef PF_SNA
	e_(PF_SNA)
#endif
#ifdef PF_IRDA
	e_(PF_IRDA)
#endif
#ifdef PF_PPPOX
	e_(PF_PPPOX)
#endif
#ifdef PF_WANPIPE
	e_(PF_WANPIPE)
#endif
#ifdef PF_BLUETOOTH
	e_(PF_BLUETOOTH)
#endif
#ifdef PF_MAX
	e_(PF_MAX)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumAF = qu_var_enum ((qu_symbol) m, "AF",

#ifdef AF_UNSPEC
	e_(AF_UNSPEC)
#endif
#ifdef AF_LOCAL
	e_(AF_LOCAL)
#endif
#ifdef AF_UNIX
	e_(AF_UNIX)
#endif
#ifdef AF_FILE
	e_(AF_FILE)
#endif
#ifdef AF_INET
	e_(AF_INET)
#endif
#ifdef AF_AX25
	e_(AF_AX25)
#endif
#ifdef AF_IPX
	e_(AF_IPX)
#endif
#ifdef AF_APPLETALK
	e_(AF_APPLETALK)
#endif
#ifdef AF_NETROM
	e_(AF_NETROM)
#endif
#ifdef AF_BRIDGE
	e_(AF_BRIDGE)
#endif
#ifdef AF_ATMPVC
	e_(AF_ATMPVC)
#endif
#ifdef AF_X25
	e_(AF_X25)
#endif
#ifdef AF_INET6
	e_(AF_INET6)
#endif
#ifdef AF_ROSE
	e_(AF_ROSE)
#endif
#ifdef AF_DECnet
	e_(AF_DECnet)
#endif
#ifdef AF_NETBEUI
	e_(AF_NETBEUI)
#endif
#ifdef AF_SECURITY
	e_(AF_SECURITY)
#endif
#ifdef AF_KEY
	e_(AF_KEY)
#endif
#ifdef AF_NETLINK
	e_(AF_NETLINK)
#endif
#ifdef AF_ROUTE
	e_(AF_ROUTE)
#endif
#ifdef AF_PACKET
	e_(AF_PACKET)
#endif
#ifdef AF_ASH
	e_(AF_ASH)
#endif
#ifdef AF_ECONET
	e_(AF_ECONET)
#endif
#ifdef AF_ATMSVC
	e_(AF_ATMSVC)
#endif
#ifdef AF_SNA
	e_(AF_SNA)
#endif
#ifdef AF_IRDA
	e_(AF_IRDA)
#endif
#ifdef AF_PPPOX
	e_(AF_PPPOX)
#endif
#ifdef AF_WANPIPE
	e_(AF_WANPIPE)
#endif
#ifdef AF_BLUETOOTH
	e_(AF_BLUETOOTH)
#endif
#ifdef AF_MAX
	e_(AF_MAX)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumINADDR = qu_var_enum_ex ((qu_symbol) m, "INADDR", "ssssssss",
	"INADDR_ANY"            , "0.0.0.0",
	"INADDR_NONE"           , "255.255.255.255",
	"INADDR_BROADCAST"      , "255.255.255.255",
	"INADDR_LOOPBACK"       , "127.0.0.1",
	"INADDR_LOCALHOST"      , "127.0.0.1",
	"INADDR_UNSPEC_GROUP"   , "224.0.0.0",
	"INADDR_ALLHOSTS_GROUP" , "224.0.0.1",
	"INADDR_ALLRTRS_GROUP"  , "224.0.0.2",
	"INADDR_MAX_LOCAL_GROUP", "224.0.0.255");


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumSOCK = qu_var_enum ((qu_symbol) m, "SOCK",

	e_(SOCK_STREAM)
	e_(SOCK_DGRAM)
#ifdef SOCK_SEQPACKET
	e_(SOCK_SEQPACKET)
#endif
#ifdef SOCK_RAW
	e_(SOCK_RAW)
#endif
#ifdef SOCK_RDM
	e_(SOCK_RDM)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumSOL = qu_var_enum ((qu_symbol) m, "SOL",

	e_(SOL_SOCKET)
	e_(SOL_TCP)
	e_(SOL_IP)
#ifdef SOL_UDP
	e_(SOL_UDP)
#endif
#ifdef SOL_IPX
	e_(SOL_IPX)
#endif
#ifdef SOL_AX25
	e_(SOL_AX25)
#endif
#ifdef SOL_ATALK
	e_(SOL_ATALK)
#endif
#ifdef SOL_NETROM
	e_(SOL_NETROM)
#endif
#ifdef SOL_ROSE
	e_(SOL_ROSE)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumSO = qu_var_enum ((qu_symbol) m, "SO",

#ifdef SO_DEBUG
	e_(SO_DEBUG)
#endif
#ifdef SO_REUSEADDR
	e_(SO_REUSEADDR)
#endif
#ifdef SO_TYPE
	e_(SO_TYPE)
#endif
#ifdef SO_ERROR
	e_(SO_ERROR)
#endif
#ifdef SO_DONTROUTE
	e_(SO_DONTROUTE)
#endif
#ifdef SO_BROADCAST
	e_(SO_BROADCAST)
#endif
#ifdef SO_SNDBUF
	e_(SO_SNDBUF)
#endif
#ifdef SO_RCVBUF
	e_(SO_RCVBUF)
#endif
#ifdef SO_KEEPALIVE
	e_(SO_KEEPALIVE)
#endif
#ifdef SO_OOBINLINE
	e_(SO_OOBINLINE)
#endif
#ifdef SO_NO_CHECK
	e_(SO_NO_CHECK)
#endif
#ifdef SO_PRIORITY
	e_(SO_PRIORITY)
#endif
#ifdef SO_LINGER
	e_(SO_LINGER)
#endif
#ifdef SO_BSDCOMPAT
	e_(SO_BSDCOMPAT)
#endif
#ifdef SO_REUSEPORT
	e_(SO_REUSEPORT)
#endif
#ifdef SO_PASSCRED
	e_(SO_PASSCRED)
#endif
#ifdef SO_PEERCRED
	e_(SO_PEERCRED)
#endif
#ifdef SO_RCVLOWAT
	e_(SO_RCVLOWAT)
#endif
#ifdef SO_SNDLOWAT
	e_(SO_SNDLOWAT)
#endif
#ifdef SO_RCVTIMEO
	e_(SO_RCVTIMEO)
#endif
#ifdef SO_SNDTIMEO
	e_(SO_SNDTIMEO)
#endif
#ifdef SO_SECURITY_AUTHENTICATION
	e_(SO_SECURITY_AUTHENTICATION)
#endif
#ifdef SO_SECURITY_ENCRYPTION_TRANSPORT
	e_(SO_SECURITY_ENCRYPTION_TRANSPORT)
#endif
#ifdef SO_SECURITY_ENCRYPTION_NETWORK
	e_(SO_SECURITY_ENCRYPTION_NETWORK)
#endif
#ifdef SO_BINDTODEVICE
	e_(SO_BINDTODEVICE)
#endif
#ifdef SO_ATTACH_FILTER
	e_(SO_ATTACH_FILTER)
#endif
#ifdef SO_DETACH_FILTER
	e_(SO_DETACH_FILTER)
#endif
#ifdef SO_PEERNAME
	e_(SO_PEERNAME)
#endif
#ifdef SO_TIMESTAMP
	e_(SO_TIMESTAMP)
#endif
#ifdef SO_ACCEPTCONN
	e_(SO_ACCEPTCONN)
#endif
#ifdef SO_NONBLOCK
	e_(SO_NONBLOCK)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	EnumMSG = qu_var_enum ((qu_symbol) m, "MSG",
#ifdef MSG_OOB
	e_(MSG_OOB)
#endif
#ifdef MSG_PEEK
	e_(MSG_PEEK)
#endif
#ifdef MSG_DONTROUTE
	e_(MSG_DONTROUTE)
#endif
#ifdef MSG_DONTWAIT
	e_(MSG_DONTWAIT)
#endif
#ifdef MSG_EOR
	e_(MSG_EOR)
#endif
#ifdef MSG_TRUNC
	e_(MSG_TRUNC)
#endif
#ifdef MSG_CTRUNC
	e_(MSG_CTRUNC)
#endif
#ifdef MSG_WAITALL
	e_(MSG_WAITALL)
#endif
#ifdef MSG_BTAG
	e_(MSG_BTAG)
#endif
#ifdef MSG_ETAG
	e_(MSG_ETAG)
#endif
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumNI = qu_var_enum ((qu_symbol) m, "NI",

	e_(NI_NOFQDN)
	e_(NI_NUMERICHOST)
	e_(NI_NAMEREQD)
	e_(NI_NUMERICSERV)
	e_(NI_DGRAM)
	NULL);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
EnumSHUT = qu_var_enum ((qu_symbol) m, "SHUT",

	e_(SHUT_RD)
	e_(SHUT_WR)
	e_(SHUT_RDWR)
	NULL);

}


#undef e_
