/* Netconst.c - Copyright (c) Marc Krisnanto */


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) struct Protoent
		name    : String # official protocol name
		aliases : Array  # alias list (of String)
		proto   : Int    # protocol number
	;;
	This struct specifies an entry in the protocol list.

	See C{man 3 getprotoent} for details.

	See also #{IPROTO}.
*/
static qu_struct ProtoentStruct;



/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) struct Servent
		name    : String # official service name
		aliases : Array  # alias list (of String)
		port    : Int    # port number
		proto   : String # protocol name to use
	;;
	This struct specifies a service in the service list.

	See C{man 3 getservent} for details.

	See also #{IPPORT}.
*/
static qu_struct ServentStruct;



/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) struct Hostent
		name     : String    # official host name
		aliases  : Array     # alias list (of String)
		addrtype : Int       # host address type, e.g. AF_INET
		length   : Int       # length of address
		Array    : addr_list #  list of addresses (of String)
	;;
	Host record as returned by #{gethostbyname} and #{gethostbyaddr}.
*/
static qu_struct HostentStruct;



/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*) struct Linger
		l_onoff  : Boolean # linger active
		l_linger : Int  # how many seconds to linger for
	;;
	This struct is used with #{getsockopt} or #{setsockopt} with SO_LINGER.

	See C{man 7 socket} for details.
*/
static qu_struct LingerStruct;



/*----------------------------------------------------------------------------
	init
----------------------------------------------------------------------------*/

static void _net_init_struct (qu_module m)
{

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
ProtoentStruct = qu_struct_new_static (3,
	"name"      , QuStringClass,
	"aliases"   , QuArrayClass,
	"proto"     , QuIntClass);

qu_module_add_const (m, "Protoent", ProtoentStruct);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
ServentStruct = qu_struct_new_static (4,
	"name"      , QuStringClass,
	"aliases"   , QuArrayClass,
	"port"      , QuIntClass,
	"proto"     , QuStringClass);

qu_module_add_const (m, "Servent", ServentStruct);


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
HostentStruct = qu_struct_new_static (5,
	"name"      , QuStringClass,
	"aliases"   , QuArrayClass,
	"addrtype"  , QuIntClass,
	"length"    , QuIntClass,
	"addr_list" , QuArrayClass);

qu_module_add_const (m, "Hostent", HostentStruct);

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

LingerStruct = qu_struct_new_static (2,
	"l_onoff"   , QuBooleanClass,
	"l_linger"  , QuIntClass);

qu_module_add_const (m, "Linger", LingerStruct);

}
