#ifndef _H_XmlParse_
#define _H_XmlParse_ 1

typedef void *XML_Parser;

/* Information is UTF-8 encoded. */
typedef char XML_Char;
typedef char XML_LChar;

enum XML_Content_Type {
	XML_CTYPE_EMPTY = 1,
	XML_CTYPE_ANY,
	XML_CTYPE_MIXED,
	XML_CTYPE_NAME,
	XML_CTYPE_CHOICE,
	XML_CTYPE_SEQ
};

enum XML_Content_Quant {
	XML_CQUANT_NONE,
	XML_CQUANT_OPT,
	XML_CQUANT_REP,
	XML_CQUANT_PLUS
};

typedef struct XML_cp XML_Content;

struct XML_cp {
	enum XML_Content_Type type;
	enum XML_Content_Quant quant;
	XML_Char *name;
	unsigned int numchildren;
	XML_Content *children;
};

typedef struct {
	int    map[256];
	void  *data;
	int  (*convert)(void *data, const char *s);
	void (*release)(void *data);
} XML_Encoding;


typedef struct {
	void *(*malloc_fcn)(size_t size);
	void *(*realloc_fcn)(void *ptr, size_t size);
	void (*free_fcn)(void *ptr);
} XML_Memory_Handling_Suite;

typedef void (*XML_ElementDeclHandler          ) (void *userData, const XML_Char *name, XML_Content *model);
typedef void (*XML_AttlistDeclHandler          ) (void *userData, const XML_Char *elname, const XML_Char *attname, const XML_Char *att_type, const XML_Char *dflt, int isrequired);
typedef void (*XML_XmlDeclHandler              ) (void *userData, const XML_Char*version, const XML_Char *encoding, int standalone);
typedef void (*XML_StartElementHandler         ) (void *userData, const XML_Char *name, const XML_Char **atts);
typedef void (*XML_EndElementHandler           ) (void *userData, const XML_Char *name);
typedef void (*XML_CharacterDataHandler        ) (void *userData, const XML_Char *s, int len);
typedef void (*XML_ProcessingInstructionHandler) (void *userData, const XML_Char *target, const XML_Char *data);
typedef void (*XML_CommentHandler              ) (void *userData, const XML_Char *data);
typedef void (*XML_StartCdataSectionHandler    ) (void *userData);
typedef void (*XML_EndCdataSectionHandler      ) (void *userData);
typedef void (*XML_DefaultHandler              ) (void *userData, const XML_Char *s, int len);
typedef void (*XML_StartDoctypeDeclHandler     ) (void *userData, const XML_Char *doctypeName, const XML_Char *sysid, const XML_Char *pubid, int has_internal_subset);
typedef void (*XML_EndDoctypeDeclHandler       ) (void *userData);
typedef void (*XML_EntityDeclHandler           ) (void *userData, const XML_Char *entityName, int is_parameter_entity, const XML_Char *value, int value_length, const XML_Char *base, const XML_Char *systemId, const XML_Char *publicId, const XML_Char *notationName);
typedef void (*XML_NotationDeclHandler         ) (void *userData, const XML_Char *notationName, const XML_Char *base, const XML_Char *systemId, const XML_Char *publicId);
typedef void (*XML_StartNamespaceDeclHandler   ) (void *userData, const XML_Char *prefix, const XML_Char *uri);
typedef void (*XML_EndNamespaceDeclHandler     ) (void *userData, const XML_Char *prefix);
typedef int  (*XML_NotStandaloneHandler        ) (void *userData);
typedef int  (*XML_ExternalEntityRefHandler    ) (XML_Parser p, const XML_Char *context, const XML_Char *base, const XML_Char *systemId, const XML_Char *publicId);
typedef int  (*XML_UnknownEncodingHandler      ) (void *encodingHandlerData, const XML_Char *name, XML_Encoding *info);

static XML_Parser (*XML_ParserCreate  )  (const XML_Char *encoding);
static XML_Parser (*XML_ParserCreateNS)  (const XML_Char *encoding, XML_Char namespaceSeparator);
static XML_Parser (*XML_ParserCreate_MM) (const XML_Char *encoding, const XML_Memory_Handling_Suite *memsuite, const XML_Char *namespaceSeparator);

static void (*XML_SetElementDeclHandler          ) (XML_Parser p, XML_ElementDeclHandler eldecl);
static void (*XML_SetAttlistDeclHandler          ) (XML_Parser p, XML_AttlistDeclHandler attdecl);
static void (*XML_SetXmlDeclHandler              ) (XML_Parser p, XML_XmlDeclHandler xmldecl);
static void (*XML_SetEntityDeclHandler           ) (XML_Parser p, XML_EntityDeclHandler handler);
static void (*XML_SetElementHandler              ) (XML_Parser p, XML_StartElementHandler start, XML_EndElementHandler end);
static void (*XML_SetStartElementHandler         ) (XML_Parser p, XML_StartElementHandler start);
static void (*XML_SetEndElementHandler           ) (XML_Parser p, XML_EndElementHandler end);
static void (*XML_SetCharacterDataHandler        ) (XML_Parser p, XML_CharacterDataHandler handler);
static void (*XML_SetProcessingInstructionHandler) (XML_Parser p, XML_ProcessingInstructionHandler handler);
static void (*XML_SetCommentHandler              ) (XML_Parser p, XML_CommentHandler handler);
static void (*XML_SetCdataSectionHandler         ) (XML_Parser p, XML_StartCdataSectionHandler start, XML_EndCdataSectionHandler end);
static void (*XML_SetStartCdataSectionHandler    ) (XML_Parser p, XML_StartCdataSectionHandler start);
static void (*XML_SetEndCdataSectionHandler      ) (XML_Parser p, XML_EndCdataSectionHandler end);
static void (*XML_SetDefaultHandler              ) (XML_Parser p, XML_DefaultHandler handler);
static void (*XML_SetDefaultHandlerExpand        ) (XML_Parser p, XML_DefaultHandler handler);
static void (*XML_SetDoctypeDeclHandler          ) (XML_Parser p, XML_StartDoctypeDeclHandler start, XML_EndDoctypeDeclHandler end);
static void (*XML_SetStartDoctypeDeclHandler     ) (XML_Parser p, XML_StartDoctypeDeclHandler start);
static void (*XML_SetEndDoctypeDeclHandler       ) (XML_Parser p, XML_EndDoctypeDeclHandler end);
static void (*XML_SetNotationDeclHandler         ) (XML_Parser p, XML_NotationDeclHandler handler);
static void (*XML_SetNamespaceDeclHandler        ) (XML_Parser p, XML_StartNamespaceDeclHandler start, XML_EndNamespaceDeclHandler end);
static void (*XML_SetStartNamespaceDeclHandler   ) (XML_Parser p, XML_StartNamespaceDeclHandler start);
static void (*XML_SetEndNamespaceDeclHandler     ) (XML_Parser p, XML_EndNamespaceDeclHandler end);
static void (*XML_SetNotStandaloneHandler        ) (XML_Parser p, XML_NotStandaloneHandler handler);
static void (*XML_SetExternalEntityRefHandler    ) (XML_Parser p, XML_ExternalEntityRefHandler handler);
static void (*XML_SetExternalEntityRefHandlerArg ) (XML_Parser, void *arg);
static void (*XML_SetUnknownEncodingHandler      ) (XML_Parser p, XML_UnknownEncodingHandler handler, void *encodingHandlerData);
static void (*XML_DefaultCurrent                 ) (XML_Parser p);

static void  (*XML_SetReturnNSTriplet        ) (XML_Parser p, int do_nst);
static void  (*XML_SetUserData               ) (XML_Parser p, void *userData);
#define XML_GetUserData(p) (*(void **)(p))
static int   (*XML_SetEncoding               ) (XML_Parser p, const XML_Char *encoding);
static void  (*XML_UseParserAsHandlerArg     ) (XML_Parser p);
static int   (*XML_SetBase                   ) (XML_Parser p, const XML_Char *base);
static const XML_Char *(*XML_GetBase         ) (XML_Parser p);
static int   (*XML_GetSpecifiedAttributeCount) (XML_Parser p);
static int   (*XML_GetIdAttributeIndex       ) (XML_Parser p);
static int   (*XML_Parse                     ) (XML_Parser p, const char *s, int len, int isFinal);
static void *(*XML_GetBuffer                 ) (XML_Parser p, int len);
static int   (*XML_ParseBuffer               ) (XML_Parser p, int len, int isFinal);

static XML_Parser (*XML_ExternalEntityParserCreate) (XML_Parser p, const XML_Char *context, const XML_Char *encoding);

enum XML_ParamEntityParsing {
	XML_PARAM_ENTITY_PARSING_NEVER,
	XML_PARAM_ENTITY_PARSING_UNLESS_STANDALONE,
	XML_PARAM_ENTITY_PARSING_ALWAYS
};

static int (*XML_SetParamEntityParsing) (XML_Parser p, enum XML_ParamEntityParsing parsing);

enum XML_Error {
	XML_ERROR_NONE,
	XML_ERROR_NO_MEMORY,
	XML_ERROR_SYNTAX,
	XML_ERROR_NO_ELEMENTS,
	XML_ERROR_INVALID_TOKEN,
	XML_ERROR_UNCLOSED_TOKEN,
	XML_ERROR_PARTIAL_CHAR,
	XML_ERROR_TAG_MISMATCH,
	XML_ERROR_DUPLICATE_ATTRIBUTE,
	XML_ERROR_JUNK_AFTER_DOC_ELEMENT,
	XML_ERROR_PARAM_ENTITY_REF,
	XML_ERROR_UNDEFINED_ENTITY,
	XML_ERROR_RECURSIVE_ENTITY_REF,
	XML_ERROR_ASYNC_ENTITY,
	XML_ERROR_BAD_CHAR_REF,
	XML_ERROR_BINARY_ENTITY_REF,
	XML_ERROR_ATTRIBUTE_EXTERNAL_ENTITY_REF,
	XML_ERROR_MISPLACED_XML_PI,
	XML_ERROR_UNKNOWN_ENCODING,
	XML_ERROR_INCORRECT_ENCODING,
	XML_ERROR_UNCLOSED_CDATA_SECTION,
	XML_ERROR_EXTERNAL_ENTITY_HANDLING,
	XML_ERROR_NOT_STANDALONE,
	XML_ERROR_UNEXPECTED_STATE
};

static enum  XML_Error  (*XML_GetErrorCode) (XML_Parser p);
static const XML_LChar *(*XML_ErrorString ) (int code);

static int  (*XML_GetCurrentLineNumber  ) (XML_Parser p);
static int  (*XML_GetCurrentColumnNumber) (XML_Parser p);
static long (*XML_GetCurrentByteIndex   ) (XML_Parser p);
static int  (*XML_GetCurrentByteCount   ) (XML_Parser p);
static const char *(*XML_GetInputContext) (XML_Parser p, int *offset, int *size);

static void (*XML_ParserFree) (XML_Parser p);
static const XML_LChar *(*XML_ExpatVersion) ();

/* not in expat.h */
static void (*XML_FreeContentModel) (XML_Parser parser, XML_Content *model);

#endif /*XmlParse.h*/
