/* Emit.c - Copyright (c) Marc Krisnanto */

#ifndef QU_EMIT_IN_CODE_C
#error This file is part of Code.c
#endif

/*============================================================================

	CHECKME

============================================================================*/

/*
	Optimizing final qu_object attributes

	__inst must not be optimzed because C classes might override it for subclassing
	This should be the only one though ...
*/
#if 0 /* HECK - this doesn't work with __pure */
static int emit_is_obj_optimizable (qu_string s)
{
	if (s->len > 2 && s->str[0] == '_' && s->str[1] == '_')
	{
		if (strcmp (s->str, "__inst") == 0)
			return 0;
	}
	return 1;
}
#else
#define emit_is_obj_optimizable(s) 0
#endif

/*============================================================================

	emit stuffs

============================================================================*/

/* pass 1 or 2 see emit_module down below */
static int EmitPass;

/* forward */
static void emit_begin ();
static void emit_end   ();


/* test if a node is a lexical variable */
#define ISLEXICAL(n) \
	((n)->type == QU_AST_name && qu_var_is ((n)->thing) && qu_var_is_lexical ((n)->thing))

/*
	PUSH and POP are used to calculate the required stack size.

	!!! After each statement the stack must be balanced
	!!! otherwise there is something wrong.
*/
#define PUSH(n) \
QU_BEGIN \
	CoSp += (n); \
	if (CoSp > CoStackSize) \
		CoStackSize = CoSp; \
QU_END

#define POP(n) \
QU_BEGIN \
	CoSp -= (n); \
	if (CoSp < 0) \
		QU_WARN ("miscalculated POP: %d", CoSp); \
QU_END



/* to make it obvious that we are going to set the argument later */
#define __LATER__  0

/* set the first argument */
#define SET1(ofs, v)  QU_BEGIN *(qu_any *) (CoBytes + (ofs) + 1) = (qu_any) v; QU_END

/* set the second argument */
#define SET2(ofs, v)  QU_BEGIN *(qu_any *) (CoBytes + (ofs) + 1 + sizeof (qu_any)) = (qu_any) v; QU_END

/* set a jump instruction target = offset of the next instruction */
#define SETJ(v) QU_BEGIN *(long *) (CoBytes + v + 1) = (long) CoNextOfs; QU_END



/*
	Emit instruction with no argument.
	The offset returned is the offset of this instruction, not of the next.
*/
#define P0(pc) emit_p0 (QU_PC_##pc)
static int emit_p0 (int pc)
{
	int i = CoNextOfs;

	CoPcLast = pc;
	CoPcLastOfs = i;
	CHECKGROW (i);
	CoBytes [i] = pc;
	CoNextOfs++;
	return i;
}



/*
	Emit instruction with 1 argument.
	The offset returned is the offset of this instruction, not of the next.
*/
#define P1(pc, arg) emit_p1 (QU_PC_##pc, (qu_any) (arg))
static int emit_p1 (int pc, qu_any arg)
{
	int i = CoNextOfs;
	int j = i + L1;

	CoPcLast = pc;
	CoPcLastOfs = i;
	CHECKGROW (j);
	CoBytes [i] = pc;
	*(qu_any *) (CoBytes + i + 1) = arg;
	CoNextOfs = j;
	CoBytes [j] = 0; /* always set zero marker */
	return i;
}



/*
	Emit bytecode with 2 arguments.
	The offset returned is the offset of this instruction, not of the next.
*/
#define P2(pc, arg1, arg2) emit_p2 (QU_PC_##pc, (qu_any) (arg1), (qu_any) (arg2))
static int emit_p2 (int pc, qu_any arg1, qu_any arg2)
{
	int i = CoNextOfs;
	int j = i + L2;

	CoPcLast = pc;
	CoPcLastOfs = i;
	CHECKGROW (j);
	CoBytes [i] = pc;
	*(qu_any *) (CoBytes + i + 1) = arg1;
	*(qu_any *) (CoBytes + i + 1 + sizeof (qu_any)) = arg2;
	CoNextOfs = j;
	CoBytes [j] = 0; /* always set zero marker */
	return i;
}



/*
	Emit line number.
*/
#define LINE(n) \
QU_BEGIN \
	if (QU_APP_LINE_NUMBER) \
		emit_line (n); \
QU_END

static long emit_get_line_from_child (qu_ast n)
{
	if (n->child)
	{
		if (n->child->line)
			return n->child->line;
		return emit_get_line_from_child (n->child);
	}
	return 0;
}



static void emit_line (qu_ast n)
{
	long l = n->line;

	if (l < 1)
		l = emit_get_line_from_child (n);

	if (l > CoLine)
	{
		P1 (LINE, l);
		CoLine = l;
	}
}



/*
	E1 = emit one node - forward declaration
*/
#define E1(n) emit_node(n)
static void emit_node (qu_ast n);



/*
	EC = emit all child nodes of n
*/
#define EC(n) emit_child_nodes(n)
inline static void emit_child_nodes (qu_ast n)
{
	for (n = n->child; n; n = n->next)
		E1 (n);
}



/*
	EF = emit nodes starting from n rightwards
*/
#define EF(n) emit_nodes_from (n)
inline void emit_nodes_from (qu_ast n)
{
	for (; n; n = n->next)
		E1 (n);
}



/*
	Emit binary operator.

	!!! The POP is not emitted by BINOP.
	!!! The caller must do it itself.
	!!! This is intentional for clarity.
*/
#define BINOP(f) emit__binop ((qu_binop_f) f)
void emit__binop (qu_binop_f f)
{
	if      (f == (qu_binop_f) qu_sys_in     ) P0 (IN);
	else if (f == (qu_binop_f) qu_sys_eq     ) P0 (EQ);
	else if (f == (qu_binop_f) qu_sys_is     ) P0 (IS);
	else if (f == (qu_binop_f) qu_sys_lt     ) P0 (LT);
	else if (f == (qu_binop_f) qu_sys_gt     ) P0 (GT);
	else if (f == (qu_binop_f) qu_sys_ne     ) P0 (NE);
	else if (f == (qu_binop_f) qu_sys_le     ) P0 (LE);
	else if (f == (qu_binop_f) qu_sys_ge     ) P0 (GE);
	else if (f == (qu_binop_f) qu_sys_cmp    ) P0 (CMP);
	else if (f == (qu_binop_f) qu_sys_like   ) P0 (LIKE);
	else if (f == (qu_binop_f) qu_sys_rin    ) P0 (RIN);
	else if (f == (qu_binop_f) qu_sys_plus   ) P0 (PLUS);
	else if (f == (qu_binop_f) qu_sys_minus  ) P0 (MINUS);
	else if (f == (qu_binop_f) qu_sys_mul    ) P0 (MUL);
	else if (f == (qu_binop_f) qu_sys_div    ) P0 (DIV);
	else if (f == (qu_binop_f) qu_sys_concat ) P0 (CONCAT);
	else if (f == (qu_binop_f) qu_sys_as     ) P0 (AS);
	else if (f == (qu_binop_f) qu_sys_push   ) P0 (PUSH);
	else if (f == (qu_binop_f) qu_sys_ior    ) P0 (IOR);
	else if (f == (qu_binop_f) qu_sys_iand   ) P0 (IAND);
	else if (f == (qu_binop_f) qu_sys_idiv   ) P0 (IDIV);
	else if (f == (qu_binop_f) qu_sys_ixor   ) P0 (IXOR);
	else if (f == (qu_binop_f) qu_sys_lshift ) P0 (LSHIFT);
	else if (f == (qu_binop_f) qu_sys_rshift ) P0 (RSHIFT);
	else if (f == (qu_binop_f) qu_sys_ratio  ) P0 (RATIO);
	else if (f == (qu_binop_f) qu_sys_rem    ) P0 (REM);
	else if (f == (qu_binop_f) qu_sys_pow    ) P0 (POW);
	else if (f == (qu_binop_f) qu_sys_rep    ) P0 (DROP1);
	else if (f == (qu_binop_f) qu_sys_of     ) P0 (OF);
	else if (f == (qu_binop_f) qu_sys_or     ) QU_FATAL ("|| should not be here");
	else if (f == (qu_binop_f) qu_sys_and    ) QU_FATAL ("&& should not be here");
	else QU_FATAL ("unknown binop");
}


#define BINOPLL(f, a, b) emit__binopll ((qu_binop_f) f, a, b)
void emit__binopll (qu_binop_f f, qu_ast a, qu_ast b)
{
	long i = ((qu_var) a->thing)->index;
	long j = ((qu_var) b->thing)->index;

	if      (f == (qu_binop_f) qu_sys_in     ) P2 (INLL, i, j);
	else if (f == (qu_binop_f) qu_sys_eq     ) P2 (EQLL, i, j);
	else if (f == (qu_binop_f) qu_sys_is     ) P2 (ISLL, i, j);
	else if (f == (qu_binop_f) qu_sys_lt     ) P2 (LTLL, i, j);
	else if (f == (qu_binop_f) qu_sys_gt     ) P2 (GTLL, i, j);
	else if (f == (qu_binop_f) qu_sys_ne     ) P2 (NELL, i, j);
	else if (f == (qu_binop_f) qu_sys_le     ) P2 (LELL, i, j);
	else if (f == (qu_binop_f) qu_sys_ge     ) P2 (GELL, i, j);
	else if (f == (qu_binop_f) qu_sys_cmp    ) P2 (CMPLL, i, j);
	else if (f == (qu_binop_f) qu_sys_like   ) P2 (LIKELL, i, j);
	else if (f == (qu_binop_f) qu_sys_rin    ) P2 (RINLL, i, j);
	else if (f == (qu_binop_f) qu_sys_plus   ) P2 (PLUSLL, i, j);
	else if (f == (qu_binop_f) qu_sys_minus  ) P2 (MINUSLL, i, j);
	else if (f == (qu_binop_f) qu_sys_mul    ) P2 (MULLL, i, j);
	else if (f == (qu_binop_f) qu_sys_div    ) P2 (DIVLL, i, j);
	else if (f == (qu_binop_f) qu_sys_concat ) P2 (CONCATLL, i, j);
	else if (f == (qu_binop_f) qu_sys_as     ) P2 (ASLL, i, j);
	else if (f == (qu_binop_f) qu_sys_push   ) P2 (PUSHLL, i, j);
	else if (f == (qu_binop_f) qu_sys_ior    ) P2 (IORLL, i, j);
	else if (f == (qu_binop_f) qu_sys_iand   ) P2 (IANDLL, i, j);
	else if (f == (qu_binop_f) qu_sys_idiv   ) P2 (IDIVLL, i, j);
	else if (f == (qu_binop_f) qu_sys_ixor   ) P2 (IXORLL, i, j);
	else if (f == (qu_binop_f) qu_sys_lshift ) P2 (LSHIFTLL, i, j);
	else if (f == (qu_binop_f) qu_sys_rshift ) P2 (RSHIFTLL, i, j);
	else if (f == (qu_binop_f) qu_sys_ratio  ) P2 (RATIOLL, i, j);
	else if (f == (qu_binop_f) qu_sys_rem    ) P2 (REMLL, i, j);
	else if (f == (qu_binop_f) qu_sys_pow    ) P2 (POWLL, i, j);
	else if (f == (qu_binop_f) qu_sys_rep    ) P2 (DROP1LL, i, j);
	else if (f == (qu_binop_f) qu_sys_of     ) P2 (OFLL, i, j);
	else if (f == (qu_binop_f) qu_sys_or     ) QU_FATAL ("|| should not be here");
	else if (f == (qu_binop_f) qu_sys_and    ) QU_FATAL ("&& should not be here");
	else QU_FATAL ("unknown binop");
}



#define BINOPLX(f, a, b) emit__binoplx ((qu_binop_f) f, a, b)
void emit__binoplx (qu_binop_f f, qu_ast a, qu_ast b)
{
	long i = ((qu_var) a->thing)->index;
	qu_any  x = b->thing;

	if      (f == (qu_binop_f) qu_sys_in     ) P2 (INLX, i, x);
	else if (f == (qu_binop_f) qu_sys_eq     ) P2 (EQLX, i, x);
	else if (f == (qu_binop_f) qu_sys_is     ) P2 (ISLX, i, x);
	else if (f == (qu_binop_f) qu_sys_lt     ) P2 (LTLX, i, x);
	else if (f == (qu_binop_f) qu_sys_gt     ) P2 (GTLX, i, x);
	else if (f == (qu_binop_f) qu_sys_ne     ) P2 (NELX, i, x);
	else if (f == (qu_binop_f) qu_sys_le     ) P2 (LELX, i, x);
	else if (f == (qu_binop_f) qu_sys_ge     ) P2 (GELX, i, x);
	else if (f == (qu_binop_f) qu_sys_cmp    ) P2 (CMPLX, i, x);
	else if (f == (qu_binop_f) qu_sys_like   ) P2 (LIKELX, i, x);
	else if (f == (qu_binop_f) qu_sys_rin    ) P2 (RINLX, i, x);
	else if (f == (qu_binop_f) qu_sys_plus   ) P2 (PLUSLX, i, x);
	else if (f == (qu_binop_f) qu_sys_minus  ) P2 (MINUSLX, i, x);
	else if (f == (qu_binop_f) qu_sys_mul    ) P2 (MULLX, i, x);
	else if (f == (qu_binop_f) qu_sys_div    ) P2 (DIVLX, i, x);
	else if (f == (qu_binop_f) qu_sys_concat ) P2 (CONCATLX, i, x);
	else if (f == (qu_binop_f) qu_sys_as     ) P2 (ASLX, i, x);
	else if (f == (qu_binop_f) qu_sys_push   ) P2 (PUSHLX, i, x);
	else if (f == (qu_binop_f) qu_sys_ior    ) P2 (IORLX, i, x);
	else if (f == (qu_binop_f) qu_sys_iand   ) P2 (IANDLX, i, x);
	else if (f == (qu_binop_f) qu_sys_idiv   ) P2 (IDIVLX, i, x);
	else if (f == (qu_binop_f) qu_sys_ixor   ) P2 (IXORLX, i, x);
	else if (f == (qu_binop_f) qu_sys_lshift ) P2 (LSHIFTLX, i, x);
	else if (f == (qu_binop_f) qu_sys_rshift ) P2 (RSHIFTLX, i, x);
	else if (f == (qu_binop_f) qu_sys_ratio  ) P2 (RATIOLX, i, x);
	else if (f == (qu_binop_f) qu_sys_rem    ) P2 (REMLX, i, x);
	else if (f == (qu_binop_f) qu_sys_pow    ) P2 (POWLX, i, x);
	else if (f == (qu_binop_f) qu_sys_rep    ) P2 (DROP1LX, i, x);
	else if (f == (qu_binop_f) qu_sys_of     ) P2 (OFLX, i, x);
	else if (f == (qu_binop_f) qu_sys_or     ) QU_FATAL ("|| should not be here");
	else if (f == (qu_binop_f) qu_sys_and    ) QU_FATAL ("&& should not be here");
	else QU_FATAL ("unknown binop");
}


/*
	Emit unary operator.
	Obviously, unary operators do not change the stack.
*/
#define UNOP(f) emit__unop ((qu_unop_f) f)
void emit__unop (qu_unop_f f)
{
	if      (f == (qu_unop_f) qu_sys_true) P0 (TRUE);
	else if (f == (qu_unop_f) qu_sys_each) P0 (EACH);
	else if (f == (qu_unop_f) qu_sys_inot) P0 (INOT);
	else if (f == (qu_unop_f) qu_sys_neg ) P0 (NEG);
	else if (f == (qu_unop_f) qu_sys_not ) P0 (NOT);
	else if (f == (qu_unop_f) qu_sys_pos ) P0 (POS);
	else if (f == (qu_unop_f) qu_sys_pop ) P0 (POP);
	else QU_FATAL ("unknown unop");
}



/*
	Get/set variable value.

	!!! VSET does not change the stack.

	!!! VGET does not emit PUSH(1).
	!!! The caller must do it itself.
	!!! This is intentional for clarity.
*/

#define VGET(v)    emit_getvar (v)
#define VSET(v, n) emit_setvar (v, n)

/*
	These are used to opmitize:

	SETLEXICAL x \
	DISCARD       >-  SETLEXICAL x
	GETLEXICAL x /

	SETGLOBAL  x \
	DISCARD       >-  GETGLOBAL x
	GETGLOBAL  x /                    <--- can't do if this is a jump target
*/
#define EMIT_OPTIMIZE_VGET

#ifdef EMIT_OPTIMIZE_VGET
static qu_var VLast    = 0;
static int    VLastofs = 0;

static int emit_is_jump_target (int where)
{
	register char *cs = (char *) CoBytes;
	register int   pc;
	register int   ip = 0;

	while (ip < where)
	{
		pc = cs [ip];
		switch (pc)
		{
			case 0:
				return 0;

			case QU_PC_CATCH:
			case QU_PC_ENTER:
			case QU_PC_JMP:
			case QU_PC_JMPFALSE:
			case QU_PC_JMPFALSEX:
			case QU_PC_JMPTRUE:
			case QU_PC_JMPTRUEX:
			case QU_PC_PARAM:
				if (where == *(long *) (cs + ip + 1))
					return 1;
		}
		ip += PSIZE (pc);
	}
	return 0;
}
#endif


static void emit_getvar (qu_var v)
{
#ifdef EMIT_OPTIMIZE_VGET
	if (VLast == v)
	{
		int i = CoBytes[VLastofs];

		if (i == QU_PC_SETLEXICAL)
		{
			i = VLastofs + L1;
		}
		else if (i == QU_PC_SETLEXICALX
			|| i == QU_PC_SETLEXICALLL
			|| i == QU_PC_SETLEXICALLX)
		{
			i = VLastofs + L2;
		}
		else
		{
			goto nope;

		}

		if (CoBytes[i] == QU_PC_DISCARD && CoNextOfs == i + L0)
		{
			if (!emit_is_jump_target (CoNextOfs))
			{
				CoNextOfs = i;
				VLast = 0;
				return;
			}
		}
	}

   nope:

#endif
	if (qu_var_is_lexical (v))
	{
		P1 (GETLEXICAL, (long) v->index);
	}
	else
	{
		if (qu_var_frozen (v))
			P1 (LITERAL, v->value);
		else
			P1 (GETGLOBAL, v);
	}
}



static void emit_setvar (qu_var v, qu_ast n)
{

#ifdef EMIT_OPTIMIZE_VGET
	VLast = v;
	VLastofs = CoNextOfs;
#endif

	if (qu_var_is_lexical (v))
	{
		/* CHECKME - toxic! we know v->type is marked by Sub */
		if (v->type)
			P2 (SETLEXICALX, v->index, v->type);
		else
			P1 (SETLEXICAL, v->index);
	}
	else
	{
		if (!qu_var_is_rdwr (v))
		{
			if (v->parent != CoMaster && CoModule != qu_var_get_module (v))
				CTHROW (n, "cant modify object");
		}
		if (v->type)
			P1 (SETGLOBALX, v);
		else
			P1 (SETGLOBAL, v);
	}
}



/*
	Emit node which value will be converted into a Boolean.

	!!! This does the PUSH (1)
*/
#define BOOL(n) emit_bool (n)
static void emit_bool (qu_ast n)
{
	switch (n->type)
	{
		case QU_AST_unop:
			E1 (n);
			if (!code_unop_return_bool ((qu_unop_f) n->op))
				P0 (TRUE);
			break;

		case QU_AST_binop:
			E1 (n);
			if (!code_binop_return_bool ((qu_binop_f) n->op))
				P0 (TRUE);
			break;

		case QU_AST_literal:
			if (qu_sys_true (n->thing) == QuTrue)
				P1 (LITERAL, QuTrue);
			else
				P1 (LITERAL, QuFalse);
			PUSH (1);
			break;

		default:
			E1 (n);
			P0 (TRUE);
	}
}



/*============================================================================

	ALIAS

	This is a hack.
	See emit_class.
============================================================================*/

/*HACK - alias */
static void emit_alias (qu_ast n)
{
	return;
}



/*============================================================================

	ARGC

	code                  stack
	--------------------  --------------------
	ARGC                  x


	This node comes from __argc__, e.g:

	sub foo (a, b)
		n = __argc__
		...
	;;

	The number of arguments passed is retrieved from the Frame object,
	i.e. Frame->argc

============================================================================*/

static void emit_argc (qu_ast n)
{
	P0   (ARGC);
	PUSH (1);
}



/*============================================================================

	ANONSUB -> thing = Sub object
	|
	... whatever

	code                  stack
	--------------------  --------------------
	CLOSURE sub           closure-object

============================================================================*/

static void emit_anonsub (qu_ast n)
{
	P1   (CLOSURE, n->thing);
	PUSH (1);
}



/*============================================================================

	ARRAY -> op = number of items
	|
	expr*

	!!! const Array is already converted to LITERAL by Reduce


	code                  stack
	--------------------  --------------------
	expr1                 x1
	expr2                 x1  x2
	...
	exprn                 x1  x2  ...  xn
	ARRAY                 r

	Note
	----
	This instruction never fails because the number of arguments is already
	limited by QU_COMPILE_MAXARGS.

============================================================================*/

static void emit_array (qu_ast n)
{
	long len = n->op;

	EC   (n);
	P1   (ARRAY, len);
	POP  (len);
	PUSH (1);
}



/*============================================================================

	ASSERT -> thing = expr string
	|
	expr

	code
	--------------------
	   {expr}
	   JMPTRUE j
	   LITERAL {thing}
	   LITERAL EAssert
	   THROW 2
	j:

============================================================================*/

static void emit_assert (qu_ast n)
{
	int j;

		LINE (n);
	    E1   (n->child);
	j = P1   (JMPTRUE, __LATER__);
	    POP  (1);
	    P1   (LITERAL, n->thing);
	    P1   (LITERAL, QuEAssertClass);
	    PUSH (2);
	    P1   (THROW, 2);
	    POP  (2); /* there is not STMT on top of ASSERT */
	    SETJ (j);
}



/*============================================================================

	  n
	ASSIGN -> op = operator (qu_sys_plus, qu_sys_mul, etc.), 0 means "="
	 /  \
	a    b

	a = b
	a += b
	a *= b
	etc.

============================================================================*/

static void emit_assign_attr   (qu_ast n);
static void emit_assign_item   (qu_ast n);
static void emit_assign_slice  (qu_ast n);
static void emit_assign_name   (qu_ast n);
static void emit_assign_list   (qu_ast n);

static void emit_assign (qu_ast n)
{
	switch (n->child->type)
	{
		case QU_AST_getattr:  emit_assign_attr  (n); return;
		case QU_AST_getitem:  emit_assign_item  (n); return;
		case QU_AST_getslice: emit_assign_slice (n); return;
		case QU_AST_name:     emit_assign_name  (n); return;
		case QU_AST_list:     emit_assign_list  (n); return;
	}
	CTHROW (n, "invalid assignment");
}



/*============================================================================

	c.d  = b
	c.d += b
	c.d /= b
	etc.

	       n
	    ASSIGN -> op = qu_sys_or, qu_sys_plus, etc.
	    /    \
	   a      b
	GETATTR
	 /  \
	c    d

============================================================================*/

static void emit_assign_attr_direct_literal (qu_ast a, qu_ast b);
static void emit_assign_attr_direct         (qu_ast a, qu_ast b);
static void emit_assign_attr_literal_or     (qu_ast a, qu_ast b);
static void emit_assign_attr_or             (qu_ast a, qu_ast b);
static void emit_assign_attr_literal_and    (qu_ast a, qu_ast b);
static void emit_assign_attr_and            (qu_ast a, qu_ast b);
static void emit_assign_attr_literal_binop  (qu_ast a, qu_ast b, qu_binop_f op);
static void emit_assign_attr_binop          (qu_ast a, qu_ast b, qu_binop_f op);

static void emit_assign_attr (qu_ast n)
{
	qu_ast     a  = n->child;
	qu_ast     b  = a->next;
	int        lt = a->child->next->type == QU_AST_literal;
	qu_binop_f op = (qu_binop_f) n->op;

	if (n->child->child->type == QU_AST_super)
		CTHROW (n, "cant set attribute via super");

	if (op == 0)
	{
		if (lt)
			emit_assign_attr_direct_literal (a, b);
		else
			emit_assign_attr_direct (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_or)
	{
		if (lt)
			emit_assign_attr_literal_or (a, b);
		else
			emit_assign_attr_or (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_and)
	{
		if (lt)
			emit_assign_attr_literal_and (a, b);
		else
			emit_assign_attr_and (a, b);
	}
	else
	{
		if (lt)
			emit_assign_attr_literal_binop (a, b, op);
		else
			emit_assign_attr_binop (a, b, op);
	}
}



/*
	c.d = b

	       n
	    ASSIGN -> op = 0
	    /    \
	   a      b
	GETATTR
	 /  \
	c    LITERAL     name = d->thing
	        d

	code                  stack
	--------------------  ----------------
	{c}                   c
	{b}                   c  b
	SETATTRX name         b

	or

	{b}                   b
	SETATTRSELF name      b
*/
static void emit_assign_attr_direct_literal (qu_ast a, qu_ast b)
{
	qu_ast    c    = a->child;
	qu_ast    d    = c->next;
	qu_string name = d->thing;

	/* checked by Reduce */
	assert (qu_string_is (name));

	if (c->type == QU_AST_self)
	{
		assert (qu_sub_is (CoMaster));

		E1 (b);
		P1 (SETATTRSELF, name);
	}
	else
	{
		E1  (c);
		E1  (b);
		P1  (SETATTRX, name);
		POP (1);
	}
}




/*
	c.d = b

	       n
	    ASSIGN -> op = 0
	    /    \
	   a      b
	GETATTR
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{b}                   c  d  b
	SETATTR               b
*/
static void emit_assign_attr_direct (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;

	E1  (c);
	E1  (d);
	E1  (b);
	P0  (SETATTR);
	POP (2);
}



/*
	c.d ||= b

	       n
	    ASSIGN -> op = qu_sys_or
	    /    \
	   a      b
	GETATTR
	 /  \
	c    LITERAL     name = d->thing
	        d

	code                  stack
	--------------------  --------------------
	    {c}               c
		DUP1              c  c
	    GETATTRX name     c  c  x
		AUTOATTR          c  x
	l1: JMPFALSEX j1      c  x
	    DROP1             x
	l2: JMP j2
	j1: DISCARD           c
	    {b}               c  b
	    SETATTRX name     b
	j2: ...
*/
static void emit_assign_attr_literal_or (qu_ast a, qu_ast b)
{
	qu_ast    c    = a->child;
	qu_ast    d    = c->next;
	qu_string name = d->thing;
	int       l1, l2;

	/* checked by Reduce */
	assert (qu_string_is (name));

	     E1   (c);
	     P0   (DUP1);
	     P1   (GETATTRX, name);
	     PUSH (2);
	     P0   (AUTOATTR);
	     POP  (1);
	l1 = P1   (JMPFALSEX, __LATER__);
	     P0   (DROP1);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     P0   (DISCARD);
	     POP  (1);
	     E1   (b);
	     P1   (SETATTRX, name);
	     POP  (1);
	     SETJ (l2);
}



/*
	c.(d) ||= b

	       n
	    ASSIGN -> op = qu_sys_or
	    /    \
	   a      b
	GETATTR
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	    {c}               c
	    {d}               c  d
	    GETATTR2          c  d  x
	l1: JMPFALSEX j1      c  d  x
	    DROP2             x
	l2: JMP j2
	j1: DISCARD           c  d
	    {b}               c  d  b
	    SETATTR           b
	j2:
*/
static void emit_assign_attr_or (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	int    l1, l2;

	     E1   (c);
	     E1   (d);
	     P0   (GETATTR2);
	     PUSH (1);
	l1 = P1   (JMPFALSEX, __LATER__);
	     P0   (DROP2);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     P0   (DISCARD);
	     POP  (1);
	     E1   (b);
	     P0   (SETATTR);
	     POP  (2);
	     SETJ (l2);
}



/*
	c.d &&= b

	       n
	    ASSIGN -> op = qu_sys_and
	    /    \
	   a      b
	GETATTR
	 /  \
	c    LITERAL     name = d->thing
	        d

	code                  stack
	--------------------  --------------------
	    {c}              c
	    GETATTRX name    c  x
	l1: JMPTRUEX j1      c  x
	    DROP1            x
	l2: JMP j2
	j1: DISCARD          c
	    {b}              c  b
	    SETATTRX name    b
	j2:
*/
static void emit_assign_attr_literal_and (qu_ast a, qu_ast b)
{
	qu_ast    c    = a->child;
	qu_ast    d    = c->next;
	qu_string name = d->thing;
	int       l1, l2;

	/* checked by Reduce */
	assert (qu_string_is (name));

	     E1   (c);
	     P1   (GETATTRX, name);
	     PUSH (1);
	l1 = P1   (JMPTRUEX, __LATER__);
	     P0   (DROP1);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     P0   (DISCARD);
	     POP  (1);
	     E1   (b);
	     P1   (SETATTRX, name);
	     POP  (1);
	     SETJ (l2);
}


/*
	c.(d) &&= b

	       n
	    ASSIGN -> op = qu_sys_and
	    /    \
	   a      b
	GETATTR
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	    {c}               c
	    {d}               c  d
	    GETATTR2          c  d  x
	l1: JMPTRUEX j1       c  d  x
	    DROP2             x
	l2: JMP j2
	j1: DISCARD           c  d
	    {b}               c  d  b
	    SETATTR           b
	j2:
*/
static void emit_assign_attr_and (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	int    l1, l2;

	     E1   (c);
	     E1   (d);
	     P0   (GETATTR2);
	     PUSH (1);
	l1 = P1   (JMPTRUEX, __LATER__);
	     P0   (DROP2);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     P0   (DISCARD);
	     POP  (1);
	     E1   (b);
	     P0   (SETATTR);
	     POP  (2);
	     SETJ (l2);
}



/*
	c.d += b
	c.d -= b
	c.d *= b
	c.d /= b
	c.d &= b
	c.d |= b

	       n
	    ASSIGN -> op = Sys_xxx
	    /    \
	   a      b
	GETATTR
	 /  \
	c    LITERAL     name = d->thing
	        d

	code                  stack
	--------------------  --------------------
	{c}                   c
	GETATTRX  name        c  x
	AUTOATTREX            c  r
	{b}                   c  r  b
	<binop>               c  y
	SETATTRX name         y
*/
static void emit_assign_attr_literal_binop (qu_ast a, qu_ast b, qu_binop_f op)
{
	qu_ast    c    = a->child;
	qu_ast    d    = c->next;
	qu_string name = d->thing;

	/* checked by Reduce */
	assert (qu_string_is (name));

	E1    (c);
	P1    (GETATTRX, name);
	PUSH  (1);
	P0    (AUTOATTREX);
	E1    (b);
	BINOP (op);
	P1    (SETATTRX, name);
	POP   (2);
}



/*
	c.d += b
	c.d -= b
	c.d *= b
	c.d /= b
	c.d &= b
	c.d |= b

	       n
	    ASSIGN -> op = Sys_xxx
	    /    \
	   a      b
	GETATTR
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETATTR2              c  d  x
	AUTOATTREX2           c  d  r
	{b}                   c  d  r  b
	<binop>               c  d  y
	SETATTR               y
*/
static void emit_assign_attr_binop (qu_ast a, qu_ast b, qu_binop_f op)
{
	qu_ast c = a->child;
	qu_ast d = c->next;

	E1    (c);
	E1    (d);
	P0    (GETATTR2);
	PUSH  (1);
	P0    (AUTOATTREX2);
	E1    (b);
	BINOP (op);
	P0    (SETATTR);
	POP   (3);
}



/*----------------------------------------------------------------------------

	       n
	    ASSIGN -> op = qu_sys_or, qu_sys_plus, etc.
	    /    \
	   a      b
	GETITEM
	 /  \
	c    d

----------------------------------------------------------------------------*/

static void emit_assign_item_direct (qu_ast a, qu_ast b);
static void emit_assign_item_or     (qu_ast a, qu_ast b);
static void emit_assign_item_and    (qu_ast a, qu_ast b);
static void emit_assign_item_binop  (qu_ast a, qu_ast b, qu_binop_f op);

static void emit_assign_item (qu_ast n)
{
	qu_ast     a  = n->child;
	qu_ast     b  = a->next;
	qu_binop_f op = (qu_binop_f) n->op;

	if (op == 0)
	{
		emit_assign_item_direct (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_or)
	{
		emit_assign_item_or (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_and)
	{
		emit_assign_item_and (a, b);
	}
	else
	{
		emit_assign_item_binop (a, b, op);
	}
}



/*
	c[d] = b

	       n
	    ASSIGN
	    /    \
	   a      b
	GETITEM
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{b}                   c  d  b
	SETITEM               x

	or

	{c}                   c
	{b}                   c  b
	SETITEMX              x


	or

	         n
	       ASSIGN
	       /    \
	      a      b
	   GETITEM
	    /  \
	   c    d
	NAME
	(lexical var)

	{d}                   d
	{b}                   d b
	SETITEML index        x
*/
static void emit_assign_item_direct (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;

	if (ISLEXICAL (c))
	{
		if (d->type == QU_AST_literal)
		{
			E1 (b);
			P2 (SETITEMLX, ((qu_var) c->thing)->index, d->thing);
		}
		else
		{
			E1  (d);
			E1  (b);
			P1  (SETITEML, ((qu_var) c->thing)->index);
			POP (1);
		}
	}
	else
	{
		if (d->type == QU_AST_literal)
		{
			E1  (c);
			E1  (b);
			P1  (SETITEMX, d->thing);
			POP (1);
		}
		else
		{
			E1  (c);
			E1  (d);
			E1  (b);
			P0  (SETITEM);
			POP (2);
		}
	}
}



/*
	c[d] ||= b

	       n
	    ASSIGN -> op = qu_sys_or
	    /    \
	   a      b
	GETITEM
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	    {c}               c
	    {d}               c  d
	    GETITEM2          c  d  x
	l1: JMPFALSEX j1      c  d  x
	    DROP2             x
	l2: JMP j2
	j1: DISCARD           c  d
	    {b}               c  d  b
	    SETITEM           y
	j2:
*/
static void emit_assign_item_or (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	int    l1, l2;

	     E1   (c);
	     E1   (d);
	     P0   (GETITEM2);
	     PUSH (1);
	l1 = P1   (JMPFALSEX, __LATER__);
	     P0   (DROP2);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     P0   (DISCARD);
	     POP  (1);
	     E1   (b);
	     P0   (SETITEM);
	     POP  (2);
	     SETJ (l2);
}



/*
	c[d] &&= b

	       n
	    ASSIGN -> op = qu_sys_and
	    /    \
	   a      b
	GETITEM
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	    {c}               c
	    {d}               c  d
	    GETITEM2          c  d  x
	j1: JMPTRUEX l1       c  d  x
	    DROP2             x
	j2: JMP l2
	l1: DISCARD           c  d
	    {b}               c  d  b
	    SETITEM           y
	l2:
*/
static void emit_assign_item_and (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	int    l1, l2;

	     E1   (c);
	     E1   (d);
	     P0   (GETITEM2);
	     PUSH (1);
	l1 = P1   (JMPTRUEX, __LATER__);
	     P0   (DROP2);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     P0   (DISCARD);
	     POP  (1);
	     E1   (b);
	     P0   (SETITEM);
	     POP  (2);
	     SETJ (l2);
}



/*
	c[d] += b
	c[d] -= b
	c[d] *= b
	c[d] /= b
	c[d] &= b
	c[d] |= b

	       n
	    ASSIGN -> op = Sys_xxx
	    /    \
	   a      b
	GETITEM
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETITEM2              c  d  x
	{b}                   c  d  x  b
	<binop>               c  d  y
	SETITEM               z
*/
static void emit_assign_item_binop (qu_ast a, qu_ast b, qu_binop_f op)
{
	qu_ast c = a->child;
	qu_ast d = c->next;

	E1    (c);
	E1    (d);
	P0    (GETITEM2);
	PUSH  (1);
	E1    (b);
	BINOP (op);
	P0    (SETITEM);
	POP   (3);
}



/*----------------------------------------------------------------------------

	       n
	    ASSIGN -> op = qu_sys_or, qu_sys_plus, etc.
	    /    \
	   a      b
	GETSLICE
	 / | \
	c  d  e

----------------------------------------------------------------------------*/

static void emit_assign_slice_direct (qu_ast a, qu_ast b);
static void emit_assign_slice_or     (qu_ast a, qu_ast b);
static void emit_assign_slice_and    (qu_ast a, qu_ast b);
static void emit_assign_slice_binop  (qu_ast a, qu_ast b, qu_binop_f op);

static void emit_assign_slice (qu_ast n)
{
	qu_ast     a  = n->child;
	qu_ast     b  = a->next;
	qu_binop_f op = (qu_binop_f) n->op;

	if (op == 0)
	{
		emit_assign_slice_direct (a, b);
	}
	else if (op == qu_sys_or)
	{
		emit_assign_slice_or (a, b);
	}
	else if (op == qu_sys_and)
	{
		emit_assign_slice_and (a, b);
	}
	else
	{
		emit_assign_slice_binop (a, b, op);
	}
}



/*
	c[d, e] = b

	       n
	    ASSIGN
	    /    \
	   a      b
	GETSLICE
	 / | \
	c  d  e

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{e}                   c  d  e  b
	{b}                   c  d  e  b
	SETSLICE              b
*/
static void emit_assign_slice_direct (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;

	E1  (c);
	E1  (d);
	E1  (e);
	E1  (b);
	P0  (SETSLICE);
	POP (3);
}



/*
	This is probably useless but ...

	c[d, e] ||= b

	       n
	    ASSIGN -> op = qu_sys_or
	    /    \
	   a      b
	GETSLICE
	 / | \
	c  d  e

	code                  stack
	--------------------  --------------------
	    {c}               c
	    {d}               c  d
	    {e}               c  d  e
	    GETSLICE2         c  d  e  x
	l1: JMPFALSEX j1      c  d  e  x
	    DROP3             x
	l2: JMP j2
	j1: DISCARD           c  d  e
	    {b}               c  d  e  b
	    SETSLICE          b
	j2:
*/
static void emit_assign_slice_or (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;
	int    l1, l2;

	     E1   (c);
	     E1   (d);
	     E1   (e);
	     P0   (GETSLICE2);
	     PUSH (1);
	l1 = P1   (JMPFALSEX, __LATER__);
	     P0   (DROP3);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     POP  (1);
	     P0   (DISCARD);
	     E1   (b);
	     P0   (SETSLICE);
	     POP  (3);
	     SETJ (l2);
}



/*
	This is probably useless but ...

	c[d, e] &&= b

	       n
	    ASSIGN -> op = qu_sys_and
	    /    \
	   a      b
	GETSLICE
	 / | \
	c  d  e

	code                  stack
	--------------------  --------------------
	    {c}               c
	    {d}               c  d
	    {e}               c  d  e
	    GETSLICE2         c  d  e  x
	l1: JMPTRUEX j1       c  d  e  x
	    DROP3             x
	l2: JMP j2
	j1: DISCARD           c  d  e
	    {b}               c  d  e  b
	    SETSLICE          b
	j2:
*/
static void emit_assign_slice_and (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;
	int    l1, l2;

	     E1   (c);
	     E1   (d);
	     E1   (e);
	     P0   (GETSLICE2);
	     PUSH (1);
	l1 = P1   (JMPTRUEX, __LATER__);
	     P0   (DROP3);
	l2 = P1   (JMP, __LATER__);
	     SETJ (l1);
	     P0   (DISCARD);
	     POP  (1);
	     E1   (b);
	     P0   (SETSLICE);
	     POP  (3);
	     SETJ (l2);
}



/*
	This is probably useless but ...

	c[d, e] += b
	c[d, e] -= b
	c[d, e] *= b
	c[d, e] /= b
	c[d, e] &= b
	c[d, e] |= b

	       n
	    ASSIGN -> op = Sys_xxx
	    /    \
	   a      b
	GETSLICE
	 / | \
	c  d  e

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{e}                   c  d  e
	GETSLICE2             c  d  e  x
	{b}                   c  d  e  x  b
	<binop>               c  d  e  y
	SETSLICE              y
*/
static void emit_assign_slice_binop (qu_ast a, qu_ast b, qu_binop_f op)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;

	E1    (c);
	E1    (d);
	E1    (e);
	P0    (GETSLICE2);
	PUSH  (1);
	E1    (b);
	BINOP (op);
	P0    (SETSLICE);
	POP   (4);
}



/*----------------------------------------------------------------------------

	     n
	   ASSIGN -> op = binary operator
	   /   \
	  a     b
	NAME

----------------------------------------------------------------------------*/

static void emit_assign_name_direct (qu_ast a, qu_ast b);
static void emit_assign_name_or     (qu_ast a, qu_ast b);
static void emit_assign_name_and    (qu_ast a, qu_ast b);
static void emit_assign_name_binop  (qu_ast a, qu_ast b, qu_binop_f op);

static void emit_assign_name (qu_ast n)
{
	qu_ast     a  = n->child;
	qu_ast     b  = a->next;
	qu_binop_f op = (qu_binop_f) n->op;

	if (op == 0)
	{
		emit_assign_name_direct (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_or)
	{
		emit_assign_name_or (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_and)
	{
		emit_assign_name_and (a, b);
	}
	else
	{
		emit_assign_name_binop  (a, b, op);
	}
}



static void emit_assign_name_direct (qu_ast a, qu_ast b)
{
	qu_any v = a->thing;

	if (qu_var_is (v))
	{
		/* this is checked by Reduce */
		assert (!qu_var_is_inst (v));

/*
		if (!qu_var_is_lexical (v))
		{
*/
			/*
				If the value is a literal and the variable is typed
				then check for a valid value.

				Note that SETGLOBAL will still do a run-time check.

				Note also that in Qu lexicals can have any value.  XXX
				Their typed are only checked if it is an function  XXX not anymore
				argument when the function is called.              XXX
			*/

			/*HACK - documentation

				too much pain trying to hack documentation eval module
				(all Sys classes are fake thus default literal values
				will always fail) so ...

				# var x: qu_string = 'foo'
				#        |
				#        +-- will not be sys.qu_string but the `qu_string` class from
				#            the eval module, thus 'foo' is not a `qu_string`
			*/

			if (!qu_module_is_doc (CoModule))
			{
				if (b->type == QU_AST_literal && ((qu_var) v)->type)
				{
					if (!qu_object_check_value (((qu_var) v)->type, b->thing))
						CTHROW (b, "invalid value");
				}
			}
/*
		}
*/

		/*
			code                  stack
			--------------------  --------------------
			{b}                   b
			SETGLOBAL/SETLEXICAL  b
		*/

		if (qu_var_is_lexical (v) && ((qu_var) v)->type == NULL && ISLEXICAL (b))
		{
#ifdef EMIT_OPTIMIZE_VGET
			VLast = v;
			VLastofs = CoNextOfs;
#endif
			P2   (SETLEXICALLL, ((qu_var) v)->index, ((qu_var) b->thing)->index);
			PUSH (1);
		}
		else if (qu_var_is_lexical (v) && ((qu_var) v)->type == NULL && b->type == QU_AST_literal)
		{
#ifdef EMIT_OPTIMIZE_VGET
			VLast = v;
			VLastofs = CoNextOfs;
#endif
			P2   (SETLEXICALLX, ((qu_var) v)->index, b->thing);
			PUSH (1);
		}
		else
		{
			E1   (b);
			VSET (v, a);
		}
	}
	else if (qu_sub_is (v))
	{
		if (qu_sub_is_inst (v))
			CTHROW (a, "cant call instance method this way");

		/*
			sub foo (x = 1)
				...
			;;

			foo = 10   ->  foo (10)


			code                  stack
			--------------------  --------------------
			LITERAL sub           sub
			{b}                   sub  b
			CALL 1                R
		*/

		P1   (LITERAL, v);
		PUSH (1);
		E1   (b);
		P1   (CALL, 1);
		POP  (1);
	}
	else
	{
		CTHROW (a, "invalid assignment");
	}
}



/*
	     n
	   ASSIGN -> op = ||
	   /   \
	  a     b
	NAME

	code                       stack
	-------------------------  --------------------
	    GETGLOBAL/GETLEXICAL   a
	l:  JMPTRUEX j             a
	    DISCARD
	    {b}                    b
	    SETGLOBAL/SETLEXICAL   b
	j: ...
*/
static void emit_assign_name_or (qu_ast a, qu_ast b)
{
	qu_any v = a->thing;

	if (qu_var_is (v))
	{
		int l;

		assert (!qu_var_is_inst (v));

		    VGET  (v);
		    PUSH  (1);
		l = P1    (JMPTRUEX, __LATER__);
		    P0    (DISCARD);
		    POP   (1);
		    E1    (b);
		    VSET  (v, a);
		    SETJ  (l);
	}
	else
	{
		CTHROW (a, "invalid assignment");
	}
}




/*
	a &&= 1

	     n
	   ASSIGN -> op = &&
	   /   \
	  a     b
	NAME

	code                       stack
	-------------------------  --------------------
	    GETGLOBAL/GETLEXICAL   a
	l:  JMPFALSEX j            a
	    DISCARD
	    {b}                    b
	    SETGLOBAL/SETLEXICAL   b
	j: ...
*/
static void emit_assign_name_and (qu_ast a, qu_ast b)
{
	qu_any v = a->thing;

	if (qu_var_is (v))
	{
		int l;

		assert (!qu_var_is_inst (v));

		    VGET  (v);
		    PUSH  (1);
		l = P1    (JMPFALSEX, __LATER__);
		    P0    (DISCARD);
		    POP   (1);
		    E1    (b);
		    VSET  (v, a);
		    SETJ  (l);
	}
	else
	{
		CTHROW (a, "invalid assignment");
	}
}



static void emit_assign_name_binop (qu_ast a, qu_ast b, qu_binop_f op)
{
	qu_any v = a->thing;

	if (qu_var_is (v))
	{
		assert (!qu_var_is_inst (v));

		/*
			code                  stack
			--------------------  --------------------
			GETGLOBAL/GETLEXICAL  x
			{b}                   x  b
			<binop>               y
			SETGLOBAL/SETLEXICAL  y
		*/
		VGET  (v);
		PUSH  (1);
		E1    (b);
		BINOP (op);
		POP   (1);
		VSET  (v, a);
	}
	else
	{
		CTHROW (a, "invalid assignment");
	}
}




/*----------------------------------------------------------------------------

	(x, y, *z) = b

	     n
	  ASSIGN -> op = 0
	   /  \
	  a    b
	LIST

----------------------------------------------------------------------------*/

static void emit_assign_list (qu_ast n)
{
	qu_ast a = n->child;
	qu_ast b = a->next;
	qu_ast c;
	int    i;

	E1 (b);
	for (c = a->child, i = 0; c; c = c->next, i++)
	{
		switch (c->type)
		{
			/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
			case QU_AST_getattr:
			{
				qu_ast d = c->child;
				qu_ast e = d->next;

				if (e->type == QU_AST_literal)
				{
					/* CHECKME - checked by reduce */
					assert (qu_string_is (e->thing));

					/*
						code                  stack
						--------------------  --------------------
						DUP1                  T
						GETITEMX i            x
						{d}                   x  d
						XCHG                  d  x
						SETATTRX name         x
						DISCARD
					*/
					P0   (DUP1);
					PUSH (1);
					P1   (GETITEMX, qu_int_new (i));
					E1   (d);
					P0   (XCHG);
					P1   (SETATTRX, e->thing);
					P0   (DISCARD);
					POP  (2);
				}
				else
				{
					/*
						code                  stack
						--------------------  --------------------
						DUP1                  T
						GETITEMX i            x
						{d}                   x  d
						{e}                   x  d  e
						ROT3                  d  e  x
						SETATTR               x
						DISCARD
					*/
					P0   (DUP1);
					PUSH (1);
					P1   (GETITEMX, qu_int_new (i));
					E1   (d);
					E1   (e);
					P0   (ROT3);
					P0   (SETATTR);
					P0   (DISCARD);
					POP  (2);
				}
				continue;
			}

			/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
			case QU_AST_getitem:
			{
				qu_ast d = c->child;
				qu_ast e = d->next;

				/*
					code                  stack
					--------------------  --------------------
					DUP1                  T
					GETITEMX i            x
					{d}                   x  d
					{e}                   x  d  e
					ROT3                  d  e  x
					SETITEM               z
					DISCARD
				*/
				P0   (DUP1);
				PUSH (1);
				P1   (GETITEMX, qu_int_new (i));
				POP  (1);
				E1   (d);
				E1   (e);
				P0   (ROT3);
				P0   (SETITEM);
				P0   (DISCARD);
				POP  (2);
				continue;
			}

			/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
			case QU_AST_getslice:
			{
				qu_ast d = c->child;
				qu_ast e = d->next;
				qu_ast f = e->next;

				/*
					code                  stack
					--------------------  --------------------
					DUP1                  T
					GETITEMX  i           x
					{d}                   x  d
					{e}                   x  d  e
					{f}                   x  d  e  f
					ROT4                  d  e  f  x
					SETSLICE              x
					DISCARD
				*/
				P0   (DUP1);
				PUSH (1);
				P1   (GETITEMX, qu_int_new (i));
				POP  (1);
				E1   (d);
				E1   (e);
				E1   (f);
				P0   (ROT4);
				P0   (SETSLICE);
				P0   (DISCARD);
				POP  (3);
				continue;
			}

			/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
			case QU_AST_name:
			{
				qu_any v = c->thing;

				if (qu_var_is (v))
				{
					if (qu_var_is_inst (v))
						CTHROW (c, "cant set instance property directly");

					/*
						code                  stack
						--------------------  --------------------
						DUP1                  T
						GETITEMX i            x
						SETLEXICAL/SETGLOBAL  x
						DISCARD
					*/
					P0   (DUP1);
					PUSH (1);
					P1   (GETITEMX, qu_int_new (i));
					VSET (v, c);
					P0   (DISCARD);
					POP  (1);
				}
				else if (qu_sub_is (v))
				{
					if (qu_sub_is_inst (v))
						CTHROW (c, "cant call instance method this way");

					/*
						code                  stack
						--------------------  --------------------
						DUP1                  T
						GETITEMX i            x
						LITERAL sub           x  sub
						XCHG                  sub  x
						CALL                  R
						DISCARD
					*/
					P0   (DUP1);
					PUSH (1);
					P1   (GETITEMX, qu_int_new (i));
					P1   (LITERAL, v);
					P0   (XCHG);
					P1   (CALL, 1);
					P0   (DISCARD);
					POP  (1);
				}
				else
				{
					CTHROW (c, "invalid assignment");
				}
				continue;
			}

			/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
			case QU_AST_literal:
			{
				/* checked by Reduce */
				assert (c->thing == QuNil);
				continue;
			}

			/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
			case QU_AST_eat:
			{
				qu_var v = c->thing;

				assert (qu_var_is (v));
				assert (qu_var_is_local (v) || qu_var_is_lexical (v));
				assert (!qu_var_frozen (v));

				/*
					code                  stack
					--------------------  --------------------
					DUP1                  T
					LITERAL i             T  i
					LITERAL -1            T  i  -1
					GETSLICE              x
					SETLOCAL/SETGLOBAL    x
					DISCARD
				*/

				P0   (DUP1);
				P1   (LITERAL, qu_int_new (i));
				P1   (LITERAL, QuInt_1);
				PUSH (3);
				P0   (GETSLICE);
				VSET (v, c);
				P0   (DISCARD);
				POP  (3);
				break;
			}

			default:
				CTHROW (c, "invalid assignment");
		}
	}
}



/*----------------------------------------------------------------------------

	ATOMIC
	|
	stmt*

	code
	------------------------------
	b:  ENTER z, QU_CODE_BLOCK_ATOMIC
	    stmt*
	    LEAVE
	z:  ...

----------------------------------------------------------------------------*/

static void emit_atomic (qu_ast n)
{
	int b;

	b = P2   (ENTER, __LATER__, QU_CODE_BLOCK_ATOMIC);
	    EC   (n);
	    P0   (LEAVE);
	    SETJ (b);
}



/*============================================================================

	  n
	BINOP -> op = operator (qu_sys_plus, qu_sys_mul, etc.)
	 / \
	a   b

============================================================================*/

static void emit_binop_are    (qu_ast a, qu_ast b);
static void emit_binop_bor    (qu_ast a, qu_ast b);
static void emit_binop_band   (qu_ast a, qu_ast b);
static void emit_binop_or     (qu_ast a, qu_ast b);
static void emit_binop_and    (qu_ast a, qu_ast b);
static void emit_binop_range  (qu_ast a, qu_ast b);
static void emit_binop_xchg   (qu_ast a, qu_ast b);
static void emit_binop_of     (qu_ast a, qu_ast b);
static void emit_binop_concat (qu_ast n);
static void emit_binop_direct (qu_ast a, qu_ast b, qu_binop_f op);

static void emit_binop (qu_ast n)
{
	qu_ast     a  = n->child;
	qu_ast     b  = a->next;
	qu_binop_f op = (qu_binop_f) n->op;

	if (op == (qu_binop_f) qu_sys_bor)
	{
		emit_binop_bor (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_band)
	{
		emit_binop_band (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_or)
	{
		emit_binop_or (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_and)
	{
		emit_binop_and (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_range)
	{
		emit_binop_range (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_xchg)
	{
		emit_binop_xchg (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_of)
	{
		emit_binop_of (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_concat)
	{
		emit_binop_concat (n);
	}
	else if (op == (qu_binop_f) qu_sys_are)
	{
		emit_binop_are (a, b);
	}
	else if (op == (qu_binop_f) qu_sys_binop)
	{
		EC (n);
		assert (qu_int_is (n->thing));
		P1 (BINOP, qu_int_tol (n->thing));
		POP (1);
		return;
	}
	else
	{
		emit_binop_direct (a, b, op);
	}
}



/*----------------------------------------------------------------------------
	a are b  -> result is true or false

	  n
	BINOP -> op = qu_sys_are
	 / \
	a   b

	if a is LIST

	code                  stack
	--------------------  --------------------
	   {b}               b

	   DUP1              b  b
	   {a0}              b  b a0
	   ROT 2             b  a0 b
	   IS                b  X
	   JMPFALSE f        b

	   DUP1              b  b
	   {a1}              b  b  a1
	   ROT 2             b  a1 b
	   IS                b  X
	   JMPFALSE f        b

	   ...
	   DISCARD
	   LITERAL true      true
	g: JMP q

	f: DISCARD
	   LITERAL false     false
	q: ...


	otherwise

	code                  stack
	--------------------  --------------------
	{a}                   a
	{b}                   b
	BINOP QU_T_are        X


----------------------------------------------------------------------------*/

static void emit_binop_are (qu_ast a, qu_ast b)
{
	if (a->type == QU_AST_list)
	{
		int    j [512];
		int    i, n = a->op;
		int    g;
		qu_ast c;

		assert (a->type == QU_AST_list);
		if (n >= sizeof (j))
			CTHROW (a, "too many items in are list (512 max)");

		E1 (b);
		for (i = 0, c = a->child; c; c = c->next, i++)
		{
				   P0   (DUP1);
				   PUSH (1);
				   E1   (c);
				   P0   (XCHG);
				   P0   (IS);
			j[i] = P1   (JMPFALSE, __LATER__);
		}

			P0 (DISCARD);
			P1 (LITERAL, QuTrue);
		g = P1 (JMP, __LATER__);

		for (i = 0; i < n; i++)
			SETJ (j[i]);

		P0 (DISCARD);
		P1 (LITERAL, QuFalse);

		SETJ (g);
	}
	else
	{
		E1  (a);
		E1  (b);
		P1  (BINOP, QU_T_are);
		POP (1);
	}
}



/*----------------------------------------------------------------------------
	a or b  -> result is true or false

	  n
	BINOP -> op = qu_sys_bor
	 / \
	a   b

	code                  stack
	--------------------  --------------------
	    {a}               a
	    [TRUE]            A
	j:  JMPTRUEX g        A
	    DISCARD
	    {b}               b
	    [TRUE]            B
	g:  ...
----------------------------------------------------------------------------*/

static void emit_binop_bor (qu_ast a, qu_ast b)
{
	if (a->type == QU_AST_literal)
	{
		if (qu_sys_true (a->thing) == QuTrue)
		{
			P1 (LITERAL, QuTrue);
			PUSH (1);
		}
		else
		{
			BOOL (b);
		}
	}
	else
	{
		int j;

		    BOOL (a);
		j = P1   (JMPTRUEX, __LATER__);
		    P0   (DISCARD);
		    POP  (1);
		    BOOL (b);
		    SETJ (j);
	}
}



/*----------------------------------------------------------------------------
	a and b  -> result is true or false

	  n
	BINOP -> op = qu_sys_band
	 / \
	a   b

	code                  stack
	--------------------  --------------------
	    {a}               a
	    [TRUE]            A
	j:  JMPFALSEX g       A
	    DISCARD
	    {b}               b
	    [TRUE]            B
	g:  ...
----------------------------------------------------------------------------*/

static void emit_binop_band (qu_ast a, qu_ast b)
{
	if (a->type == QU_AST_literal)
	{
		if (qu_sys_true (a->thing) == QuTrue)
		{
			BOOL (b);
		}
		else
		{
			P1 (LITERAL, QuFalse);
			PUSH (1);
		}
	}
	else
	{
		int j;

		    BOOL (a);
		j = P1   (JMPFALSEX, __LATER__);
		    P0   (DISCARD);
		    POP  (1);
		    BOOL (b);
		    SETJ (j);
	}
}



/*----------------------------------------------------------------------------
	a || b

	  n
	BINOP -> op = qu_sys_or
	 / \
	a   b

	code                  stack
	--------------------  --------------------
	    {a}               a
	j:  JMPTRUEX g        a
	    DISCARD
	    {b}               b
	g:  ...
----------------------------------------------------------------------------*/

static void emit_binop_or (qu_ast a, qu_ast b)
{
	if (a->type == QU_AST_literal)
	{
		if (qu_sys_true (a->thing) == QuTrue)
		{
			P1   (LITERAL, a->thing);
			PUSH (1);
		}
		else
		{
			E1 (b);
		}
	}
	else
	{
		int j;

		    E1   (a);
		j = P1   (JMPTRUEX, __LATER__);
		    P0   (DISCARD);
		    POP  (1);
		    E1   (b);
		    SETJ (j);
	}
}



/*----------------------------------------------------------------------------
	a && b

	  n
	BINOP -> op = qu_sys_and
	 / \
	a   b

	code                  stack
	--------------------  --------------------
	    {a}               a
	j:  JMPFALSEX g       a
	    DISCARD
	    {b}               b
	g:  ...
----------------------------------------------------------------------------*/

static void emit_binop_and (qu_ast a, qu_ast b)
{
	if (a->type == QU_AST_literal)
	{
		if (qu_sys_true (a->thing) == QuTrue)
		{
			E1 (b);
		}
		else
		{
			P1   (LITERAL, a->thing);
			PUSH (1);
		}
	}
	else
	{
		int j;

		    E1   (a);
		j = P1   (JMPFALSEX, __LATER__);
		    P0   (DISCARD);
		    POP  (1);
		    E1   (b);
		    SETJ (j);
	}
}



/*----------------------------------------------------------------------------
	a .. b

	  n
	BINOP -> op = qu_sys_range
	 / \
	a   b

	code                  stack
	--------------------  --------------------
	{a}                   a
	{b}                   b
	RANGE                 R
----------------------------------------------------------------------------*/

static void emit_binop_range (qu_ast a, qu_ast b)
{
	E1  (a);
	E1  (b);
	P0  (RANGE);
	POP (1);
}



/*----------------------------------------------------------------------------
	a of b

	  n
	BINOP -> op = qu_sys_of
	 / \
	a   b

	code                  stack
	--------------------  --------------------
	    {a}               a
		{b}               a b
		OF
----------------------------------------------------------------------------*/

static void emit_binop_of (qu_ast a, qu_ast b)
{
	E1 (a);

	if (b->type == QU_AST_literal)
	{
		P1 (LITERAL, b->thing);
		PUSH (1);
	}
	else if (b->type == QU_AST_list)
	{
		qu_ast c, d;

		assert (b->op == 2);
		c = b->child;
		d = c->next;

		if (c->type != QU_AST_literal || d->type != QU_AST_literal)
			CTHROW (b, "invalid type of");

		P1 (LITERAL, c->thing);
		P1 (LITERAL, d->thing);
		P1 (ARRAY, 2);
		PUSH (1);
	}
	else
	{
		/* should never happen */
		CTHROW (b, "invalid type of");
	}
	P0 (OF);
	POP (1);
}



/*----------------------------------------------------------------------------
	a <-> b  the result must be b

	  n
	BINOP -> op = qu_sys_xchg
	 / \
	a   b
----------------------------------------------------------------------------*/

static void emit_binop_xchg_attr_attr   (qu_ast a, qu_ast b);
static void emit_binop_xchg_attr_item   (qu_ast a, qu_ast b);
static void emit_binop_xchg_attr_slice  (qu_ast a, qu_ast b);
static void emit_binop_xchg_attr_name   (qu_ast a, qu_ast b);

static void emit_binop_xchg_item_attr   (qu_ast a, qu_ast b);
static void emit_binop_xchg_item_item   (qu_ast a, qu_ast b);
static void emit_binop_xchg_item_slice  (qu_ast a, qu_ast b);
static void emit_binop_xchg_item_name   (qu_ast a, qu_ast b);

static void emit_binop_xchg_slice_attr  (qu_ast a, qu_ast b);
static void emit_binop_xchg_slice_item  (qu_ast a, qu_ast b);
static void emit_binop_xchg_slice_slice (qu_ast a, qu_ast b);
static void emit_binop_xchg_slice_name  (qu_ast a, qu_ast b);

static void emit_binop_xchg_name_attr   (qu_ast a, qu_ast b);
static void emit_binop_xchg_name_item   (qu_ast a, qu_ast b);
static void emit_binop_xchg_name_slice  (qu_ast a, qu_ast b);
static void emit_binop_xchg_name_name   (qu_ast a, qu_ast b);


static void emit_binop_xchg (qu_ast a, qu_ast b)
{
	switch (a->type)
	{
		case QU_AST_getattr:
			switch (b->type)
			{
				case QU_AST_name:     emit_binop_xchg_attr_name  (a, b); return;
				case QU_AST_getattr:  emit_binop_xchg_attr_attr  (a, b); return;
				case QU_AST_getitem:  emit_binop_xchg_attr_item  (a, b); return;
				case QU_AST_getslice: emit_binop_xchg_attr_slice (a, b); return;
			}
			break;

		case QU_AST_getitem:
			switch (b->type)
			{
				case QU_AST_name:     emit_binop_xchg_item_name  (a, b); return;
				case QU_AST_getattr:  emit_binop_xchg_item_attr  (a, b); return;
				case QU_AST_getitem:  emit_binop_xchg_item_item  (a, b); return;
				case QU_AST_getslice: emit_binop_xchg_item_slice (a, b); return;
			}
			break;

		case QU_AST_getslice:
			switch (b->type)
			{
				case QU_AST_name:     emit_binop_xchg_slice_name  (a, b); return;
				case QU_AST_getattr:  emit_binop_xchg_slice_attr  (a, b); return;
				case QU_AST_getitem:  emit_binop_xchg_slice_item  (a, b); return;
				case QU_AST_getslice: emit_binop_xchg_slice_slice (a, b); return;
			}
			break;

		case QU_AST_name:
			switch (b->type)
			{
				case QU_AST_name:     emit_binop_xchg_name_name  (a, b); return;
				case QU_AST_getattr:  emit_binop_xchg_name_attr  (a, b); return;
				case QU_AST_getitem:  emit_binop_xchg_name_item  (a, b); return;
				case QU_AST_getslice: emit_binop_xchg_name_slice (a, b); return;
			}
			break;
	}
	CTHROW (a, "invalid operand");
}



/*
	c.d <-> b

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETATTR   NAME
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETATTR2              c  d  x
	GETGLOBAL/GETLEXICAL  c  d  x  b
	XCHG                  c  d  b  x
	SETGLOBAL/SETLEXICAL  c  d  b  x
	DISCARD               c  d  b
	SETATTR               b
*/
static void emit_binop_xchg_attr_name (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_any x = b->thing;

	if (!qu_var_is (x))
		CTHROW (b, "invalid operand");

	E1   (c);
	E1   (d);
	P0   (GETATTR2);
	VGET (x);
	PUSH (2);
	P0   (XCHG);
	VSET (x, b);
	P0   (DISCARD);
	P0   (SETATTR);
	POP  (3);
}



/*
	c.d <-> e.f

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETATTR  GETATTR
	 /  \     /  \
	c    d   e    f

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETATTR2              c  d  x
	{e}                   c  d  x  e
	{f}                   c  d  x  e  f  y
	GETATTR2              c  d  x  e  f  y
	XCHG3                 c  d  y  e  f  x
	SETATTR               c  d  y  x
	DISCARD               c  d  y
	SETATTR               y
*/
static void emit_binop_xchg_attr_attr (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = b->child;
	qu_ast f = e->next;

	E1   (c);
	E1   (d);
	P0   (GETATTR2);
	E1   (e);
	E1   (f);
	P0   (GETATTR2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETATTR);
	P0   (DISCARD);
	P0   (SETATTR);
	POP  (5);
}



/*
	c.d <-> e[f]

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETATTR  GETITEM
	 /  \     /  \
	c    d   e    f

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETATTR2              c  d  x
	{e}                   c  d  x  e
	{f}                   c  d  x  e  f
	GETITEM2              c  d  x  e  f  y
	XCHG3                 c  d  y  e  f  x
	SETITEM               c  d  y  z
	DISCARD               c  d  y
	SETATTR               y
*/
static void emit_binop_xchg_attr_item (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = b->child;
	qu_ast f = e->next;

	E1   (c);
	E1   (d);
	P0   (GETATTR2);
	E1   (e);
	E1   (f);
	P0   (GETITEM2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETITEM);
	P0   (DISCARD);
	P0   (SETATTR);
	POP  (5);
}



/*
	c.d <-> e[f, g]

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETATTR  GETSLICE
	 /  \     / | \
	c    d   e  f  g

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETATTR2              c  d  x
	{e}                   c  d  x  e
	{f}                   c  d  x  e  f
	{g}                   c  d  x  e  f  g
	GETSLICE2             c  d  x  e  f  g  y
	XCHG4                 c  d  y  e  f  g  x
	SETSLICE              c  d  y  x
	DISCARD               c  d  y
	SETATTR               y
*/
static void emit_binop_xchg_attr_slice (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = b->child;
	qu_ast f = e->next;
	qu_ast g = f->next;

	E1   (c);
	E1   (d);
	P0   (GETATTR2);
	E1   (e);
	E1   (f);
	E1   (g);
	P0   (GETSLICE2);
	PUSH (2);
	P0   (XCHG4);
	P0   (SETSLICE);
	P0   (DISCARD);
	P0   (SETATTR);
	POP  (6);
}



/*
	c[d] <-> b

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETITEM   NAME
	 /  \
	c    d

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETITEM2              c  d  x
	GETGLOBAL/GETLEXICAL  c  d  x  b
	XCHG                  c  d  b  x
	SETGLOBAL/SETLEXICAL  c  d  b  x
	DISCARD               c  d  b
	SETITEM               z
*/
static void emit_binop_xchg_item_name (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_any x = b->thing;

	if (!qu_var_is (x))
		CTHROW (b, "invalid operand");

	E1   (c);
	E1   (d);
	P0   (GETITEM2);
	VGET (x);
	PUSH (2);
	P0   (XCHG);
	VSET (x, b);
	P0   (DISCARD);
	P0   (SETITEM);
	POP  (3);
}



/*
	c[d] <-> e.f

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETITEM  GETATTR
	 /  \     /  \
	c    d   e    f

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETITEM2              c  d  x
	{e}                   c  d  x  e
	{f}                   c  d  x  e  f
	GETATTR2              c  d  x  e  f  y
	XCHG3                 c  d  y  e  f  x
	SETATTR               c  d  y  x
	DISCARD               c  d  y
	SETITEM               z
*/
static void emit_binop_xchg_item_attr (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = b->child;
	qu_ast f = e->next;

	E1   (c);
	E1   (d);
	P0   (GETITEM2);
	E1   (e);
	E1   (f);
	P0   (GETATTR2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETATTR);
	P0   (DISCARD);
	P0   (SETITEM);
	POP  (5);
}



/*
	c[d] <-> e.f

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETITEM  GETITEM
	 /  \     /  \
	c    d   e    f

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETITEM2              c  d  x
	{e}                   c  d  x  e
	{f}                   c  d  x  e  f
	GETITEM2              c  d  x  e  f  y
	XCHG3                 c  d  y  e  f  x
	SETITEM               c  d  y  z
	DISCARD               c  d  y
	SETITEM               y
*/
static void emit_binop_xchg_item_item (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = b->child;
	qu_ast f = e->next;

	E1   (c);
	E1   (d);
	P0   (GETITEM2);
	E1   (e);
	E1   (f);
	P0   (GETITEM2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETITEM);
	P0   (DISCARD);
	P0   (SETITEM);
	POP  (5);
}



/*
	c[d] <-> e[f, g]

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETITEM  GETSLICE
	 /  \     / | \
	c    d   e  f  g

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	GETITEM2              c  d  x
	{e}                   c  d  x  e
	{f}                   c  d  x  e  f
	{g}                   c  d  x  e  f  g
	GETSLICE2             c  d  x  e  f  g  y
	XCHG4                 c  d  y  e  f  g  x
	SETSLICE              c  d  y  x
	DISCARD               c  d  y
	SETITEM               z
*/
static void emit_binop_xchg_item_slice (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = b->child;
	qu_ast f = e->next;
	qu_ast g = f->next;

	E1   (c);
	E1   (d);
	P0   (GETITEM2);
	E1   (e);
	E1   (f);
	E1   (g);
	P0   (GETSLICE2);
	PUSH (2);
	P0   (XCHG4);
	P0   (SETSLICE);
	P0   (DISCARD);
	P0   (SETITEM);
	POP  (6);
}



/*
	c[d, e] <-> b

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETSLICE  NAME
	 / | \
	c  d  e

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{e}                   c  d  e
	GETSLICE2             c  d  e  x
	GETGLOBAL/GETLEXICAL  c  d  e  x  b
	XCHG                  c  d  e  b  x
	SETGLOBAL/SETLEXICAL  c  d  e  b  x
	DISCARD               c  d  e  b
	SETSLICE              b
*/
static void emit_binop_xchg_slice_name (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;
	qu_any x = b->thing;

	if (!qu_var_is (x))
		CTHROW (b, "invalid operand");

	E1   (c);
	E1   (d);
	E1   (e);
	P0   (GETSLICE2);
	VGET (x);
	PUSH (2);
	P0   (XCHG);
	VSET (x, b);
	P0   (DISCARD);
	P0   (SETSLICE);
	POP  (4);
}



/*
	c[d, e] <-> f.g

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETSLICE GETATTR
	 / | \    /  \
	c  d  e  f    g

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{e}                   c  d  e
	GETSLICE2             c  d  e  x
	{f}                   c  d  e  x  f
	{g}                   c  d  e  x  f  g
	GETATTR2              c  d  e  x  f  g  y
	XCHG3                 c  d  e  y  f  g  x
	SETATTR               c  d  e  y  x
	DISCARD               c  d  e  y
	SETSLICE              y
*/
static void emit_binop_xchg_slice_attr (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;
	qu_ast f = b->child;
	qu_ast g = f->next;

	E1   (c);
	E1   (d);
	E1   (e);
	P0   (GETSLICE2);
	E1   (f);
	E1   (g);
	P0   (GETATTR2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETATTR);
	P0   (DISCARD);
	P0   (SETSLICE);
	POP  (6);
}



/*
	c[d, e] <-> f.g

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETSLICE GETITEM
	 / | \    /  \
	c  d  e  f    g

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{e}                   c  d  e
	GETSLICE2             c  d  e  x
	{f}                   c  d  e  x  f
	{g}                   c  d  e  x  f  g
	GETITEM2              c  d  e  x  f  g  y
	XCHG3                 c  d  e  y  f  g  x
	SETITEM               c  d  e  y  z
	DISCARD               c  d  e  y
	SETSLICE              y
*/
static void emit_binop_xchg_slice_item (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;
	qu_ast f = b->child;
	qu_ast g = f->next;

	E1   (c);
	E1   (d);
	E1   (e);
	P0   (GETSLICE2);
	E1   (f);
	E1   (g);
	P0   (GETITEM2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETITEM);
	P0   (DISCARD);
	P0   (SETSLICE);
	POP  (6);
}



/*
	c[d, e] <-> f[g, h]

	       n
	     BINOP -> op = qu_sys_xchg
	    /     \
	   a       b
	GETSLICE GETSLICE
	 / | \    / | \
	c  d  e  f  g  h

	code                  stack
	--------------------  --------------------
	{c}                   c
	{d}                   c  d
	{e}                   c  d  e
	GETSLICE2             c  d  e  x
	{f}                   c  d  e  x  f
	{g}                   c  d  e  x  f  g
	{h}                   c  d  e  x  f  g  h
	GETSLICE2             c  d  e  x  f  g  h  y
	XCHG3                 c  d  e  y  f  g  h  x
	SETSLICE              c  d  e  y  x
	DISCARD               c  d  e  y
	SETSLICE              y
*/
static void emit_binop_xchg_slice_slice (qu_ast a, qu_ast b)
{
	qu_ast c = a->child;
	qu_ast d = c->next;
	qu_ast e = d->next;
	qu_ast f = b->child;
	qu_ast g = f->next;
	qu_ast h = g->next;

	E1   (c);
	E1   (d);
	E1   (e);
	P0   (GETSLICE2);
	E1   (f);
	E1   (g);
	E1   (h);
	P0   (GETSLICE2);
	PUSH (2);
	P0   (XCHG4);
	P0   (SETSLICE);
	P0   (DISCARD);
	P0   (SETSLICE);
	POP  (7);
}



/*
	a <-> b

	      n
	    BINOP -> op = qu_sys_xchg
	   /     \
	  a       b
	NAME     NAME

	code                  stack
	--------------------  --------------------
	GETGLOBAL/GETLEXICAL  a
	GETGLOBAL/GETLEXICAL  a  b
	XCHG                  b  a
	SETGLOBAL/SETLEXICAL  b  a
	DISCARD
	SETGLOBAL/SETLEXICAL  b
*/
static void emit_binop_xchg_name_name (qu_ast a, qu_ast b)
{
	qu_any x = a->thing;
	qu_any y = b->thing;

	if (!qu_var_is (x))
		CTHROW (a, "invalid operand");

	if (!qu_var_is (y))
		CTHROW (b, "invalid operand");

	VGET (x);
	VGET (y);
	PUSH (2);
	P0   (XCHG);
	VSET (y, b);
	P0   (DISCARD);
	VSET (x, a);
	POP  (1);
}



/*
	a <-> c.d

	      n
	    BINOP -> op = qu_sys_xchg
	   /     \
	  a       b
	NAME    GETATTR
	         /  \
	        c    d

	code                  stack
	--------------------  --------------------
	GETGLOBAL/GETLEXICAL  a
	{c}                   a  c
	{d}                   a  c  d
	GETATTR2              a  c  d  x
	XCHG3                 x  c  d  a
	SETATTR               x  a
	DISCARD               x
	SETGLOBAL/SETLEXICAL  x
*/
static void emit_binop_xchg_name_attr (qu_ast a, qu_ast b)
{
	qu_any x = a->thing;
	qu_ast c = b->child;
	qu_ast d = c->next;

	if (!qu_var_is (x))
		CTHROW (b, "invalid operand");

	VGET (x);
	E1   (c);
	E1   (d);
	P0   (GETATTR2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETATTR);
	P0   (DISCARD);
	VSET (x, a);
	POP  (3);
}



/*
	a <-> c[d]

	      n
	    BINOP -> op = qu_sys_xchg
	   /     \
	  a       b
	NAME    GETITEM
	         /  \
	        c    d

	code                  stack
	--------------------  --------------------
	GETGLOBAL/GETLEXICAL  a
	{c}                   a  c
	{d}                   a  c  d
	GETITEM2              a  c  d  x
	XCHG3                 x  c  d  a
	SETITEM               x  y
	DISCARD               x
	SETGLOBAL/SETLEXICAL  x
*/
static void emit_binop_xchg_name_item (qu_ast a, qu_ast b)
{
	qu_any x = a->thing;
	qu_ast c = b->child;
	qu_ast d = c->next;

	if (!qu_var_is (x))
		CTHROW (b, "invalid operand");

	VGET (x);
	E1   (c);
	E1   (d);
	P0   (GETITEM2);
	PUSH (2);
	P0   (XCHG3);
	P0   (SETITEM);
	P0   (DISCARD);
	VSET (x, a);
	POP  (3);
}


/*
	a <-> c[d, e]

	      n
	    BINOP -> op = qu_sys_xchg
	   /     \
	  a       b
	NAME   GETSLICE
	         / | \
	        c  d  e

	code                  stack
	--------------------  --------------------
	GETGLOBAL/GETLEXICAL  a
	{c}                   a  c
	{d}                   a  c  d
	{e}                   a  c  d  e
	GETSLICE2             a  c  d  e  x
	XCHG4                 x  c  d  e  a
	SETSLICE              x  a
	DISCARD               x
	SETGLOBAL/SETLEXICAL  x
*/
static void emit_binop_xchg_name_slice (qu_ast a, qu_ast b)
{
	qu_any x = a->thing;
	qu_ast c = b->child;
	qu_ast d = c->next;
	qu_ast e = d->next;

	if (!qu_var_is (x))
		CTHROW (a, "invalid operand");

	VGET (x);
	E1   (c);
	E1   (d);
	E1   (e);
	P0   (GETSLICE2);
	PUSH (2);
	P0   (XCHG4);
	P0   (SETSLICE);
	P0   (DISCARD);
	VSET (x, a);
	POP  (4);
}



/*----------------------------------------------------------------------------

	   n
	 BINOP -> op = qu_sys_concat
	  / \
	 a   b

----------------------------------------------------------------------------*/

static void emit_binop_concat (qu_ast n)
{
	qu_ast     a  = n->child;
	qu_ast     b  = a->next;
	qu_binop_f op = (qu_binop_f) n->op;

	if (   (a->type == QU_AST_binop && (qu_binop_f) a->op == op)
		|| (b->type == QU_AST_binop && (qu_binop_f) b->op == op))
	{
		/*
			optimize a _ b _ c _ ... so that there is not intermediate string
			that will be allocated
		*/

		qu_ast c;

		n->type = QU_AST_cat;
		n->op   = 2;

		while (a->type == QU_AST_binop && (qu_binop_f) a->op == op)
		{
			a = a->child;
			n->child = a;
			c = a->next;
			c->next = b;
			b = c;
			n->op++;
		}
		a = n->child;
		for (c = a->next; c; a = c, c = c->next)
		{
			while (c->type == QU_AST_binop && (qu_binop_f) c->op == op)
			{
				b = c->child;
				a->next = b;
				b->next->next = c->next;
				n->op++;
				c = b;
			}
		}
		E1 (n);
	}
	else
	{
		emit_binop_direct (a, b, op);
	}
}



/*
	BINOP  -> op = qu_sys_plus, qu_sys_minus, etc.
	 / \
	a   b

	code                  stack
	--------------------  --------------------
	{a}                   a
	{b}                   a  b
	<binop>               R
*/
static void emit_binop_direct (qu_ast a, qu_ast b, qu_binop_f op)
{
	if (ISLEXICAL (a))
	{
		if (ISLEXICAL (b))
		{
			BINOPLL (op, a, b);
			PUSH    (1);
			return;
		}
		if (b->type == QU_AST_literal)
		{
			BINOPLX (op, a, b);
			PUSH    (1);
			return;
		}
	}

	E1    (a);
	E1    (b);
	BINOP (op);
	POP   (1);
}



/*============================================================================

	BREAK - op = number of blocks to break

	code                  stack
	--------------------  --------------------
	BREAK n

============================================================================*/

static void emit_break (qu_ast n)
{
	P1   (BREAK, n->op);
	PUSH (1);
}



/*============================================================================

	    CALL -> op = number of arguments
	    /  \
	expr    args

	args: (expr | flat)*

	flat:
		FLAT
		|
		expr

	Note that argument flattening may be repeated any number of times:

	a = [1 2 3]
	foo (*a, x, y, *a, z)

	code                  stack
	--------------------  --------------------
	expr                  expr
	arg                   expr  arg
	.                     expr  arg ...
	.
	.
	CALL                  result

============================================================================*/

static void emit_call (qu_ast n)
{
	/* TODO - check typed arguments */

	qu_ast   c    = n->child;
	int      op   = QU_PC_CALL;
	int      flat = 0;
	int      narg = n->op;
	qu_any   obj  = NULL;
	qu_class k;
	qu_ast   g;

	assert (QU_COMPILE_MAXARGS <= 0xFFF);

	if (c->type == QU_AST_getattr)
	{
		op = QU_PC_CALLATTR;

		if (c->child->next->type == QU_AST_literal)
		{
			/*
				optimizing final qu_object attributes
			*/
			qu_string s = c->child->next->thing;

			assert (qu_string_is (s));
			if (emit_is_obj_optimizable (s))
			{
				qu_sub u = qu_attr_find (QuObjectClass->attrs, s);

				if (u && qu_sub_is (u) && qu_sub_is_final (u))
				{
					E1   (c->child);
					P1   (LITERAL, u);
					PUSH (1);
					goto callargs;
				}
			}
		}
		E1 (c);
	}
	else if (c->type == QU_AST_name)
	{
		obj = c->thing;
		k   = qu_classof (obj);

		if (k == QuClassClass)
		{
			qu_sub f;

			if (qu_class_is_abstract (obj))
			{
				/*
					special cases - Number, Real, Integer and Decimal have a
					new method that return an instance of a real number class
					(Int, Long, etc.)
				*/
				if (!qu_class_is_cast (obj))
					CTHROW (n, "cant instantiate abstract class");
			}

			f = qu_class_search_attr (obj, QU_NAME_new);

			/* if not defined in the class hierarchy, inherit from Object */
			if (f == 0)
				f = __QuObjectNew;

			assert (f && qu_sub_is (f) && (qu_sub_is_class (f) || qu_sub_is_static (f)));

			if (qu_sub_arg_check (f, narg, NULL, 0) == 0)
				CRETHROW (n);

			P1   (LITERAL, obj);
			P1   (LITERAL, f);
			PUSH (2);

			op = QU_PC_CALLATTR;
		}
		else if (k == QuStructClass)
		{
			P1   (LITERAL, QuStructMethodNew);
			PUSH (1);
		}
		else if (k == QuSubClass)
		{
			qu_any master = ((qu_sub) obj)->parent;

			if (qu_class_is (master))
			{
				/* FIXME - this should be an assertion because it is already
					checked by Reduce */
				if (qu_sub_is_inst (obj))
					CTHROW (n, "cant access instance method directly");

#if 0 /* WRONG */
				op = QU_PC_CALLATTR;
				P1   (LITERAL, master);
				P1   (LITERAL, obj);
				PUSH (2);
#else
				P1   (LITERAL, obj);
				PUSH (1);
#endif
			}
			else
			{
				/*
					FIXME - currently we do not allow a child sub calling
					a nested parent sub because this generally means bad
					design. This eliminates the need to create closures.

					If we have too many complains then we will have to
					create a closure.
				*/

				if (qu_sub_is (master) && !qu_sub_isroot ((qu_sub) master))
				{
					if (qu_sub_isparent ((qu_sub) master, (qu_sub) obj)
							&& !qu_sub_isroot ((qu_sub) obj))
						CTHROW (n, "cant call nested parent sub");
				}
				P1   (LITERAL, obj);
				PUSH (1);
			}
		}
		else if (k == QuVarClass)
		{
			if (qu_var_is_inst (obj))
				CTHROW (n, "cant call instance property this way");

			VGET (obj);
			PUSH (1);
		}
		else if (k == QuModuleClass)
		{
			/* for convenience if the module contains a class  or
				function named as the module */

			qu_any x = qu_module_getattr (obj, qu_file_name (qu_file_strip (((qu_module) obj)->name)));

			if (x && (qu_class_is (x) || qu_sub_is (x)))
				obj = x;

			/* for the optimizer */
			c->type  = QU_AST_name;
			c->thing = obj;

			P1   (LITERAL, obj);
			PUSH (1);
		}
		else
		{
			CTHROW (n, "object not callable");
		}
	}
	else
	{
		E1 (c);
	}

  callargs:

	for (c = c->next; c; c = c->next)
	{
		if (c->type == QU_AST_flat)
		{
			flat = 1;
			g = c->child;
			switch (g->type)
			{
				case QU_AST_name:
					k = qu_classof (g->thing);
					if (k != QuVarClass && k != QuSubClass)
						CTHROW (g, "cant flatten object");
					break;

				case QU_AST_literal:
					if (!qu_array_is (g->thing))
						CTHROW (g, "cant flatten object");
					break;
			}
			E1 (g);
			P0 (FLAT);
		}
		else
		{
			E1 (c);
		}
	}

	/* Check for valid number of arguments, if possible */
	if (obj)
	{
		k = qu_classof (obj);
		if (k == QuClassClass)
		{
			/* already checked above */
		}
		else if (k == QuSubClass)
		{
			if (flat)
			{
				/*
					sub foo
						...
					;;
					a = []
					foo (*a)
						-> is not allowed even if the flattened argument
							turns out to be empty

					This saves a check at run-time, i.e. qu_frame_new_flat does
					not have to recheck this.
				*/
				if (((qu_sub) obj)->argc == 0 && narg != 0)
				{
					if (qu_sub_arg_check ((qu_sub) obj, 1, NULL, 0) == 0)
						CRETHROW (n);
				}
			}
			else
			{
				if (qu_sub_arg_check ((qu_sub) obj, narg, NULL, 0) == 0)
					CRETHROW (n);
			}
		}
	}
	if (op == QU_PC_CALLATTR)
	{
		if (flat)
			P1 (CALLATTRFLAT, narg);
		else
			P1 (CALLATTR, narg);
		POP (narg + 1);
	}
	else
	{
		if (flat)
			P1 (CALLFLAT, narg);
		else
			P1 (CALL, narg);

		POP (narg);
	}
}



/*============================================================================

	CAT -> op = number of arguments
	|
	expr+

	code                  stack
	--------------------  --------------------
	expr                  expr
	.                     expr ...
	.
	XCAT n                result

============================================================================*/

static void emit_cat (qu_ast n)
{
	EC  (n);
	P1  (XCAT, n->op);
	POP (n->op - 1);
}




/*============================================================================

	CLASS -> thing = class object
	|
	COMPOUND          !!! ISA and MIXIN nodes removed by Reduce
	|
	stmt*

	code
	------------------
	stmt*


	Note
	----
	We don't generate a Code object for classes.
	The code will be executed together with the module code.

	Side-effect: in the main program, the code might not be initialized.

	WRONG
	-----

		# main program

		x = Foo ()
		println (x.bar)    #--> nil

		class Foo
			var class bar = something ()
		;;

		sub something
			return 10
		;;


	CORRECT
	-------

		# main program

		class Foo
			var class bar = something ()
		;;

		sub something
			return 10
		;;

		x = Foo ()
		println (x.bar)

============================================================================*/

static void emit_class (qu_ast n)
{
	qu_class k = n->thing;

	assert (qu_class_is (k));

	code_enter_master (k);

	assert (n->child->type == QU_AST_compound);
	E1 (n->child);

	/* FIXME - this should probably be done by compile.c at the end of compilation
		by enddecl-ing superclasses first recursively, otherwise complex module
		cyclic dependency might fail.

		use Gtk
		use Gtk.Gdk
		use Gtk.Gdk.Rgb    -> but this doesn't
		use Gtk.Gdk.Window
		use Gtk.Gdk.Rgb    -> this causes Window to not have __inst etc. from GObject
	*/
	if (qu_class_enddecl (k) == 0)
		CRETHROW (n);

	/*HACK - now we can process ALIAS */
	{
		qu_ast    c, g;
		qu_class  owner;
		qu_string name;
		qu_any    x;

		for (c = n->child->child; c; c = c->next)
		{
			if (c->type != QU_AST_alias)
				continue;

			name = c->thing;
			assert (name && qu_string_is (name));

			g = c->child;
			if (g->child)
				CTHROW (c, "undeclared symbol");

			if (qu_string_is (g->thing))
			{
				x = qu_class_search_attr (k, g->thing);
				if (x == 0)
					CTHROW (c, "undeclared symbol");
			}
			else
			{
				assert (g->type == QU_AST_literal);
				x = g->thing;
			}

			switch (qu_classid (x))
			{
				case QU_CLASS_ID_Sub:
					if (qu_sub_is_local (x))
						CTHROW (c, "cant alias a local attribute");
					if (qu_classof (((qu_sub) x)->parent) != QuModuleClass)
					{
						owner = qu_sub_get_class (x);
						if (!(owner == k || owner == QuObjectClass || owner == QuClassClass))
						{
							if (!qu_class_is_super_or_mixin (k, owner))
								CTHROW (c, "cant alias foreign class attribute");
						}
					}
					break;

				case QU_CLASS_ID_Var:
					if (qu_var_is_local (x) || qu_var_is_lexical (x))
						CTHROW (c, "cant alias a local attribute");
					if (qu_classof (((qu_var) x)->parent) != QuModuleClass)
					{
						owner = qu_var_get_class (x);
						if (!(owner == k || owner == QuObjectClass || owner == QuClassClass))
						{
							if (!qu_class_is_super_or_mixin (k, owner))
								CTHROW (c, "cant alias foreign class attribute");
						}
					}
					break;

				default:
					CTHROW (c, "invalid alias");
			}

			/*HACK - documentation
				so it doesn't appear twice in the doc and alias work in the doc too */
			if (!qu_module_is_doc (CoModule))
				qu_attr_set (k->attrs, name, x);
		}
	}
	code_leave_master ();
}



/*============================================================================

	COMPOUND
	|
	stmt*


	code
	------------------------------
	stmt*

============================================================================*/

static void emit_compound (qu_ast n)
{
	/* so that ctrl-c works */
	if (n->child == 0)
	{
		LINE (n);
		P1   (LITERAL, QuNil);
		PUSH (1);
		P0   (DISCARD);
		POP  (1);
	}
	else
	{
		EC (n);
	}
}



/*============================================================================

	CONTINUE - op = number of blocks to break

	code
	------------------------------
	CONTINUE n

============================================================================*/

static void emit_continue (qu_ast n)
{
	P1   (CONTINUE, n->op);
	PUSH (1);
}



/*============================================================================

	DICT -> op = number of items (8 means 4 keys/value items)
	|
	expr*

	code                   stack
	---------------------  ---------------------
	expr1                 expr1
	expr2                 expr1 expr2
	...                   expr1 expr2 ...
	DICT    n             dict

	Note
	----
	This instruction never fails because the number of arguments is already
	limited by COMPILE_MAXARGS.

============================================================================*/

static void emit_arg (qu_ast n)
{
	long len = n->op;

	assert ((len & 1) == 0);

	EC   (n);
	P1   (ARG, len);
	POP  (len);
	PUSH (1);
}


static void emit_dict (qu_ast n)
{
	long len = n->op;

	assert ((len & 1) == 0);

	EC   (n);
	P1   (DICT, len);
	POP  (len);
	PUSH (1);
}



/*----------------------------------------------------------------------------

	DO
	|
	COMPOUND | IF | SWITCH | LOOP | WHILE | FOR
	|
	stmt*

	code
	------------------------------
	b:  ENTER z, QU_CODE_BLOCK_DO
	    stmt*
	    LEAVE
	z:  ...

----------------------------------------------------------------------------*/

static void emit_do (qu_ast n)
{
	int b;

	b = P2   (ENTER, __LATER__, QU_CODE_BLOCK_DO);
	    EC   (n);
		P0   (LEAVE);
		SETJ (b);
}



/*----------------------------------------------------------------------------

	EXTERN -> thing: [Module object, name, spec]

	code
	------------------------------
	EXTERN Module object
	LITERAL name
	LITERAL spec

	This is a special instruction. If the Module object is turned into
	a Method then skip the LITERAL instructions and push that method instead.

----------------------------------------------------------------------------*/

static void emit_extern (qu_ast n)
{
	qu_array  a = n->thing;
	qu_module mod  = a->objs[0];
	qu_string name = a->objs[1];
	qu_string spec = a->objs[2];

	assert (qu_module_is (mod));
	P1 (EXTERN, mod);
	P1 (LITERAL, name);
	P1 (LITERAL, spec);
	PUSH (1);
}



/*============================================================================

	FOR
	|
	(NAME | LIST) - expr - COMPOUND
	        |              |
	        NAME+ - [EAT]  stmt*


	code                            stack
	------------------------------  --------------------
	    expr                        expr
	b:  ENTER z,  QU_CODE_BLOCK_FOR | QU_CODE_BLOCK_RANGE
	x:  ITERNEXT var|varlist
	    stmt*
	    JMP x
	    LEAVE
	z:  ...

============================================================================*/

static void emit_for (qu_ast n)
{
	qu_ast c = n->child;
	int    b, x, k = QU_CODE_BLOCK_FOR;
	qu_any o;

	if (c->type == QU_AST_name)
	{
		o = c->thing;

		assert (qu_var_is (o));
		assert (!qu_var_frozen (o));

		if (qu_var_is_lexical (o))
			o = qu_int_new (((qu_var) o)->index);
	}
	else
	{
		qu_ast g;

		assert (c->type == QU_AST_list);
		assert (c->op != 0);

		o = qu_array_new (c->op);
		for (g = c->child; g; g = g->next)
		{
			qu_var v = g->thing;

			assert (qu_var_is (v));
			assert (!qu_var_frozen (v));

			if (g->type == QU_AST_eat)
			{
				qu_array_forlast_set (o);
				assert (g->next == NULL);
			}

			if (qu_var_is_lexical (v))
				qu_array_push_quick (o, qu_int_new (v->index));
			else
				qu_array_push_quick (o, v);
		}

		qu_array_freeze (o);
		qu_set_push (CoLiterals, o);
	}

	/* for the optimizer */
	c->thing = o;

	c = c->next;

	LINE (c);

	if (c->type == QU_AST_literal)
	{
		P1 (LITERAL, c->thing);
		PUSH (1);
	}
	else if (c->type == QU_AST_list)
	{
		if (c->op < 2)
			CTHROW (c, "need at least two arguments");

		k = QU_CODE_BLOCK_RANGE;
		EC (c);
		if (c->op < 3)
		{
			P1   (LITERAL, QuInt1);
			PUSH (1);
		}
	}
	else
	{
		E1 (c);
	}

	if (k == QU_CODE_BLOCK_FOR)
	{
		b = P2  (ENTER, __LATER__, QU_CODE_BLOCK_FOR);
		    POP (1);
	}
	else
	{
		b = P2  (ENTER, __LATER__, QU_CODE_BLOCK_RANGE);
		    POP (3);
	}
	x = P1   (ITERNEXT, o);
	    E1   (c->next);

    P1   (JMP, x);
    P0   (LEAVE);
	SETJ (b);
}



/*============================================================================

	FRAME

	code                  stack
	--------------------  --------------------
	FRAME                 current-frame-object

============================================================================*/

static void emit_frame (qu_ast n)
{
	P0   (FRAME);
	PUSH (1);
}



/*============================================================================

	GETATTR -> op = 1: followed by call(), 0:not
	 /  \
	a    b

	code                  stack
	--------------------  --------------------
	a                     a
	b                     a  b
	GETATTR               a  x

	Note
	----
	Notice the unusual stack arrangement. This is because we have GETATTR
	is followed by AUTOATTR or ATTRCALL thus we need the 'self' object
	passed to the method.

============================================================================*/

static void emit_getattr_super (qu_ast a, qu_ast b, int op);
static void emit_getattr_any   (qu_ast a, qu_ast b, int op);

static void emit_getattr (qu_ast n)
{
	qu_ast a = n->child;
	qu_ast b = a->next;

	switch (a->type)
	{
		case QU_AST_super:
			emit_getattr_super (a, b, n->op);
			break;

		default:
			emit_getattr_any (a, b, n->op);
	}
}



/*
	    GETATTR -> op = 1: followed by call(), 0:not
	     /  \
	SUPER    b = (LITERAL | expr)


	SUPER -> thing = Class object
*/
static void emit_getattr_super (qu_ast a, qu_ast b, int op)
{
	qu_class c, u;

	if (qu_array_is (a->thing))
	{
		qu_array  v = a->thing;
		qu_string s;

		assert (v->len == 2);
		c = v->objs[0];
		s = v->objs[1];
		assert (qu_class_is (c));
		assert (qu_string_is (s));
		u = qu_class_find_class (v->objs[0], v->objs[1]);
		if (u == NULL)
			CTHROW (a, "no such super or mixin class");
		a->thing = c;
	}
	else
	{
		c = (qu_class) a->thing;
		assert (c && qu_class_is (c));
		u = c->super;
		if (u == 0)
			CTHROW (a, "class has no super class");
	}


	if (b->type == QU_AST_literal)
	{
		qu_any x;

		assert (qu_string_is (b->thing));

		x = qu_attr_find (u->attrs, b->thing);

		if (x == 0 && !qu_module_is_doc (CoModule)) /* documentation hack */
			CTHROW (b, "super class has no such attribute");

		if (x)
		{
			if (qu_sub_is (x))
			{
				assert (qu_sub_is (CoMaster));
				if (qu_sub_is_class (CoMaster))
				{
					if (qu_sub_is_inst (x))
						CTHROW (b, "invalid call");

					P0   (SELF);
					P1   (LITERAL, x);
					PUSH (2);
					goto done;
				}
				else if (qu_sub_is_inst (CoMaster))
				{
					P0   (SELF);
					P1   (LITERAL, x);
					PUSH (2);
					goto done;
				}
				else
				{
					CTHROW (b, "super class has no such attribute");
				}
			}
		}

		P2   (SUPERX, u, b->thing);
		PUSH (2);
	}
	else
	{
		E1   (b);
		P1   (SUPER, u);
		PUSH (1);
	}

  done:

	if (op == 0)
	{
		P0  (AUTOATTR);
		POP (1);
	}
}



/*
	GETATTR -> op = 1: followed by call(), 0:not
	 /  \
	a    b = (LITERAL | expr)
*/
static void emit_getattr_any (qu_ast a, qu_ast b, int op)
{
	if (b->type == QU_AST_literal)
	{
		qu_string s = b->thing;
		qu_any    x;

		assert (qu_string_is (s));

		if (a->type == QU_AST_name)
		{
			if (qu_class_is (a->thing))
			{
				/* optimize class attribute */

				x = qu_class_search_attr (a->thing, s);
#if 0
				if (x == 0)
					CTHROW (a, "class has no such public attribute");
#else
				if (x == 0)
					goto nope; /* leave it to runtime */
#endif

				assert (qu_sub_is (x) || qu_var_is (x));
				if (qu_symbol_is_inst (x))
					CTHROW (a, "cant access instance attribute directly");

				if (qu_sub_is (x))
				{
					P1   (LITERAL, a->thing);
					P1   (LITERAL, x);
					PUSH (2);
					goto done;
				}
			}
			else if (qu_var_is (a->thing))
			{
				/* optimize typed variable */

				qu_var v = a->thing;

				if (v->type && qu_class_is (v->type))
				{
					x = qu_class_search_attr (v->type, s);
					/* can only do if attribute is method, for properties we
						will need to use slots :-( */
					if (x && qu_sub_is (x))
					{
						if (qu_sub_is_class (x))
						{
							P1   (LITERAL, qu_sub_get_class (x));
							P1   (LITERAL, x);
							PUSH (2);
						}
						else
						{
							VGET (v);
							P1   (LITERAL, x);
							PUSH (2);
						}
						goto done;
					}
				}
			}
		}

		/*
			optimizing final qu_object attributes
		*/
		x = 0;
		if (emit_is_obj_optimizable (s))
			x = qu_attr_find (QuObjectClass->attrs, s);

		if (x && qu_sub_is (x) && qu_sub_is_final (x))
		{
			E1   (a);
			P1   (LITERAL, x);
			PUSH (1);
		}
		else
		{
			if (a->type == QU_AST_self)
			{
				P1   (GETATTRSELF, s);
				PUSH (2);
			}
			else
			{
				E1   (a);
				P1   (GETATTRX, s);
				PUSH (1);
			}
		}
	}
	else
	{

  nope:
		E1 (a);
		E1 (b);
		P0 (GETATTR);
	}

  done:

	if (op == 0)
	{
		P0  (AUTOATTR);
		POP (1);
	}
}



/*============================================================================

	GETITEM
	 /  \
	a    b

	code                  stack
	--------------------  --------------------
	a                     a
	b                     a  b
	GETITEM               x

	or

	a                     a
	GETITEMX              x

	or

		  GETITEM
		    / \
		   a   b
		NAME
		(lexical var)

	b                     b
	GETITEML idx          x

	or

		  GETITEM
		    / \
		   a   b
		NAME   LITERAL
		(lexical var)

	GETITEMLX idx, index  x

============================================================================*/

static void emit_getitem (qu_ast n)
{
	qu_ast a = n->child;
	qu_ast b = a->next;

	if (ISLEXICAL (a))
	{
		if (b->type == QU_AST_literal)
		{
			P2   (GETITEMLX, ((qu_var) a->thing)->index, b->thing);
			PUSH (1);
		}
		else
		{
			E1 (b);
			P1 (GETITEML, ((qu_var) a->thing)->index);
		}
	}
	else
	{
		E1 (a);

		if (b->type == QU_AST_literal)
		{
			P1  (GETITEMX, b->thing);
		}
		else
		{
			E1  (b);
			P0  (GETITEM);
			POP (1);
		}
	}
}



/*============================================================================

	GETSLICE
	 / | \
	a  b  c

	code                  stack
	--------------------  --------------------
	a                     a
	b                     a  b
	c                     a  b  c
	GETSLICE              x

============================================================================*/

static void emit_getslice (qu_ast n)
{
	EC  (n);
	P0  (GETSLICE);
	POP (2);
}



/*============================================================================

	htopen   (thing: String, op:close|linebreak)
	htclose  (thing: String, op:linebreak)
	htentity (thing: String)

	code
	-----------------------
	HTOPEN   (String, op)
	HTCLOSE  (String, op)
	HTENTITY (String)

============================================================================*/

static void emit_htopen (qu_ast n)
{
	P2 (HTOPEN, n->thing, n->op);
}



static void emit_htclose (qu_ast n)
{
	P2 (HTCLOSE, n->thing, n->op);
}



static void emit_htentity (qu_ast n)
{
	P1 (HTENTITY, n->thing);
}


/*============================================================================

	IF
	|
	expr - COMPOUND -  [IF*] - [COMPOUND (else)]
	       |                    |
	       stmt*                stmt*

	code
	-----------------------
	    {expr}
	f:  JMPFALSE/JMPTRUE k
	    {stmt*}
	j:  JMP z
	k:  {expr}
	    JMPFALSE/JMPTRUE x
	    {stmt*}
	    JMP z
	x:  {stmt*}
	z:  ...

============================================================================*/

static void emit_if (qu_ast n)
{
	qu_ast c = n->child, d, g;
	int    f, j;

	LINE (n);

	if (c->type == QU_AST_unop && (qu_unop_f) c->op == (qu_unop_f) qu_sys_not)
	{
			E1  (c->child);
		f = P1  (JMPTRUE, __LATER__);
			POP (1);
	}
	else
	{
			E1  (c);
		f = P1  (JMPFALSE, __LATER__);
			POP (1);
	}

	c = c->next;
	assert (c->type == QU_AST_compound);

	    E1   (c);
	j = P1   (JMP, __LATER__);
	    SETJ (f);

	d = c->next;
	for (c = d; c && c->type == QU_AST_if; c = c->next)
	{
		g = c->child;
		if (g->type == QU_AST_unop && (qu_unop_f) g->op == (qu_unop_f) qu_sys_not)
		{
				E1  (g->child);
			f = P1  (JMPTRUE, __LATER__);
				POP (1);
		}
		else
		{
				E1  (g);
			f = P1  (JMPFALSE, __LATER__);
				POP (1);
		}
		g = g->next;

		assert (g->type == QU_AST_compound);

		        E1   (g);
		c->op = P1   (JMP, __LATER__);
		        SETJ (f);
	}

	if (c)
	{
		EF (c);
#ifdef EMIT_OPTIMIZE_VGET
	VLast = 0;
#endif
	}

    SETJ (j);
	for (c = d; c && c->type == QU_AST_if; c = c->next)
		SETJ (c->op);
}



/*============================================================================

	IMPORT -> thing = array of Module objects or one Module object

============================================================================*/

static void emit_import (qu_ast n)
{
	qu_any o = n->thing;

	if (qu_array_is (o))
	{
		qu_any *p = ((qu_array) o)->objs;
		long l = ((qu_array) o)->len;
		while (l--)
		{
			o = *p++;
			assert (qu_module_is (o));
			if (!qu_module_initialized (o))
			{
				P1   (INITMOD, o);
				PUSH (1);
				POP  (1);
			}
		}
	}
	else
	{
		assert (qu_module_is (o));

		if (!qu_module_initialized (o))
		{
			P1   (INITMOD, o);
			PUSH (1);
			POP  (1);
		}
	}

	/* to avoid duplicates since QU_AST_import is under QU_AST_todo */
	n->type  = QU_AST_nop;
	n->child = 0;
}



/*============================================================================

	LITERAL -> thing = value

	code                  stack
	--------------------  --------------------
	LITERAL value         value

============================================================================*/

static void emit_literal (qu_ast n)
{
	P1   (LITERAL, n->thing);
	PUSH (1);
}



/*============================================================================

	LOCAL -> thing = Var object
	|
	[expr]      !!! the [SYMBOL] node removed by Reduce.c

	code                  stack
	--------------------  --------------------
	expr                  expr
	SETLEXICAL            expr
	DISCARD

============================================================================*/

static void emit_local (qu_ast n)
{
	qu_var v = n->thing;

	assert (qu_var_is (v) && (qu_var_is_local (v) || qu_var_is_lexical (v)));
	assert (n->child);

	E1   (n->child);
	VSET (v, n);
	P0   (DISCARD);
	POP  (1);
}



/*============================================================================

	LOOP
	|
	[expr] - COMPOUND - [expr]
	         |
	         stmt*

	code
	------------------------------
	f:  expr
	    JMPFALSE z
	b:  ENTER z, QU_CODE_BLOCK_LOOP
	x:  stmt*
	    expr
	    JMPFALSE x
	l:  LEAVE
	z:  ...

============================================================================*/

static void emit_loop (qu_ast n)
{
	qu_ast c = n->child;
	int    f = 0, b, x;

	if (c->type != QU_AST_compound)
	{
		if (c->type == QU_AST_unop && (qu_unop_f) c->op == (qu_unop_f) qu_sys_not)
		{
			    E1  (c->child);
			f = P1  (JMPTRUE, __LATER__);
			    POP (1);
		}
		else
		{
			    E1  (c);
			f = P1  (JMPFALSE, __LATER__);
			    POP (1);
		}
		c = c->next;
	}

	b = P2 (ENTER, __LATER__, QU_CODE_BLOCK_LOOP);
	x = CoNextOfs;
	    E1 (c);

	c = c->next;
	if (c)
	{
		E1   (c);
		P1   (JMPFALSE, x);
		POP  (1);
	}
	else
	{
		P1 (JMP, x);
	}
	P0   (LEAVE);
	SETJ (b);

	if (f)
		SETJ (f);
}



/*============================================================================

	ME

	code                  stack
	--------------------  --------------------
	LITERAL me            me

============================================================================*/

static void emit_me (qu_ast n)
{
	P1   (LITERAL, CoModule);
	PUSH (1);
}



/*============================================================================

	METHOD -> op = C function address
	 /  \
	a    b may be a list

============================================================================*/

static void emit_method (qu_ast n)
{
	qu_ast    a = n->child;
	qu_ast    b = a->next;
	qu_subn_f f = (qu_subn_f) n->op;
	long      k;

	E1 (a);
	if (b->type == QU_AST_list)
	{
		k = b->op;
		EC (b);
	}
	else
	{
		k = 1;
		E1 (b);
	}

	if (f == (qu_subn_f) qu_sys_format)
	{
		P1 (FORMAT, k);
	}
	else
	{
		QU_FATAL ("unknown method");
	}
	POP (k);
}




/*============================================================================

	NAME -> thing = Module, Class, Sub or Var object
	        op    = 1:followed by call, 0: not

	code                  stack
	--------------------  --------------------
	LITERAL x             x
	[CALL]                y

============================================================================*/

static void emit_name (qu_ast n)
{
	qu_any   x = n->thing;
	qu_class c = qu_classof (x);

	if (c == QuSubClass)
	{
		assert (!qu_sub_is_inst (x));

		P1   (LITERAL, x);
		PUSH (1);

		/*if (n->op == 0)*/
		if (!qu_sub_noauto (x))
			P1 (CALL, 0);
	}
	else if (c == QuVarClass)
	{
		assert (!qu_var_is_inst (x));
		VGET (x);
		PUSH (1);
	}
	else
	{
		P1   (LITERAL, x);
		PUSH (1);
	}
}



/*============================================================================

	NOP

	This node might be produced by Reduce to "eliminate" the node.

============================================================================*/

static void emit_nop (qu_ast n)
{
	return;
}



/*============================================================================

	PARAM -> thing = Var object
	|
	[expr]                 !!! the [SYMBOL] node is removed by Reduce


		code                         stack
		--------------------------   -------------
		j:  PARAM position, g
			{expr}                   x
			SETLEXICAL               x
			DISCARD
		g:  ...


	PARAM does this: if the number of arguments passed is more than 'position'
	then jump to 'g', otherwise set the default value

============================================================================*/

static void emit_param (qu_ast n)
{
	qu_ast c = n->child;

	if (c)
	{
		qu_var v = n->thing;
		qu_sub u = (qu_sub) v->parent;
		int j;

		assert (qu_var_is (v));
		assert (qu_var_is_lexical (v));

		/*
			REMEMBER!
			interp.c does this:
				if argc > x then jump
				where x = v->index - u->first

			sub outer (...
				sub inner (a, b, c = 1)
				          #6 #7 #8
			argc = 3
			if 3 >= #8 - #6 -> c passed by caller, otherwise not
		*/

		j = P2   (PARAM, __LATER__, v->index - u->first);
			E1   (c);
			VSET (v, n);
			P0   (DISCARD);
			POP  (1);
			P0   (NOP); /* for OPTIMIZE_VGET otherwise we get a wrong jump */
			SETJ (j);
	}
}



/*============================================================================

	PASS

	code                   stack
	--------------------   --------------------
	PASS                   nil

============================================================================*/

static void emit_pass (qu_ast n)
{
	P0   (PASS);
	PUSH (1);
}



/*============================================================================

	PRINT -> thing = object, op = 1:linebreak

	code                  stack
	--------------------  --------------------
	PRINT object

	!!! This is a full statement by its own -> it does not change the stack

============================================================================*/

static void emit_print (qu_ast n)
{
	P1 (PRINT, n->thing);
	if (n->op)
		P1 (PRINT, qu_string_byte ('\n'));
}



/*============================================================================

	PRINTEX -> op = 0:print 1:html escape

	code                  stack
	--------------------  --------------------
	EXPR                  expr
	PRINTTOP/PRINTHTML
	...

	or

	PRINT
	...

============================================================================*/

static void emit_printex (qu_ast n)
{
	qu_ast c;
	int    html = n->op;

	for (c = n->child; c; c = c->next)
	{
		if (html)
		{
			E1  (c);
			P0  (PRINTHTML);
			POP (1);
		}
		else
		{
			if (c->type == QU_AST_literal)
			{
				P1  (PRINT, c->thing);
			}
			else
			{
				E1  (c);
				P0  (PRINTTOP);
				POP (1);
			}
		}
	}
}



/*============================================================================

	  QUEST
	 /  |  \
	a   b  [c]


	code                 stack
	-------------------  --------------------
		{a}              a
	j:  JMPFALSE h
		{b}              b
	g:  JMP x
	h:  {c}              c
	x:  ...

============================================================================*/

static void emit_quest (qu_ast n)
{
	qu_ast a = n->child;
	qu_ast b = a->next;
	qu_ast c = b->next;
	int    j, g;

	if (a->type == QU_AST_literal)
	{
		if (qu_sys_true (a->thing) == QuTrue)
		{
			E1 (b);
		}
		else
		{
			if (c)
			{
				E1 (c);
			}
			else
			{
				P1   (LITERAL, QuNil);
				PUSH (1);
			}
		}
	}
	else
	{
			E1   (a);
		j = P1   (JMPFALSE, __LATER__);
			POP  (1);
			E1   (b);
		g = P1   (JMP, __LATER__);
			SETJ (j);

		if (c)
		{
			E1   (c);
		}
		else
		{
			P1   (LITERAL, QuNil);
			PUSH (1);
		}
		POP  (1);
		SETJ (g);
	}
}



/*============================================================================

	REDO - op = number of blocks to break

============================================================================*/

#define emit_redo emit_continue



/*============================================================================

	REF -> thing = Module, Class, Sub or Var


	(1)
		sub foo
			sub bar
			;;
			return @bar
		;;

		code
		---------------------
		CLOSURE  Sub


	(2)
		otherwise

		code
		---------------------
		LITERAL  thing

============================================================================*/

static void emit_ref (qu_ast n)
{
	if (n->child == 0 && n->op == 2)
	{
		/* @self.name */
		assert (n->thing && qu_string_is (n->thing));
		P1 (GETATTRSELF, n->thing);
		PUSH (2);
		P0 (XCHG);
		P0 (DISCARD);
		POP (1);
	}
	else
	{
		qu_any x = n->thing;

		if (qu_sub_is (x) && !(qu_sub_is_extern (x) || ((qu_sub) x)->root == x))
		{
			P1   (CLOSURE, x);
			PUSH (1);
		}
		else
		{
			P1   (LITERAL, x);
			PUSH (1);
		}
	}
}



/*==========================================================================

	QU_RETURN
	|
	[expr]

	code                  stack
	--------------------  --------------------
	expr                  expr
	RET

==========================================================================*/

static void emit_return (qu_ast n)
{
	qu_ast c = n->child;

	if (c)
	{
		E1 (c);
	}
	else
	{
		P1   (LITERAL, QuNil);
		PUSH (1);
	}
	P0  (RET);
}



/*============================================================================

	SELF

	code
	----------
	SELF

============================================================================*/

static void emit_self (qu_ast n)
{
	P0   (SELF);
	PUSH (1);
}



/*============================================================================

	SET -> op = number of items
	|
	expr*

	!!! const qu_set is already converted to LITERAL by Reduce

	code                  stack
	--------------------  --------------------
	expr1                 x1
	expr2                 x1  x2
	...
	exprn                 x1  x2  ...  xn
	SET                   r

	Note
	----
	This instruction never fails because the number of arguments is already
	limited by COMPILE_MAXARGS.

============================================================================*/

static void emit_set (qu_ast n)
{
	long len = n->op;

	EC   (n);
	P1   (SET, len);
	POP  (len);
	PUSH (1);
}



/*============================================================================

	STACKTOP

============================================================================*/

static void emit_stacktop (qu_ast n)
{
	P0   (DUP1);
	PUSH (1);
}



/*============================================================================

	STMT
	|
	expr

	code
	------------------
	{expr}
	DISCARD

============================================================================*/

static void emit_stmt (qu_ast n)
{
	qu_ast c = n->child;

	assert (CoSp == 0);

	LINE (c);
	E1   (c);
	P0   (DISCARD);
	POP  (1);

	assert (CoSp == 0);
}



/*============================================================================

	SUB -> thing = Sub object
	|
	[SYMBOL] - PARAM* - COMPOUND  !!! the [SYMBOL] node is removed by Reduce
	                    |
	                    stmt*

	code
	-----------------
	param*
	stmt*
	QU_END

============================================================================*/

static void emit_sub (qu_ast n)
{
	qu_ast c;

	if (EmitPass == 1)
		return;

	for (c = n->child; c; c = c->next)
	{
		if (c->child)
			emit_sub (c);
	}

	if (n->type == QU_AST_sub || n->type == QU_AST_anonsub)
	{
		qu_code c;
		qu_sub  u = n->thing;

		assert (qu_sub_is (u));
		code_begin (u);
		emit_begin ();
		EmitPass = 1;

		if (qu_sub_is_abstract (u))
		{
			P1 (LITERAL, QuEAbstractClass);
			P1 (THROW, 1);
			PUSH (1);
			POP  (1);
		}
		else
		{
#if 0
			if (qu_sub_is_unsafe (u))
				P0 (UNSAFE);
#endif
			EC (n);
		}

		EmitPass = 2;
		emit_end ();
		c = code_make ();
		u->value = c;
		qu_sub_enddecl (u);
		code_end ();
	}
}



/*============================================================================

	SWAP
	|
	expr

	code               stack
	-----------------  -----------------
	expr               x
	SWAP               x

============================================================================*/

static void emit_swap (qu_ast n)
{
	assert (qu_sub_is (CoMaster));

	LINE (n);
	E1   (n->child);
	P0   (SWAP);
}



/*============================================================================

	SWITCH
	|       (default)
	expr - CASE | NOP - CASE*

	CASE -> op = C function address, 0 for default
	|
	[expr | COMPOUND] - COMPOUND

============================================================================*/

static void emit_switch (qu_ast n)
{
	qu_ast c, d, g;
	int    j, m = 0, k = 0;

	LINE (n);

	c = n->child;
	d = c->next;

        E1  (c);
	j = P2  (ENTER, __LATER__, QU_CODE_BLOCK_SWITCH);
	    POP (1);

	n = 0;
	for (c = d->next; c; c = c->next)
	{
		assert (c->type == QU_AST_case);

		if (k)
			SETJ (k);

		g = c->child;
		if (g->type == QU_AST_compound)
		{
			/* case expr1, expr2, expr3, ...

				| z1: {expr}
				|     GETSWITCH
				|     EQ
				|     JMPTRUE p1
				|     {expr}
				|     GETSWITCH
				|     EQ
				|     JMPTRUE p1
				|     ...
				|     JMP z2
				| p1: {compound}
				|     ...
				|     JMP p2
				|
				| z2: {expr}
				|      ...
				| p2: {compound}
			*/

			int    z [1024];
			int    l, e;
			qu_ast x;

			for (l = 0, x = g->child; x; x = x->next, l++)
			{
				if (l >= 1024)
					CTHROW (x, "too many options in case");

				       P0    (GETSWITCH);
				       PUSH  (1);
				       E1    (x);
				       BINOP (c->op);
				z[l] = P1    (JMPTRUE, __LATER__);
				       POP   (2);
			}
			k = P1 (JMP, __LATER__);
			e = CoNextOfs;
			while (l--)
				SET1 (z[l], e);
		}
		else if (g->type == QU_AST_literal && g->op == 1)
		{
			/*
				case .method:
			*/
			    P0    (GETSWITCH);
			    P1    (GETATTRX, g->thing);
			    PUSH  (2);
			    P1    (CALLATTR, 0);
			    POP   (1);
			k = P1    (JMPFALSE, __LATER__);
			    POP   (1);
		}
		else if (g->type == QU_AST_call && g->thing == QuNil)
		{
			/*
				case .method (a, b, c):
			*/
			    P0    (GETSWITCH);
			    P1    (LITERAL, g->child->thing);
			    PUSH  (2);
				P0    (GETATTR);
				EF    (g->child->next);
			    P1    (CALLATTR, g->op);
			    POP   (g->op);
			k = P1    (JMPFALSE, __LATER__);
		}
		else if (g->type == QU_AST_unop && g->thing == QuNil)
		{
			/*
				case ? expr:
			*/
			    E1    (g->child);
			k = P1    (JMPFALSE, __LATER__);
			    POP   (1);
		}
		else
		{
			/*
				case expr:

				| z1: {expr}
				|     GETSWITCH
				|     <binop>
				| k1: JMPFALSE z2
				| p1: {compound}
				| m1: JMP p2
				|
				| z2: ...
			*/
			    P0    (GETSWITCH);
			    PUSH  (1);
			    E1    (g);
			    BINOP (c->op);
			k = P1    (JMPFALSE, __LATER__);
			    POP   (2);
		}

		if (m)
			SETJ (m);

		E1 (g->next);

		if (c->next == NULL)
			m = 0;
		else
			m = P1 (JMP, __LATER__);
	}

	if (k)
		SETJ (k);

	assert (m == 0);
	if (d->type == QU_AST_case)
		E1 (d->child);

	P0   (LEAVE);
	SETJ (j);
}



/*============================================================================

	TAG -> thing = name  op = Var or Sub object
	|
	expr




----------------------------------------------------------------------------*/

static void emit_tag (qu_ast n)
{
	qu_ast    c    = n->child;
	qu_symbol sym  = (qu_symbol) n->op;
	qu_string name = (qu_string) n->thing;

	if (c->type == QU_AST_literal || c->type == QU_AST_ref)
	{
		if (qu_sub_is (sym))
		{
			qu_sub_settag ((qu_sub) sym, name, c->thing);
		}
		else
		{
			assert (qu_var_is (sym));
			qu_var_settag ((qu_var) sym, name, c->thing);
		}
	}
	else
	{
		P1   (LITERAL, sym);
		P1   (LITERAL, name);
		PUSH (2);
		E1   (c);
		P0   (SETITEM);
		P0   (DISCARD);
		POP  (3);
	}

	n->type  = QU_AST_nop;
	n->child = 0;
}



/*============================================================================

	THROW
	|
	[expr - [expr]]

	code
	--------------------
	THROW 0

	or

	LITERAL (exception class)
	THROW 1

	or

	LITERAL (exception class)
	expr
	QU_THROW 2

============================================================================*/

static void emit_throw (qu_ast n)
{
	qu_ast c = n->child;

	if (c == NULL)
	{
		P1 (THROW, 0);
		PUSH (1);
	}
	else
	{
#if 0
		qu_any o = c->thing;

		assert (qu_exception_class_is (o));
		c = c->next;
		if (c == 0)
		{
			P1   (LITERAL, o);
			PUSH (1);
			P1   (THROW, 1);
		}
		else
		{
			E1   (c);
			P1   (LITERAL, o);
			PUSH (1);
			P1   (THROW, 2);
			POP  (1);
		}
#else
		E1 (c);
		c = c->next;
		if (c == 0)
		{
			P1   (THROW, 1);
		}
		else
		{
			E1   (c);
			P1   (THROW, 2);
			POP  (1);
		}
#endif
	}
}



/*============================================================================

	TRY
	|
	[expr] - COMPOUND [- CATCH* - [ELSE] - [FINALLY]]
	         |                     |        |
	         stmt*                 stmt*    stmt*

	CATCH -> op = 0 or 1           \__ set by Reduce
	|        thing = what to catch /
	|
	SYMBOL* - COMPOUND   !!! SYMBOL removed by Reduce
	          |
	          stmt*

	code
	-----------------------------------------
	    expr
	x:  JMPFALSE z

	t:  ENTER catch, QU_CODE_BLOCK_TRY
	    stmt*
	    JMP elsetry

	catch:
	    ENTER finally, QU_CODE_BLOCK_CATCH
	c1: CATCH c2, exceptions
	    stmt*
	    JMP u
	c2: CATCH cd, exceptions
	    stmt*
	    JMP u
	cd: CATCH u, nil
	    stmt*
	    JMP u             -> a cheap waste
	u:  LEAVE

	    LEAVE             -> a cheap waste

	elsetry:
	    ENTER finally, QU_CODE_BLOCK_ELSETRY   -> must fake depth
	    stmt*
	    LEAVE

	finally:
	   ENTER z, QU_CODE_BLOCK_FINALLY
	   stmt*

	z: ...

	Note the unusual location of the LEAVE of the try block.
	This is required by optimizers to get the correct ENTER/LEAVE pairs.
	The CATCH block must be a child of the TRY block.

============================================================================*/

static void emit_try (qu_ast n)
{
	long   x = 0, t = 0, j, h;
	qu_ast c, g;

	c = n->child;
	if (c->type != QU_AST_compound)
	{
		if (c->type == QU_AST_literal)
		{
			/* try if false -> produces no code */
			if (qu_sys_true (c->thing) != QuTrue)
				return;
		}

		LINE (c);
		if (c->type == QU_AST_unop && n->op == (long) qu_sys_not)
		{
			    E1 (c->child);
			x = P1 (JMPTRUE, __LATER__);
			    POP (1);
		}
		else
		{
			    E1  (c);
			x = P1  (JMPFALSE, __LATER__);
			    POP (1);
		}
		c = c->next;
	}

	/*
		try
	*/
	t = P2   (ENTER, __LATER__, QU_CODE_BLOCK_TRY);
	    E1   (c);
	j = P1   (JMP, __LATER__);
	    SETJ (t);

	/*
		catch
	*/
	h = P2 (ENTER, __LATER__, QU_CODE_BLOCK_CATCH);
	c = c->next;
	g = c;
	for (; c && c->type == QU_AST_catch; c = c->next)
	{
		qu_any x = c->thing;
		qu_any e;

		if (c->op == 1)
		{
			assert (qu_array_is (x) && ((qu_array) x)->len == 2);
			e = ((qu_array) x)->objs[0];
		}
		else
		{
			e = x;
		}

		t = P2 (CATCH, __LATER__, e);

			if (c->op == 1)
			{
				qu_var v = ((qu_array) x)->objs[1];
				assert (qu_var_is (v));

				if (qu_var_is_lexical (v))
					P1 (CATCHSET, qu_int_new (v->index));
				else
					P1 (CATCHSET, v);
			}

		        E1   (c->child);
		c->op = P1 (JMP, __LATER__);
		        SETJ (t);
	}
	for (; g && g->type == QU_AST_catch; g = g->next)
		SETJ (g->op);

	P0 (LEAVE); /* catch */
    P0 (LEAVE); /* try */

	/*
		else
	*/
	SETJ (j);

	j = P2 (ENTER, __LATER__, QU_CODE_BLOCK_ELSETRY);
	if (c && c->type == QU_AST_else)
	{
	    EC (c);
		c = c->next;
	}
    P0   (LEAVE);
    SETJ (j);
	SETJ (h);

	/*
		finally
	*/
	if (c)
	{
		j = P2   (ENTER, __LATER__, QU_CODE_BLOCK_FINALLY);
	        EC   (c);
	        P0   (LEAVE);
	        SETJ (j);
	}

	if (x)
		SETJ (x);
}



/*============================================================================

	UNOP -> op = operator (qu_sys_true, qu_sys_not, etc.)
	|
	a

============================================================================*/

static void emit_incr (qu_ast n);

static void emit_unop (qu_ast n)
{
	qu_unop_f op = (qu_unop_f) n->op;

	if (op == qu_sys_incr || op == qu_sys_decr)
	{
		emit_incr (n);
	}
	else
	{
		E1   (n->child);
		UNOP (n->op);
	}
}



/*============================================================================

	 n
	UNOP -> op = qu_sys_incr || qu_sys_decr
	|       thing = 0:pre  otherwise:post
	a

	++a
	--a

============================================================================*/

static void emit_incr_attr   (qu_ast n);
static void emit_incr_item   (qu_ast n);
static void emit_incr_name   (qu_ast n);

static void emit_incr (qu_ast n)
{
	switch (n->child->type)
	{
		case QU_AST_getattr:
			emit_incr_attr (n);
			return;

		case QU_AST_getitem:
			emit_incr_item (n);
			return;

		case QU_AST_name:
			emit_incr_name (n);
			return;
	}
	CTHROW (n, "invalid ++ or -- operand");
}



/*
	++b.c

	  n
	 UNOP -> op = qu_sys_incr | qu_sys_decr
	   |     thing = 0:pre  otherwise:post
	   a
	GETATTR
	 /  \
	b    c

	code                  stack
	--------------------  --------------------
	post-increment

	{b}                   b
	{c}                   b  c
	GETATTR2              b  c  x
	DUP1                  b  c  x  x
	ROT4X                 x  b  c  x
	INCR                  x  b  c  y
	SETATTR               x  y
	DISCARD               x

	pre-increment

	{b}                   b
	{c}                   b  c
	GETATTR2              b  c  x
	INCR/DECR             b  c  y
	SETATTR               y
*/
static void emit_incr_attr (qu_ast n)
{
	qu_ast a = n->child;
	qu_ast b = a->child;
	qu_ast c = b->next;

	E1    (b);
	E1    (c);
	P0    (GETATTR2);
	PUSH  (1);

	if (n->thing)
	{
		P0   (DUP1);
		PUSH (1);
		P0   (ROT4X);
		if ((qu_unop_f) n->op == qu_sys_incr)
			P0 (INCR);
		else
			P0 (DECR);
		P0   (SETATTR);
		P0   (DISCARD);
		POP  (3);
	}
	else
	{
		if ((qu_unop_f) n->op == qu_sys_incr)
			P0 (INCR);
		else
			P0 (DECR);
		P0  (SETATTR);
		POP (2);
	}
}



/*
	++b[c]

	  n
	 UNOP -> op = qu_sys_incr | qu_sys_decr
	   |     thing = 0:pre  otherwise:post
	   a
	GETITEM
	 /  \
	b    c

	code                  stack
	--------------------  --------------------
	post-increment

	{b}                   b
	{c}                   b  c
	GETITEM2              b  c  x
	DUP1                  b  c  x  x
	ROT4X                 x  b  c  x
	INCR                  x  b  c  y
	SETITEM               x  z
	DISCARD               x

	pre-increment

	{b}                   b
	{c}                   b  c
	GETITEM2              b  c  x
	INCR/DECR             b  c  y
	SETITEM               z
*/
static void emit_incr_item (qu_ast n)
{
	qu_ast a = n->child;
	qu_ast b = a->child;
	qu_ast c = b->next;

	E1    (b);
	E1    (c);
	P0    (GETITEM2);
	PUSH  (1);

	if (n->thing)
	{
		P0   (DUP1);
		PUSH (1);
		P0   (ROT4X);
		if ((qu_unop_f) n->op == qu_sys_incr)
			P0 (INCR);
		else
			P0 (DECR);
		P0   (SETITEM);
		P0   (DISCARD);
		POP  (3);
	}
	else
	{
		if ((qu_unop_f) n->op == qu_sys_incr)
			P0 (INCR);
		else
			P0 (DECR);
		P0  (SETITEM);
		POP (2);
	}
}



/*
	++a

	 n
	UNOP -> op = qu_sys_incr | qu_sys_decr
	  |
	  a
	NAME

	code                  stack
	--------------------  --------------------
	post-increment
	- - - - - - - -

	GETGLOBAL             a
	DUP1                  a  a
	INCR/DECR             a  y
	SETGLOBAL             a  y
	DISCARD               a

	or

	INCRLPOST/DECRLPOST index   a

	pre-increment
	- - - - - - - -
	GETGLOBAL             a
	INCR/DECR             x
	SETGLOBAL             x

	or

	INCRLPRE/DECRLPRE index

----------------------------------------------------------------------------*/

static void emit_incr_name (qu_ast n)
{
	qu_ast a = n->child;
	qu_var v = a->thing;
	int    incr = (qu_unop_f) n->op == qu_sys_incr;

	if (!qu_var_is (v))
		CTHROW (a, "invalid ++ or -- operand");

	/* this is checked by Reduce */
	assert (!qu_var_is_inst (v));

	if (n->thing) /* post */
	{
		if (qu_var_is_lexical (v))
		{
			if (incr)
				P1 (INCRLPOST, v->index);
			else
				P1 (DECRLPOST, v->index);
			PUSH (1);
		}
		else
		{
			VGET (v);
			P0   (DUP1);
			PUSH (2);
			if (incr)
				P0 (INCR);
			else
				P0 (DECR);
			VSET (v, a);
			P0   (DISCARD);
			POP  (1);
		}
	}
	else /* pre */
	{
		if (qu_var_is_lexical (v))
		{
			if (incr)
				P1 (INCRLPRE, v->index);
			else
				P1 (DECRLPRE, v->index);
			PUSH (1);
		}
		else
		{
			VGET (v);
			PUSH (1);
			if (incr)
				P0 (INCR);
			else
				P0 (DECR);
			VSET (v, a);
		}
	}
}



/*============================================================================

	VAR -> thing = Var object
	|
	[SYMBOL] - [expr] - [SUB]    !!! SYMBOL node removed by Reduce.c

============================================================================*/

static void emit_var (qu_ast n)
{
	qu_var v;
	qu_ast c = n->child;

	v = n->thing;

	if (c && c->type != QU_AST_sub)
	{
		assert (qu_var_is (v) && !qu_var_is_lexical (v));

		if (c->type == QU_AST_literal && v->type == 0)
		{
			v->value = c->thing;
		}
		else
		{
			E1   (c);
			VSET (v, n);
			P0   (DISCARD);
			POP  (1);
		}

		c = c->next;
		if (c)
		{
			assert (c->type == QU_AST_sub);
			v->type = c->thing;
		}
	}
}



/*============================================================================

	WHILE
	|
	expr - COMPOUND
	       |
	       stmt*

	code
	------------------------------
	b:  ENTER z, QU_CODE_BLOCK_WHILE
	x:  expr
	j:  JMPFALSE l
	    stmt*
	    JMP x
	l:  LEAVE
	z:  ...

============================================================================*/

static void emit_while (qu_ast n)
{
	int b, x, j;

	b = P2   (ENTER, __LATER__, QU_CODE_BLOCK_WHILE);
	x = CoNextOfs;
	    E1   (n->child);
	j = P1   (JMPFALSE, __LATER__);
	    POP  (1);
	    E1   (n->child->next);
		P1   (JMP, x);
		SETJ (j);
		P0   (LEAVE);
		SETJ (b);
}



/*============================================================================

	WITH

	code
	------------------------------

============================================================================*/

static void emit_with (qu_ast n)
{
	QU_FATAL ("NOT YET");
}



/*============================================================================

	YIELD
	|
	expr+

	code                  stack
	--------------------  --------------------
	expr1                 expr1
	YIELD                 expr1
	DISCARD
	expr2                 expr2
	YIELD                 expr2
	DISCARD
	...

============================================================================*/

static void emit_yield (qu_ast n)
{
	qu_ast c;
	long   k = 0;

	assert (qu_sub_is (CoMaster));
	assert (qu_sub_is_generator (CoMaster));

	for (c = n->child; c; c = c->next)
	{
		/* yield is an expression - the last one is the result */
		if (k++)
		{
			P0  (DISCARD);
			POP (1);
		}
		E1  (c);
		P0  (YIELD);
	}
}



/*============================================================================
	emit_node
============================================================================*/
static void emit_module ();

static void emit__bug (qu_ast n)
{
	QU_FATAL ("bug: never");
}

#define emit_DO_NOT_USE emit__bug
#define emit_case       emit__bug
#define emit_catch      emit__bug
#define emit_const      emit__bug
#define emit_eat        emit__bug
#define emit_else       emit__bug
#define emit_enum       emit__bug
#define emit_enumitem   emit__bug
#define emit_finally    emit__bug
#define emit_flat       emit__bug
#define emit_isa        emit__bug
#define emit_mixin      emit__bug
#define emit_static     emit__bug
#define emit_struct     emit__bug
#define emit_super      emit__bug
#define emit_symbol     emit__bug
#define emit_todo       emit__bug

#define emit_list    emit_array

typedef void (*emit_f) (qu_ast n);

#define n_(name) emit_##name,
static emit_f EmitFunctions [QU_AST_LAST + 1] =
{
#include "Asts.h"
};
#undef n_




static void emit_node (qu_ast n)
{

#if 0
{
	qu_string s = qu_ast_name (n->type);
	fprintf (stderr, "EMIT %s line:%ld %s\n", CoModule->name->str, n->line, s->str);
}
#endif

	assert (n->type > 0 && n->type <= QU_AST_LAST);
	EmitFunctions [n->type] (n);
}




/*----------------------------------------------------------------------------
	almost start
----------------------------------------------------------------------------*/

static void emit_begin ()
{
	return;
}



/*
	Must be called after emitting Module/Sub.
*/
static void emit_end ()
{
	P0 (END);
}



/*----------------------------------------------------------------------------
	start here
----------------------------------------------------------------------------*/

/*
	Pass-1: emit Module
	Pass-2: emit Sub

	MODULE -> thing = Module object
	|
	TODO - COMPOUND
	|
	(TAG | IMPORT) *
*/
static void emit_module ()
{
	qu_ast c = CoModule->tree->child;

	EmitPass = 1;
	emit_begin ();
	emit_child_nodes (c);
	c = c->next;
	if (c)
	{
		assert (c->type = QU_AST_compound);
		E1 (c);
	}
	emit_end ();
	CoModule->code = code_make ();

	EmitPass = 2;
	c = CoModule->tree->child;
	emit_sub (c);
	c = c->next;
	if (c)
	{
		assert (c->type = QU_AST_compound);
		emit_sub (c);
	}
}
