/* Opaque.c - Copyright (c) Marc Krisnanto */

#include "qu.h"

/*) class Opaque __final
	This class is used to wrap C pointers.

	You can not create Opaque objects directly but you can pass it
	around just like any other objects.
*/
qu_class QuOpaqueClass;


struct qu_opaqueex_s
{
	QU_OPAQUE_HEADEX;
};
typedef struct qu_opaqueex_s *qu_opaque_ex;


/* flags */
#define qu_opaque_set_ex(self)       qu_flag_set (self, QU_OPAQUE_FLAG_EX)
#define qu_opaque_set_strfield(self) qu_flag_set (self, QU_OPAQUE_FLAG_STRFIELD)


/*----------------------------------------------------------------------------
	low-level instance methods
----------------------------------------------------------------------------*/

static void qu__opaque__destroy (qu_opaque self)
{
	if (self->free)
		self->free (self);
}



static void qu__opaque__mark (qu_opaque self)
{
	if (self->mark)
		self->mark (self);
}



/*----------------------------------------------------------------------------
	standard instance mehods
----------------------------------------------------------------------------*/

/*) sub __getitem (x) -> Any
	Some Opaque objects might provide this method to get field value.

	@exc
	.EImplement
	.EKey
*/
static qu_any qu__opaque__getitem (qu_opaque self, qu_any x)
{
	QU_REQUIRE (QuEImplement, qu_opaque_is_ex (self) && ((qu_opaque_ex) self)->getitem);
	if (qu_opaque_is_strfield (self))
		QU_REQUIRE (QuEKey, qu_string_is (x));
	return ((qu_opaque_ex) self)->getitem (self, x);
}



/*) sub __setitem (x, v) -> Any
	Some Opaque objects might provide this method to set a field value.

	@exc
	.EImplement
	.EKey
*/
static qu_any qu__opaque__setitem (qu_opaque self, qu_any x, qu_any v)
{
	QU_REQUIRE (QuEImplement, qu_opaque_is_ex (self) && ((qu_opaque_ex) self)->setitem);
	if (qu_opaque_is_strfield (self))
		QU_REQUIRE (QuEKey, qu_string_is (x));
	return ((qu_opaque_ex) self)->setitem (self, x, v);
}



/*----------------------------------------------------------------------------
	specific instance mehods
----------------------------------------------------------------------------*/

/*) sub canget -> Boolean
	Whether or not #{__getitem} works. That is, whether or not the Opaque
	object provides readable field values.

	@noexc
*/
static qu_boolean qu__opaque_canget (qu_opaque self)
{
	if (!qu_opaque_is_ex (self))
		return QuFalse;
	return ((qu_opaque_ex) self)->getitem ? QuTrue : QuFalse;
}



/*) sub canset -> Boolean
	Whether or not #{__setitem} works. That is, whether or not the Opaque
	object provides writable field values.

	@noexc
*/
static qu_boolean qu__opaque_canset (qu_opaque self)
{
	if (!qu_opaque_is_ex (self))
		return QuFalse;
	return ((qu_opaque_ex) self)->setitem ? QuTrue : QuFalse;
}



/*-----------------------------------------------------------------------
	interface
-----------------------------------------------------------------------*/

qu_opaque qu_opaque__new (qu_opaquemark_f m, qu_opaquefree_f f, size_t z)
{
	register qu_opaque o;

	assert (z >= sizeof (qu_opaque_s));
	o = (qu_opaque) qu_gc__new (QuOpaqueClass, z);
	o->mark = m;
	o->free = f;
	return o;
}



qu_opaque qu_opaque__new_ex (qu_opaquemark_f m, qu_opaquefree_f f, size_t z, qu_sub1_f g, qu_sub2_f s)
{
	register qu_opaque o;

	assert (z >= sizeof (qu_opaque_s));
	o = (qu_opaque) qu_gc__new (QuOpaqueClass, z);
	qu_opaque_set_ex (o);
	o->mark = m;
	o->free = f;
	((qu_opaque_ex) o)->getitem = g;
	((qu_opaque_ex) o)->setitem = s;
	return o;
}



/*-----------------------------------------------------------------------
	init
-----------------------------------------------------------------------*/

void qu_opaque__setup ()
{
	qu_class_setup_sys (QuOpaqueClass, qu_opaque_s, qu__opaque__destroy, qu__opaque__mark);
}


void qu_opaque__init ()
{
	/* standard instance methods */
	qu_class_method_inst  (QuOpaqueClass, QU_NAME__getitem, qu__opaque__getitem, 1);
	qu_class_method_inst  (QuOpaqueClass, QU_NAME__setitem, qu__opaque__setitem, 2);

	/* specific instance methods */
	qu_class_method_instz (QuOpaqueClass, "canget", qu__opaque_canget, 0);
	qu_class_method_instz (QuOpaqueClass, "canset", qu__opaque_canset, 0);

	/* finalize declaration */
	qu_class_enddecl (QuOpaqueClass);
}
