/* Sub.c - Copyright (c) Marc Krisnanto */

#include "qu.h"


/*) class Sub is Function __final
	Functions and methods are instances of this class. This includes those
	created with the k{lambda} keyword.

	You can not create Sub objects directly. The compiler creates them
	for you from your source code.

	@xmp
		sub foo               #-> a Sub object
			return 1
		;;

		class Foo
		    sub bar           #-> a Sub object
		        return 1
		    ;;
		;;

	@see
		The builtin function -{call}.
*/
qu_class QuSubClass;


/* specialized structs so we dont waste too much memory */
struct func_s
{
	QU_SUB_HEAD;
};
struct sub_s
{
	QU_SUB_HEAD;
	QU_SUB_SCRIPT_FIELDS;
};
struct closure_s
{
	QU_SUB_HEAD;
	QU_SUB_SCRIPT_FIELDS;
	QU_SUB_CLOSURE_FIELDS;
};


static qu_any _sub_call_error () { QU_THROWZ (QuECall, "cant call sub at compile time"); }

static qu_any _sub_call_error_n (qu_sub self, qu_any theself, int argc, qu_any *argv)                     { return _sub_call_error (); }
static qu_any _sub_call_error_4 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3, qu_any a4) { return _sub_call_error (); }
static qu_any _sub_call_error_3 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3)            { return _sub_call_error (); }
static qu_any _sub_call_error_2 (qu_sub self, qu_any theself, qu_any a1, qu_any a2)                       { return _sub_call_error (); }
static qu_any _sub_call_error_1 (qu_sub self, qu_any theself, qu_any a1)                                  { return _sub_call_error (); }
static qu_any _sub_call_error_0 (qu_sub self, qu_any theself)                                             { return _sub_call_error (); }

static qu_any _sub_call_once_n  (qu_sub self, qu_any theself, int argc, qu_any *argv)                     { return self->value; }
static qu_any _sub_call_once_4  (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3, qu_any a4) { return self->value; }
static qu_any _sub_call_once_3  (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3)            { return self->value; }
static qu_any _sub_call_once_2  (qu_sub self, qu_any theself, qu_any a1, qu_any a2)                       { return self->value; }
static qu_any _sub_call_once_1  (qu_sub self, qu_any theself, qu_any a1)                                  { return self->value; }
static qu_any _sub_call_once_0  (qu_sub self, qu_any theself)                                             { return self->value; }

static qu_any _sub_call_en (qu_sub self, qu_any theself, int argc, qu_any *argv);
static qu_any _sub_call_e4 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3, qu_any a4);
static qu_any _sub_call_e3 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3);
static qu_any _sub_call_e2 (qu_sub self, qu_any theself, qu_any a1, qu_any a2);
static qu_any _sub_call_e1 (qu_sub self, qu_any theself, qu_any a1);
static qu_any _sub_call_e0 (qu_sub self, qu_any theself);

static qu_any _sub_call_interface_n (qu_sub self, qu_any theself, int argc, qu_any *argv);
static qu_any _sub_call_interface_4 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3, qu_any a4);
static qu_any _sub_call_interface_3 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3);
static qu_any _sub_call_interface_2 (qu_sub self, qu_any theself, qu_any a1, qu_any a2);
static qu_any _sub_call_interface_1 (qu_sub self, qu_any theself, qu_any a1);
static qu_any _sub_call_interface_0 (qu_sub self, qu_any theself);


static qu_subcall_s CallError =
{
	_sub_call_error_n,
	_sub_call_error_4,
	_sub_call_error_3,
	_sub_call_error_2,
	_sub_call_error_1,
	_sub_call_error_0
};

static qu_subcall_s CallGenerator =
{
	(qu_subcalln_f) qu_interp_call_gn,
	(qu_subcall4_f) qu_interp_call_g4,
	(qu_subcall3_f) qu_interp_call_g3,
	(qu_subcall2_f) qu_interp_call_g2,
	(qu_subcall1_f) qu_interp_call_g1,
	(qu_subcall0_f) qu_interp_call_g0
};

static qu_subcall_s CallExtern =
{
	_sub_call_en,
	_sub_call_e4,
	_sub_call_e3,
	_sub_call_e2,
	_sub_call_e1,
	_sub_call_e0
};

static qu_subcall_s CallNonGeneratorCFrame =
{
	qu_interp_call_cn,
	qu_interp_call_c4,
	qu_interp_call_c3,
	qu_interp_call_c2,
	qu_interp_call_c1,
	qu_interp_call_c0
};

static qu_subcall_s CallNonGeneratorAFrame =
{
	qu_interp_call_an,
	qu_interp_call_a4,
	qu_interp_call_a3,
	qu_interp_call_a2,
	qu_interp_call_a1,
	qu_interp_call_a0
};

static qu_subcall_s CallNonGeneratorCFrameJit =
{
	qu_interp_call_jit_cn,
	qu_interp_call_jit_c4,
	qu_interp_call_jit_c3,
	qu_interp_call_jit_c2,
	qu_interp_call_jit_c1,
	qu_interp_call_jit_c0
};

static qu_subcall_s CallNonGeneratorAFrameJit =
{
	qu_interp_call_jit_an,
	qu_interp_call_jit_a4,
	qu_interp_call_jit_a3,
	qu_interp_call_jit_a2,
	qu_interp_call_jit_a1,
	qu_interp_call_jit_a0
};

static qu_subcall_s CallOnceCFrame =
{
	qu_interp_call_once_cn,
	qu_interp_call_once_c4,
	qu_interp_call_once_c3,
	qu_interp_call_once_c2,
	qu_interp_call_once_c1,
	qu_interp_call_once_c0
};

static qu_subcall_s CallOnceAFrame =
{
	qu_interp_call_once_an,
	qu_interp_call_once_a4,
	qu_interp_call_once_a3,
	qu_interp_call_once_a2,
	qu_interp_call_once_a1,
	qu_interp_call_once_a0
};

static qu_subcall_s CallOnceCalled =
{
	_sub_call_once_n,
	_sub_call_once_4,
	_sub_call_once_3,
	_sub_call_once_2,
	_sub_call_once_1,
	_sub_call_once_0
};

static qu_subcall_s CallInterface =
{
	_sub_call_interface_n,
	_sub_call_interface_4,
	_sub_call_interface_3,
	_sub_call_interface_2,
	_sub_call_interface_1,
	_sub_call_interface_0
};



/*----------------------------------------------------------------------------
	specific class methods
----------------------------------------------------------------------------*/

static qu_sub SubInterfaceError = 0;

/* dont publish doc yet */
/*  sub class interface_error (*arg) -> qu_any
	@def
	sub class interface_error (qu_sub wrapper) -> qu_any
	sub class interface_error -> qu_any
	@p
	Get or set the qu_sub to call when a call to an __interface qu_sub fails.

	@exc
	.QuEArgValue invalid v{wrapper}

	@p
	The qu_sub v{wrapper} must take one argument: the interface name that was
	called. In this qu_sub, k{self} will be whatever k{self} was when the
	interface was called.
*/
static qu_any qu__sub_interface_error (qu_class self, int argc, qu_any *argv)
{
	QU_ARG_VCOUNT (0, 1);
	if (argc == 0)
	{
		if (SubInterfaceError)
			return SubInterfaceError;
	}
	else
	{
		qu_sub u   = argv[0];
		qu_sub old = SubInterfaceError;

		QU_ARG_CHECK (qu_sub_is (u) && qu_sub_is_static (u) && u->argc == 1);
		SubInterfaceError = u;
		if (old)
			return old;
	}
	return QuNil;
}



/*----------------------------------------------------------------------------
	low-level instance methods
----------------------------------------------------------------------------*/

static void qu__sub__destroy (qu_sub self)
{
	if (qu_sub_is_extern (self))
	{
		if (qu_sub_is_jit (self))
			qu_mem_free (self->value);

		if (self->ltype)
			qu_mem_free (self->ltype);
	}
	else
	{
		if (!qu_sub_is_closure (self))
		{
			if (self->ltype)
				qu_mem_free (self->ltype);
		}
	}
}



static void qu__sub__mark (qu_sub self)
{
	qu_gc_mark  (self->name);
	qu_gc_mark  (self->parent);
	qu_gc_xmark (self->type);
	qu_gc_xmark (self->tag);

	if (self->ltype)
	{
		int    i = self->nlocal;
		qu_any x;

		while (i)
		{
			x = self->ltype[--i];
			qu_gc_xmark (x);
		}
	}

	if (!qu_sub_is_extern (self))
	{
		qu_gc_xmark (self->value);
		qu_gc_mark  (self->root);

		if (qu_sub_is_closure (self))
		{
			qu_gc_xmark (self->clex);
			qu_gc_mark  (self->corg);
		}
	}
}



/*----------------------------------------------------------------------------
	standard instance methods
----------------------------------------------------------------------------*/

/*) sub __cmp (x)
	If v{x} is a Sub then do a comparisson suitable for documentation.
	Otherwise delegate to -{Object.__cmp}.
*/
static int _sub_cmp_val (qu_sub self)
{
	if (qu_sub_get_class (self))
	{
		if (qu_sub_is_class (self))
			return 0;
		return qu_sub_is_static (self) ? 1 : 2;
	}
	return 3;
}

static qu_int qu__sub__cmp (qu_sub self, qu_sub x)
{
	qu_class c = qu_classof (x);

	if (c == QuClassClass)
		return QuInt_1;

	else if (c == QuVarClass)
		return QuInt1;

	else if (c == QuSubClass)
	{
		int a = _sub_cmp_val (self);
		int b = _sub_cmp_val (x);

		if (a == b)
			return qu_string_cmpint (self->name, x->name);
		return (a > b) ? QuInt1 : QuInt_1;
	}
	else
		return qu_object_default_cmp (self, x);
}



/*) sub __getitem (name: String) -> Any
	Get value in the tag list.

	@exc
	.EKey

	@xmp
		sub foo (x)
			\_ title = 'Fooey'
		;;

		f = @foo
		println (f->title)

	@out
		Fooey
*/
qu_any qu__sub__getitem (qu_sub self, qu_string name)
{
	qu_any x = qu_sub_gettag (self, name);
	QU_REQUIRE (QuEKey, qu_string_is (name));
	x = qu_sub_gettag (self, name);
	QU_REQUIRE (QuENone, x);
	return x;
}



/*) sub __getslice (name: String, defval: Any) -> Any
	Get value in the tag list.

	This does the same as #{__getitem} except that if the name does not
	exist then v{defval} is returned instead of throwing an exception.

	@noexc
*/
qu_any qu__sub__getslice (qu_sub self, qu_string name, qu_any defval)
{
	if (qu_string_is (name))
	{
		qu_any x = qu_sub_gettag (self, name);
		if (x)
			return x;
	}
	return defval;
}



/*) sub __print -> Null
	Print self to -{Stream.stdout}.

	See -{print} for possible exceptions.

	@xmp
		sub foo
			return 1
		;;

		println (@foo)

	@out
		<Sub foo>
*/
static qu_null qu__sub__print (qu_sub self)
{
	qu_string s = qu_sub_as_string (self);
	QU_ENSURE (qu_sys_printz (s->str, s->len));
	return QuNil;
}



/*) sub __push (x) -> self
	Call the Sub with one argument and the current 'self' then return the
	Sub on success.

	@exc
	.Exception anything thrown by the Sub

	@xmp
		sub foo (a)
			println (a)
		;;

		f = @foo
		f <<< 'hello' <<< 'world'

	@out
		hello
		world
*/
static qu_sub qu__sub__push (qu_sub self, qu_any x)
{
	QU_ENSURE (qu_sub_call1 (self, qu_interp_self (), x));
	return self;
}



/*) sub __setitem (name: String, value: Any) -> Any
	Set value in the tag list.

	@exc
	.EArgValue name is not a String

	@xmp
		sub foo (x)
			\_ title = 'Fooey'
		;;

		f = @foo
		f->greeting = 'Hi'
		println (f->greeting)

	@out
		Hi
*/
qu_any qu__sub__setitem (qu_sub self, qu_string name, qu_any val)
{
	QU_ARG_TYPE (name, qu_string);
	qu_sub_settag (self, name, val);
	return val;
}



/*) sub __string -> String
	Return a string representation of self.

	@noexc

	@xmp
		sub foo
			return 1
		;;

		f = @foo
		println (f as String)

	@out
		<Sub foo>
*/
qu_string qu__sub__string (qu_sub self)
{
	if (qu_sub_is_closure (self))
		return qu_object_where_string (self, "<Closure ", ">");
	return qu_object_where_string (self, "<Sub ", ">");
}



/*----------------------------------------------------------------------------
	specific instance methods
----------------------------------------------------------------------------*/

/*) sub argc -> Int
	Return the minimum number of arguments required by the Sub.

	A value of -1 means that the Sub can take any number of arguments.

	@noexc

	@xmp
		sub foo (x)
			return x + 1
		;;

		f = @foo
		println (f.argc)

	@out
		1
*/
static qu_int qu__sub_argc (qu_sub self)
{
	return qu_int_new (self->argc);
}



/*) sub argn -> Int
	Return the actual number of arguments declared by the Sub.

	@noexc

	@xmp
		sub foo (x = 0)
			return x + 1
		;;

		f = @foo
		println (f.argn)

	@out
		1
*/
static qu_int qu__sub_argn (qu_sub self)
{
	if (qu_sub_is_extern (self))
		return qu_int_newi (self->argc);
	else
		return qu_int_newi (self->narg);
}



/*) sub argnames -> Any
	If the __args decorator is specified then this returns the list of
	argument names in an Array, nil otherwise.

	@noexc

	@xmp
		sub foo (x, y, z) __args
		;;

		f = @foo
		println (f.argnames)

	@out
		['x', 'y', 'z']
*/
qu_any qu__sub_argnames (qu_sub self)
{
	if (!qu_flag_test (self, QU_SUB_FLAG_ARGS)
			|| qu_flag_test (self, QU_SUB_FLAG_JIT | QU_SUB_FLAG_EXTERN))
		return QuNil;

	if (self->attrs && self->narg)
	{
		long   i = 0;
		qu_string k;
		qu_var    v;
		int    n = self->narg;
		qu_array  a = qu_array_new (n);

		QU_OBJECT_LIST_SETVAL (a->objs, QuNil, n);
		while ((v = (qu_var) qu_attr_walk (self->attrs, &k, &i)))
		{
			if (qu_var_is (v) && qu_var_is_arg (v))
			{
				assert (0 <= v->index && v->index < n);
				a->objs [v->index] = v->name;
			}
		}
		a->len = n;
		return a;
	}
	return qu_array_new (0);
}



/*) sub argtypes -> Any
	If any of the arguments are typed then return an Array containing
	those types, nil otherwise. Untyped arguments are reported as nil.

	@noexc

	@xmp
		sub foo (x: String, y, z:Int)
		;;

		f = @foo
		println (f.argtypes)

	@out
		[<Class Sys.String>, nil, <Class Sys.Int>]
*/
qu_any qu__sub_argtypes (qu_sub self)
{
	if (qu_sub_need_arg_check (self) && self->ltype)
	{
		int   n = qu_sub_is_extern (self) ? self->argc : self->narg;
		qu_any  *p = self->ltype;
		qu_array a = qu_array_new (self->narg);

		while (n--)
		{
			qu_any type = *p++;
			if (type == 0)
				type = QuNil;
			qu_array_push (a, type);
		}
		return a;
	}
	return QuNil;
}



/*) sub argcheck (*arg) -> self
	This does the same as #{argmatch} except that if the arguments do not
	match then an exception is thrown.

	Unlike #{argmatch}, if the argument contains validators then the validators
	are actually called.

	@exc
	.EArgValue
	.EArgCount
	.EAbstract
*/
static qu_sub qu__sub_argcheck (qu_sub self, int argc, qu_any *argv)
{
	QU_ENSURE (qu_sub_arg_check (self, argc, argv, 1));
	return self;
}



/*) sub argmatch (*arg) -> Int
	Return 0 if v{arg} is a valid argument list for this Sub, otherwise:
	@ul
	.-1      insufficient argument
	.integer position of the invalid argument, first is 1

	@noexc

	@xmp
		sub foo (a: Int, b: String)
			println (a ' ' b)
		;;
		println ((@foo).argmatch (123, 456))

	@out
		2
*/
qu_int qu__sub_argmatch (qu_sub self, int argc, qu_any *argv)
{
	if (self->argc != -1)
	{
		if (qu_sub_is_vararg (self))
		{
			if (argc < self->argc)
				return QuInt_1;
		}
		else if (qu_sub_is_argmin (self))
		{
			if (argc < self->argc || argc > self->narg)
				return QuInt_1;
		}
		else if (self->argc != argc)
		{
			return QuInt_1;
		}
	}

	if (qu_sub_need_arg_check (self) && self->ltype && argv)
	{
		int  n = qu_sub_is_extern (self) ? self->argc : self->narg;
		int  k = 0;
		qu_any *p = self->ltype;

		while (n-- && argc)
		{
			qu_any type = *p++;
			++k;
			if (type)
			{
/*				if (!qu_object_check_value (type, *argv)) */
				if (!qu_object_validate (self, type, *argv))
					return qu_int_newi (k);
			}
			argv++;
			argc--;
		}
	}
	return QuInt0;
}



/*) sub change (x: Sub) -> Null
	Change the Sub to contain the code of v{x}.

	v{x} must be a Sub with the same signature as self and both v{x} and
	self may not be external functions nor closures.

	@warn
	This is a dangerous operation. Use with care!

	@exc
	.EArgValue

	@xmp
		sub foo (x)
			println (x)
		;;

		f = eval (`
			sub bar (x)
				println ('x = ' x)
			;;
			@bar`)

		foo (10)
		(@foo).change (f)
		foo (10)

	@out
		10
		x = 10
*/
static qu_null qu__sub_change (qu_sub self, qu_sub x)
{
	QU_REQUIRE (QuEArgValue,
		qu_sub_is (x)
		&& !qu_sub_is_extern (x)
		&& !qu_sub_is_extern (self)
		&& !qu_sub_is_closure (x)
		&& !qu_sub_is_closure (x)
		&& x->flags  == self->flags
		&& x->argc   == self->argc
		&& x->narg   == self->narg
		&& x->nlocal == self->nlocal
		&& x->first  == self->first
		&& x->type   == self->type
		&& x->call   == self->call);

	self->root  = x->root;
	self->value = x->value;
	self->tag   = x->tag;

	if (x->ltype)
		self->ltype = qu_mem_copy (x->ltype);
	else
		self->ltype = 0;

	return QuNil;
}



/*) sub code -> Code
	Return the -{Code} object of this Sub.

	@exc
	.ENone the Sub is external or has native JIT code

	@xmp
		sub foo
			return 1
		;;

		f = @foo
		c = f.code
		println (c.__len)

	@out
		12
*/
qu_code qu__sub_code (qu_sub self)
{
	QU_REQUIREZ (QuENone, !qu_sub_is_extern (self) && self->value && qu_code_is (self->value), "Sub has no code");
	return (qu_code) self->value;
}



/*) sub classof -> Class
	Return the Class object where the Sub is declared.

	@exc
	.ENone not a method

	@xmp
		class Foo
			sub static foo
				return 1
			;;

			println ((@foo).classof)
		;;

	@out
		<Class __main__.Foo>
*/
static qu_class qu__sub_classof (qu_sub self)
{
	qu_class c = qu_sub_get_class (self);
	QU_REQUIREZ (QuENone, c, "not a method");
	return c;
}



/*) sub ident -> String
	Return the identifier name.

	@noexc

	@xmp
		sub foo
			return 1
		;;

		f = @foo
		println (f.ident)

	@out
		foo
*/
static qu_string qu__sub_ident (qu_sub self)
{
	return self->name;
}



/*) sub isclass -> Boolean
	Return true if this is a class method, false otherwise.

	@noexc

	@xmp
		class Foo
			sub class foo    #-> isclass
			;;
		;;
*/
static qu_boolean qu__sub_isclass (qu_sub self)
{
	return qu_sub_is_class (self) ? QuTrue : QuFalse;
}



/*) sub isclosure -> Boolean
	Return true if this is a closure false otherwise.

	@noexc

	@xmp
		sub foo (x)
			sub bar (y)
				return x + y
			;;
			return @bar
		;;

		f = foo (10)
		println (f.isclosure)

	@out
		true
*/
static qu_boolean qu__sub_isclosure (qu_sub self)
{
	return qu_sub_is_closure (self) ? QuTrue : QuFalse;
}



/*) sub isextern -> Boolean
	Return true if this is a C Sub.

	@noexc

	@xmp
		sub foo
			return 1
		;;

		f = @foo
		println (f.isextern)

		f = @print
		println (f.isextern)

	@out
		false
		true
*/
static qu_boolean qu__sub_isextern (qu_sub self)
{
	return qu_sub_is_extern (self) ? QuTrue : QuFalse;
}



/*) sub isfunction -> Boolean
	Return true if this is a module function, false otherwise.
	The result of this method is the reverse of #{ismethod}.

	@noexc

	@xmp
		sub foo (x)              #-> isfunction
			return x
		;;

		f = lambda (x) x + 1     #-> isfunction
		f = sub (x) return x ;;  #-> isfunction
*/
static qu_boolean qu__sub_isfunction (qu_sub self)
{
	return qu_sub_get_class (self) ? QuFalse : QuTrue;
}



/*) sub isinst -> Boolean
	Return true if this is an instance method, false otherwise.

	@noexc

	@xmp
		class Foo
		    sub bar  #-> isinst
		    ;;
		;;
*/
static qu_boolean qu__sub_isinst (qu_sub self)
{
	return qu_sub_is_inst (self) ? QuTrue : QuFalse;
}



/*) sub ismethod -> Boolean
	Return true if this is a method, false otherwise.
	The result of this method is the reverse of #{isfunction}.

	@noexc

	@xmp
		class Foo
		    sub class foo   #-> ismethod
		    ;;

		    sub static bar  #-> ismethod
		    ;;

		    sub baz         #-> ismethod
		    ;;
		;;
*/
static qu_boolean qu__sub_ismethod (qu_sub self)
{
	return qu_sub_get_class (self) ? QuTrue : QuFalse;
}



/*) sub isprivate -> Boolean
	Return true if this is a private Sub.

	@noexc
*/
static qu_boolean qu__sub_isprivate (qu_sub self)
{
	return qu_flag_test (self, QU_SUB_FLAG_LOCAL | QU_SUB_FLAG_PRIVATE) ? QuTrue : QuFalse;
}



/*) sub isstatic -> Boolean
	Return true if this is a static Sub, false otherwise.

	A I{static} Sub means that the Sub will have an arbitrary value
	of self.

	@noexc

	@xmp
		sub foo                  #-> static
			return 1
		;;

		f = lambda (x) x + 1     #-> static
		f = sub (x) return x ;;  #-> static

		class Foo

		    sub static foo       #-> static
		    ;;

		    sub local bar        #-> static
		    ;;
		;;
*/
static qu_boolean qu__sub_isstatic (qu_sub self)
{
	return qu_sub_is_static (self) ? QuTrue : QuFalse;
}



/*) sub tag (*args) -> Any
	@def
	sub tag -> Any
	sub tag (items: Dict) -> self
	@p
	Set tag values or get an item in the tag list.

	If getting and the Sub does not have any tag then nil is returned.

	@exc
	.EArgValue

	@xmp
		sub foo (x)
			\_ hello = 0
		;;

		(@foo).tag (bar:1, baz:2)
		println ((@foo).tag)

	@out
		['hello':0, 'bar':1, 'baz':2]
*/
static qu_any qu__sub_tag (qu_sub self, int argc, qu_any *argv)
{
	QU_ARG_VCOUNT (0, 1);
	if (argc == 0)
	{
		qu_dict d = qu_sub_gettag_dict (self);
		if (d == 0)
			return QuNil;
		return d;
	}
	else
	{
		qu_dict d;
		long    b, i;
		qu_any  k, v;

		QU_ARG_VTYPE (0, d, qu_dict);
		b = i = 0;
		while ((v = qu_dict_walk (d, &k, &b, &i)) != 0)
		{
			QU_ARG_TYPE (k, qu_string);
			qu_sub_settag (self, k, v);
		}
		return self;
	}
}



/*) sub typeof -> Any
	Return the type returned by this Sub.

	@exc
	.ENone
*/
static qu_any qu__sub_typeof (qu_sub self)
{
	QU_REQUIREZ (QuENone, self->type, "Sub not typed");
	return self->type;
}



/*----------------------------------------------------------------------------
	interface - creating
----------------------------------------------------------------------------*/

/* !!! this must only create the Sub, it may not set block->attrs */
qu_sub qu_sub_new (qu_symbol block, qu_string name, qu_any value, int argc, long flags)
{
	register qu_sub s;

	assert (block);
	assert (qu_class_is (block) || qu_module_is (block) || qu_sub_is (block));
	assert (-1 <= argc && argc <= 4);

	if (flags & QU_SUB_FLAG_EXTERN)
	{
		s = (qu_sub) qu_gc__new (QuSubClass, sizeof (struct func_s));
		s->call   = &CallExtern;
		s->flags  = flags;
		s->name   = name;
		s->parent = block;
		s->argc   = argc;
		s->value  = value;
		s->type   = 0;
		s->ltype  = 0;
		s->tag    = 0;
	}
	else
	{
		s = (qu_sub) qu_gc__new (QuSubClass, sizeof (struct sub_s));
		s->call   = &CallError;
		s->flags  = flags;
		s->name   = name;
		s->parent = block;
		s->type   = 0;
		s->ltype  = 0;
		s->tag    = 0;
		/*
			Get root of the sub.

			sub foo            -> the root of foo is foo itself
				sub bar        -> the root of bar is foo
					sub baz    -> the root of baz is foo
		*/
		if (qu_class_is (block) || qu_module_is (block))
			s->root = s;
		else
			s->root = ((qu_sub) block)->root;

		s->argc  = argc;
		s->narg  = 0;
		s->value = (value == 0) ? QuNil : value;
		s->first = s->nlocal = 0; /* not a mistake: s->root may be s itself */
		s->first = s->nlocal = s->root->nlocal;
	}
	return s;
}



/*
	sub foo (n)
		return sub bar (x) return n + x ;;
	;;

		--> same as

			sub foo (n)
				sub bar (x) return n + x ;;
				return @bar
			;;


	x = foo (10)

	- create frame foo = f
	- run frame f
	- QU_PC_CLOSURE -> qu_sub_new_closure (bar, f)


	# module/class-level
	c = sub bar (x) return x + 1 ;;

	--> does not create qu_sub_new_closure
		because it is exactly the same as
		sub bar (x) return x + 1 ;;
		c = @bar

	Thus f is the frame used to run the parent sub. It is not neccessarily
	the current frame. So C functions may not call qu_sub_new_closure directly.
*/
qu_sub qu_sub_new_closure (qu_sub self, qu_frame f)
{
	register qu_sub u;
	int n;

	if (qu_sub_is_extern (self))
	{
		if (!qu_sub_is_jit (self))
			return self;
	}

	if (self == self->root)
		return self;

	n = self->nlocal;

	assert (n == self->root->nlocal);

	u = (qu_sub) qu_gc__new (QuSubClass, sizeof (struct closure_s));
	u->call   = self->call;
	u->flags  = self->flags | QU_SUB_FLAG_CLOSURE;
	u->name   = self->name;
	u->parent = self->parent;
	u->root   = self->root;
	u->argc   = self->argc;
	u->type   = self->type;
	u->ltype  = self->ltype;
	u->narg   = self->narg;
	u->value  = self->value;
	u->first  = self->first;
	u->tag    = self->tag;
	u->nlocal = n;
	u->clex   = 0;
	u->corg   = self;

	if (n > 0)
	{
		qu_array a;
		u->clex = a = qu_array_new (n);
		a->len = n;
		QU_OBJECT_LIST_COPY (a->objs, f->ss, n);
	}
	return u;
}



/*
	Get attribute - for the compiler only.
	This function returns a qu_sub or a qu_var because we dont allow declaring
	classes within subs.

	Note that if the attribute does not exist then 0 is returned.
	No exception raised.
*/
qu_symbol qu_sub_getattr (qu_sub self, qu_string name)
{
	if (self->attrs)
		return qu_attr_find (self->attrs, name);
	return 0;
}



qu_any qu_sub_gettag (qu_sub self, qu_string name)
{
	if (self->tag)
		return qu_dict_getitem (self->tag, name);
	return 0;
}



void qu_sub_settag (qu_sub self, qu_string name, qu_any value)
{
	qu_dict d = self->tag;

	if (d == 0)
		d = self->tag = qu_dict_new (4);
	qu_dict_setitem (d, name, value);
}



qu_dict qu_sub_gettag_dict (qu_sub self)
{
	if (self->tag)
		return qu_dict_copy (self->tag);
	return 0;
}


/*
	Reindexing

	                           #                     +-- 1 higher than
	                           #                     |   the actual value
	         #0 #1             #                     |
	sub foo (a, *b)            #-> first=0   nlocal=15   narg=2  argc=1

		      #2
		local c                #-> created by Compile.c

		    #3
		d = 1                  #-> auto-created by Code.c

		         #6 #7 #8
		sub bar (k, l, m)      #-> first=6   nlocal=15   narg=3   argc=3

			      #9
			local p            #-> created by Compile.c

			    #10
			q = 1              #-> auto-created by Code.c

			        #12 #13
			sub baz (x, y)     #-> first=12  nlocal=15   narg=2   argc=2

				      #14
				local z        #-> created by Compile.c
			;;

			    #11
			r = 1              #-> auto-created by Code.c
		;;

		    #4
		u = 1                  #-> auto-created by Code.c
		      #5
		local v                #-> created by Compile.c
	;;
*/
static void _sub_reindex (qu_sub self, int *i)
{
	int       k = *i;
	long      e;
	qu_string s;
	qu_var    v;

	assert (self->attrs);

	self->first = k;
	self->nlocal = self->root->nlocal;

	e = 0;
	while ((v = qu_attr_walk (self->attrs, &s, &e)) != 0)
	{
		if (qu_var_is (v) && qu_var_is_arg (v))
		{
			v->index = k++;
		}
	}
	e = 0;
	while ((v = qu_attr_walk (self->attrs, &s, &e)) != 0)
	{
		if (qu_var_is (v) && !qu_var_is_arg (v))
		{
			v->index = k++;
		}
	}
	*i = k;
	e = 0;
	while ((v = qu_attr_walk (self->attrs, &s, &e)) != 0)
	{
		if (qu_sub_is (v))
		{
			if (((qu_sub) v)->attrs)
				_sub_reindex ((qu_sub) v, i);
			else
			{
				((qu_sub) v)->nlocal = self->nlocal;
			}
		}
	}

#if 1
	{
		long      e;
		qu_var    v;
		qu_string s;

		/* have to do it this way because we run out of flags
			--- should have a QU_SUB_FLAG_LEXICALTYPED or something */
		e = 0;
		while ((v = qu_attr_walk (self->attrs, &s, &e)) != 0)
		{
			if (qu_var_is (v) && v->type)
			{
				self->ltype = qu_mem_calloc (self->nlocal, sizeof (qu_any));
				e = 0;
				while ((v = qu_attr_walk (self->attrs, &s, &e)) != 0)
				{
					if (qu_var_is (v))
						self->ltype[v->index] = v;
						/* we'll change the v to v->type in qu_sub_enddecl */
				}
				break;
			}
		}
	}
#endif

}



/* !!! only Code.c may call this */
void qu_sub_reindex (qu_sub self)
{
	int i = 0;

	assert (self->root == self);
	if (self->attrs)
	{
#if 0
		long   e;
		qu_var    v;
		qu_string s;
#endif
		_sub_reindex (self, &i);

#if 0
		/* have to do it this way because we run out of flags
			--- should have a QU_SUB_FLAG_LEXICALTYPED or something */
		e = 0;
		while ((v = qu_attr_walk (self->attrs, &s, &e)) != 0)
		{
			if (qu_var_is (v) && v->type)
			{
				self->ltype = qu_mem_calloc (self->nlocal, sizeof (qu_any));
				e = 0;
				while ((v = qu_attr_walk (self->attrs, &s, &e)) != 0)
				{
					if (qu_var_is (v))
						self->ltype[v->index] = v;
						/* we'll change the v to v->type in qu_sub_enddecl */
				}
				break;
			}
		}
#endif
	}
}




/* !!! only Code.c may call this */
void qu_sub_enddecl (qu_sub self)
{
	assert (self->nlocal == self->root->nlocal);

	/*
		What we do here is to get rid of all lexical qu_var objects.
		We don't need them at run-time thus freeing a substantial amount
		of memory.

		We keep the arguments type in self->ltype. Not the qu_var object.
	*/
	if (self->ltype)
	{
		int i;

		for (i = 0; i < self->nlocal; i++)
		{
			qu_var v = self->ltype[i];
			if (v)
				self->ltype[i] = v->type;
		}
	}
	/*
		However, its optional via the -x option or the __args decorator
	*/
	if (self->attrs && QU_APP_FREE_LEXICAL)
	{
		if (!qu_flag_test (self, QU_SUB_FLAG_ARGS))
			qu_attr_end_declsub (self);
	}

	assert (!qu_sub_is_extern (self));

	if (qu_sub_is_generator (self))
	{
		self->call = &CallGenerator;
	}
	else if (qu_sub_is_once (self))
	{
		if (((qu_code) self->value)->stacksize < QU_FRAME_CSTACK_MAX)
			self->call = &CallOnceCFrame;
		else
			self->call = &CallOnceAFrame;
	}
	else if (qu_flag_test (self, QU_SUB_FLAG_INTERFACE))
	{
		self->call = &CallInterface;
	}
	else
	{
		if (((qu_code) self->value)->stacksize < QU_FRAME_CSTACK_MAX)
			self->call = &CallNonGeneratorCFrame;
		else
			self->call = &CallNonGeneratorAFrame;
	}
}



/* !!! only JIT Generate may call this */
void qu_sub_jit_set (qu_sub self)
{
	qu_code c = (qu_code) self->value;

	qu_flag_set (self, QU_SUB_FLAG_JIT);
	qu_flag_set (c, QU_CODE_FLAG_JIT);

	if (c->stacksize < QU_FRAME_CSTACK_MAX)
		self->call = &CallNonGeneratorCFrameJit;
	else
		self->call = &CallNonGeneratorAFrameJit;
}



/* !!! only JIT Generate may call this */
void qu_sub_jit_set_extern (qu_sub self)
{
	qu_flag_set (self, QU_SUB_FLAG_EXTERN | QU_SUB_FLAG_JIT);
	self->call = &CallExtern;
	if (self->attrs)
	{
		qu_attr_free (self->attrs);
		self->attrs = 0;
	}

	/* dont do this!
	if (self->ltype)
		qu_mem_free (self->ltype); */
}



void qu_sub_set_once (qu_sub self, qu_any result)
{
	self->value = result;
	self->call = &CallOnceCalled;
	if (self->ltype)
	{
		qu_mem_free (self->ltype);
		self->ltype = 0;
	}
	qu_flag_unset (self, QU_SUB_FLAG_ARGCHECK);
}



/*
	Set attribute - for the compiler only.
	This function never fails thus the compiler must call qu_sub_getattr to
	make sure there are no duplicate symbols.
*/
void qu_sub_setattr (qu_sub self, qu_string name, qu_symbol thing)
{
	assert (!qu_sub_is_extern (self));

	if (self->attrs == 0)
		self->attrs = qu_attr_alloc ();

	if (qu_var_is (thing))
	{
		qu_var v = (qu_var) thing;

		v->flags |= QU_VAR_FLAG_LOCAL | QU_VAR_FLAG_PRIVATE;

		if (!qu_var_is_static (v))
			v->flags |= QU_VAR_FLAG_LEXICAL;

		v->index = self->root->nlocal++;
		if (qu_var_is_arg (v))
			self->narg++;
		if (qu_var_is_vararg (v))
			qu_flag_set (self, QU_SUB_FLAG_VARARG);
	}
	else
	{
		assert (qu_sub_is (thing));
		qu_flag_set (thing, QU_SUB_FLAG_PRIVATE | QU_SUB_FLAG_STATIC);
	}
	qu_attr_set (self->attrs, name, thing);
}



qu_sub qu_sub_arg_check (qu_sub self, int argc, qu_any *argv, int runtime)
{
	if (self->argc != -1)
	{
		if (qu_sub_is_vararg (self))
		{
			if (argc < self->argc)
			{
				qu_string s = qu_object_where_string (self, "", "");
				QU_THROWF (QuEArgCount, "%s needs at least %d argument(s) but %d given",
					s->str, self->argc, argc);
			}
		}
		else if (qu_sub_is_argmin (self))
		{
			assert (!qu_sub_is_extern (self));

			if (argc < self->argc)
			{
				qu_string s = qu_object_where_string (self, "", "");
				QU_THROWF (QuEArgCount, "%s needs at least %d argument(s) but %d given",
					s->str, self->argc, argc);
			}
			else if (argc > self->narg)
			{
				qu_string s = qu_object_where_string (self, "", "");
				QU_THROWF (QuEArgCount, "%s needs at most %d argument(s) but %d given",
					s->str, self->argc, argc);
			}
		}
		else if (argc != self->argc)
		{
			qu_string s = qu_object_where_string (self, "", "");
			QU_THROWF (QuEArgCount, "%s needs %d argument(s) but %d given",
					s->str, self->argc, argc);
		}
	}

	if (qu_sub_need_arg_check (self) && self->ltype && argv)
	{
		int  n = qu_sub_is_extern (self) ? self->argc : self->narg;
		qu_any *p = self->ltype;
		int  k = 1;

		while (n && argc)
		{
			qu_any type = *p++;
			if (type)
			{
				if (runtime)
				{
					qu_any r = qu_object_validate (QuNil, type, *argv);
					if (r == 0)
					{
						qu_string s = qu_object_where_string (self, "", "");
						QU_ARG_THROWF ("mismatch argument %d for %s", k, s->str);
					}

	/* CHECKME
	 	I am doing something dangerous here.
		I am changing the value of the arguments passed.
	*/
	*argv = r;

				}
				else
				{
					if (!qu_object_check_value (type, *argv))
					{
						qu_string s = qu_object_where_string (self, "", "");
						QU_ARG_THROWF ("mismatch argument %d for %s", k, s->str);
					}
				}
			}
			k++;
			n--;
			argv++;
			argc--;
		}
	}
	return self;
}



/*----------------------------------------------------------------------------
	call extern
----------------------------------------------------------------------------*/

#undef SUB_SHOW_EXTERNAL_CALLS

qu_any _sub_call_en (qu_sub self, qu_any theself, int argc, qu_any *argv)
{
	assert (qu_sub_is_extern (self));

#ifdef QU_DEBUG_SEGV
	__qu_sub_trace (self);
#endif

#ifdef SUB_SHOW_EXTERNAL_CALLS
	{
		qu_module m = qu_sub_get_module (self);
		qu_class  c = qu_sub_get_class (self);
		qu_any   *p = argv;
		int    i;

		if (c)
			fprintf (stderr, "::: %s.%s.%s (", m->name->str, c->name->str, self->name->str);
		else
			fprintf (stderr, "::: %s.%s (", m->name->str, self->name->str);
		for (i = 0; i < argc; i++, p++)
		{
			if (i)
				fprintf (stderr, ", ");
			c = qu_classof (*p);
			fprintf (stderr, "%s", c->name->str);
		}
		fprintf (stderr, ")\n");
	}
#endif

	QU_ENSURE (qu_sub_arg_check (self, argc, argv, 1));

	if (qu_sub_is_cfunc (self))
	{
		switch (self->argc)
		{
			case  0: return ((qu_subc0_f) self->value) ();
			case  1: return ((qu_subc1_f) self->value) (argv[0]);
			case  2: return ((qu_subc2_f) self->value) (argv[0], argv[1]);
			case  3: return ((qu_subc3_f) self->value) (argv[0], argv[1], argv[2]);
			case  4: return ((qu_subc4_f) self->value) (argv[0], argv[1], argv[2], argv[3]);
			default: return ((qu_subcn_f) self->value) (argc, argv);
		}
	}
	else
	{
		switch (self->argc)
		{
			case  0: return ((qu_sub0_f) self->value) (theself);
			case  1: return ((qu_sub1_f) self->value) (theself, argv[0]);
			case  2: return ((qu_sub2_f) self->value) (theself, argv[0], argv[1]);
			case  3: return ((qu_sub3_f) self->value) (theself, argv[0], argv[1], argv[2]);
			case  4: return ((qu_sub4_f) self->value) (theself, argv[0], argv[1], argv[2], argv[3]);
			default: return ((qu_subn_f) self->value) (theself, argc, argv);
		}
	}
	QU_FATAL ("never");
}



qu_any _sub_call_e4 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3, qu_any a4)
{
	qu_any argv[4] = {a1, a2, a3, a4};
	return _sub_call_en (self, theself, 4, argv);
}



qu_any _sub_call_e3 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3)
{
	qu_any argv[3] = {a1, a2, a3};
	return _sub_call_en (self, theself, 3, argv);
}



qu_any _sub_call_e2 (qu_sub self, qu_any theself, qu_any a1, qu_any a2)
{
	qu_any argv[2] = {a1, a2};
	return _sub_call_en (self, theself, 2, argv);
}



qu_any _sub_call_e1 (qu_sub self, qu_any theself, qu_any a1)
{
	return _sub_call_en (self, theself, 1, &a1);
}



qu_any _sub_call_e0 (qu_sub self, qu_any theself)
{
	return _sub_call_en (self, theself, 0, 0);
}



/*----------------------------------------------------------------------------
	call __interface
----------------------------------------------------------------------------*/

static qu_dict Interfaces;
#define INTERFACECHECKPERIOD 3
#define INTERFACEMAXENTRY    100


static qu_module _sub_load_interface (qu_sub self)
{
	static qu_string qu = 0;
	static double lasttime = 0;

	qu_class  c, u;
	qu_module m;
	qu_string dir, path, s = 0;

	if (qu == 0)
		qu = qu_string_new_staticz (".qu");

	c = u = (qu_class) self->parent;
	assert (qu_class_is (c));

	while (s == 0)
	{
		m = qu_class_get_module (c);
		assert (m && qu_module_is (m));
		dir = qu_file_dir (m->name);
		path = qu_file_make (c->name, dir, qu);

		m = qu_dict_getitem (Interfaces, path);
		if (m)
		{
			double now = qu_stream_now ();
			if (now > lasttime + INTERFACECHECKPERIOD && qu_file_modified (path, lasttime))
			{
				lasttime = now;
				qu_dict_del (Interfaces, path);
			}
			else
			{
				lasttime = now;
				return m;
			}
		}

		s = qu_file_load (path);
		if (s)
			break;

		u = u->super;
		if (u == 0)
			QU_THROWF (QuEImplement, "interface %s.%s not implemented", c->name->str, self->name->str);
	}

	m = qu_interp_eval (qu_interp_get_module (), s, 0, QU_INTERP_EVAL_MODULE | QU_INTERP_EVAL_TRUSTED, path);
	if (m == 0)
		QU_THROWF (QuESystem, "interface %s.%s (%s) error: %s", c->name->str, self->name->str, path->str, qu_thread_err_string()->str);

	if (Interfaces->len >= INTERFACEMAXENTRY)
		qu_dict_del_any (Interfaces);
	qu_dict_setitem (Interfaces, path, m);
	return m;
}



qu_any _sub_call_interface_n (qu_sub self, qu_any theself, int argc, qu_any *argv)
{
	qu_module m;
	qu_any    u;
	qu_class  c;

	m = _sub_load_interface (self);

	if (m == 0)
	{
		if (SubInterfaceError)
			return qu_sub_call1 (SubInterfaceError, theself, self->name);
		QU_RETHROW;
	}

	u = qu_attr_find (m->attrs, self->name);
	if (u)
	{
		if (qu_sub_is (u))
		{
			qu_any r = qu_sub_calln (u, theself, argc, argv);
			if (r == 0)
			{
				if (SubInterfaceError)
					return qu_sub_call1 (SubInterfaceError, theself, self->name);
				QU_RETHROW;
			}
			return r;
		}
		/* allow "calling" a variable with no arguments */
		else if (qu_var_is (u) && argc == 0)
		{
			return ((qu_var) u)->value;
		}
	}
	if (SubInterfaceError)
		return qu_sub_call1 (SubInterfaceError, theself, self->name);
	else
	{
		c = (qu_class) self->parent;
		assert (c && qu_class_is (c));
		QU_THROWF (QuEImplement, "interface %s.%s not implemented", c->name->str, self->name->str);
	}
}



qu_any _sub_call_interface_4 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3, qu_any a4)
{
	qu_any argv[4] = {a1, a2, a3, a4};
	return _sub_call_interface_n (self, theself, 4, argv);
}



qu_any _sub_call_interface_3 (qu_sub self, qu_any theself, qu_any a1, qu_any a2, qu_any a3)
{
	qu_any argv[3] = {a1, a2, a3};
	return _sub_call_interface_n (self, theself, 3, argv);
}



qu_any _sub_call_interface_2 (qu_sub self, qu_any theself, qu_any a1, qu_any a2)
{
	qu_any argv[2] = {a1, a2};
	return _sub_call_interface_n (self, theself, 2, argv);
}



qu_any _sub_call_interface_1 (qu_sub self, qu_any theself, qu_any a1)
{
	return _sub_call_interface_n (self, theself, 1, &a1);
}



qu_any _sub_call_interface_0 (qu_sub self, qu_any theself)
{
	return _sub_call_interface_n (self, theself, 0, 0);
}



/*----------------------------------------------------------------------------
	interface - others
----------------------------------------------------------------------------*/

qu_module qu_sub_get_module (qu_sub self)
{
	qu_any b;

	if (qu_sub_is_extern (self))
		b = self->parent;
	else
		b = self->root->parent;

	if (qu_class_is (b))
		return qu_class_get_module ((qu_class) b);
	else
	{
		assert (qu_module_is (b));
		return (qu_module) b;
	}
}



qu_class qu_sub_get_class (qu_sub self)
{
	qu_any b;

	if (qu_sub_is_extern (self))
		b = self->parent;
	else
		b = self->root->parent;

	if (qu_class_is (b))
		return (qu_class) b;
	else
		return 0;
}



int qu_sub_isroot  (qu_sub self)
{
	return qu_sub_is_extern (self) || (self->root == self);
}



int qu_sub_isparent (qu_sub self, qu_sub p)
{
	if (!qu_sub_is_extern (self))
	{
		while ((self = (qu_sub) self->parent) && qu_sub_is (self))
		{
			if (self == p)
				return 1;
		}
	}
	return 0;
}



/*----------------------------------------------------------------------------
	interface - argument
----------------------------------------------------------------------------*/

void qu_arg__throw_boolean (const char *cfunc)
{
	qu_string msg = qu_string_newz ("Boolean argument required");
	qu_thread__throw (QuEArgValueClass, msg, cfunc);
}



void qu_arg__throw_integer (const char *cfunc)
{
	qu_string msg = qu_string_newz ("not an integer argument or value out of range");
	qu_thread__throw (QuEArgValueClass, msg, cfunc);
}



void qu_arg__throw_float (const char *cfunc)
{
	qu_string msg = qu_string_newz ("not an float argument or value out of range");
	qu_thread__throw (QuEArgValueClass, msg, cfunc);
}



void qu_arg__throw_enum (const char *cfunc, qu_var v)
{
	qu_string msg = qu_string_newf ("invalid enum value %s", v->name->str);
	qu_thread__throw (QuEArgValueClass, msg, cfunc);
}



void qu_arg__throw_count (const char *cfunc, int argc, int min, int max)
{
	qu_string msg = qu_string_newf ("%d to %d arguments required %d given", min, max, argc);
	qu_thread__throw (QuEArgCountClass, msg, cfunc);
}



void qu_arg__throw_min (const char *cfunc, int argc, int min)
{
	qu_string msg = qu_string_newf ("at least %d argument(s) required %d given", min, argc);
	qu_thread__throw (QuEArgCountClass, msg, cfunc);
}



void qu_arg__throw_max (const char *cfunc, int argc, int max)
{
	qu_string msg = qu_string_newf ("at most %d argument(s) required %d given", max, argc);
	qu_thread__throw (QuEArgCountClass, msg, cfunc);
}



void qu_arg__throw_exact (const char *cfunc, int argc, int n)
{
	qu_string msg = qu_string_newf ("%d argument(s) required %d given", n, argc);
	qu_thread__throw (QuEArgCountClass, msg, cfunc);
}



/* this is similar as qu_pointer_set */
static int _sub_setarg (void *p, char type, qu_any x, const char *funcname)
{
	switch (type)
	{
		case QU_STRUCT_ID_char:
			if (qu_string_is (x) && ((qu_string) x)->len > 0)
			{
				*((char *) p) = ((qu_string) x)->str[0];
				return 0;
			}
			break;


		case QU_STRUCT_ID_byte:
		{
			int v = qu_number_toi (x);
			if (errno == 0 && -127 <= v && v <= 127)
			{
				*((int *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_ubyte:
		{
			unsigned int v = qu_number_toui (x);
			if (errno == 0 && v <= 255)
			{
				*((unsigned char *) p) = (unsigned char) v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_short:
		{
			short v = qu_number_toh (x);
			if (errno == 0)
			{
				*((short *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_ushort:
		{
			unsigned short v = qu_number_touh (x);
			if (errno == 0)
			{
				*((unsigned short *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_int:
		{
			int v = qu_number_toi (x);
			if (errno == 0)
			{
				*((int *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_uint:
		{
			unsigned int v = qu_number_toui (x);
			if (errno == 0)
			{
				*((unsigned int *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_int16:
		{
#if SIZEOF_SHORT == 2
			short v = qu_number_toh (x);
			if (errno == 0)
			{
				*((short *) p) = v;
				return 0;
			}
#endif
			break;
		}

		case QU_STRUCT_ID_uint16:
		{
#if SIZEOF_SHORT == 2
			unsigned short v = qu_number_touh (x);
			if (errno == 0)
			{
				*((unsigned short *) p) = v;
				return 0;
			}
#endif
			break;
		}


		case QU_STRUCT_ID_int32:
		{
#if SIZEOF_INT == 4
			int v = qu_number_toi (x);
			if (errno == 0)
			{
				*((int *) p) = v;
				return 0;
			}
#endif
			break;
		}

		case QU_STRUCT_ID_uint32:
		{
#if SIZEOF_INT == 4
			unsigned int v = qu_number_toui (x);
			if (errno == 0)
			{
				*((unsigned int *) p) = v;
				return 0;
			}
#endif
			break;
		}

		case QU_STRUCT_ID_long:
		{
			long v = qu_number_tol (x);
			if (errno == 0)
			{
				*((long *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_ulong:
		{
			unsigned long v = qu_number_toul (x);
			if (errno == 0)
			{
				*((unsigned long *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_longlong:
		{
#ifdef HAVE_LONG_LONG
			long long v = qu_number_toll (x);
			if (errno == 0)
			{
				*((long long *) p) = v;
				return 0;
			}
#endif
			break;
		}

		case QU_STRUCT_ID_ulonglong:
		{
#ifdef HAVE_LONG_LONG
			unsigned long long v = qu_number_toll (x);
			if (errno == 0)
			{
				*((unsigned long long *) p) = v;
				return 0;
			}
#endif
			break;
		}

		case QU_STRUCT_ID_float:
		{
			double v = qu_number_tod (x);
			if (errno == 0)
			{
				double f = v;

				if (v < 0)
					f = -v;

				if ((double) FLT_MIN <= f && f <= (double) FLT_MAX)
				{
					*((float *) p) = (float) v;
					return 0;
				}
			}
			break;
		}


		case QU_STRUCT_ID_double:
		{
			double v = qu_number_tod (x);
			if (errno == 0)
			{
				*((double *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_longdouble:
		{
#ifdef HAVE_LONG_DOUBLE
			long double v = qu_number_told (x);
			if (errno == 0)
			{
				*((long double *) p) = v;
				return 0;
			}
#endif
			break;
		}

		case QU_STRUCT_ID_ssize_t:
		{
#if SIZEOF_SIZE_T == SIZEOF_INT
			int v = qu_number_toi (x);
#elif SIZEOF_SIZE_T == SIZEOF_LONG
			long v = qu_number_tol (x);
#else
            long long v = qu_number_toll (x);
#endif
			if (errno == 0)
			{
				*((ssize_t *) p) = (ssize_t) v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_size_t:
		{
#if SIZEOF_SIZE_T == SIZEOF_INT
			unsigned int v = qu_number_toui (x);
#elif SIZEOF_SIZE_T == SIZEOF_LONG
			unsigned long v = qu_number_toul (x);
#else
            unsigned long long v = qu_number_toull (x);
#endif
			if (errno == 0)
			{
				*((size_t *) p) = (size_t) v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_off_t:
		{
#if SIZEOF_OFF_T == SIZEOF_INT
			int v = qu_number_toi (x);
#elif SIZEOF_OFF_T == SIZEOF_LONG
			long v = qu_number_tol (x);
#else
            long long v = qu_number_toll (x);
#endif
			if (errno == 0)
			{
				*((off_t *) p) = (off_t) v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_Null:
			if (x == QuNil)
			{
				*((void **) p) = NULL;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Boolean:
			if (qu_boolean_is (x))
			{
				*((int *) p) = (qu_sys_true (x) == QuTrue);
				return 0;
			}
			break;


		case QU_STRUCT_ID_Int:
		{
			long v = qu_number_toli (x);
			if (errno == 0)
			{
				*((long *) p) = v;
				return 0;
			}
			break;
		}


		case QU_STRUCT_ID_String:
			if (qu_string_is (x))
			{
				*((qu_string *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Stringy:
			if (qu_stringy_is (x))
			{
				*((qu_stringy *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Buffer:
			if (qu_buffer_is (x))
			{
				*((qu_buffer *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Pointer:
			if (qu_pointer_is (x))
			{
				*((qu_pointer *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Array:
			if (qu_array_is (x))
			{
				*((qu_array *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Dict:
			if (qu_dict_is (x))
			{
				*((qu_dict *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Set:
			if (qu_set_is (x))
			{
				*((qu_set *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Range:
			if (qu_range_is (x))
			{
				*((qu_range *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Class:
			if (qu_class_is (x))
			{
				*((qu_class *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Function:
			if (qu_function_is (x))
			{
				*((qu_function *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Sub:
			if (qu_sub_is (x))
			{
				*((qu_sub *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Method:
			if (qu_method_is (x))
			{
				*((qu_method *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Queue:
			if (qu_queue_is (x))
			{
				*((qu_queue *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Stream:
			if (qu_stream_is (x))
			{
				*((qu_stream *) p) = x;
				return 0;
			}
			break;


		case QU_STRUCT_ID_Proc:
			if (qu_proc_is (x))
			{
				*((qu_proc *) p) = x;
				return 0;
			}
			break;


#ifdef QU_SUPPORT_GLIB
		case QU_STRUCT_ID_GObject:
			if (QuGObjectClass && qu_gobject_is (x))
			{
				*((qu_gobject *) p) = x;
				return 0;
			}
			break;
#endif


		case QU_STRUCT_ID_Any:
			*((qu_any *) p) = x;
			return 0;


		default:
			QU_FATAL ("%s: '%c' invalid arg spec", funcname, type);
	}
	return 1;
}



int qu_arg_vparse (const char *funcname, const char *spec, int argc, qu_any *argv, ...)
{
	va_list va;
	char    type;
	void   *v;
	qu_any  x;
	int     req = 0, got = 0, err = 0, k = 1;

	va_start (va, argv);
	for (;;)
	{
		type = *spec++;
		if (type == ':')
		{
			k = 0;
			continue;
		}
		req += k;

		v = va_arg (va, void *);

		if (argc == 0)
			continue;
		x = *argv++;
		argc--;

		got++;
		err = _sub_setarg (v, type, x, funcname);
	}
	va_end (va);

	if (err)
	{
		qu_thread__throw (QuEValueClass, 0, funcname);
		return 0;
	}
	if (got < req)
	{
		qu_thread__throw (QuEArgCountClass, 0, funcname);
		return 0;
	}
	return 1;
}




int qu_arg_xparse (const char *funcname, char **argx, int argc, qu_any *argv, ...)
{
	va_list va;
	char   *name, type;
	void   *v;
	qu_any  x;
	int     req = 0, got = 0, err = 0, k = 1;
	int     isarg = (argc == 1) && qu_arg_is (argv[0]);
	qu_dict    d = 0;

	if (isarg)
		d = argv[0];

	va_start (va, argv);
	for (;;)
	{
		name = *argx++;
		if (name == NULL)
			break;

		type = name[0];
		if (type == ':')
		{
			k = 0;
			continue;
		}
		name = *argx++;
		req += k;

		v = va_arg (va, void *);

		if (isarg)
		{
			x = qu_dict_getitemz (d, name, strlen (name));
			if (x == 0)
				continue;
		}
		else
		{
			if (argc == 0)
				continue;
			x = *argv++;
			argc--;
		}

		got++;
		err = _sub_setarg (v, type, x, funcname);
	}
	va_end (va);

	if (err)
	{
		qu_thread__throw (QuEValueClass, 0, funcname);
		return 0;
	}
	if (got < req)
	{
		qu_thread__throw (QuEArgCountClass, 0, funcname);
		return 0;
	}
	return 1;
}




int qu_arg_dparse (const char *funcname, char **argx, qu_dict argd, ...)
{
	va_list va;
	char   *name, type;
	void   *v;
	qu_any  x;
	int     req = 0, got = 0, err = 0, k = 1;

	assert (qu_dict_is (argd));

	va_start (va, argd);
	for (;;)
	{
		name = *argx++;
		if (name == NULL)
			break;
		type = name[0];
		if (type == ':')
		{
			k = 0;
			continue;
		}
		name = *argx++;
		req += k;

		v = va_arg (va, void *);

		x = qu_dict_getitemz (argd, name, strlen (name));
		if (x == 0)
			continue;

		got++;
		err = _sub_setarg (v, type, x, funcname);
	}
	va_end (va);

	if (err)
	{
		qu_thread__throw (QuEValueClass, 0, funcname);
		return 0;
	}
	if (got < req)
	{
		qu_thread__throw (QuEArgCountClass, 0, funcname);
		return 0;
	}
	return 1;
}




int qu_arg_validate (qu_any type, int argc, qu_any *argv)
{
	while (argc--)
	{
		if (!qu_object_validate (QuNil, type, *argv++))
			return 0;
	}
	return 1;
}



int qu_sub_can_access (qu_sub self, qu_symbol from)
{
	qu_symbol owner = self->parent;

	if (owner == from)
		return 1;

	if (qu_sub_is_local (self))
	{
		if (qu_module_is (self->parent))
		{
			/* module local function
				--- accessible from anywhere within the module */

			if (qu_module_is (from))
			{
				return 0;
			}
			else if (qu_class_is (from))
			{
				from = (qu_symbol) ((qu_class) from)->parent;
			}
			else
			{
				assert (qu_sub_is (from));
				from = (qu_symbol) qu_sub_get_module ((qu_sub) from);
			}
			return owner == from;
		}
		else if (qu_class_is (self->parent))
		{
			/* class local function
				--- accessible from within the class itself */

			if (qu_module_is (from))
			{
				return 0;
			}
			else if (qu_class_is (from))
			{
				/* do nothing */
			}
			else
			{
				assert (qu_sub_is (from));
				from = (qu_symbol) qu_sub_get_class ((qu_sub) from);
				if (from == 0)
					return 0;
			}
			return qu_class_isa ((qu_class) from, (qu_class) self->parent);
		}
		else
		{
			/* child function
				--- accessible from parent only */

			assert (qu_sub_is (self->parent));
			if (self->parent == owner)
				return 1;
		}
	}
	if (!qu_sub_is_private (self))
	{
		return 1;
	}
	else
	{
		if (qu_module_is (owner))
		{
			if (qu_module_is (from))
			{
				return 0;
			}
			else if (qu_class_is (from))
			{
				from = (qu_symbol) ((qu_class) from)->parent;
			}
			else
			{
				assert (qu_sub_is (from));
				from = (qu_symbol) qu_sub_get_module ((qu_sub) from);
			}
			return owner == from;
		}
		else if (qu_class_is (owner))
		{
			switch (qu_classid (from))
			{
				case QU_CLASS_ID_Sub:
					from = (qu_symbol) qu_sub_get_class ((qu_sub) from);
					if (from == 0 || from == owner)
						break;
					/*nobreak*/
				case QU_CLASS_ID_Class:
					return qu_class_isa ((qu_class) from, (qu_class) owner);
			}
		}
		return 0;
	}
}



/*----------------------------------------------------------------------------
	debug
----------------------------------------------------------------------------*/

#ifdef QU_DEBUG_SEGV

#define TRACEMAX 200
#define TRACELEN 512

static int   TraceIndex = -1;
static int   TraceLen = 0;
static char *TraceList [TRACEMAX];

void __qu_sub_trace (qu_sub u)
{
	char *s;

	if (++TraceLen >= TRACEMAX)
		TraceLen = TRACEMAX;

	if (++TraceIndex >= TRACEMAX)
		TraceIndex = 0;

	s = TraceList [TraceIndex];
	sprintf (s, QU_THREAD_IDFORMAT " ", qu_thread_id ());
	s += strlen (s);

	if (qu_sub_is_jit (u))
	{
		*s++ = ':';
		*s++ = ' ';
	}
	else if (qu_sub_is_extern (u))
	{
		*s++ = '*';
		*s++ = ' ';
	}
	else
	{
		*s++ = ' ';
		*s++ = ' ';
	}

	if (u->parent)
	{
		qu_string m = u->parent->name;

		memcpy (s, m->str, m->len);
		s += m->len;
		*s++ = '.';
	}

	memcpy (s, u->name->str, u->name->len);
	s += u->name->len;
	*s = '\0';
	assert (s < TraceList [TraceIndex] + TRACELEN);
}


void __qu_sub_trace_show ()
{
	char t[64], *z;
	int n = TraceLen;
	int i = TraceIndex;
	int l;

	sprintf (t, QU_THREAD_IDFORMAT " ", qu_thread_id ());
	l = strlen (t);

	fprintf (stderr, "----------TRACE---------\n");
	while (n--)
	{
		z = TraceList [i];
/*		if (strncmp (t, z, l) == 0) */
			fprintf (stderr, "%s\n", z);

		if (--i < 0)
			i = TRACEMAX - 1;
	}
	fprintf (stderr, "------------------------\n");
}

#endif /*QU_DEBUG_SEGV*/


/*----------------------------------------------------------------------------
	init
----------------------------------------------------------------------------*/

qu_sub QuSubYield;


void qu_sub__setup ()
{
	qu_class_setup_sys (QuSubClass, qu_sub_s, qu__sub__destroy, qu__sub__mark);

#ifdef QU_DEBUG_SEGV
	{
		int i;
		for (i = 0; i < TRACEMAX; ++i)
		{
			TraceList [i] = calloc (1, TRACELEN);
			assert (TraceList [i]);
		}
	}
#endif

}



void qu_sub__init ()
{
	/* specific class methods */
	qu_class_method_classz (QuSubClass, "interface_error", qu__sub_interface_error, -1);

	/* standard instance methods */
	qu_class_method_inst  (QuSubClass, QU_NAME__cmp     , qu__sub__cmp      ,  1);
	qu_class_method_inst  (QuSubClass, QU_NAME__getitem , qu__sub__getitem  ,  1);
	qu_class_method_inst  (QuSubClass, QU_NAME__getslice, qu__sub__getslice ,  2);
	qu_class_method_inst  (QuSubClass, QU_NAME__print   , qu__sub__print    ,  0);
	qu_class_method_inst  (QuSubClass, QU_NAME__push    , qu__sub__push     ,  1);
	qu_class_method_inst  (QuSubClass, QU_NAME__setitem , qu__sub__setitem  ,  2);
	qu_class_method_inst  (QuSubClass, QU_NAME__string  , qu__sub__string   ,  0);

	/* specific instance methods */
	qu_class_method_inst  (QuSubClass, QU_NAME_classof  , qu__sub_classof   ,  0);
	qu_class_method_inst  (QuSubClass, QU_NAME_ident    , qu__sub_ident     ,  0);
	qu_class_method_instz (QuSubClass, "argc"           , qu__sub_argc      ,  0);
	qu_class_method_instz (QuSubClass, "argn"           , qu__sub_argn      ,  0);
	qu_class_method_instz (QuSubClass, "argnames"       , qu__sub_argnames  ,  0);
	qu_class_method_instz (QuSubClass, "argtypes"       , qu__sub_argtypes  ,  0);
	qu_class_method_instz (QuSubClass, "argcheck"       , qu__sub_argcheck  , -1);
	qu_class_method_instz (QuSubClass, "argmatch"       , qu__sub_argmatch  , -1);
	qu_class_method_instz (QuSubClass, "change"         , qu__sub_change    ,  1);
	qu_class_method_instz (QuSubClass, "code"           , qu__sub_code      ,  0);
	qu_class_method_instz (QuSubClass, "isclass"        , qu__sub_isclass   ,  0);
	qu_class_method_instz (QuSubClass, "isclosure"      , qu__sub_isclosure ,  0);
	qu_class_method_instz (QuSubClass, "isextern"       , qu__sub_isextern  ,  0);
	qu_class_method_instz (QuSubClass, "isfunction"     , qu__sub_isfunction,  0);
	qu_class_method_instz (QuSubClass, "isinst"         , qu__sub_isinst    ,  0);
	qu_class_method_instz (QuSubClass, "ismethod"       , qu__sub_ismethod  ,  0);
	qu_class_method_instz (QuSubClass, "isprivate"      , qu__sub_isprivate ,  0);
	qu_class_method_instz (QuSubClass, "isstatic"       , qu__sub_isstatic  ,  0);
	qu_class_method_instz (QuSubClass, "tag"            , qu__sub_tag       , -1);
	qu_class_method_instz (QuSubClass, "typeof"         , qu__sub_typeof    ,  0);

	/* interface */
	Interfaces = qu_dict_new (8);
	qu_class_prop (QuSubClass, qu_string_new_staticz ("__interfaces__"),
		Interfaces, QU_VAR_FLAG_FINAL | QU_VAR_FLAG_CLASS | QU_VAR_FLAG_PRIVATE | QU_VAR_FLAG_LOCAL);

	/* finalize declaration */
	qu_class_set_super (QuSubClass, QuFunctionClass);
	qu_class_enddecl   (QuSubClass);
}
