/* err.c - Copyright (c) Marc Krisnanto */

#include "qu.h"


static qu_thread_mutex_t Locker = QU_THREAD_MUTEX_INITIALIZER;
#define LOCKME   qu_thread_mutex_lock   (&Locker)
#define UNLOCKME qu_thread_mutex_unlock (&Locker)


static char *Prefix[] =
{
	"REMARK",
	"WARNING",
	"EXPERIMENTAL",
	"DEPRECATED",
	"FATAL"
};


void qu_err_do (const char *file, const char *func, int kind, const char *fmt, ...)
{
	va_list va;
	int     eno = errno;

	if (kind < 0)
		kind = 0;
	else if (kind > 4)
		kind = 4;
	LOCKME;
	fseek (stderr, 0, SEEK_END);
	fprintf (stderr, "%s [" QU_THREAD_IDFORMAT "] %s::%s ", Prefix [kind], qu_thread_id (), file, func);
	va_start (va, fmt);
	vfprintf (stderr, fmt, va);
	va_end (va);
	fprintf (stderr, "\n");
	fflush (stderr);
	UNLOCKME;
	if (kind >= 4)
	{
		fprintf (stderr, QU_APP_REPORT_BUG);
		abort ();
	}
	errno = eno;
}
