/* qu.c - Copyright (c) Marc Krisnanto */

#include "qu.h"


int   QU_APP_DEBUG              = 0;
int   QU_APP_VERBOSE            = 0;
int   QU_APP_LINE_NUMBER        = 0;
int   QU_APP_INCLUDE_TEST       = 0;
int   QU_APP_NO_WARNING         = 1;
int   QU_APP_KEEP_TREE          = 0;
int   QU_APP_OPTIMIZE_CODE      = 0;
int   QU_APP_WITH_DOC           = 0;
int   QU_APP_FREE_LEXICAL       = 1;
int   QU_APP_NOVALIDATION       = 0;
int   QU_APP_MORE_COMPILE_CHECK = 0;
int   QU_APP_DAEMON             = 0;
char *QU_APP_USEPACKAGE         = 0;


const char *QU_APP_HOMEDIR = QU_APP_PREFIX QU_PATH_SEPSTR QU_APP_PACKAGE;


static void _qu_usage ()
{
	puts
	(
		"\nUsage\n"
		"    " QU_APP_EXENAME " [Options] Filename [args]\n"
		"\nOptions\n"
		"    -a  Initially disable garbage collector.\n"
		"    -d  Set __debug__ to true also activates assert.\n"
		"    -e  Initially enable trace, implies -l.\n"
		"    -f  Ignore finalizers.\n"
		"    -g  Do not block signal.\n"
		"    -h  Display this help then exit.\n"
		"    -l  Generate line numbers.\n"
		"    -o  Set __verbose__ to print.\n"
		"    -p  Do not free parsed tree.\n"
		"    -r  Compile only do not run.\n"
		"    -t  Do not ignore code below __test__.\n"
		"    -v  Display version then exit.\n"
		"    -x  Do not free lexicals after compiling.\n"
		"    -C  Do more compile-time checking.\n"
		"    -D  Include documentation strings.\n"
		"    -E  <string> Evaluate this program string.\n"
		"    -I  Install a new package.\n"
		"    -K  <path> set __package__ to this directory instead.\n"
		"    -M  <path> Pretend Qu is installed here.\n"
		"    -O  Optimize code.\n"
		"    -P  List all installed packages.\n"
		"    -R  Remove package specified by Filename.\n"
		"    -S  Run as daemon (forks).\n"
		"    -V  Disable all validators.\n"
		"    -W  Enable warnings.\n"
		"\nExamples\n"
		"    " QU_APP_EXENAME " -h\n"
		"    " QU_APP_EXENAME " -v\n"
		"    " QU_APP_EXENAME " foo/bar.qu\n"
		"    " QU_APP_EXENAME " foo/bar.qu one 2 three\n"
		"    " QU_APP_EXENAME " -P\n"
		"    " QU_APP_EXENAME " -R Uri\n"
		"    " QU_APP_EXENAME " -M /opt/qu1.05 foo/bar.qu\n"
	);
}



static void _qu_version ()
{
	printf ("%s\n", QU_APP_VERSION);
}



static int hasext (const char *path)
{
	char *e = strrchr (path, '.');
	char *p;

	if (e == NULL)
		return 0;
	p = strrchr (path, QU_PATH_SEPCHR);
	return (p == NULL) || (e > p);
}



static void list_packages ()
{
	qu_string s = qu_string_newf ("%s" QU_PATH_SEPSTR "remove", QU_APP_HOMEDIR);
	qu_set    a = qu_file_list (s, qu_string_byte ('f'), NULL, 1);
	long      b, i;

	if (a == 0)
	{
		puts ("No packages found.");
		return;
	}
	b = i = 0;
	while ((s = qu_set_walk (a, &b, &i)) != 0)
	{
		printf ("%s\n", s->str);
	}
	printf ("Total: %ld package(s) found\n", a->len);
}



int main (int argc, char **argv)
{
	char      *path = NULL, *eval = NULL;
	int        blocksig = 1, run = 1, trace = 0, gc = 1;
	int        plist = 0, premove = 0, pinstall = 0;
	qu_module  mod;
	qu_string  prog, arg0 = 0;
	int        i;
	char      *g;

	goto start;

  nope:

	_qu_usage ();
	exit (EXIT_FAILURE);

  start:

	argc--;
	for (i = 1; argc > 0 && argv[i][0] == '-'; i++, argc--)
	{
		g = argv[i];
		g++;
		while (*g)
		{
			switch (*g++)
			{
				case 'a':
					gc = 0;
					break;

				case 'd':
					QU_APP_DEBUG = 1;
					break;

				case 'e':
					trace = 1;
					QU_APP_LINE_NUMBER = 1;
					break;

				case 'g':
					blocksig = 0;
					break;

				case 'h':
					_qu_usage ();
					exit (EXIT_SUCCESS);

				case 'l':
					QU_APP_LINE_NUMBER = 1;
					break;

				case 'o':
					QU_APP_VERBOSE = 1;
					break;

				case 'p':
					QU_APP_KEEP_TREE = 1;
					break;

				case 'r':
					run = 0;
					break;

				case 't':
					QU_APP_INCLUDE_TEST = 1;
					break;

				case 'v':
					_qu_version ();
					exit (EXIT_SUCCESS);

				case 'x':
					QU_APP_FREE_LEXICAL = 0;
					break;

				case 'C':
					QU_APP_MORE_COMPILE_CHECK = 1;
					break;

				case 'D':
					QU_APP_WITH_DOC = 1;
					break;

				case 'E':
					if (*g != '\0' || argc-- < 1)
						goto nope;
					eval = argv[++i];
					goto next;

				case 'I':
					pinstall = 1;
					break;

				case 'K':
					if (*g != '\0' || argc-- < 1)
						goto nope;
					QU_APP_USEPACKAGE = argv[++i];
					goto next;

				case 'M':
					if (*g != '\0' || argc-- < 1)
						goto nope;
					QU_APP_HOMEDIR = argv[++i];
					goto next;

				case 'O':
					QU_APP_OPTIMIZE_CODE = 1;
					break;

				case 'P':
					plist = 1;
					break;

				case 'R':
					premove = 1;
					break;

				case 'S':
					QU_APP_DAEMON = 1;
					break;

				case 'V':
					QU_APP_NOVALIDATION = 1;
					break;

				case 'W':
					QU_APP_NO_WARNING = 0;
					break;

				default:
					goto nope;
			}
		}
	  next:
		continue; /* avoid stupid compiler warning */
	}

	if (QU_APP_DAEMON)
	{
		pid_t p;

		p = fork ();
		if (p < 0)
		{
			fprintf (stderr, "fork failed\n");
			exit (EXIT_FAILURE);
		}

		if (p != 0)
		{
			sleep (1);
			exit (EXIT_SUCCESS);
		}
	}

	if (plist + premove + pinstall > 1)
	{
		fprintf (stderr, "Cant list, remove and install a package at the same time.\n");
		exit (EXIT_FAILURE);
	}

	if (!plist && !eval)
	{
		if (argc-- < 1)
			goto nope;
		path = argv[i++];
	}

	setlocale (LC_ALL, "POSIX");

	__QuGcDisable = 1;

	qu_gc__init      ();
	qu_class__init   ();
	qu_sys__init     ();
	qu_compile__init ();
	qu_interp__init  ();
	qu_sig__init     (blocksig);

	if (plist)
	{
		list_packages ();
		goto done;
	}
	else if (premove)
	{
		prog = qu_string_newf ("%s" QU_PATH_SEPSTR "remove" QU_PATH_SEPSTR "%s.qu", QU_APP_HOMEDIR, path);
	}
	else if (pinstall)
	{
		prog = qu_string_newf ("%s" QU_PATH_SEPSTR "bin" QU_PATH_SEPSTR "install.qu", QU_APP_HOMEDIR);
		arg0 = qu_string_newz (path);
	}
	else if (eval)
	{
		prog = qu_string_newz (eval);
	}
	else
	{
		if (path == 0)
			goto nope;

		prog = qu_file_toabs (qu_string_newz (path), 0);
		if (prog == 0)
		{
			qu_thread_print_err ();
			exit (EXIT_FAILURE);
		}
		if (!hasext (path))
		{
			if (!qu_file_exist (prog, qu_string_byte ('f')))
				prog = qu_string_cat (prog, qu_string_new_staticz (".qu"), 0);
		}
	}

	/*
		Even if 'trace' is on, dont do it now to prevent printing of
		intermmediate exceptions during compilation.
	*/

	/* so that 'use Lightning' is not necessary */
	if (QU_APP_OPTIMIZE_CODE)
		qu_module_load (qu_string_new_staticf ("%s" QU_PATH_SEPSTR "lib" QU_PATH_SEPSTR "Lightning", QU_APP_HOMEDIR));

	mod = qu_compile_main (prog, eval == NULL);
	if (mod == 0)
	{
		fprintf (stderr, "%s\n", prog->str);
		qu_thread_print_err ();
		exit (EXIT_FAILURE);
	}

	__QuGcDisable = !gc;

	if (run)
	{
		if (arg0)
			qu_array_push (QuSys__arg__->value, arg0);

		while (argc-- > 0)
			qu_array_push (QuSys__arg__->value, qu_string_newz (argv [i++]));

		qu_array_freeze (QuSys__arg__->value);

		QuSysTrace = trace;

		if (qu_interp_run_main (mod) == 0)
		{
			if (__QuExceptionExitCode != -1)
				exit (__QuExceptionExitCode);

			/*
				Dont print the error if trace is on since it must have
				already been done by the interpreter. Otherwise we will
				print it twice.
			*/

			if (!QuSysTrace)
				qu_thread_print_err ();

			exit (EXIT_FAILURE);
		}
	}

  done:

	exit (EXIT_SUCCESS);
	return 0;
}
