/* threads.c - Copyright (c) Marc Krisnanto */

#include "threads.h"
#include "base.h"
#ifdef HAVE_SCHED_H
#include <sched.h>
#endif


/*
	Create a new thread.
*/
int thread_create (thread_t *tid, void *(*start)(void *), void *arg,
                   int stacksize, int detach)
{
    pthread_attr_t attr;

#if 0
	int stackdef;
#endif

	errno = pthread_attr_init (&attr);
	if (errno)
		return -1;

#if 0
	if (stacksize > 0)
	{
		if (pthread_attr_getstacksize (&attr, &stackdef) == 0)
		{
			if (stacksize < THREAD_STACK_MIN)
				stacksize = THREAD_STACK_MIN;
			else if (stacksize > stackdef)
				stacksize = stackdef;
			pthread_attr_setstacksize (&attr, stacksize);
		}
	}
#endif

	if (detach)
	{
		errno = pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
		if (errno)
			return -1;
	}

	errno = pthread_create (tid, &attr, start, arg);
	if (errno)
		return -1;

	return 0;
}



/*
	Give up CPU to other thread.
*/
#ifndef HAVE_SCHED_YIELD
void thread_yield ()
{
	struct timespec req, rem;
	req.tv_sec = 0;
	req.tv_nsec = 0;
	nanosleep (&req, &rem);
}
#endif
